"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCassandraUsers = exports.GetCassandraVCCs = exports.IntegrationCassandraSql = void 0;
const moment = require("moment");
const log4js = require("log4js");
let ab2str = require('arraybuffer-to-string');
let logger = log4js.getLogger('Cassandra');
let cassandraConn;
function IntegrationCassandraSql(app) {
    cassandraConn = app.get('cassandraConn');
}
exports.IntegrationCassandraSql = IntegrationCassandraSql;
function GetCassandraVCCs() {
    return new Promise((resolve, reject) => {
        cassandraConn.execute('SELECT * FROM VCCs', [], function (err, rows) {
            if (err) {
                let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
                logger.error(errorMsg);
                reject(err);
            }
            else {
                let VCCs = [];
                let vcc = '';
                let description = '';
                let nowUtc = moment().utc().toDate();
                let allegroIntegration = '';
                rows.map(row => {
                    description = '';
                    allegroIntegration = '';
                    row.cols.map(col => {
                        vcc = '';
                        if (ab2str(col.name, 'utf8') === 'Id')
                            vcc = ab2str(col.value, 'utf8');
                        if (ab2str(col.name, 'utf8') === 'Description')
                            description = ab2str(col.value, 'utf8');
                        if (ab2str(col.name, 'utf8') === 'AllegroIntegration')
                            allegroIntegration = ab2str(col.value, 'utf8');
                        if (vcc !== '') {
                            let VCC = {};
                            VCC.vcc = vcc;
                            VCC.id = vcc;
                            VCC.description = description;
                            VCC.allegroIntegration = allegroIntegration;
                            VCC.country = null;
                            VCC.language = null;
                            VCC.timeZone = null;
                            VCC.createdDate = nowUtc;
                            VCC.createdByUserId = 'admin';
                            VCC.lastModifiedDate = nowUtc;
                            VCC.lastModifiedByUserId = 'admin';
                            VCCs.push(VCC);
                        }
                    });
                });
                resolve(VCCs);
            }
        });
    });
}
exports.GetCassandraVCCs = GetCassandraVCCs;
function GetCassandraUsers() {
    return new Promise((resolve, reject) => {
        cassandraConn.execute('SELECT * FROM Users', [], function (err, rows) {
            if (err) {
                let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
                logger.error(errorMsg);
                reject(err);
            }
            else {
                resolve(rows);
            }
        });
    });
}
exports.GetCassandraUsers = GetCassandraUsers;
