"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDispositionCodes = exports.getCampaignWebdesignerURL = exports.getCampaigns = void 0;
const campaign_model_1 = require("../../../models/campaign.model");
const log4js = require("log4js");
const Knex = require("knex");
const knex_config_1 = require("../knex.config");
const disposition_code_model_1 = require("../../../models/disposition-code.model");
let logger = log4js.getLogger('ServerScripts');
const inconcertDB = Knex(knex_config_1.connections.inconcert);
const campaignTable = inconcertDB.raw('Campaign WITH (NOLOCK)');
const agentTable = inconcertDB.raw('Agent AS a WITH (NOLOCK)');
const userCampaignTable = inconcertDB.raw('UserCampaign AS u WITH (NOLOCK)');
const campaignPopupTable = inconcertDB.raw('CampaignPopup WITH (NOLOCK)');
const campaignDispositionCodes = inconcertDB.raw('CampaignGoalResultManagement WITH (NOLOCK)');
function getCampaigns(vcc, campaigns = null) {
    return new Promise(function (resolve, reject) {
        let builder = inconcertDB.select(['Id', 'Description']).from(campaignTable).where({
            VCC: vcc
        });
        if (campaigns && campaigns.length)
            builder = builder.whereIn('Campaign', campaigns);
        builder.then(result => {
            let c = [];
            if (result) {
                result.forEach(r => {
                    c.push(new campaign_model_1.Campaign(r.Id, r.Id));
                });
            }
            resolve(c);
        }, error => {
            reject(error);
        });
    });
}
exports.getCampaigns = getCampaigns;
function getCampaignWebdesignerURL(vcc, campaignId, interactionType) {
    return new Promise(function (resolve, reject) {
        let builder = inconcertDB.select('PopupData').from(campaignPopupTable).where({
            VCC: vcc,
            PopupType: 'WEBDESIGNER',
            CampaignId: campaignId
        }).whereIn('InteractionType', [interactionType, 'ALL']);
        builder.then(result => {
            let url = '';
            if (result && result.length > 0) {
                url = result[0].PopupData;
            }
            resolve(url);
        }, error => {
            reject(error);
        });
    });
}
exports.getCampaignWebdesignerURL = getCampaignWebdesignerURL;
function getDispositionCodes(vcc, campaigns) {
    return new Promise(function (resolve, reject) {
        logger.debug('[MMProdat::getDispositionCodes] Vcc received: ' + vcc);
        logger.debug('[MMProdat::getDispositionCodes] Campaigns received: ' + JSON.stringify(campaigns));
        let builder = inconcertDB.select(['CampaignId', 'Code', 'Description', 'Category1', 'Category2', 'Category3']).from(campaignDispositionCodes).where({
            VCC: vcc
        });
        if (campaigns && campaigns.length > 0)
            builder.whereIn('CampaignId', campaigns);
        builder.then(result => {
            let dispositionCodes = [];
            logger.debug('[MMProdat::getDispositionCodes] Received result: ' + JSON.stringify(result));
            result.forEach(row => {
                let treepath = (row.Category1 ? row.Category1 + '/' : '') + (row.Category2 ? row.Category2 + '/' : '') + (row.Category3 ? row.Category3 + '/' : '') + row.Description;
                let dp = new disposition_code_model_1.DispositionCode(vcc, row.Description, treepath);
                dp.code = row.Code;
                dispositionCodes.push(dp);
            });
            resolve(dispositionCodes);
        }, error => {
            logger.error('[MMProdat::getDispositionCodes] requesting error: ' + error);
            reject(error);
        });
    });
}
exports.getDispositionCodes = getDispositionCodes;
exports.default = {
    getCampaigns,
    getCampaignWebdesignerURL,
    getDispositionCodes
};
