"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllegroBillingInteractionsForEvaluationPlan = exports.GetAllegroBillingInteractions = exports.GetAllegroInteractionVideos = exports.GetAllegroManagementResults = exports.GetAllegroCampaignMembers = exports.GetAllegroCampaigns = exports.GetAllegroAgents = exports.GetAllegroSupervisors = exports.AllegroMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
const arrays_shared_1 = require("../../shared/arrays.shared");
let logger = log4js.getLogger('Sql');
let sqlConn;
function AllegroMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AllegroMSSql = AllegroMSSql;
function GetAllegroSupervisors(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Bit, type)
        .execute('GetAllegroSupervisors')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let supervisors = [];
        let uniqueUserIds = arrays_shared_1.GetUniqueFromObjectArray(recordset, 'userId');
        uniqueUserIds.map(uui => {
            let supervisor = {};
            supervisor.userId = uui.userId;
            supervisor.userName = uui.userName;
            supervisor.campaigns = [];
            let currentSupervisor = recordset.filter(r => r.userId === uui.userId);
            let currentSupervisorCampaigns = arrays_shared_1.GetUniqueFromObjectArray(currentSupervisor, 'campaign');
            currentSupervisorCampaigns.map(csc => {
                let campaign = {};
                campaign.campaignId = csc.campaignId;
                campaign.workGroups = [];
                currentSupervisor.filter(cs => cs.campaignId === campaign.campaign).map(cs => {
                    if (cs.workGroupId)
                        campaign.workGroups.push(cs.workGroupId);
                });
                supervisor.campaigns.push(campaign);
            });
            supervisors.push(supervisor);
        });
        return Promise.resolve(supervisors);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroSupervisors = GetAllegroSupervisors;
function GetAllegroAgents(ip, vcc, type, campaign) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('Campaigns', sql.VarChar(sql.MAX), campaign)
        .execute('GetAllegroAgents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroAgents = GetAllegroAgents;
function GetAllegroCampaigns(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .execute('GetAllegroCampaigns')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroCampaigns = GetAllegroCampaigns;
function GetAllegroCampaignMembers(ip, vcc, campaigns, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns.toString())
        .input('Type', sql.Int, type)
        .execute('GetAllegroCampaignMembers')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let campaignsMembers = [];
        campaigns.map(c => {
            let campaignMembers = {};
            campaignMembers.id = c;
            let agents = [];
            let supervisors = [];
            let agentWg = [];
            let supervisorWg = [];
            recordset.filter(r => r.campaignId === c).map(r => {
                if (r.isAgent) {
                    agents.push(r.userId);
                    let agent = {};
                    agent.id = r.userId;
                    agent.workGroupId = r.agentWorkgroup ? r.agentWorkgroup : null;
                    agentWg.push(agent);
                }
                if (r.isSupervisor) {
                    supervisors.push(r.userId);
                    let alreadyAdded = false;
                    let supervisor = {};
                    if (supervisorWg.find(swg => swg.id === r.userId)) {
                        alreadyAdded = true;
                        supervisor = supervisorWg.find(swg => swg.id === r.userId);
                    }
                    supervisor.id = r.userId;
                    supervisor.workGroups = supervisor.workGroups ? supervisor.workGroups : [];
                    if (r.supervisorWorkgroup)
                        supervisor.workGroups.push(r.supervisorWorkgroup);
                    if (!alreadyAdded)
                        supervisorWg.push(supervisor);
                }
            });
            campaignMembers.supervisorAgents = [];
            supervisorWg.map(swg => {
                let supervisorAgents = {};
                supervisorAgents.id = swg.id;
                supervisorAgents.agents = [];
                if (swg.workGroups.length) {
                    swg.workGroups.map(wg => {
                        agentWg.filter(awg => awg.workGroupId === wg).map(awg => supervisorAgents.agents.push(awg.id));
                    });
                }
                else {
                    agentWg.map(awg => supervisorAgents.agents.push(awg.id));
                }
                campaignMembers.supervisorAgents.push(supervisorAgents);
            });
            campaignMembers.agents = arrays_shared_1.GetUniqueFromArray(agents);
            campaignMembers.supervisors = arrays_shared_1.GetUniqueFromArray(supervisors);
            campaignsMembers.push(campaignMembers);
        });
        return Promise.resolve(campaignsMembers);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroCampaignMembers = GetAllegroCampaignMembers;
function GetAllegroManagementResults(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .execute('GetAllegroManagementResults')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroManagementResults = GetAllegroManagementResults;
function GetAllegroInteractionVideos(ip, vcc, type, interactionIds = '') {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .execute('GetAllegroInteractionVideos')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroInteractionVideos = GetAllegroInteractionVideos;
function GetAllegroBillingInteractions(vcc, ip, type, secondsPerRequest, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, formats) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Ip', sql.VarChar(50), ip)
        .input('Type', sql.Int, type)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('Campaigns', sql.NVarChar(sql.MAX), campaigns)
        .input('Agents', sql.NVarChar(sql.MAX), agents)
        .input('Origins', sql.NVarChar(sql.MAX), origins)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('MinDuration', sql.Int, minDuration)
        .input('MaxDuration', sql.Int, maxDuration)
        .input('Results', sql.NVarChar(sql.MAX), results)
        .input('Formats', sql.NVarChar(50), formats)
        .execute('GetAllegroBillingInteractions')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let allegroBillingInteractions = (recordset && recordset.length > 0 ? recordset[0] : null);
        allegroBillingInteractions.interactionType = 'CALL';
        logger.info("[AllegroMSSql::GetAllegroBillingInteractions] Total Records: " + allegroBillingInteractions.quantity);
        return Promise.resolve(allegroBillingInteractions);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroBillingInteractions = GetAllegroBillingInteractions;
function GetAllegroBillingInteractionsForEvaluationPlan(vcc, processId, secondsPerRequest) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .execute('GetAllegroBillingInteractionsForEvaluationPlan')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let result = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info("[AllegroMSSql::GetAllegroBillingInteractionsForEvaluationPlan] Total Records :" + result.length);
        return Promise.resolve(result);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroBillingInteractionsForEvaluationPlan = GetAllegroBillingInteractionsForEvaluationPlan;
