"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAzureSubscription = exports.SaveAzureSubscription = exports.SaveAzureSubscriptionKey = exports.GetAzureSubscriptionKeys = exports.GetAzureSubscriptionKeyByKey = exports.GetAzureSubscriptionKeyByName = exports.AzureSubscriptionMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function AzureSubscriptionMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AzureSubscriptionMSSql = AzureSubscriptionMSSql;
function GetAzureSubscriptionKeyByName(VCC, subscriptionName) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), VCC)
        .input('SubscriptionName', sql.VarChar(50), subscriptionName)
        .execute('GetAzureSubscriptionKeyByName')
        .then((sqlResult) => {
        logger.info('[AzureSubscriptionKeyMSSql::GetAzureSubscriptionKeyByName] Received result from SP GetAzureSubscriptionKeyByName: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let azureSubscriptionKey = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(azureSubscriptionKey);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AzureSubscriptionKeyMSSql::GetAzureSubscriptionKeyByName] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAzureSubscriptionKeyByName = GetAzureSubscriptionKeyByName;
function GetAzureSubscriptionKeyByKey(VCC, subscriptionKey) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), VCC)
        .input('SubscriptionKey', sql.VarChar(50), subscriptionKey)
        .execute('GetAzureSubscriptionKeyByKey')
        .then((sqlResult) => {
        logger.info('[AzureSubscriptionKeyMSSql::GetAzureSubscriptionKeyByKey] Received result from SP GetAzureSubscriptionKeyByKey: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let azureSubscriptionKey = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(azureSubscriptionKey);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AzureSubscriptionKeyMSSql::GetAzureSubscriptionKeyByKey] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAzureSubscriptionKeyByKey = GetAzureSubscriptionKeyByKey;
function GetAzureSubscriptionKeys(instance) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .execute('GetAzureSubscriptionKeys')
        .then((sqlResult) => {
        logger.info('[AzureSubscriptionKeyMSSql::GetAzureSubscriptionKeys] Received result from SP GetAzureSubscriptionKeys: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AzureSubscriptionKeyMSSql::GetAzureSubscriptionKeys] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAzureSubscriptionKeys = GetAzureSubscriptionKeys;
function SaveAzureSubscriptionKey(vcc, subscriptionKey, subscriptionRegion, isDemo, subscriptionName) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SubscriptionKey', sql.VarChar(50), subscriptionKey)
        .input('SubscriptionRegion', sql.VarChar(50), subscriptionRegion)
        .input('SubscriptionName', sql.VarChar(50), subscriptionName)
        .input('IsDemo', sql.Bit, isDemo)
        .execute('SaveAzureSubscriptionKey')
        .then((sqlResult) => {
        logger.info('[AzureSubscriptionKeyMSSql::SaveAzureSubscriptionKey] Received result from SP SaveAzureSubscriptionKey: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AzureSubscriptionKeyMSSql::SaveAzureSubscriptionKey] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveAzureSubscriptionKey = SaveAzureSubscriptionKey;
function SaveAzureSubscription(vcc, azureSubscription, resourceGroup, accountName) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AzureSubscription', sql.VarChar(50), azureSubscription)
        .input('ResourceGroup', sql.VarChar(50), resourceGroup)
        .input('AccountName', sql.VarChar(50), accountName)
        .execute('SaveAzureSubscription')
        .then((sqlResult) => {
        logger.info('[AzureSubscriptionKeyMSSql::SaveAzureSubscription] Received result from SP SaveAzureSubscription: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AzureSubscriptionKeyMSSql::SaveAzureSubscription] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveAzureSubscription = SaveAzureSubscription;
function GetAzureSubscription(VCC) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), VCC)
        .execute('GetAzureSubscription')
        .then((sqlResult) => {
        logger.info('[AzureSubscriptionKeyMSSql::GetAzureSubscription] Received result from SP GetAzureSubscription: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let azureSubscription = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(azureSubscription);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AzureSubscriptionKeyMSSql::GetAzureSubscription] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAzureSubscription = GetAzureSubscription;
