"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetBatchScheduledTemplateItemResults = exports.GetBatchScheduledTemplateResults = exports.DeleteBatchScheduleExecution = exports.SaveBatchScheduleExecution = exports.GetBatchScheduleExecutionsByBatchType = exports.GetBatchScheduleExecutionsByCriteria = exports.DeleteBatchSchedule = exports.SaveBatchSchedule = exports.GetBatchScheduleById = exports.GetBatchSchedulesByCriteria = exports.GetBatchSchedulesSummary = exports.BatchScheduleMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function BatchScheduleMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.BatchScheduleMSSql = BatchScheduleMSSql;
function GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression, batchType) {
    let recordset = null;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(500), searchExpression)
        .input('BatchType', sql.VarChar(50), batchType)
        .execute('GetBatchSchedulesSummary')
        .then((sqlResult) => {
        recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[BatchScheduleMSSql::GetBatchSchedulesSummary] getting schedules recordset:', recordset.length);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchSchedulesSummary] getting errorMsg:', errorMsg, 'of GetBatchSchedules');
        return Promise.reject(err);
    });
}
exports.GetBatchSchedulesSummary = GetBatchSchedulesSummary;
function GetBatchSchedulesByCriteria(vcc, searchCriteria, searchExpression, extendedContent = false) {
    let recordset = null;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(500), searchExpression)
        .input('ExtendedContent', sql.Bit, extendedContent)
        .execute('GetBatchSchedules')
        .then((sqlResult) => {
        recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] getting schedules recordset:', recordset.length);
        if (recordset.length && extendedContent) {
            recordset.map((r) => {
                try {
                    r.batchData = JSON.parse(r.batchData);
                }
                catch (error) {
                    logger.info('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] Failed to parse content.', r.batchData);
                    r.batchData = r.batchData;
                }
            });
        }
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] getting errorMsg:', errorMsg, 'of GetBatchSchedules');
        return Promise.reject(err);
    });
}
exports.GetBatchSchedulesByCriteria = GetBatchSchedulesByCriteria;
function GetBatchScheduleById(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), id)
        .execute('GetBatchScheduleById')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let process = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(process);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetBatchScheduleById = GetBatchScheduleById;
function SaveBatchSchedule(vcc, scheduleData, userId) {
    let saveBatchScheduleResult = false;
    let batchData = typeof scheduleData.batchData === 'string' ? scheduleData.batchData : JSON.stringify(scheduleData.batchData);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), scheduleData.id)
        .input('BatchType', sql.VarChar(50), scheduleData.batchType)
        .input('Frequency', sql.VarChar(50), scheduleData.frequency)
        .input('Weekday', sql.Int, scheduleData.weekDay)
        .input('Monthday', sql.Int, scheduleData.monthDay)
        .input('DaysInterval', sql.Int, scheduleData.daysInterval)
        .input('HoursInterval', sql.Int, scheduleData.hoursInterval)
        .input('DaysToExpire', sql.Int, scheduleData.daysToExpire)
        .input('ExecutionHour', sql.VarChar(50), scheduleData.executionHour)
        .input('FirstExecutionDate', sql.DateTime, scheduleData.firstExecutionDate)
        .input('ExecutionDelay', sql.Int, scheduleData.executionDelay)
        .input('StartDate', sql.DateTime, scheduleData.startDate)
        .input('EndDate', sql.DateTime, scheduleData.endDate)
        .input('BatchData', sql.VarChar(sql.MAX), batchData)
        .input('ProcessFiltersId', sql.VarChar(50), scheduleData.processFiltersId)
        .input('Status', sql.VarChar(50), scheduleData.status)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveBatchSchedule')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        saveBatchScheduleResult = output && output.result === 1 ? true : false;
        let promiseArray = [];
        if (scheduleData.executionList && scheduleData.executionList.length)
            scheduleData.executionList.map(el => promiseArray.push(SaveBatchScheduleExecution(vcc, el, userId)));
        return Promise.all(promiseArray);
    })
        .then((_) => {
        return Promise.resolve(saveBatchScheduleResult);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing error ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveBatchSchedule = SaveBatchSchedule;
function DeleteBatchSchedule(vcc, batchType, batchScheduleId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('BatchType', sql.VarChar(50), batchType)
        .input('ID', sql.VarChar(50), batchScheduleId)
        .execute('DeleteBatchSchedule')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteBatchSchedule = DeleteBatchSchedule;
function GetBatchScheduleExecutionsByCriteria(vcc, searchCriteria, searchExpression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(500), searchExpression)
        .execute('GetBatchScheduleExecutionsByCriteria')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        logger.info('[BatchScheduleMSSql::GetBatchScheduleExecutionsByCriteria] getting schedules recordset:', recordset.length);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchScheduleExecutionsByCriteria] getting errorMsg:', errorMsg, 'of GetBatchScheduleExecutionsByCriteria');
        return Promise.reject(err);
    });
}
exports.GetBatchScheduleExecutionsByCriteria = GetBatchScheduleExecutionsByCriteria;
function GetBatchScheduleExecutionsByBatchType(vcc, batchScheduleId, batchType) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('BatchScheduleId', sql.VarChar(50), batchScheduleId)
        .input('BatchType', sql.VarChar(500), batchType)
        .execute('GetBatchScheduleExecutionsByBatchType')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        logger.info('[BatchScheduleMSSql::GetBatchScheduleExecutionsByBatchType] getting schedules recordset:', recordset.length);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchScheduleExecutionsByBatchType] getting errorMsg:', errorMsg, 'of GetBatchScheduleExecutionsByCriteria');
        return Promise.reject(err);
    });
}
exports.GetBatchScheduleExecutionsByBatchType = GetBatchScheduleExecutionsByBatchType;
function SaveBatchScheduleExecution(vcc, scheduleExecution, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), scheduleExecution.id)
        .input('BatchScheduleId', sql.VarChar(50), scheduleExecution.batchScheduleId)
        .input('BatchId', sql.VarChar(50), scheduleExecution.batchId)
        .input('ExecutionDate', sql.DateTime, scheduleExecution.executionDate)
        .input('Order', sql.Int, scheduleExecution.order)
        .input('Status', sql.VarChar(50), scheduleExecution.status)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveBatchScheduleExecution')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing error ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveBatchScheduleExecution = SaveBatchScheduleExecution;
function DeleteBatchScheduleExecution(vcc, scheduleExecutionId) {
    logger.info('[IntentMSSql::DeleteEntity] getting vcc:', vcc, 'executionId:', scheduleExecutionId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), scheduleExecutionId)
        .execute('DeleteBatchScheduleExecution')
        .then((sqlResult) => {
        logger.info('[BatchScheduleMSSql::DeleteBatchScheduleExecution] getting sqlResult.recordsets:', sqlResult.recordsets, 'of DeleteBatchScheduleExecution');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::DeleteBatchScheduleExecution] getting errorMsg:', errorMsg, 'of DeleteEntity');
        return Promise.reject(err);
    });
}
exports.DeleteBatchScheduleExecution = DeleteBatchScheduleExecution;
function GetBatchScheduledTemplateResults(vcc, batchScheduleIds = [], filterData) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('BatchScheduledIds', sql.VarChar(sql.MAX), batchScheduleIds.toString())
        .input('StartDate', sql.DateTime, filterData.startDate)
        .input('EndDate', sql.DateTime, filterData.endDate)
        .execute('GetBatchScheduledTemplateResults')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        logger.info('[BatchScheduleMSSql::GetBatchScheduledTemplateResults] getting sqlResult.recordsets:', sqlResult.recordsets);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchScheduledTemplateResults] getting errorMsg:', errorMsg, 'of GetBatchScheduledTemplateResults');
        return Promise.reject(err);
    });
}
exports.GetBatchScheduledTemplateResults = GetBatchScheduledTemplateResults;
function GetBatchScheduledTemplateItemResults(vcc, filterData) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('BatchScheduledId', sql.VarChar(50), filterData.batchScheduledId)
        .input('StartDate', sql.DateTime, filterData.startDate)
        .input('EndDate', sql.DateTime, filterData.endDate)
        .input('Category1', sql.VarChar(50), filterData.category1)
        .input('Category2', sql.VarChar(50), filterData.category2)
        .input('Description', sql.VarChar(50), filterData.description)
        .execute('GetBatchScheduledTemplateItemResults')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        logger.info('[BatchScheduleMSSql::GetBatchScheduledTemplateItemResults] getting sqlResult.recordsets:', sqlResult.recordsets);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchScheduledTemplateItemResults] getting errorMsg:', errorMsg, 'of GetBatchScheduledTemplateItemResults');
        return Promise.reject(err);
    });
}
exports.GetBatchScheduledTemplateItemResults = GetBatchScheduledTemplateItemResults;
