"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteCustomFieldResults = exports.SaveCustomFieldResult = exports.GetCustomFieldResults = exports.GetCustomFieldsUsage = exports.DeleteCustomFields = exports.DeleteCustomField = exports.SaveCustomField = exports.GetCustomField = exports.GetCustomFields = exports.CustomFieldMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
const response_1 = require("../utils/response");
let logger = log4js.getLogger('Sql');
let sqlConn;
function CustomFieldMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.CustomFieldMSSql = CustomFieldMSSql;
function GetCustomFields(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetCustomFields')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomFields = GetCustomFields;
function GetCustomField(vcc, customFieldId, evaluationPlanId = null) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(100), customFieldId)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .execute('GetCustomField')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let customField = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(customField);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomField = GetCustomField;
function SaveCustomField(vcc, id, description, type, label, createdByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(100), id)
        .input('Description', sql.VarChar(250), description)
        .input('Type', sql.VarChar(25), type)
        .input('Label', sql.VarChar(100), label)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .execute('SaveCustomField')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomField = SaveCustomField;
function DeleteCustomField(vcc, customFieldId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldId', sql.VarChar(100), customFieldId)
        .execute('DeleteCustomField')
        .then((sqlResult) => response_1.responseSaveOrDelete(sqlResult.recordsets))
        .catch((err) => response_1.responseError(err));
}
exports.DeleteCustomField = DeleteCustomField;
function DeleteCustomFields(vcc, customFieldIds) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldIds', sql.VarChar(sql.MAX), customFieldIds)
        .execute('DeleteCustomFields')
        .then((sqlResult) => response_1.responseSaveOrDelete(sqlResult.recordsets))
        .catch((err) => response_1.responseError(err));
}
exports.DeleteCustomFields = DeleteCustomFields;
function GetCustomFieldsUsage(vcc, customFieldIds) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldIds', sql.VarChar(sql.MAX), customFieldIds)
        .execute('GetCustomFieldsUsage')
        .then((sqlResult) => response_1.responseArrayGet(sqlResult.recordsets))
        .catch((err) => response_1.responseError(err));
}
exports.GetCustomFieldsUsage = GetCustomFieldsUsage;
function GetCustomFieldResults(vcc, customFieldId, evaluationPlanId = null) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldId', sql.VarChar(100), customFieldId)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .execute('GetCustomFieldResults')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCustomFieldResults = GetCustomFieldResults;
function SaveCustomFieldResult(vcc, id, description, order, customFieldId, createdByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(100), id)
        .input('Description', sql.VarChar(150), description)
        .input('Order', sql.Int, order)
        .input('CustomFieldId', sql.VarChar(150), customFieldId)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .execute('SaveCustomFieldResult')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCustomFieldResult = SaveCustomFieldResult;
function DeleteCustomFieldResults(vcc, customFieldId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('CustomFieldId', sql.VarChar(100), customFieldId)
        .execute('DeleteCustomFieldResults')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteCustomFieldResults = DeleteCustomFieldResults;
