"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetSoundLikesFromDictionaryWords = exports.GetDictionaryUsage = exports.DeleteDictionaryWordSoundLike = exports.SaveDictionaryWordSoundLike = exports.GetDictionaryWordSoundLikes = exports.DeleteDictionaryWord = exports.SaveDictionaryWord = exports.GetDictionaryWords = exports.GetDictionaryWord = exports.SaveDictionary = exports.GetDictionaries = exports.GetDictionary = exports.DictionaryMSSql = void 0;
const log4js = require("log4js");
const response_1 = require("../utils/response");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function DictionaryMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.DictionaryMSSql = DictionaryMSSql;
function GetDictionary(vcc, dictionaryId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryId', sql.VarChar(50), dictionaryId)
        .execute('GetDictionary')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let dictionary = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(dictionary);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDictionary = GetDictionary;
function GetDictionaries(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetDictionaries')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDictionaries = GetDictionaries;
function SaveDictionary(vcc, name, description, language, isSystemDictionary, saveUser) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Description', sql.VarChar(250), description)
        .input('Language', sql.VarChar(5), language)
        .input('IsSystemDictionary', sql.Bit, isSystemDictionary)
        .input('SaveUser', sql.VarChar(50), saveUser)
        .execute('SaveDictionary')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveDictionary = SaveDictionary;
function GetDictionaryWord(vcc, dictionaryWordId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryWordId', sql.VarChar(50), dictionaryWordId)
        .execute('GetDictionaryWord')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let dictionaryWord = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(dictionaryWord);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDictionaryWord = GetDictionaryWord;
function GetDictionaryWords(vcc, search_criteria, search_expression, dictionaryId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryId', sql.VarChar(50), dictionaryId)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetDictionaryWords')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDictionaryWords = GetDictionaryWords;
function SaveDictionaryWord(vcc, dictionaryId, id, word, saveUser) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryId', sql.VarChar(50), dictionaryId)
        .input('ID', sql.VarChar(50), id)
        .input('Word', sql.VarChar(50), word)
        .input('SaveUser', sql.VarChar(50), saveUser)
        .execute('SaveDictionaryWord')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveDictionaryWord = SaveDictionaryWord;
function DeleteDictionaryWord(vcc, dictionaryWordId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryWordId', sql.VarChar(50), dictionaryWordId)
        .execute('DeleteDictionaryWord')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteDictionaryWord = DeleteDictionaryWord;
function GetDictionaryWordSoundLikes(vcc, dictionaryWordId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryWordIds', sql.VarChar(sql.MAX), dictionaryWordId)
        .execute('GetDictionaryWordSoundLikes')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDictionaryWordSoundLikes = GetDictionaryWordSoundLikes;
function SaveDictionaryWordSoundLike(vcc, dictionaryWordId, id, soundLike, saveUser) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryWordId', sql.VarChar(50), dictionaryWordId)
        .input('ID', sql.VarChar(50), id)
        .input('SoundLike', sql.VarChar(50), soundLike)
        .input('SaveUser', sql.VarChar(50), saveUser)
        .execute('SaveDictionaryWordSoundLike')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveDictionaryWordSoundLike = SaveDictionaryWordSoundLike;
function DeleteDictionaryWordSoundLike(vcc, dictionaryWordId, dictionaryWordSoundLikeId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryWordId', sql.VarChar(50), dictionaryWordId)
        .input('DictionaryWordSoundLikeId', sql.VarChar(50), dictionaryWordSoundLikeId)
        .execute('DeleteDictionaryWordSoundLike')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteDictionaryWordSoundLike = DeleteDictionaryWordSoundLike;
function GetDictionaryUsage(vcc, dictionaryId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryId', sql.VarChar(50), dictionaryId)
        .execute('GetDictionaryUsage')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDictionaryUsage = GetDictionaryUsage;
function GetSoundLikesFromDictionaryWords(vcc, dictionaryWordId, wordsSoundlike) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DictionaryWordId', sql.VarChar(sql.MAX), dictionaryWordId)
        .input('SoundLikes', sql.VarChar(sql.MAX), wordsSoundlike)
        .execute('GetSoundLikesFromDictionaryWords')
        .then((sqlResult) => response_1.responseArrayGet(sqlResult.recordsets))
        .catch((err) => response_1.responseError(err));
}
exports.GetSoundLikesFromDictionaryWords = GetSoundLikesFromDictionaryWords;
