"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteEntity = exports.SaveEntity = exports.GetEntity = exports.GetEntities = exports.EntityMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function EntityMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.EntityMSSql = EntityMSSql;
function GetEntities(vcc, appId, search_criteria, search_expression, entities) {
    logger.info('[EntityMSSql::GetEntities] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'entities:', entities, 'appid', appId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(100), appId)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Entities', sql.NVarChar(sql.MAX), entities)
        .execute('GetEntities')
        .then((sqlResult) => {
        logger.info('[EntityMSSql::GetEntities] getting sqlResult.recordsets:', sqlResult.recordsets, 'of GetEntities');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[EntityMSSql::GetEntities] getting recordset:', recordset, 'of GetEntities');
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EntityMSSql::GetEntities] getting errorMsg:', errorMsg, 'of GetEntities');
        return Promise.reject(err);
    });
}
exports.GetEntities = GetEntities;
function GetEntity(vcc, appId, id, name) {
    logger.info('[EntityMSSql::GetEntity] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('ID', sql.VarChar(50), id)
        .input('Name', sql.VarChar(50), name)
        .execute('GetEntity')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let intent = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(intent);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EntityMSSql::GetEntity] getting errorMsg:', errorMsg, 'of GetEntity');
        return Promise.reject(err);
    });
}
exports.GetEntity = GetEntity;
function SaveEntity(vcc, appId, id, name, type, createdByUserId, lastModifiedByUserId) {
    logger.info('[EntityMSSql::SaveEntity] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('ID', sql.VarChar(50), id)
        .input('Name', sql.VarChar(50), name)
        .input('Type', sql.VarChar(50), type)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveEntity')
        .then((sqlResult) => {
        logger.info('[EntityMSSql::SaveEntity] getting sqlResult.recordsets:', sqlResult.recordsets, 'of SaveEntity');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EntityMSSql::SaveEntity] getting errorMsg:', errorMsg, 'of SaveEntity');
        return Promise.reject(err);
    });
}
exports.SaveEntity = SaveEntity;
function DeleteEntity(vcc, appId, intentId) {
    logger.info('[IntentMSSql::DeleteEntity] getting vcc:', vcc, 'entityId:', intentId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('ID', sql.VarChar(50), intentId)
        .execute('DeleteEntity')
        .then((sqlResult) => {
        logger.info('[IntentEntityMSSqlMSSql::DeleteEntity] getting sqlResult.recordsets:', sqlResult.recordsets, 'of DeleteEntity');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[EntityMSSql::DeleteEntity] getting recordset:', recordset, 'of DeleteEntity');
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[EntityMSSql::DeleteEntity] getting output:', output, 'of DeleteEntity');
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EntityMSSql::DeleteEntity] getting errorMsg:', errorMsg, 'of DeleteEntity');
        return Promise.reject(err);
    });
}
exports.DeleteEntity = DeleteEntity;
