"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteException = exports.SaveException = exports.GetException = exports.GetExceptions = exports.ExceptionMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function ExceptionMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.ExceptionMSSql = ExceptionMSSql;
function GetExceptions(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetExceptions')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetExceptions = GetExceptions;
function GetException(VCC, idException, description) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), VCC)
        .input('ID', sql.VarChar(50), idException)
        .input('Description', sql.VarChar(50), description)
        .execute('GetException')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let exception = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(exception);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetException = GetException;
function SaveException(VCC, exception) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), VCC)
        .input('ID', sql.VarChar(50), exception.id)
        .input('Description', sql.VarChar(100), exception.description)
        .input('CreatedDate', sql.DateTime, exception.createdDate)
        .input('CreatedByUserId', sql.VarChar(50), exception.createdByUserId)
        .input('LastModifiedDate', sql.DateTime, exception.lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), exception.lastModifiedByUserId)
        .execute('SaveException')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveException = SaveException;
function DeleteException(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .execute('DeleteException')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteException = DeleteException;
