"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteGenerativeIntent = exports.SaveGenerativeIntentId = exports.SaveGenerativeIntent = exports.GetGenerativeIntents = exports.GenerativeIntentMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function GenerativeIntentMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.GenerativeIntentMSSql = GenerativeIntentMSSql;
function GetGenerativeIntents(vcc, searchCriteria, searchExpression, intentIds) {
    let logLabel = `[GenerativeIntentMSSql::GetGenerativeIntents] [vcc => ${vcc}]: `;
    if (searchCriteria && searchExpression)
        logger.info(`${logLabel}[search_criteria::search_expression => ${searchCriteria}::${searchExpression}]`);
    if (intentIds?.length)
        logger.info(`${logLabel}[appIds => ${intentIds}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .input('IntentIds', sql.NVarChar(sql.MAX), intentIds)
        .execute('GetGenerativeIntents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info(`${logLabel}getting recordset: `, recordset?.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenerativeIntents = GetGenerativeIntents;
function SaveGenerativeIntent(vcc, intent, currentUser) {
    let logLabel = `[GenerativeIntentMSSql::SaveGenerativeIntent] [vcc::name => ${vcc}::${intent.name}]: `;
    if (intent.id)
        logger.info(`${logLabel}[appId => ${intent.id}]`);
    if (intent.description)
        logger.info(`${logLabel}[description => ${intent.description}]`);
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), intent.appId)
        .input('Id', sql.VarChar(50), intent.id)
        .input('Name', sql.VarChar(50), intent.name)
        .input('Description', sql.VarChar(250), intent.description)
        .input('CurrentUser', sql.VarChar(50), currentUser)
        .execute('SaveGenerativeIntent')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveGenerativeIntent = SaveGenerativeIntent;
function SaveGenerativeIntentId(vcc, appId, oldId, newId, currentUser) {
    let logLabel = `[GenerativeMSSql::SaveGenerativeIntentId] [vcc::oldId::newId => ${vcc}::${oldId}::${newId}]: `;
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('appId', sql.VarChar(50), appId)
        .input('OldId', sql.VarChar(50), oldId)
        .input('NewId', sql.VarChar(50), newId)
        .input('CurrentUser', sql.VarChar(50), currentUser)
        .execute('SaveGenerativeIntentId')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveGenerativeIntentId = SaveGenerativeIntentId;
function DeleteGenerativeIntent(vcc, intent, currentUser) {
    let logLabel = `[GenerativeIntentMSSql::DeleteGenerativeIntent] [vcc::name => ${vcc}::${intent.name}]: `;
    if (intent.id)
        logger.info(`${logLabel}[id => ${intent.id}]`);
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), intent.appId)
        .input('Id', sql.VarChar(50), intent.id)
        .execute('DeleteGenerativeIntent')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteGenerativeIntent = DeleteGenerativeIntent;
