"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetUniqueModelsByIntents = exports.DeleteGenerative = exports.SaveGenerativeId = exports.SaveGenerative = exports.GetGenerative = exports.GetGeneratives = exports.GenerativeMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function GenerativeMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.GenerativeMSSql = GenerativeMSSql;
function GetGeneratives(vcc, searchCriteria, searchExpression, appIds) {
    let logLabel = `[GenerativeMSSql::GetGeneratives] [vcc => ${vcc}]: `;
    if (searchCriteria && searchExpression)
        logger.info(`${logLabel}[search_criteria::search_expression => ${searchCriteria}::${searchExpression}]`);
    if (appIds?.length)
        logger.info(`${logLabel}[appIds => ${appIds}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .input('AppIds', sql.NVarChar(sql.MAX), appIds)
        .execute('GetGeneratives')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info(`${logLabel}getting recordset: `, recordset?.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGeneratives = GetGeneratives;
function GetGenerative(vcc, searchCriteria, searchExpression) {
    let logLabel = `[GenerativeMSSql::GetGenerative] [vcc::${searchCriteria} => ${vcc}::${searchExpression}]: `;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .execute('GetGenerative')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        let generative = (recordset?.length > 0 ? recordset[0] : null);
        if (sqlResult?.recordsets?.length > 1) {
            let intents = (sqlResult.recordsets?.length > 1 ? sqlResult.recordsets[1] : []);
            let examples = (sqlResult.recordsets?.length > 2 ? sqlResult.recordsets[2] : []);
            intents.forEach(i => {
                i.examples = examples.filter(e => e.intentId === i.id);
            });
            generative.intents = intents;
        }
        logger.info(`${logLabel}getting recordset:`, generative);
        return Promise.resolve(generative);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenerative = GetGenerative;
function SaveGenerative(vcc, generative, currentUser) {
    let logLabel = `[GenerativeMSSql::SaveGenerative] [vcc::name => ${vcc}::${generative.name}]: `;
    if (generative.id)
        logger.info(`${logLabel}[appId => ${generative.id}]`);
    if (generative.description)
        logger.info(`${logLabel}[description => ${generative.description}]`);
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), generative.name)
        .input('AppId', sql.VarChar(50), generative.id)
        .input('Description', sql.VarChar(250), generative.description)
        .input('Language', sql.VarChar(5), generative.language)
        .input('IsPublished', sql.Bit, generative.isPublished)
        .input('NeedPublish', sql.Bit, generative.needPublish)
        .input('LastPublishedDate', sql.DateTime, generative.lastPublishedDate)
        .input('StatusCode', sql.VarChar(50), generative.status)
        .input('CurrentUser', sql.VarChar(50), currentUser)
        .execute('SaveGenerative')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveGenerative = SaveGenerative;
function SaveGenerativeId(vcc, oldId, newId, currentUser) {
    let logLabel = `[GenerativeMSSql::SaveGenerativeId] [vcc::oldId::newId => ${vcc}::${oldId}::${newId}]: `;
    if (currentUser)
        logger.info(`${logLabel}[currentUser => ${currentUser}]`);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('OldId', sql.VarChar(50), oldId)
        .input('NewId', sql.VarChar(50), newId)
        .input('CurrentUser', sql.VarChar(50), currentUser)
        .execute('SaveGenerativeId')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}getting output:`, output);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveGenerativeId = SaveGenerativeId;
function DeleteGenerative(vcc, id, currentUser) {
    let logLabel = `[GenerativeMSSql::SaveGenerativeId] [vcc::id::user => ${vcc}::${id}::${currentUser}]: `;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), id)
        .execute('DeleteGenerative')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info(`${logLabel}}getting output:`, output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(`${logLabel}}getting errorMsg:`, errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteGenerative = DeleteGenerative;
function GetUniqueModelsByIntents(vcc, templateId, analysisType, intents) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(100), templateId)
        .input('AnalysisType', sql.VarChar(50), analysisType)
        .input('Intents', sql.NVarChar(sql.MAX), intents)
        .execute('GetUniqueModelsByIntents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        if (recordset?.length)
            recordset = recordset.map((x) => {
                return x.appId;
            });
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[GenerativeMSSql::GetUniqueModelsByIntents] getting errorMsg:', errorMsg, 'vcc:', vcc, 'intents:', intents);
        return Promise.reject(err);
    });
}
exports.GetUniqueModelsByIntents = GetUniqueModelsByIntents;
