"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteGenericQuery = exports.SaveGenericQuery = exports.GetGenericQueryUsage = exports.GetGenericQueryValidate = exports.GetGenericQuery = exports.GetGenericQueries = exports.GenericQueryMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function GenericQueryMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.GenericQueryMSSql = GenericQueryMSSql;
function GetGenericQueries(vcc, search_criteria, search_expression, startDate, endDate) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('StartDate', sql.VarChar(10), startDate)
        .input('EndDate', sql.VarChar(10), endDate)
        .execute('GetGenericQueries')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenericQueries = GetGenericQueries;
function GetGenericQuery(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetGenericQuery')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let genericQuery = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(genericQuery);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenericQuery = GetGenericQuery;
function GetGenericQueryValidate(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetGenericQueryValidate')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let genericQuery = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(genericQuery);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGenericQueryValidate = GetGenericQueryValidate;
function GetGenericQueryUsage(vcc, QueryName) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('QueryName', sql.VarChar(50), QueryName)
        .execute('GetGenericQueryUsage')
        .then((sqlResult) => {
        let abilitiesRecordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let templateThemeItemsRecordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[1] : null);
        let evaluationPlanRecordeset = (sqlResult.recordsets?.length ? sqlResult.recordsets[2] : null);
        let abilities = (abilitiesRecordset && abilitiesRecordset.length > 0 ? abilitiesRecordset : null);
        let templateThemeItems = (templateThemeItemsRecordset && templateThemeItemsRecordset.length > 0 ? templateThemeItemsRecordset[0] : null);
        let evaluationsPlan = (evaluationPlanRecordeset && evaluationPlanRecordeset.length > 0 ? evaluationPlanRecordeset : null);
        let result = {
            abilities: abilities,
            templateThemeItems: templateThemeItems,
            evaluationsPlan: evaluationsPlan
        };
        return Promise.resolve(result);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[GenericQueryMSSql:GetGenericQueryUsage] Getting error:', errorMsg, 'vcc:', vcc, 'queryName:', QueryName);
        return Promise.reject(err);
    });
}
exports.GetGenericQueryUsage = GetGenericQueryUsage;
function SaveGenericQuery(vcc, name, description, type, query, rawData, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId, version = 1) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(100), name)
        .input('Description', sql.VarChar(500), description)
        .input('Type', sql.VarChar(25), type)
        .input('Query', sql.VarChar(sql.MAX), query)
        .input('RawData', sql.VarChar(sql.MAX), rawData)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .input('Version', sql.SmallInt, version)
        .execute('SaveGenericQuery')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveGenericQuery = SaveGenericQuery;
function DeleteGenericQuery(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteGenericQuery')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteGenericQuery = DeleteGenericQuery;
