"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetGlobalSearchInteractions = exports.GetGlobalSearchFilters = exports.GlobalSearchMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function GlobalSearchMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.GlobalSearchMSSql = GlobalSearchMSSql;
function GetGlobalSearchFilters(instance) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), instance)
        .execute('GetGlobalSearchFilters')
        .then((sqlResult) => {
        let campigns = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let agents = (sqlResult.recordsets?.length ? sqlResult.recordsets[1] : null);
        let origins = (sqlResult.recordsets?.length ? sqlResult.recordsets[2] : null);
        let dispositionCodes = (sqlResult.recordsets?.length ? sqlResult.recordsets[3] : null);
        let interactionTypes = (sqlResult.recordsets?.length ? sqlResult.recordsets[4] : null);
        return Promise.resolve({
            agents: agents,
            campaigns: campigns,
            dispositionCodes: dispositionCodes,
            interactionTypes: interactionTypes,
            origins: origins
        });
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGlobalSearchFilters = GetGlobalSearchFilters;
function GetGlobalSearchInteractions(vcc, agents, campaigns, dateFrom, dateTo, origins, interactionTypes, interactionIds, contactAddress, dispositionCodes) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Agents', sql.VarChar(sql.MAX), agents)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns)
        .input('InteractionDateFrom', sql.VarChar(50), dateFrom)
        .input('InteractionDateTo', sql.VarChar(50), dateTo)
        .input('InteractionOrigins', sql.VarChar(50), origins)
        .input('InteractionTypes', sql.VarChar(50), interactionTypes)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .input('ContactAddress', sql.VarChar(300), contactAddress)
        .input('DispositionCodes', sql.VarChar(sql.MAX), dispositionCodes)
        .execute('GetGlobalSearchInteractions')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetGlobalSearchInteractions = GetGlobalSearchInteractions;
