"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteIntent = exports.SaveIntent = exports.GetIntentByAppId = exports.GetIntent = exports.GetIntents = exports.IntentMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function IntentMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.IntentMSSql = IntentMSSql;
function GetIntents(vcc, appId, search_criteria, search_expression, intents) {
    logger.info('[IntentMSSql::GetIntents] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'intents:', intents);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(100), appId)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Intents', sql.NVarChar(sql.MAX), intents)
        .execute('GetIntents')
        .then((sqlResult) => {
        logger.info('[IntentMSSql::GetIntents] getting sqlResult.recordsets:', sqlResult.recordsets, 'of GetIntents');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[IntentMSSql::GetIntents] getting recordset:', recordset, 'of GetIntents');
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[IntentMSSql::GetIntents] getting errorMsg:', errorMsg, 'of GetIntents');
        return Promise.reject(err);
    });
}
exports.GetIntents = GetIntents;
function GetIntent(vcc, name) {
    logger.info('[IntentMSSql::GetIntent] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetIntent')
        .then((sqlResult) => {
        logger.info('[IntentMSSql::GetIntent] getting sqlResult.recordsets:', sqlResult.recordsets, 'of GetIntent');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[IntentMSSql::GetIntent] getting recordset:', recordset, 'of GetIntent');
        let intent = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[IntentMSSql::GetIntent] getting intent:', intent, 'of GetIntent');
        return Promise.resolve(intent);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[IntentMSSql::GetIntent] getting errorMsg:', errorMsg, 'of GetIntent');
        return Promise.reject(err);
    });
}
exports.GetIntent = GetIntent;
function GetIntentByAppId(vcc, appId, name) {
    logger.info('[IntentMSSql::GetIntentByAppId] getting vcc:', vcc, 'appId:', appId, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('Name', sql.VarChar(50), name)
        .execute('GetIntentByAppId')
        .then((sqlResult) => {
        logger.info('[IntentMSSql::GetIntentByAppId] getting sqlResult.recordsets:', sqlResult.recordsets, 'of GetIntentByAppId');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[IntentMSSql::GetIntentByAppId] getting recordset:', recordset, 'of GetIntentByAppId');
        let intent = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[IntentMSSql::GetIntentByAppId] getting intent:', intent, 'of GetIntentByAppId');
        return Promise.resolve(intent);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[IntentMSSql::GetIntentByAppId] getting errorMsg:', errorMsg, 'of GetIntentByAppId');
        return Promise.reject(err);
    });
}
exports.GetIntentByAppId = GetIntentByAppId;
function SaveIntent(vcc, name, intentId, appId, color, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    logger.info('[IntentMSSql::SaveIntent] getting vcc:', vcc, 'name:', name, 'intentId:', intentId, 'appId:', appId, 'color:', color, 'createdDate:', createdDate, 'createdByUserId:', createdByUserId, 'lastModifiedDate:', lastModifiedDate, 'lastModifiedByUserId:', lastModifiedByUserId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('AppId', sql.VarChar(50), appId)
        .input('Color', sql.VarChar(50), color)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveIntent')
        .then((sqlResult) => {
        logger.info('[IntentMSSql::SaveIntent] getting sqlResult.recordsets:', sqlResult.recordsets, 'of SaveIntent');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[IntentMSSql::SaveIntent] getting recordset:', recordset, 'of SaveIntent');
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[IntentMSSql::SaveIntent] getting output:', output, 'of SaveIntent');
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[IntentMSSql::GetIntent] getting errorMsg:', errorMsg, 'of SaveIntent');
        return Promise.reject(err);
    });
}
exports.SaveIntent = SaveIntent;
function DeleteIntent(vcc, appId, intentId) {
    logger.info('[IntentMSSql::DeleteIntent] getting vcc:', vcc, 'intentId:', intentId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .execute('DeleteIntent')
        .then((sqlResult) => {
        logger.info('[IntentMSSql::DeleteIntent] getting sqlResult.recordsets:', sqlResult.recordsets, 'of DeleteIntent');
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[IntentMSSql::DeleteIntent] getting recordset:', recordset, 'of SaveIntent');
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[IntentMSSql::DeleteIntent] getting output:', output, 'of SaveIntent');
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[IntentMSSql::DeleteIntent] getting errorMsg:', errorMsg, 'of DeleteIntent');
        return Promise.reject(err);
    });
}
exports.DeleteIntent = DeleteIntent;
