"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveInteractionType = exports.GetInteractionTypes = exports.InteractionTypeMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function InteractionTypeMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.InteractionTypeMSSql = InteractionTypeMSSql;
function GetInteractionTypes(instance, search_criteria, search_expression) {
    logger.info('[InteractionTypeMSSql::GetInteractionTypes] Received search_criteria: ' + search_criteria);
    logger.info('[InteractionTypeMSSql::GetInteractionTypes] Received search_expression: ' + search_expression);
    logger.info('[InteractionTypeMSSql::GetInteractionTypes] Calling SP GetInteractionTypes');
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetInteractionTypes')
        .then((sqlResult) => {
        logger.info('[InteractionTypeMSSql::GetInteractionTypes] Received result from SP GetInteractionTypes: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionTypeMSSql::GetInteractionTypes] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionTypes = GetInteractionTypes;
function SaveInteractionType(instance, id, description) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('ID', sql.VarChar(50), id)
        .input('Description', sql.VarChar(100), description)
        .execute('SaveInteractionType')
        .then((sqlResult) => {
        logger.info('[InteractionTypeMSSql::SaveInteractionType] Received result from SP SaveInteractionType: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionTypeMSSql::SaveInteractionType] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveInteractionType = SaveInteractionType;
