"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateRasaNluMigration = exports.UpdateNluStatus = exports.SaveCognitiveEngineMigrateStatus = exports.GetNlusUsage = exports.UpdateNluAppId = exports.UpdateVccStatus = exports.UpdateIntentId = exports.UpdateUtteranceId = exports.UpdateEntityId = exports.GetNluIntentUtteranceToExport = exports.DeleteNLU = exports.SaveNLU = exports.GetNLUByAppId = exports.GetNLUByName = exports.GetNLU = exports.GetNLUsStudy = exports.GetNLUs = exports.GetInteractionAgents = exports.GetInteractionCampaigns = exports.NLUMSSql = void 0;
const log4js = require("log4js");
const config_1 = require("../../config/config");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function NLUMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.NLUMSSql = NLUMSSql;
function GetInteractionCampaigns(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetInteractionCampaigns')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionCampaigns = GetInteractionCampaigns;
function GetInteractionAgents(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetInteractionAgents')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionAgents = GetInteractionAgents;
function GetNLUs(vcc, search_criteria, search_expression, apps, isSystemApp, showInMigrationProcess = false, useRasaApi) {
    logger.info('[NLUMSSql::GetNLUs] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'apps:', apps);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Apps', sql.NVarChar(sql.MAX), apps)
        .input('IsSystemApp', sql.Bit, isSystemApp)
        .input('ShowInMigrationProcess', sql.Bit, showInMigrationProcess)
        .input('UseRasaApi', sql.Bit, useRasaApi)
        .execute('GetNLUs')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUs] getting recordset: ', recordset && recordset.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUs] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUs = GetNLUs;
function GetNLUsStudy(vcc, search_criteria, search_expression, names) {
    logger.info('[NLUMSSql::GetNLUsStudy] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'names:', names);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Names', sql.NVarChar(sql.MAX), names)
        .execute('GetNLUsStudy')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::GetNLUsStudy] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUsStudy] getting recordset:', recordset);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUsStudy] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUsStudy = GetNLUsStudy;
function GetNLU(vcc, name) {
    logger.info('[NLUMSSql::GetNLU] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetNLU')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::GetNLU] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLU] getting recordset:', recordset);
        let nlu = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::GetNLU] getting nlu:', nlu);
        return Promise.resolve(nlu);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLU = GetNLU;
function GetNLUByName(vcc, name) {
    logger.info('[NLUMSSql::GetNLUByName] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetNLU')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::GetNLUByName] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUByName] getting recordset:', recordset);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUByName] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUByName = GetNLUByName;
function GetNLUByAppId(vcc, appId) {
    logger.info('[NLUMSSql::GetNLUByAppId] getting vcc:', vcc, ', appId:', appId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .execute('GetNLUByAppId')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::GetNLUByAppId] getting sqlResult.recordsets: ', sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let nlu = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(nlu);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUByAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUByAppId = GetNLUByAppId;
function SaveNLU(vcc, name, culture, description, appId, subscriptionKey, isSystemApp, needTrain, needPublish, isPublished, lastPublishedDate, isTrained, lastTrainedDate, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId, trainVersion, trainId) {
    logger.info('[NLUMSSql::SaveNLU] getting vcc:', vcc, 'name:', name, 'culture:', culture, 'description:', description, 'appId:', appId, 'subscriptionKey:', subscriptionKey, 'createdDate:', createdDate, 'createdByUserId:', createdByUserId, 'lastModifiedDate:', lastModifiedDate, 'lastModifiedByUserId:', lastModifiedByUserId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Culture', sql.VarChar(50), culture)
        .input('Description', sql.VarChar(50), description)
        .input('AppId', sql.VarChar(100), appId)
        .input('SubscriptionKey', sql.VarChar(50), subscriptionKey)
        .input('IsSystemApp', sql.Bit, isSystemApp)
        .input('NeedTrain', sql.Bit, needTrain)
        .input('NeedPublish', sql.Bit, needPublish)
        .input('IsPublished', sql.Bit, isPublished)
        .input('LastPublishedDate', sql.DateTime, lastPublishedDate)
        .input('IsTrained', sql.Bit, isTrained)
        .input('LastTrainedDate', sql.DateTime, lastTrainedDate)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .input('TrainVersion', sql.VarChar(250), trainVersion)
        .input('TrainId', sql.VarChar(250), trainId)
        .input('UseRasaApi', sql.Bit, config_1.USE_RASA_API)
        .execute('SaveNLU')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::SaveNLU] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::SaveNLU] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::SaveNLU] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveNLU = SaveNLU;
function DeleteNLU(vcc, name) {
    logger.info('[NLUMSSql::DeleteNLU] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteNLU')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::DeleteNLU] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::DeleteNLU] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::DeleteNLU] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteNLU = DeleteNLU;
function GetNluIntentUtteranceToExport(vcc, listMode) {
    logger.info('[NLUMSSql::GetNluIntentUtteranceToExport] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ListMode', sql.Int, listMode)
        .input('UseRasaApi', sql.Bit, config_1.USE_RASA_API)
        .input('RatioRasaMigration', sql.Int, config_1.RATIO_RASA_MIGRATION)
        .execute('GetNluIntentUtteranceToExport')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::GetNluIntentUtteranceToExport] getting sqlResult.recordsets: ', sqlResult.recordsets.length);
        let response = {};
        if (sqlResult.recordsets.length) {
            if (listMode === 1 || listMode === 3) {
                response.nlus = sqlResult.recordsets[0];
                response.intents = sqlResult.recordsets[1];
                response.utterances = sqlResult.recordsets[2];
                response.entities = sqlResult.recordsets[3];
            }
            else {
                response = {
                    quantity: sqlResult.recordsets[0][0],
                    usage: sqlResult.recordsets[1][0]
                };
            }
        }
        return Promise.resolve(response);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNluIntentUtteranceToExport] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNluIntentUtteranceToExport = GetNluIntentUtteranceToExport;
function UpdateEntityId(vcc, appId, entityId, newEntityId) {
    logger.info('[NLUMSSql::UpdateEntityId] getting UpdateEntityId:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('EntityId', sql.VarChar(50), entityId)
        .input('NewEntityId', sql.VarChar(50), newEntityId)
        .execute('UpdateEntityId')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateEntityId] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateEntityId] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateEntityId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateEntityId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateEntityId = UpdateEntityId;
function UpdateUtteranceId(vcc, appId, intentId, utteranceId, newUtteranceId) {
    logger.info('[NLUMSSql::UpdateUtteranceId] getting UpdateUtteranceId:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('NewUtteranceId', sql.VarChar(50), newUtteranceId)
        .execute('UpdateUtteranceId')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateUtteranceId] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateUtteranceId] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateUtteranceId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateUtteranceId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateUtteranceId = UpdateUtteranceId;
function UpdateIntentId(vcc, appId, intentId, newIntentId) {
    logger.info('[NLUMSSql::UpdateIntentId] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('NewIntentId', sql.VarChar(50), newIntentId)
        .execute('UpdateIntentId')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateIntentId] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateIntentId] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateIntentId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateIntentId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateIntentId = UpdateIntentId;
function UpdateVccStatus(vcc, isMigrating = false) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IsMigrating', sql.Bit, isMigrating)
        .execute('UpdateVccStatus')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateNluAppId] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateNluAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateVccStatus = UpdateVccStatus;
function UpdateNluAppId(vcc, appId, newAppId, culture, cognitiveEngine) {
    logger.info('[NLUMSSql::UpdateNluAppId] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('NewAppId', sql.VarChar(50), newAppId)
        .input('Culture', sql.VarChar(10), culture)
        .input('CognitiveEngine', sql.VarChar(50), cognitiveEngine)
        .input('UseRasaApi', sql.Bit, config_1.USE_RASA_API)
        .execute('UpdateNluAppId')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateNluAppId] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateNluAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateNluAppId = UpdateNluAppId;
function GetNlusUsage(vcc, nluNames) {
    logger.info('[NLUMSSql::GetNlusUsage] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('NluNames', sql.VarChar(sql.MAX), nluNames)
        .execute('GetNlusUsage')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::GetNlusUsage] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::GetNlusUsage] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::GetNlusUsage] getting output:', output);
        return Promise.resolve(output && output.inUse == true);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNlusUsage] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNlusUsage = GetNlusUsage;
function SaveCognitiveEngineMigrateStatus(vcc, model, key, status, statusCode, nlusToMigrate) {
    logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] Save for ', key);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Model', sql.VarChar(50), model)
        .input('Key', sql.VarChar(100), key)
        .input('Status', sql.Int, status)
        .input('StatusCode', sql.VarChar(50), statusCode)
        .input('NlusToMigrate', sql.VarChar(sql.MAX), nlusToMigrate)
        .execute('SaveCognitiveEngineMigrateStatus')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting output:', output);
        return Promise.resolve(output && output.result == 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCognitiveEngineMigrateStatus = SaveCognitiveEngineMigrateStatus;
function UpdateNluStatus(vcc, appId, newStatus) {
    logger.info('[NLUMSSql::UpdateNluStatus]', vcc, appId, newStatus);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('NewStatus', sql.VarChar(100), newStatus)
        .execute('UpdateNLUStatus')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateNluStatus] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateNluStatus] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateNluStatus] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateNluStatus] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateNluStatus = UpdateNluStatus;
function UpdateRasaNluMigration(vcc, oldAppId, newAppId, useRasaApi) {
    logger.info('[NLUMSSql::UpdateRasaNluMigration]', vcc, oldAppId, newAppId, useRasaApi);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('OldAppId', sql.VarChar(50), oldAppId)
        .input('NewAppId', sql.VarChar(50), newAppId)
        .input('UseRasaApi', sql.Bit, useRasaApi)
        .execute('UpdateRasaNluMigration')
        .then((sqlResult) => {
        logger.info('[NLUMSSql::UpdateRasaNluMigration] getting sqlResult.recordsets:', sqlResult.recordsets);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateRasaNluMigration] getting recordset:', recordset);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateRasaNluMigration] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateRasaNluMigration] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateRasaNluMigration = UpdateRasaNluMigration;
