"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveNotification = exports.GetUserNotifications = exports.GetNotifications = exports.NotificationMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let Promise = require('bluebird');
let logger = log4js.getLogger('Sql');
let sqlConn;
function NotificationMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.NotificationMSSql = NotificationMSSql;
function GetNotifications(type) {
    return new sql.Request(sqlConn)
        .input('Type', sql.VarChar(50), type)
        .execute('GetNotifications')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNotifications = GetNotifications;
function GetUserNotifications(instance, userId, type) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('UserId', sql.VarChar(50), userId)
        .input('Type', sql.VarChar(50), type)
        .execute('GetUserNotifications')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUserNotifications = GetUserNotifications;
function SaveNotification(userIndicatorId, instance, indicatorId, userId, type, opened, sendDate, title, description, createdDate) {
    return new sql.Request(sqlConn)
        .input('UserIndicatorId', sql.VarChar(50), userIndicatorId)
        .input('Instance', sql.VarChar(50), instance)
        .input('IndicatorId', sql.VarChar(50), indicatorId)
        .input('UserId', sql.VarChar(50), userId)
        .input('Opened', sql.Bit, opened)
        .input('Type', sql.VarChar(50), type)
        .input('SendDate', sql.DateTime, sendDate)
        .input('Title', sql.VarChar(150), title)
        .input('Description', sql.VarChar(500), description)
        .input('CreatedDate', sql.DateTime, createdDate)
        .execute('SaveUserNotification')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveNotification = SaveNotification;
