"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetProcessEvaluationPlanTask = exports.GetProcessTask = exports.GetProcessInteractionsHistory = exports.GetProcessI6InteractionsToUpdateTask = exports.GetProcessInteractionsTask = exports.BulkCSVBillingInteractions = exports.BulkCSVSummaryInteractions = exports.BulkCSVFiltersInteractions = exports.DeleteProcessAudioByProcess = exports.DeleteProcessFilter = exports.SaveProcessFilter = exports.GetProcessFilters = exports.SaveProcessStatus = exports.DeleteProcess = exports.GetInteractionsResult = exports.GetProcessInteractionsByProcessId = exports.GetProcessChannelByProcessId = exports.GetProcessUsage = exports.SaveProcess = exports.GetLinguisticProcessStatus = exports.GetProcessValidDurationTime = exports.GetProcessFiltersData = exports.GetProcessByName = exports.GetProcess = exports.GetProcessAudios = exports.GetProcesses = exports.ProcessMSSql = void 0;
const log4js = require("log4js");
const moment = require("moment");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function ProcessMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.ProcessMSSql = ProcessMSSql;
function GetProcesses(vcc, search_criteria, search_expression, startDate, endDate, type, environment, language) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('StartDate', sql.VarChar(10), startDate)
        .input('EndDate', sql.VarChar(10), endDate)
        .input('Type', sql.VarChar(50), type)
        .input('Environment', sql.VarChar(50), environment)
        .input('Language', sql.VarChar(50), language)
        .execute('GetProcesses')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcesses = GetProcesses;
function GetProcessAudios(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('GetProcessAudios')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessAudios = GetProcessAudios;
function GetProcess(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetProcess')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let process = (recordset && recordset.length > 0 ? recordset[0] : null);
        if (process) {
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
            process.processFilters = (recordset ? recordset : null);
        }
        return Promise.resolve(process);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcess = GetProcess;
function GetProcessByName(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetProcessByName')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let process = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(process);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessByName = GetProcessByName;
function GetProcessFiltersData(vcc, processId, evaluationFormatId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('EvaluationFormatId', sql.VarChar(50), evaluationFormatId)
        .execute('GetProcessFiltersData')
        .then((sqlResult) => {
        let recordset = {};
        if (sqlResult.recordsets?.length)
            recordset.channels = sqlResult.recordsets[0] || [];
        if (sqlResult.recordsets?.length >= 2)
            recordset.campaigns = sqlResult.recordsets[1] || [];
        if (sqlResult.recordsets?.length >= 3)
            recordset.agents = sqlResult.recordsets[2] || [];
        if (sqlResult.recordsets?.length >= 4)
            recordset.dispositionCodes = sqlResult.recordsets[3] || [];
        if (sqlResult.recordsets?.length >= 5)
            recordset.dates = sqlResult.recordsets[4] || [];
        if (sqlResult.recordsets?.length >= 6)
            recordset.interactions = sqlResult.recordsets[5] || [];
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessFiltersData = GetProcessFiltersData;
function GetProcessValidDurationTime(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Process', sql.VarChar(50), processId)
        .execute('GetProcessValidDurationTime')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessValidDurationTime = GetProcessValidDurationTime;
function GetLinguisticProcessStatus(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetLinguisticProcessStatus')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let process = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(process);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetLinguisticProcessStatus = GetLinguisticProcessStatus;
function SaveProcess(vcc, name, description, clientId, status, type, startDate, endDate, minDurationTime, maxDurationTime, language, dictionaryId, environment, queryId, queryDataId, queryDataIdDate, batchScheduleId, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId, retranscription, diarization, channelchange) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Description', sql.VarChar(50), description)
        .input('ClientId', sql.VarChar(50), clientId)
        .input('Status', sql.VarChar(50), status)
        .input('Type', sql.VarChar(50), type)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('MinDurationTime', sql.Numeric(18, 2), minDurationTime)
        .input('MaxDurationTime', sql.Numeric(18, 2), maxDurationTime)
        .input('Language', sql.VarChar(25), language)
        .input('DictionaryId', sql.VarChar(50), dictionaryId)
        .input('Environment', sql.VarChar(50), environment)
        .input('QueryId', sql.VarChar(50), queryId)
        .input('QueryDataId', sql.VarChar(50), queryDataId)
        .input('QueryDataIdDate', sql.DateTime, queryDataIdDate)
        .input('BatchScheduleId', sql.VarChar(50), batchScheduleId)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .input('Retranscription', sql.Int, retranscription)
        .input('Diarization', sql.Int, diarization)
        .input('ChannelChange', sql.Int, channelchange)
        .execute('SaveProcess')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveProcess = SaveProcess;
function GetProcessUsage(vcc, ProcessId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), ProcessId)
        .execute('GetProcessUsage')
        .then((sqlResult) => {
        let processesRecordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let studiesRecordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[1] : null);
        let scheduledTemplatesRecordset = (sqlResult.recordsets?.length > 1 ? sqlResult.recordsets[2] : null);
        let processes = (processesRecordset && processesRecordset.length > 0 ? processesRecordset : null);
        let studies = (studiesRecordset && studiesRecordset.length > 0 ? studiesRecordset : null);
        let scheduledTemplates = (scheduledTemplatesRecordset?.length ? scheduledTemplatesRecordset : null);
        let result = {
            processes: processes,
            studies: studies,
            scheduledTemplates: scheduledTemplates
        };
        return Promise.resolve(result);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[ProcessMSSql:GetProcessUsage] Getting error:', errorMsg, 'vcc:', vcc, 'ProcessId:', ProcessId);
        return Promise.reject(err);
    });
}
exports.GetProcessUsage = GetProcessUsage;
function GetProcessChannelByProcessId(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(100), processId)
        .execute('GetProcessChannelByProcessId')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[ProcessMSSql:GetInteractionsResult] Getting error:', errorMsg, 'vcc:', vcc, 'ProcessId:', processId);
        return Promise.reject(err);
    });
}
exports.GetProcessChannelByProcessId = GetProcessChannelByProcessId;
function GetProcessInteractionsByProcessId(vcc, processId, resultType) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('ResultType', sql.VarChar(50), resultType)
        .execute('GetProcessInteractionsByProcessId')
        .then((sqlResult) => {
        let processCallInteractions = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : []);
        let processTextInteractions = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : []);
        return Promise.resolve(processCallInteractions.concat(processTextInteractions));
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessInteractionsByProcessId = GetProcessInteractionsByProcessId;
function GetInteractionsResult(vcc, processId, channels) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('Channels', sql.VarChar(100), channels)
        .execute('GetInteractionsResult')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset && recordset.length > 0 ? recordset[0] : null);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[ProcessMSSql:GetInteractionsResult] Getting error:', errorMsg, 'vcc:', vcc, 'ProcessId:', processId);
        return Promise.reject(err);
    });
}
exports.GetInteractionsResult = GetInteractionsResult;
function DeleteProcess(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteProcess')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteProcess = DeleteProcess;
function SaveProcessStatus(vcc, name, status, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Status', sql.VarChar(50), status)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveProcessStatus')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveProcessStatus = SaveProcessStatus;
function GetProcessFilters(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('GetProcessFilters')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessFilters = GetProcessFilters;
function SaveProcessFilter(vcc, processId, key, value, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('Key', sql.VarChar(50), key)
        .input('Value', sql.VarChar(250), value)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveProcessFilter')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveProcessFilter = SaveProcessFilter;
function DeleteProcessFilter(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('DeleteProcessFilter')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteProcessFilter = DeleteProcessFilter;
function DeleteProcessAudioByProcess(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('DeleteProcessAudioByProcess')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing error ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteProcessAudioByProcess = DeleteProcessAudioByProcess;
function BulkCSVFiltersInteractions(data) {
    let table = SetProcessCsvToTemporaryTable(data);
    return GetCSVFiltersInteractionsFromTemporaryTable(table);
}
exports.BulkCSVFiltersInteractions = BulkCSVFiltersInteractions;
function BulkCSVSummaryInteractions(data, vcc, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, isSummary, interactionIds = '', limitIntByAgent) {
    let table = SetProcessCsvToTemporaryTable(data);
    return GetCSVSummaryInteractionsFromTemporaryTable(table, vcc, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, isSummary, interactionIds, limitIntByAgent);
}
exports.BulkCSVSummaryInteractions = BulkCSVSummaryInteractions;
function BulkCSVBillingInteractions(data, vcc, secondsPerRequest, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, interactionIds = '') {
    let table = SetProcessCsvToTemporaryTable(data);
    return GetCSVBillingInteractionsFromTemporaryTable(table, vcc, secondsPerRequest, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, interactionIds);
}
exports.BulkCSVBillingInteractions = BulkCSVBillingInteractions;
function SetProcessCsvToTemporaryTable(data) {
    let table = new sql.Table('#ProcessCsvTemp');
    table.create = true;
    table.columns.add('ID', sql.VarChar(100), { nullable: true });
    table.columns.add('Number', sql.VarChar(50), { nullable: true });
    table.columns.add('Gender', sql.VarChar(50), { nullable: true });
    table.columns.add('SilenceTime', sql.Int, { nullable: true });
    table.columns.add('DurationTime', sql.Int, { nullable: true });
    table.columns.add('Category1', sql.VarChar(100), { nullable: true });
    table.columns.add('Category2', sql.VarChar(100), { nullable: true });
    table.columns.add('Category3', sql.VarChar(100), { nullable: true });
    table.columns.add('Description', sql.VarChar(100), { nullable: true });
    table.columns.add('Campaign', sql.VarChar(100), { nullable: true });
    table.columns.add('Agent', sql.VarChar(50), { nullable: true });
    table.columns.add('Direction', sql.VarChar(50), { nullable: true });
    table.columns.add('StartDate', sql.DateTime, { nullable: true });
    table.columns.add('EndDate', sql.DateTime, { nullable: true });
    table.columns.add('DispositionCode', sql.VarChar(50), { nullable: true });
    table.columns.add('RelativePath', sql.VarChar(500), { nullable: true });
    table.columns.add('RelativeFileName', sql.VarChar(200), { nullable: true });
    data.map(d => {
        if (d.RelativePath) {
            let relativePathArray = d.RelativePath.split('/');
            if (d.RelativePath.match(/\.(mp3|wav|ogg)$/)) {
                d.FileName = relativePathArray.pop();
            }
            d.RelativePath = relativePathArray.filter(rpa => rpa.length).join('/');
            if (!d.RelativePath.startsWith('/'))
                d.RelativePath = '/' + d.RelativePath;
            if (d.RelativePath.endsWith('/'))
                d.RelativePath = d.RelativePath.replace(/\/$/, '');
        }
        table.rows.add(d.ID, d.Number ? d.Number : null, d.Gender ? d.Gender : null, d.SilenceTime ? d.SilenceTime : null, d.DurationTime ? d.DurationTime : null, d.Category1 ? d.Category1 : null, d.Category2 ? d.Category2 : null, d.Category3 ? d.Category3 : null, d.Description ? d.Description : null, d.Campaign ? d.Campaign : null, d.Agent ? d.Agent : null, d.Direction ? d.Direction : 'E', d.StartDate ? moment(d.StartDate + 'Z').toDate() : null, d.EndDate ? moment(d.EndDate + 'Z').toDate() : null, d.DispositionCode ? d.DispositionCode : null, d.RelativePath ? d.RelativePath : null, d.FileName ? d.FileName : null);
    });
    return table;
}
function GetCSVFiltersInteractionsFromTemporaryTable(temporaryTable) {
    let transaction = new sql.Transaction(sqlConn);
    let request;
    return transaction.begin()
        .then((result) => {
        logger.info('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Creating transaction');
        return new sql.Request(transaction);
    }).then((result) => {
        request = result;
        return request.bulk(temporaryTable)
            .then((rowCount) => {
            logger.info('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch((err) => {
            logger.error('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Error on bulk: ' + err.message);
            throw err;
        });
    }).then(result => {
        return request
            .execute('GetCSVFiltersInteractions')
            .then((sqlResult) => {
            logger.info('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Ejecucion Correcta');
            let campaignsRecords = sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null;
            let agentsRecords = sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null;
            let dispositionCodesRecords = sqlResult.recordsets && sqlResult.recordsets.length > 2 ? sqlResult.recordsets[2] : null;
            return Promise.resolve({
                agents: agentsRecords,
                campaigns: campaignsRecords,
                dispositionCodes: dispositionCodesRecords
            });
        })
            .catch((err) => {
            logger.error('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Al ejecutar el procedure: ' + err.message);
            return Promise.reject(err);
        });
    }).then((result) => {
        logger.info('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Saving transaction');
        transaction.commit();
        return Promise.resolve(result);
    }).catch((err) => {
        logger.error('[ProcessMSSql::GetCSVFiltersInteractionsFromTemporaryTable] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
function GetCSVSummaryInteractionsFromTemporaryTable(temporaryTable, vcc, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, isSummary, interactionIds = '', limitIntByAgent) {
    let transaction = new sql.Transaction(sqlConn);
    let request;
    return transaction.begin()
        .then((result) => {
        logger.info('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Creating transaction');
        return new sql.Request(transaction);
    })
        .then((result) => {
        request = result;
        return request.bulk(temporaryTable)
            .then((rowCount) => {
            logger.info('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch((err) => {
            logger.error('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(result => {
        return request
            .input('VCC', sql.VarChar(50), vcc)
            .input('StartDate', sql.DateTime, startDate)
            .input('EndDate', sql.DateTime, endDate)
            .input('Campaigns', sql.VarChar(50), campaigns)
            .input('Agents', sql.VarChar(50), agents)
            .input('Origins', sql.VarChar(50), origins)
            .input('StartHour', sql.VarChar(50), startHour)
            .input('EndHour', sql.VarChar(50), endHour)
            .input('MinDuration', sql.Int, minDuration)
            .input('MaxDuration', sql.Int, maxDuration)
            .input('DispositionCodes', sql.VarChar(50), results)
            .input("InteractionIds", sql.VarChar(sql.MAX), interactionIds)
            .input('IsSummary', sql.Bit, isSummary)
            .input('LimitIntByAgent', sql.Int, limitIntByAgent)
            .execute('GetCSVSummaryInteractions')
            .then((sqlResult) => {
            logger.info('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Ejecucion Correcta');
            let interactionView = {};
            interactionView.interactionType = 'CALL';
            if (isSummary) {
                let records = sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null;
                let filtersAgent = agents === '' ? sqlResult.recordsets[1] : agents;
                let channelRecord = records && records.length > 0 ? records[0] : null;
                interactionView.totalInteractions = channelRecord.totalInteractions;
                interactionView.totalInteractionDetails = channelRecord.totalInteractions;
                interactionView.totalInteractionTime = channelRecord.totalAttentionTime;
                interactionView.filtersAgent = filtersAgent;
            }
            else {
                let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
                interactionView.totalInteractions = recordset.length;
                interactionView.totalInteractionDetails = recordset.length;
                interactionView.interactions = recordset ? recordset : [];
            }
            let promiseArray = [];
            promiseArray.push(Promise.resolve(interactionView));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch((err) => {
            logger.error('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Al ejecutar el procedure: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then((result) => {
        logger.info('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Saving transaction');
        return Promise.resolve(result[0]);
    })
        .catch((err) => {
        logger.error('[ProcessMSSql::GetCSVSummaryInteractionsFromTemporaryTable] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
function GetCSVBillingInteractionsFromTemporaryTable(temporaryTable, vcc, secondsPerRequest, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, interactionIds) {
    let transaction = new sql.Transaction(sqlConn);
    let request;
    return transaction.begin()
        .then((result) => {
        logger.info('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Creating transaction');
        return new sql.Request(transaction);
    })
        .then((result) => {
        request = result;
        return request.bulk(temporaryTable)
            .then((rowCount) => {
            logger.info('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch((err) => {
            logger.error('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(result => {
        return request
            .input('VCC', sql.VarChar(50), vcc)
            .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
            .input('StartDate', sql.DateTime, startDate)
            .input('EndDate', sql.DateTime, endDate)
            .input('Campaigns', sql.VarChar(50), campaigns)
            .input('Agents', sql.VarChar(50), agents)
            .input('Origins', sql.VarChar(50), origins)
            .input('StartHour', sql.VarChar(50), startHour)
            .input('EndHour', sql.VarChar(50), endHour)
            .input('MinDuration', sql.Int, minDuration)
            .input('MaxDuration', sql.Int, maxDuration)
            .input('Results', sql.VarChar(50), results)
            .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
            .execute('GetCSVBillingInteractions')
            .then((sqlResult) => {
            logger.info('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Ejecucion Correcta');
            let results = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
            let promiseArray = [];
            promiseArray.push(Promise.resolve(results && results.length > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch((err) => {
            logger.error('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Al ejecutar el procedure: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then((result) => {
        logger.info('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Saving transaction');
        return Promise.resolve(result[0]);
    })
        .catch((err) => {
        logger.error('[ProcessMSSql::GetCSVBillingInteractionsFromTemporaryTable] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
function GetProcessInteractionsTask(vcc, processId) {
    let genericResult = {};
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('GetProcessInteractionsTask')
        .then((sqlResult) => {
        let callProcessInteractions = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let asyncProcessInteractions = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
        callProcessInteractions = callProcessInteractions ? callProcessInteractions : [];
        asyncProcessInteractions = asyncProcessInteractions ? asyncProcessInteractions : [];
        genericResult.status = true;
        genericResult.description = 'OK';
        genericResult.data = callProcessInteractions.concat(asyncProcessInteractions);
        return Promise.resolve(genericResult);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessInteractionsTask = GetProcessInteractionsTask;
function GetProcessI6InteractionsToUpdateTask(vcc, processId) {
    let genericResult = {};
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(250), processId)
        .execute('GetProcessI6InteractionsToUpdate')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        genericResult.status = true;
        genericResult.description = 'OK';
        genericResult.data = recordset;
        return Promise.resolve(genericResult);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessI6InteractionsToUpdateTask = GetProcessI6InteractionsToUpdateTask;
function GetProcessInteractionsHistory(vcc, processId, secondsPerRequest) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .execute('GetProcessInteractionsHistory')
        .then((sqlResult) => {
        let processCallInteractionsSummary = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let processAsyncInteractionsSummary = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
        return Promise.resolve(processCallInteractionsSummary.concat(processAsyncInteractionsSummary));
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessInteractionsHistory = GetProcessInteractionsHistory;
function GetProcessTask(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), name)
        .execute('GetProcessTask')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let process = (recordset && recordset.length > 0 ? recordset[0] : null);
        if (process) {
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
            process.processFilters = (recordset ? recordset : null);
        }
        return Promise.resolve(process);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessTask = GetProcessTask;
function GetProcessEvaluationPlanTask(vcc, name, evaluationPlanId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), name)
        .input('EvaluationPlanId', sql.VarChar(200), evaluationPlanId)
        .execute('GetProcessEvaluationPlanTask')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let process = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(process);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessEvaluationPlanTask = GetProcessEvaluationPlanTask;
