"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteSinglePermission = exports.SaveSinglePermission = exports.SaveCategoryPermissions = exports.GetCategoryPermissions = exports.GetRolePermissionsGroups = exports.DeleteRolePermission = exports.DeleteRole = exports.SaveRoutePermission = exports.SaveRolePermissionsGroup = exports.SaveRolePermission = exports.SaveRole = exports.GetRole = exports.GetRoles = exports.RoleMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function RoleMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.RoleMSSql = RoleMSSql;
function GetRoles(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetRoles')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetRoles = GetRoles;
function GetRole(vcc, roleId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('RoleId', sql.VarChar(50), roleId)
        .execute('GetRole')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let role = (recordset && recordset.length > 0 ? recordset[0] : null);
        if (role) {
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
            role.permissions = (recordset ? recordset : null);
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 2 ? sqlResult.recordsets[2] : null);
            role.routePermissions = (recordset ? recordset : null);
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 3 ? sqlResult.recordsets[3] : null);
            role.singlePermissions = (recordset ? recordset : null);
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 3 ? sqlResult.recordsets[3] : null);
            role.singlePermissions = (recordset ? recordset : null);
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 4 ? sqlResult.recordsets[4] : null);
            role.categories = (recordset ? recordset : null);
        }
        return Promise.resolve(role);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetRole = GetRole;
function SaveRole(vcc, id, description, systemRole, type, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .input('Description', sql.VarChar(50), description)
        .input('SystemRole', sql.Bit, systemRole)
        .input('Type', sql.VarChar(50), type)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveRole')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRole = SaveRole;
function SaveRolePermission(vcc, roleId, groupId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('RoleId', sql.VarChar(50), roleId)
        .input('GroupId', sql.VarChar(50), groupId)
        .execute('SaveRolePermission')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRolePermission = SaveRolePermission;
function SaveRolePermissionsGroup(vcc, groupId, category, type) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('GroupId', sql.VarChar(200), groupId)
        .input('Category', sql.VarChar(50), category)
        .input('Type', sql.VarChar(50), type)
        .execute('SaveRolePermissionsGroup')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRolePermissionsGroup = SaveRolePermissionsGroup;
function SaveRoutePermission(vcc, id, groupId, route, level) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .input('GroupId', sql.VarChar(200), groupId)
        .input('Route', sql.VarChar(250), route)
        .input('Level', sql.VarChar(50), level)
        .execute('SaveRoutePermission')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRoutePermission = SaveRoutePermission;
function DeleteRole(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .execute('DeleteRole')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteRole = DeleteRole;
function DeleteRolePermission(vcc, roleId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('RoleId', sql.VarChar(50), roleId)
        .execute('DeleteRolePermission')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteRolePermission = DeleteRolePermission;
function GetRolePermissionsGroups(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetRolePermissionsGroups')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetRolePermissionsGroups = GetRolePermissionsGroups;
function GetCategoryPermissions(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetCategoryPermissions')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetCategoryPermissions = GetCategoryPermissions;
function SaveCategoryPermissions(vcc, category, position) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Category', sql.VarChar(50), category)
        .input('Position', sql.Int, position)
        .execute('SaveCategoryPermission')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCategoryPermissions = SaveCategoryPermissions;
function SaveSinglePermission(vcc, type, element) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.VarChar(50), type)
        .input('Element', sql.VarChar(50), element)
        .execute('SaveSinglePermission')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveSinglePermission = SaveSinglePermission;
function DeleteSinglePermission(vcc, type, element) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.VarChar(50), type)
        .input('Element', sql.VarChar(50), element)
        .execute('DeleteSinglePermission')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteSinglePermission = DeleteSinglePermission;
