"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetStudyCognitiveInteractionsTask = exports.GetStudyGenericQueryTask = exports.GetStudiesInProcess = exports.GetStudyReport = exports.SaveSQLStudyDetail = exports.GetExecuteTemplateProcess = exports.GetStudyResultSummary = exports.DeleteStudy = exports.SaveStudyStatus = exports.SaveStudy = exports.GetStudyByName = exports.GetStudy = exports.GetStudies = exports.StudyMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function StudyMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.StudyMSSql = StudyMSSql;
function GetStudies(vcc, search_criteria, search_expression, startDate, endDate, type) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('StartDate', sql.VarChar(10), startDate)
        .input('EndDate', sql.VarChar(10), endDate)
        .input('Type', sql.VarChar(50), type)
        .execute('GetStudies')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudies = GetStudies;
function GetStudy(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StudyId', sql.VarChar(200), id)
        .execute('GetStudy')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let study = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(study);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudy = GetStudy;
function GetStudyByName(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(200), name)
        .execute('GetStudyByName')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let study = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(study);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudyByName = GetStudyByName;
function SaveStudy(vcc, processId, templateId, name, description, status, type, batchScheduleId, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('TemplateId', sql.VarChar(50), templateId)
        .input('Name', sql.VarChar(200), name)
        .input('Description', sql.VarChar(100), description)
        .input('Status', sql.Float, status)
        .input('Type', sql.VarChar(50), type)
        .input('BatchScheduleId', sql.VarChar(50), batchScheduleId)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveStudy')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveStudy = SaveStudy;
function SaveStudyStatus(vcc, processId, templateId, studyId, status, lastQueryId, errorMessage, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('TemplateId', sql.VarChar(50), templateId)
        .input('StudyId', sql.VarChar(200), studyId)
        .input('Status', sql.Float, status)
        .input('LastQueryId', sql.VarChar(50), lastQueryId)
        .input('ErrorMessage', sql.VarChar(sql.MAX), errorMessage)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveStudyStatus')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveStudyStatus = SaveStudyStatus;
function DeleteStudy(vcc, studyId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StudyId', sql.VarChar(200), studyId)
        .execute('DeleteStudy')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteStudy = DeleteStudy;
function GetStudyResultSummary(vcc, studyId, newStructure) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StudyId', sql.VarChar(200), studyId)
        .input('NewStructure', sql.VarChar(1), newStructure)
        .execute('GetStudyResultSummary')
        .then((sqlResult) => {
        let studyResultDetail = (sqlResult.recordsets?.length ? sqlResult.recordsets[0][0] : null);
        let studyResultSummary = (sqlResult.recordsets?.length ? sqlResult.recordsets[1] : null);
        let studyResultSummaryDetailViewModel = (sqlResult.recordsets?.length ? sqlResult.recordsets[2] : null);
        let studyResultViewModel = {};
        studyResultViewModel.studyResultDetail = studyResultDetail;
        studyResultViewModel.studyResultSummary = studyResultSummary;
        studyResultViewModel.studyResultSummaryDetailViewModel = studyResultSummaryDetailViewModel;
        return Promise.resolve(studyResultViewModel);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(err);
        return Promise.reject(err);
    });
}
exports.GetStudyResultSummary = GetStudyResultSummary;
function GetExecuteTemplateProcess(vcc, studyId, newStructure) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StudyId', sql.VarChar(200), studyId)
        .input('NewStructure', sql.VarChar(1), newStructure)
        .execute('GetExecuteTemplateProcess')
        .then((sqlResult) => {
        let studyResultDetail = (sqlResult.recordsets?.length ? sqlResult.recordsets[0][0] : null);
        let processAudioStudy = (sqlResult.recordsets?.length ? sqlResult.recordsets[1] : null);
        let studyResultSummary = (sqlResult.recordsets?.length ? sqlResult.recordsets[2] : null);
        let studyResultSummaryDetailViewModel = (sqlResult.recordsets?.length ? sqlResult.recordsets[3] : null);
        let studyResultViewModel = {};
        studyResultViewModel.studyResultDetail = studyResultDetail;
        studyResultViewModel.processAudioStudy = processAudioStudy;
        studyResultViewModel.studyResultSummary = studyResultSummary;
        studyResultViewModel.studyResultSummaryDetailViewModel = studyResultSummaryDetailViewModel;
        return Promise.resolve(studyResultViewModel);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(err);
        return Promise.reject(err);
    });
}
exports.GetExecuteTemplateProcess = GetExecuteTemplateProcess;
function SaveSQLStudyDetail(vcc, studyId, templateId, SQL, restart) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StudyId', sql.VarChar(200), studyId)
        .input('TemplateId', sql.VarChar(50), templateId)
        .input('SQL', sql.VarChar(sql.MAX), SQL)
        .input('Restart', sql.Int, restart)
        .execute('SaveSQLStudyDetail')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar el procedimiento: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(err);
        return Promise.reject(err);
    });
}
exports.SaveSQLStudyDetail = SaveSQLStudyDetail;
function GetStudyReport(startDate, endDate, processes) {
    return new sql.Request(sqlConn)
        .input('StartDate', sql.VarChar(10), startDate)
        .input('EndDate', sql.VarChar(10), endDate)
        .input('Processes', sql.VarChar(800), processes)
        .execute('GetStudyReport')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudyReport = GetStudyReport;
function GetStudiesInProcess() {
    return new sql.Request(sqlConn)
        .execute('GetStudiesInProcess')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudiesInProcess = GetStudiesInProcess;
function GetStudyGenericQueryTask(vcc, processId, studyId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('StudyId', sql.VarChar(200), studyId)
        .execute('GetStudyGenericQueryTask')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudyGenericQueryTask = GetStudyGenericQueryTask;
function GetStudyCognitiveInteractionsTask(vcc, processId, studyId) {
    logger.info('[StudyMSSql::GetStudyCognitiveInteractionsTask] getting vcc:', vcc, 'processId:', processId);
    let genericResult = {};
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('StudyId', sql.VarChar(50), studyId)
        .execute('GetStudyCognitiveInteractionsTask')
        .then((sqlResult) => {
        let callCognitiveInteractions = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let asyncCognitiveInteractions = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
        callCognitiveInteractions = callCognitiveInteractions ? callCognitiveInteractions : [];
        asyncCognitiveInteractions = asyncCognitiveInteractions ? asyncCognitiveInteractions : [];
        genericResult.status = true;
        genericResult.description = 'OK';
        genericResult.data = callCognitiveInteractions.concat(asyncCognitiveInteractions);
        return Promise.resolve(genericResult);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetStudyCognitiveInteractionsTask = GetStudyCognitiveInteractionsTask;
