"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetSubGroupsUsage = exports.DeleteSubGroups = exports.DeleteSubgroup = exports.GetSubgroup = exports.SaveSubgroup = exports.GetSubgroupDescription = exports.GetSubgroups = exports.SubgroupMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
const response_1 = require("../utils/response");
let logger = log4js.getLogger('Sql');
let sqlConn;
function SubgroupMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.SubgroupMSSql = SubgroupMSSql;
function GetSubgroups(instance, search_criteria, search_expression, disabled) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Disabled', sql.Bit(), disabled)
        .execute('GetSubgroups')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSubgroups = GetSubgroups;
function GetSubgroupDescription(instance, description) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('Description', sql.VarChar(50), description)
        .execute('GetSubgroupDescription')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let subgroup = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(subgroup);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSubgroupDescription = GetSubgroupDescription;
function SaveSubgroup(instance, id, description, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId, enabled) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('ID', sql.VarChar(50), id)
        .input('Description', sql.VarChar(100), description)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .input('Enabled', sql.VarChar(50), enabled)
        .execute('SaveSubgroup')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveSubgroup = SaveSubgroup;
function GetSubgroup(instance, id) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('ID', sql.VarChar(50), id)
        .execute('GetSubgroup')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let subgroup = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(subgroup);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSubgroup = GetSubgroup;
function DeleteSubgroup(instance, id) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('ID', sql.VarChar(50), id)
        .execute('DeleteSubgroup')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteSubgroup = DeleteSubgroup;
function DeleteSubGroups(vcc, ids) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), vcc)
        .input('SubGroupIDs', sql.VarChar(sql.MAX), ids)
        .execute('DeleteSubGroups')
        .then((sqlResult) => response_1.responseSaveOrDelete(sqlResult.recordsets))
        .catch(err => response_1.responseError(err));
}
exports.DeleteSubGroups = DeleteSubGroups;
function GetSubGroupsUsage(vcc, ids) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), vcc)
        .input('SubGroupIDs', sql.VarChar(sql.MAX), ids)
        .execute('GetSubGroupsUsage')
        .then((sqlResult) => response_1.responseArrayGet(sqlResult.recordsets))
        .catch(err => response_1.responseError(err));
}
exports.GetSubGroupsUsage = GetSubGroupsUsage;
