"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveTenantVCC = exports.GetTenantVCCByVCC = exports.TenantVCCMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function TenantVCCMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.TenantVCCMSSql = TenantVCCMSSql;
function GetTenantVCCByVCC(VCC) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), VCC)
        .execute('GetTenantVCCByVCC')
        .then((sqlResult) => {
        logger.info('[TenantVCCMSSql::GetTenantVCCByVCC] Received result from SP GetTenantVCCByVCC: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let tenantVCC = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(tenantVCC);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[TenantVCCMSSql::GetTenantVCCByVCC] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTenantVCCByVCC = GetTenantVCCByVCC;
function SaveTenantVCC(vcc, tenantId, clientId, clientSecret, resource, createdByUserId, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TenantId', sql.VarChar(50), tenantId)
        .input('ClientId', sql.VarChar(50), clientId)
        .input('ClientSecret', sql.VarChar(50), clientSecret)
        .input('Resource', sql.VarChar(50), resource)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveTenantVCC')
        .then((sqlResult) => {
        logger.info('[TenantVCCMSSql::SaveTenantVCC] Received result from SP SaveTenantVCC: ' + sqlResult.recordsets.length);
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing request: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[TenantVCCMSSql::SaveTenantVCC] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveTenantVCC = SaveTenantVCC;
