"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteTipificationItem = exports.SaveTipificationItem = exports.GetTipificationItems = exports.DeleteTipificationQuery = exports.SaveTipificationQuery = exports.DeleteTipification = exports.SaveTipification = exports.GetTipification = exports.GetTipifications = exports.TipificationMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function TipificationMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.TipificationMSSql = TipificationMSSql;
function GetTipifications(vcc, searchCriteria, searchExpression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(50), searchExpression)
        .execute('GetTipifications')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTipifications = GetTipifications;
function GetTipification(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .execute('GetTipification')
        .then((sqlResult) => {
        let tipificationRecordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let tipificationQueryRecordset = (sqlResult.recordsets?.length > 1 ? sqlResult.recordsets[1] : null);
        let tipification = tipificationRecordset?.length ? tipificationRecordset[0] : null;
        let tipificationQuery = tipificationQueryRecordset?.length ? tipificationQueryRecordset[0] : null;
        tipification.tipificationQuery = tipificationQuery;
        return Promise.resolve(tipification);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTipification = GetTipification;
function SaveTipification(vcc, id, description, detail, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .input('Description', sql.VarChar(250), description)
        .input('Detail', sql.VarChar(500), detail)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveTipification')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length ? recordset[0] : null);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveTipification = SaveTipification;
function DeleteTipification(vcc, tipificationId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), tipificationId)
        .execute('DeleteTipification')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length ? recordset[0] : null);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteTipification = DeleteTipification;
function SaveTipificationQuery(vcc, tipificationId, queryId, queryContent, acceptancePercentage, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TipificationId', sql.VarChar(50), tipificationId)
        .input('QueryId', sql.VarChar(50), queryId)
        .input('QueryContent', sql.VarChar(sql.MAX), queryContent)
        .input('AcceptancePercentage', sql.Decimal(18, 2), acceptancePercentage)
        .execute('SaveTipificationQuery')
        .then((sqlResult) => {
        let recordset = sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null;
        let output = recordset?.length ? recordset[0] : null;
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveTipificationQuery = SaveTipificationQuery;
function DeleteTipificationQuery(vcc, tipificationId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TipificationId', sql.VarChar(50), tipificationId)
        .execute('DeleteTipificationQuery')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length ? recordset[0] : null);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteTipificationQuery = DeleteTipificationQuery;
function GetTipificationItems(vcc, tipificationId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TipificationId', sql.VarChar(50), tipificationId)
        .execute('GetTipificationItems')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch((err) => {
        let errorMsg = (err?.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTipificationItems = GetTipificationItems;
function SaveTipificationItem(vcc, tipificationId, id, secondLevelItemId, description, order, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TipificationId', sql.VarChar(50), tipificationId)
        .input('ID', sql.VarChar(50), id)
        .input('SecondLevelItemId', sql.VarChar(50), secondLevelItemId)
        .input('Description', sql.VarChar(250), description)
        .input('Order', sql.Int, order)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveTipificationItem')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length ? recordset[0] : null);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveTipificationItem = SaveTipificationItem;
function DeleteTipificationItem(vcc, tipificationItemId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), tipificationItemId)
        .execute('DeleteTipificationItem')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset?.length ? recordset[0] : null);
        return Promise.resolve(output?.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteTipificationItem = DeleteTipificationItem;
