"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetUserAPI = exports.GetUsersTask = exports.DeleteUser = exports.SaveUserClient = exports.SaveUser = exports.GetUser = exports.GetUsers = exports.UserMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function UserMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.UserMSSql = UserMSSql;
function GetUsers(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetUsers')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUsers = GetUsers;
function GetUser(vcc, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UserId', sql.VarChar(50), userId)
        .execute('GetUser')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let user = (recordset && recordset.length > 0 ? recordset[0] : null);
        if (user) {
            recordset = (sqlResult.recordsets && sqlResult.recordsets.length > 1 ? sqlResult.recordsets[1] : null);
            user.userClients = (recordset ? recordset : null);
        }
        return Promise.resolve(user);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUser = GetUser;
function SaveUser(vcc, id, email, firstname, lastname, password, role, country, language, timeZone, preferences, isSystemUser, isLocalSupervisor, isExternalSupervisor, isExternalAgent, limitClients, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .input('Email', sql.VarChar(50), email)
        .input('Firstname', sql.VarChar(50), firstname)
        .input('Lastname', sql.VarChar(50), lastname)
        .input('Password', sql.VarChar(50), password)
        .input('Role', sql.VarChar(50), role)
        .input('Country', sql.VarChar(50), country)
        .input('Language', sql.VarChar(50), language)
        .input('TimeZone', sql.VarChar(50), timeZone)
        .input('Preferences', sql.VarChar(200), preferences)
        .input('IsSystemUser', sql.Bit, isSystemUser)
        .input('IsLocalSupervisor', sql.Bit, isLocalSupervisor)
        .input('IsExternalSupervisor', sql.Bit, isExternalSupervisor)
        .input('IsExternalAgent', sql.Bit, isExternalAgent)
        .input('LimitClients', sql.Bit, limitClients)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveUser')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveUser = SaveUser;
function SaveUserClient(vcc, userId, clientId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UserId', sql.VarChar(50), userId)
        .input('ClientId', sql.VarChar(50), clientId)
        .execute('SaveUserClient')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveUserClient = SaveUserClient;
function DeleteUser(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .execute('DeleteUser')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteUser = DeleteUser;
function GetUsersTask() {
    return new sql.Request(sqlConn)
        .execute('GetUsersTask')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUsersTask = GetUsersTask;
function GetUserAPI(vcc, ID) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), ID)
        .execute('GetUserAPI')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUserAPI = GetUserAPI;
