"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveVCCLicense = exports.DeleteVCC = exports.SaveVCC = exports.RollbackVcc = exports.GetVCC = exports.GetSubscriptionKey = exports.GetSubscriptionKeys = exports.GetIntegratedVCCs = exports.GetVCCs = exports.GetVccCleanerConfig = exports.GetCleanerData = exports.DeleteInteraction = exports.DeleteProcessAsync = exports.SaveCleanerSummary = exports.VCCMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
const config_1 = require("../../config/config");
let logger = log4js.getLogger('Sql');
let sqlConn;
function VCCMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.VCCMSSql = VCCMSSql;
function genericVccErrorHandler(err) {
    let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
    logger.error(errorMsg);
    return Promise.reject(err);
}
function SaveCleanerSummary(task, status, description) {
    return new sql.Request(sqlConn)
        .input('Task', sql.VarChar(15), task)
        .input('Status', sql.Bit, status)
        .input('Description', sql.VarChar(250), description)
        .execute('SaveCleanerSummary')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.SaveCleanerSummary = SaveCleanerSummary;
function DeleteProcessAsync(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('DeleteProcessAsync')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.DeleteProcessAsync = DeleteProcessAsync;
function DeleteInteraction(vcc, id, isAudio) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(15), id)
        .input('IsAudio', sql.Bit, isAudio)
        .execute('DeleteInteraction')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.DeleteInteraction = DeleteInteraction;
function GetCleanerData(vcc, days) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('DAYS', sql.Int, days)
        .execute('GetCleanerData')
        .then((sqlResult) => {
        return Promise.resolve(sqlResult.recordsets);
    }).catch(genericVccErrorHandler);
}
exports.GetCleanerData = GetCleanerData;
function GetVccCleanerConfig() {
    return new sql.Request(sqlConn)
        .execute('GetVccCleanerConfig')
        .then((sqlResult) => {
        let rows = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(rows);
    }).catch(genericVccErrorHandler);
}
exports.GetVccCleanerConfig = GetVccCleanerConfig;
function GetVCCs(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetVCCs')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(genericVccErrorHandler);
}
exports.GetVCCs = GetVCCs;
function GetIntegratedVCCs(vcc, ip, type) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IP', sql.VarChar(50), ip)
        .input('Type', sql.Int, type)
        .execute('GetIntegratedVCCs')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(genericVccErrorHandler);
}
exports.GetIntegratedVCCs = GetIntegratedVCCs;
function GetSubscriptionKeys(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetSubscriptionKeys')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(genericVccErrorHandler);
}
exports.GetSubscriptionKeys = GetSubscriptionKeys;
function GetSubscriptionKey(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetSubscriptionKey')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let record = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(record);
    }).catch(genericVccErrorHandler);
}
exports.GetSubscriptionKey = GetSubscriptionKey;
function GetVCC(id) {
    return new sql.Request(sqlConn)
        .input('ID', sql.VarChar(50), id)
        .execute('GetVCC')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let record = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(record);
    }).catch(genericVccErrorHandler);
}
exports.GetVCC = GetVCC;
function RollbackVcc(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('RollbackVcc')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.RollbackVcc = RollbackVcc;
function SaveVCC(vcc, id, description, type, transcriptionDefaultLanguage, cognitiveEngine, country, language, timeZone, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId, deleteDataDays = 360, changedCognitiveEngine = false) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .input('Description', sql.VarChar(50), description)
        .input('Type', sql.VarChar(50), type)
        .input('TranscriptionDefaultLanguage', sql.VarChar(25), transcriptionDefaultLanguage)
        .input('CognitiveEngine', sql.VarChar(50), cognitiveEngine)
        .input('Country', sql.VarChar(50), country)
        .input('Language', sql.VarChar(50), language)
        .input('TimeZone', sql.VarChar(50), timeZone)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .input('DeleteDataDays', sql.Int, deleteDataDays)
        .input('ChangedCognitiveEngine', sql.Bit, changedCognitiveEngine)
        .input('UseRasaApi', sql.Bit, config_1.USE_RASA_API)
        .execute('SaveVCC')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.SaveVCC = SaveVCC;
function DeleteVCC(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), id)
        .execute('DeleteVCC')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.DeleteVCC = DeleteVCC;
function SaveVCCLicense(product, status, vcc, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Product', sql.VarChar(50), product)
        .input('Status', sql.Bit, status)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveVccLicense')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(genericVccErrorHandler);
}
exports.SaveVCCLicense = SaveVCCLicense;
