"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveVersionRegistration = exports.GetVersion = exports.VersionMSSql = void 0;
const log4js = require("log4js");
let sql = require('mssql');
let logger = log4js.getLogger('Sql');
let sqlConn;
function VersionMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.VersionMSSql = VersionMSSql;
function GetVersion() {
    return new sql.Request(sqlConn)
        .execute('GetVersion')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let version = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(version);
    }).catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[VersionMSSql::GetVersion] Getting error:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetVersion = GetVersion;
function SaveVersionRegistration(installationId, buildVersion) {
    return new sql.Request(sqlConn)
        .input('InstallationId', sql.VarChar(100), installationId)
        .input('BuildVersion', sql.VarChar(25), buildVersion)
        .execute('SaveVersionRegistration')
        .then((sqlResult) => {
        let recordset = (sqlResult.recordsets?.length ? sqlResult.recordsets[0] : null);
        let output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch((err) => {
        let errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[VersionMSSql::SaveVersionRegistration] Getting error:', errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveVersionRegistration = SaveVersionRegistration;
