"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisInit = void 0;
const redis = require("redis");
const bluebird = require("bluebird");
const log4js = require("log4js");
const redis_config_1 = require("../config/redis.config");
const batch_schedule_redis_1 = require("./redis/batch-schedule.redis");
const domain_redis_1 = require("./redis/domain.redis");
const evaluation_redis_1 = require("./redis/evaluation.redis");
const evaluation_plan_ability_redis_1 = require("./redis/evaluation-plan-ability.redis");
const generative_redis_1 = require("./redis/generative.redis");
const i6_integration_redis_1 = require("./redis/i6-integration.redis");
const vcc_redis_1 = require("./redis/vcc.redis");
const role_redis_1 = require("./redis/role.redis");
const session_redis_1 = require("./redis/session.redis");
const user_redis_1 = require("./redis/user.redis");
const interaction_redis_1 = require("./redis/interaction.redis");
const process_redis_1 = require("./redis/process.redis");
const cognitive_redis_1 = require("./redis/cognitive.redis");
const queue_redis_1 = require("./redis/queue.redis");
const platform_redis_1 = require("./redis/platform.redis");
const nlu_redis_1 = require("./redis/nlu.redis");
const study_template_theme_redis_1 = require("./redis/study-template-theme.redis");
const speechmatics_redis_1 = require("./redis/speechmatics.redis");
const tipification_redis_1 = require("./redis/tipification.redis");
const cleaner_redis_1 = require("./redis/cleaner.redis");
let logger = log4js.getLogger('Redis');
function RedisInit(app) {
    return new Promise((resolve, reject) => {
        logger.info('Iniciando conexión a Redis');
        console.log('Iniciando conexión a Redis');
        let redisClient = redis.createClient(redis_config_1.REDIS_OPTIONS)
            .on('error', function (err) {
            let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('No se pudo inicializar cliente Redis: ' + errMsg);
            console.error('No se pudo inicializar cliente Redis: ' + errMsg);
            reject(err);
        })
            .on('connect', function () {
            logger.info('Conexión a Redis establecida');
            console.log('Conexión a Redis establecida');
            resolve(true);
        });
        bluebird.promisifyAll(redisClient);
        app.set('redisClient', redisClient);
        batch_schedule_redis_1.BatchScheduleRedis(app);
        domain_redis_1.DomainRedis(app);
        evaluation_redis_1.EvaluationRedis(app);
        evaluation_plan_ability_redis_1.EvaluationPlanAbilityRedis(app);
        generative_redis_1.GenerativeRedis(app);
        i6_integration_redis_1.I6IntegrationRedis(app);
        vcc_redis_1.VccRedis(app);
        role_redis_1.RoleRedis(app);
        session_redis_1.SessionRedis(app);
        speechmatics_redis_1.SpeechmaticsRedis(app);
        study_template_theme_redis_1.StudyTemplateThemeRedis(app);
        tipification_redis_1.TipificationRedis(app);
        user_redis_1.UserRedis(app);
        interaction_redis_1.InteractionRedis(app);
        process_redis_1.ProcessRedis(app);
        cognitive_redis_1.CognitiveRedis(app);
        queue_redis_1.QueueRedis(app);
        platform_redis_1.PlatformRedis(app);
        nlu_redis_1.NluRedis(app);
        cleaner_redis_1.CleanerRedis(app);
    });
}
exports.RedisInit = RedisInit;
