"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteAllCleanerTasks = exports.DeleteCleanerData = exports.GetCleanerData = exports.SaveCleanerError = exports.SaveCleanerData = exports.CleanerRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
const promises_shared_1 = require("../../shared/promises.shared");
let logger = log4js.getLogger('Redis');
let redisClient;
function CleanerRedis(app) {
    redisClient = app.get('redisClient');
}
exports.CleanerRedis = CleanerRedis;
function CleanerRedisGenericErrorHandler(err) {
    let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
    logger.error(errorMsg);
    return Promise.reject(err);
}
function SaveCleanerData(key, idColumn, vcc, mixin, rows) {
    return promises_shared_1.DummyPromise().then(_ => {
        let promises = [];
        let keys = [];
        for (let row of rows) {
            let _key = `${vcc}:${row[idColumn]}`;
            promises.push(redisClient.hsetAsync(key, _key, JSON.stringify(Object.assign({ vcc }, mixin, row))));
            keys.push(_key);
        }
        promises.push(redisClient.lpushAsync(`${key}:Keys`, keys));
        return Promise.all(promises);
    })
        .then(_ => {
        return redisClient.expireAsync(key, redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(() => {
        return redisClient.expireAsync(`${key}:Keys`, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .catch(CleanerRedisGenericErrorHandler);
}
exports.SaveCleanerData = SaveCleanerData;
function SaveCleanerError(key, data) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.lpushAsync(key, JSON.stringify(data));
    }).then(_ => {
        return redisClient.expireAsync(key, redis_config_1.REDIS_DATA_PERSISTENCE);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.SaveCleanerError = SaveCleanerError;
function GetCleanerData(key, keys) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hmgetAsync(key, keys);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.GetCleanerData = GetCleanerData;
function DeleteCleanerData(key, id) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hdelAsync(key, id);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.DeleteCleanerData = DeleteCleanerData;
function DeleteAllCleanerTasks(keys) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.delAsync(keys);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.DeleteAllCleanerTasks = DeleteAllCleanerTasks;
