"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllRunningRasaModelFromTest = exports.GetRunningRasaModels = exports.DeleteRunningRasaModelFromTest = exports.DeleteRunningRasaModel = exports.GetRunningRasaModelFromTest = exports.GetRunningRasaModel = exports.SaveRunningRasaModelFromTest = exports.SaveRunningRasaModel = exports.GetAllReadyModelAbilityStorages = exports.GetReadyModelAbilities = exports.SaveReadyModelAbility = exports.GetReadyModelInteractions = exports.SaveReadyModelInteraction = exports.DeleteModelInteraction = exports.SaveModelInteraction = exports.GetModelInteraction = exports.CognitiveRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function CognitiveRedis(app) {
    redisClient = app.get('redisClient');
}
exports.CognitiveRedis = CognitiveRedis;
function GetModelInteraction(key) {
    let storage = 'Model';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(interaction);
    })
        .catch((err) => {
        logger.error('[GetModelInteraction] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetModelInteraction = GetModelInteraction;
function SaveModelInteraction(key, interaction) {
    let storage = 'Model';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(_ => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveModelInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveModelInteraction = SaveModelInteraction;
function DeleteModelInteraction(key) {
    let storage = 'Model';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveModelInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteModelInteraction = DeleteModelInteraction;
function SaveReadyModelInteraction(key, interaction, executionCode) {
    let storage = 'Model';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(_ => {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveReadyModelInteraction] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyModelInteraction = SaveReadyModelInteraction;
function GetReadyModelInteractions(executionCode) {
    let hash = 'Model:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then((result) => {
        let interactions = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyModelInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch((err) => {
        logger.error('[GetReadyModelInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyModelInteractions = GetReadyModelInteractions;
function SaveReadyModelAbility(key, interaction, executionCode) {
    let storage = 'AbilityModel';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(_ => {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[SaveReadyModelAbility] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyModelAbility = SaveReadyModelAbility;
function GetReadyModelAbilities(executionCode) {
    let hash = 'AbilityModel:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then((result) => {
        let interactions = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyModelAbilities] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch((err) => {
        logger.error('[GetReadyModelAbilities] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyModelAbilities = GetReadyModelAbilities;
function GetAllReadyModelAbilityStorages(executionCode) {
    let config = redis_config_1.REDIS_OPTIONS;
    let pattern = `${config.prefix}AbilityModel:*`;
    return redisClient.keysAsync([pattern])
        .then((result) => {
        if (result?.length > 1)
            logger.info(`[CognitiveRedis::GetAllReadyModelAbilityStorages] Getting ${result.length} storages for execution ${executionCode}`);
        let executionCodes = [];
        result.forEach(r => {
            executionCodes.push(r.replace(`${config.prefix}AbilityModel:`, ''));
        });
        executionCodes = executionCodes.sort();
        return executionCodes;
    })
        .catch((err) => {
        logger.error('[CognitiveRedis::GetAllReadyModelAbilityStorages] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllReadyModelAbilityStorages = GetAllReadyModelAbilityStorages;
function SaveRunningRasaModel(key, runningModel) {
    let storage = 'RasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(runningModel))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, 60 * 60);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::SaveRunningRasaModel] modelId:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveRunningRasaModel = SaveRunningRasaModel;
function SaveRunningRasaModelFromTest(key, runningModel) {
    let storage = 'RasaModelFromTest';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(runningModel))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, 60 * 15);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::SaveRunningRasaModelFromTest] modelId:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveRunningRasaModelFromTest = SaveRunningRasaModelFromTest;
function GetRunningRasaModel(key) {
    let storage = 'RasaModel';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interactions = [];
        try {
            let nlu = (typeof result === 'string') ? JSON.parse(result) : result;
            return Promise.resolve(nlu);
        }
        catch (err) {
            logger.error('[cognitive.redis:GetRunningRasaModel] getting err value:', result);
            return Promise.resolve(null);
        }
    }).catch((err) => {
        logger.error('[cognitive.redis:GetRunningRasaModel] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModel = GetRunningRasaModel;
function GetRunningRasaModelFromTest(key) {
    let storage = 'RasaModelFromTest';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interactions = [];
        try {
            let nlu = (typeof result === 'string') ? JSON.parse(result) : result;
            return Promise.resolve(nlu);
        }
        catch (err) {
            logger.error('[cognitive.redis:GetRunningRasaModelFromTest] getting err value:', result);
            return Promise.resolve(null);
        }
    }).catch((err) => {
        logger.error('[cognitive.redis:GetRunningRasaModelFromTest] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModelFromTest = GetRunningRasaModelFromTest;
function DeleteRunningRasaModel(key) {
    let storage = 'RasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::DeleteRunningRasaModel] getting error: ', err, 'key: ', key);
        return Promise.resolve(false);
    });
}
exports.DeleteRunningRasaModel = DeleteRunningRasaModel;
function DeleteRunningRasaModelFromTest(key) {
    let storage = 'RasaModelFromTest';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[cognitive.redis::DeleteRunningRasaModelFromTest] getting error: ', err, 'key: ', key);
        return Promise.resolve(false);
    });
}
exports.DeleteRunningRasaModelFromTest = DeleteRunningRasaModelFromTest;
function GetRunningRasaModels() {
    let storage = 'RasaModel';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let runningRasaModelsList = [];
        try {
            runningRasaModelsList = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(runningRasaModelsList);
    }).catch((err) => {
        logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModels = GetRunningRasaModels;
function GetAllRunningRasaModelFromTest() {
    let storage = 'RasaModelFromTest';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let runningRasaModelsList = [];
        try {
            runningRasaModelsList = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[cognitive.redis::GetRunningRasaModelsFromTest] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(runningRasaModelsList);
    }).catch((err) => {
        logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllRunningRasaModelFromTest = GetAllRunningRasaModelFromTest;
