"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoveEvaluationPlanAbilities = exports.GetEvaluationPlanAbilities = exports.SaveEvaluationPlanAbilities = exports.EvaluationPlanAbilityRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function EvaluationPlanAbilityRedis(app) {
    redisClient = app.get('redisClient');
}
exports.EvaluationPlanAbilityRedis = EvaluationPlanAbilityRedis;
function SaveEvaluationPlanAbilities(vcc, evaluationPlanId, abilities) {
    let storage = 'EvaluationPlanAbility';
    let key = '';
    key += vcc;
    key += ':';
    key += evaluationPlanId;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(abilities))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[EvaluationPlanAbilityRedis::SaveEvaluationPlanAbility] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveEvaluationPlanAbilities = SaveEvaluationPlanAbilities;
function GetEvaluationPlanAbilities(vcc, evaluationPlanId) {
    let key = '';
    key += vcc;
    key += ':';
    key += evaluationPlanId;
    return redisClient.hgetAsync('EvaluationPlanAbility', key)
        .then((result) => {
        let interaction = [];
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch((err) => {
        logger.error('[EvaluationPlanAbilityRedis::GetEvaluationPlanAbilities] getting error:', err, 'key:', key);
        return Promise.reject(err);
    });
}
exports.GetEvaluationPlanAbilities = GetEvaluationPlanAbilities;
function RemoveEvaluationPlanAbilities(vcc, evaluationPlanId) {
    let storage = 'EvaluationPlanAbility';
    let key = '';
    key += vcc;
    key += ':';
    key += evaluationPlanId;
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(result);
    })
        .catch((err) => {
        logger.error('[EvaluationPlanAbilityRedis::RemoveEvaluationPlanAbilities] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.RemoveEvaluationPlanAbilities = RemoveEvaluationPlanAbilities;
