"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllGenerativeResultStorages = exports.DeleteGenerativeResult = exports.GetGenerativeResult = exports.SaveGenerativeResult = exports.DeleteGenerativeUsage = exports.GetGenerativeUsages = exports.GetGenerativeUsage = exports.SaveGenerativeUsage = exports.DeleteGenerativeToken = exports.GetGenerativeToken = exports.SaveGenerativeToken = exports.GenerativeRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function GenerativeRedis(app) {
    redisClient = app.get('redisClient');
}
exports.GenerativeRedis = GenerativeRedis;
function SaveGenerativeToken(vcc, token) {
    let hash = 'GenerativeToken';
    return redisClient.hsetAsync(hash, vcc, token)
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::SaveGenerativeToken] getting error:', err, 'with vcc:', vcc);
        return Promise.resolve(false);
    });
}
exports.SaveGenerativeToken = SaveGenerativeToken;
function GetGenerativeToken(vcc) {
    let hash = 'GenerativeToken';
    return redisClient.hgetAsync(hash, vcc)
        .then(result => {
        return Promise.resolve(result);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::GetGenerativeToken] getting err value:', err, 'hash:', hash, 'vcc:', vcc);
        return Promise.resolve(null);
    });
}
exports.GetGenerativeToken = GetGenerativeToken;
function DeleteGenerativeToken(vcc) {
    let hash = 'GenerativeToken';
    return redisClient.hdelAsync(hash, vcc)
        .then((result) => {
        return Promise.resolve(result ? true : false);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::DeleteGenerativeToken] getting error:', err, 'with vcc:', vcc);
        return Promise.resolve(null);
    });
}
exports.DeleteGenerativeToken = DeleteGenerativeToken;
function SaveGenerativeUsage(vcc, evaluationPlanId, interactionId, appId, tokens) {
    let hash = `${vcc}:GenerativeUsage:${evaluationPlanId}`;
    return redisClient.hsetAsync(hash, `${interactionId}:${appId}`, tokens)
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::SaveGenerativeUsage] getting error:', err, 'with vcc:', vcc);
        return Promise.resolve(false);
    });
}
exports.SaveGenerativeUsage = SaveGenerativeUsage;
function GetGenerativeUsage(vcc, evaluationPlanId, interactionId, appId) {
    let hash = `${vcc}:GenerativeUsage:${evaluationPlanId}`;
    return redisClient.hgetAsync(hash, `${interactionId}:${appId}`)
        .then(result => {
        return Promise.resolve(result);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::GetGenerativeUsage] getting err value:', err, 'hash:', hash, 'vcc:', vcc);
        return Promise.resolve(null);
    });
}
exports.GetGenerativeUsage = GetGenerativeUsage;
function GetGenerativeUsages(vcc, evaluationPlanId, interactionId) {
    let hash = `${vcc}:GenerativeUsage:${evaluationPlanId}`;
    let interactionAppIds = [];
    let usages = {};
    return redisClient.hkeysAsync(hash)
        .then(result => {
        if (!result)
            result = [];
        interactionAppIds = result.filter(r => r.includes(`${interactionId}:`));
        return redisClient.hmgetAsync(hash, interactionAppIds);
    })
        .then(result => {
        if (!result)
            result = [];
        interactionAppIds.forEach((iai, i) => {
            let app = iai.replace(`${interactionId}:`, '');
            let usage = Number(result[i]) || 0;
            usages[app] = usage;
        });
        return Promise.resolve(usages);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::GetGenerativeUsages] getting err value:', err, 'hash:', hash, 'vcc:', vcc);
        return Promise.resolve(null);
    });
}
exports.GetGenerativeUsages = GetGenerativeUsages;
function DeleteGenerativeUsage(vcc, evaluationPlanId, interactionId, appId) {
    let hash = `${vcc}:GenerativeUsage:${evaluationPlanId}`;
    return redisClient.hdelAsync(hash, `${interactionId}:${appId}`)
        .then((result) => {
        return Promise.resolve(result ? true : false);
    })
        .catch((err) => {
        logger.error('[GenerativeRedis::DeleteGenerativeUsage] getting error:', err, 'with vcc:', vcc);
        return Promise.resolve(null);
    });
}
exports.DeleteGenerativeUsage = DeleteGenerativeUsage;
function SaveGenerativeResult(interaction, index, results) {
    let logLabel = `[GenerativeRedis::${this.SaveGenerativeResult.name}] `;
    let regex = /\.(mp3|wav|ogg|json)$/;
    let interactionId = interaction.fileName.replace(regex, '');
    let hash = `GenerativeResult:${interaction.vcc}:${interactionId}`;
    return redisClient.hsetAsync(hash, `${interactionId}:${index}:${interaction.modelId}`, JSON.stringify({ index, results }))
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error(`${logLabel}getting error:`, err);
        return Promise.resolve(false);
    });
}
exports.SaveGenerativeResult = SaveGenerativeResult;
function GetGenerativeResult(key) {
    let logLabel = `[GenerativeRedis::${this.GetGenerativeResult.name}] `;
    let storage = 'GenerativeResult:';
    storage += key;
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let evaluationResults = [];
        try {
            let evaluationResult = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && evaluationResult) {
                for (let key in evaluationResult) {
                    let evaluation = JSON.parse(evaluationResult[key]);
                    evaluation.model = key.split(':')[2];
                    evaluationResults.push(evaluation);
                }
            }
        }
        catch (err) {
            logger.error(`${logLabel}getting err value:`, result);
            return Promise.resolve(null);
        }
        return evaluationResults;
    })
        .catch((err) => {
        logger.error(`${logLabel}getting err value:`, err);
        return Promise.resolve(err);
    });
}
exports.GetGenerativeResult = GetGenerativeResult;
function DeleteGenerativeResult(key) {
    let logLabel = `[GenerativeRedis::${this.DeleteGenerativeResult.name}] `;
    let storage = 'GenerativeResult:';
    storage += key;
    if (!key)
        return Promise.resolve(false);
    return redisClient.delAsync(storage)
        .then(_ => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error(`${logLabel}getting error:`, err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteGenerativeResult = DeleteGenerativeResult;
function GetAllGenerativeResultStorages() {
    let logLabel = `[GenerativeRedis::${this.GetAllGenerativeResultStorages.name}] `;
    let config = redis_config_1.REDIS_OPTIONS;
    let pattern = `${config.prefix}GenerativeResult:*`;
    return redisClient.keysAsync([pattern])
        .then((result) => {
        if (result?.length > 1)
            logger.info(`${logLabel}Getting ${result.length} storages`);
        let executionCodes = [];
        result.forEach(r => {
            executionCodes.push(r.replace(`${config.prefix}GenerativeResult:`, ''));
        });
        executionCodes = executionCodes.sort();
        return Promise.resolve(executionCodes);
    })
        .catch((err) => {
        logger.error(`${logLabel}getting err value:`, err);
        return Promise.resolve(err);
    });
}
exports.GetAllGenerativeResultStorages = GetAllGenerativeResultStorages;
