"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveNLUToTrainDueToRasaUpdate = exports.GetNLUToTrainDueToRasaUpdate = exports.SaveNLUToUtterancesBulk = exports.GetNLUToUtterancesBulk = exports.GetCurrentBatchesRasaModels = exports.DeleteBatchRasaModels = exports.SaveCurrentBatchesRasaModels = exports.DeleteFileNLU = exports.SaveFileNLU = exports.GetFileNLU = exports.DeleteNLU = exports.SaveNLU = exports.GetNlu = exports.NluRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function NluRedis(app) {
    redisClient = app.get('redisClient');
}
exports.NluRedis = NluRedis;
function GetNlu(key) {
    let storage = 'NLU';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let nlu;
        try {
            nlu = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return nlu;
    })
        .catch((err) => {
        logger.error('[NluRedis::GetNLU] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetNlu = GetNlu;
function SaveNLU(key, nlu) {
    let storage = 'NLU';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(nlu))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[NluRedis::SaveNLU] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveNLU = SaveNLU;
function DeleteNLU(key) {
    let storage = 'NLU';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[NluRedis::DeleteNLU] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteNLU = DeleteNLU;
function GetFileNLU(storage, key) {
    return redisClient.hgetAsync(storage + ":NLUFile", key)
        .then((result) => {
        let nluData;
        try {
            nluData = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return nluData;
    }).catch((err) => {
        logger.error('[NluRedis::GetFileNLU] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetFileNLU = GetFileNLU;
function SaveFileNLU(storage, key, fileContent) {
    if (!key || !storage)
        return Promise.resolve(false);
    let body = {
        timeInit: new Date(),
        data: fileContent
    };
    return redisClient.hsetAsync(storage + ":NLUFile", key, JSON.stringify(body))
        .then((result) => {
        return redisClient.expireAsync(storage + ":NLUFile", redis_config_1.REDIS_DATA_PERSISTENCE_ONE_HOUR);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[nlu.redis::SaveFileNLU] getting error:', err, 'with key: ', key);
        return Promise.resolve(false);
    });
}
exports.SaveFileNLU = SaveFileNLU;
function DeleteFileNLU(storage, key) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage + ":NLUFile", key)
        .then((result) => {
        return redisClient.expireAsync(storage + ":NLUFile", redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[nku.redis::DeleteFileNLU] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeleteFileNLU = DeleteFileNLU;
function SaveCurrentBatchesRasaModels(key, nlus) {
    let storage = 'BatchRasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(nlus))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        logger.error('[NluRedis::SaveCurrentBachesRASANLUs] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveCurrentBatchesRasaModels = SaveCurrentBatchesRasaModels;
function DeleteBatchRasaModels(key) {
    let storage = 'BatchRasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[NluRedis::DeleteBatchRasaModels] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteBatchRasaModels = DeleteBatchRasaModels;
function GetCurrentBatchesRasaModels() {
    let storage = 'BatchRasaModel';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let batchNluList = [];
        try {
            batchNluList = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[nlu.redis::GetCurrentBachesRasaModels] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(batchNluList);
    })
        .catch((err) => {
        logger.error('[nlu.redis::GetCurrentBachesRasaModels] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetCurrentBatchesRasaModels = GetCurrentBatchesRasaModels;
function GetNLUToUtterancesBulk() {
    let storage = 'NLUToOperate';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let objectsList = [];
        let batchNluList = [];
        try {
            objectsList = (typeof result === 'string') ? JSON.parse(result) : result;
            batchNluList = objectsList ? objectsList['UtterancesBulk'] : null;
            batchNluList = (typeof batchNluList === 'string') ? JSON.parse(batchNluList) : batchNluList;
        }
        catch (err) {
            logger.error('[nlu.redis::GetNLUToUtterancesBulk] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(batchNluList);
    })
        .catch((err) => {
        logger.error('[nlu.redis::GetNLUToUtterancesBulk] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetNLUToUtterancesBulk = GetNLUToUtterancesBulk;
function SaveNLUToUtterancesBulk(nlus) {
    let storage = 'NLUToOperate';
    let key = 'UtterancesBulk';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(nlus))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[NluRedis::SaveNLUToUtterancesBulk] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveNLUToUtterancesBulk = SaveNLUToUtterancesBulk;
function GetNLUToTrainDueToRasaUpdate() {
    let storage = 'NLUToOperate';
    return redisClient.hgetallAsync(storage)
        .then((result) => {
        let objectsList = [];
        let batchNluList = [];
        try {
            objectsList = (typeof result === 'string') ? JSON.parse(result) : result;
            batchNluList = objectsList ? objectsList['TrainDueToRasaUpdate'] : null;
            batchNluList = (typeof batchNluList === 'string') ? JSON.parse(batchNluList) : batchNluList;
        }
        catch (err) {
            logger.error('[nlu.redis::GetNLUToTrainDueToRasaUpdate] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(batchNluList);
    })
        .catch((err) => {
        logger.error('[nlu.redis::GetNLUToTrainDueToRasaUpdate] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetNLUToTrainDueToRasaUpdate = GetNLUToTrainDueToRasaUpdate;
function SaveNLUToTrainDueToRasaUpdate(nlus) {
    let storage = 'NLUToOperate';
    let key = 'TrainDueToRasaUpdate';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(nlus))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[NluRedis::SaveNLUToUtterancesBulk] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveNLUToTrainDueToRasaUpdate = SaveNLUToTrainDueToRasaUpdate;
