"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoveStudyTemplateThemes = exports.GetStudyTemplateThemes = exports.SaveStudyTemplateThemes = exports.StudyTemplateThemeRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function StudyTemplateThemeRedis(app) {
    redisClient = app.get('redisClient');
}
exports.StudyTemplateThemeRedis = StudyTemplateThemeRedis;
function SaveStudyTemplateThemes(vcc, studyId, themes) {
    let storage = 'StudyTemplateTheme';
    let key = '';
    key += vcc;
    key += ':';
    key += studyId;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(themes))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[StudyThemeRedis::SaveStudyTemplateThemes] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveStudyTemplateThemes = SaveStudyTemplateThemes;
function GetStudyTemplateThemes(vcc, studyId) {
    let key = '';
    key += vcc;
    key += ':';
    key += studyId;
    return redisClient.hgetAsync('StudyTemplateTheme', key)
        .then((result) => {
        let themes = [];
        try {
            themes = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return themes;
    })
        .catch((err) => {
        logger.error('[StudyThemeRedis::GetStudyTemplateThemes] getting error:', err, 'key:', key);
        return Promise.reject(err);
    });
}
exports.GetStudyTemplateThemes = GetStudyTemplateThemes;
function RemoveStudyTemplateThemes(vcc, studyId) {
    let storage = 'StudyTemplateTheme';
    let key = '';
    key += vcc;
    key += ':';
    key += studyId;
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(result);
    })
        .catch((err) => {
        logger.error('[StudyTemplateThemeRedis::RemoveStudyTemplateThemes] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.RemoveStudyTemplateThemes = RemoveStudyTemplateThemes;
