"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAllReadyTipificationInteractionStorages = exports.GetReadyTipificationInteractions = exports.SaveReadyTipificationInteraction = exports.DeleteTipificationInteraction = exports.SaveTipificationInteraction = exports.GetTipificationInteraction = exports.TipificationRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
let logger = log4js.getLogger('Redis');
let redisClient;
function TipificationRedis(app) {
    redisClient = app.get('redisClient');
}
exports.TipificationRedis = TipificationRedis;
function GetTipificationInteraction(key) {
    let storage = 'Tipification';
    return redisClient.hgetAsync(storage, key)
        .then((result) => {
        let interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch((err) => {
        logger.error('[TipificationRedis::GetTipificationInteraction] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetTipificationInteraction = GetTipificationInteraction;
function SaveTipificationInteraction(key, interaction) {
    let storage = 'Tipification';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then((result) => {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[TipificationRedis::SaveTipificationInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveTipificationInteraction = SaveTipificationInteraction;
function DeleteTipificationInteraction(key) {
    let storage = 'Tipification';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[TipificationRedis::DeleteTipificationInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteTipificationInteraction = DeleteTipificationInteraction;
function SaveReadyTipificationInteraction(key, interaction, executionCode) {
    let storage = 'ReadyTipificationInteraction';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then((result) => {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then((result) => {
        return Promise.resolve(true);
    })
        .catch((err) => {
        logger.error('[TipificationRedis::SaveReadyTipificationInteraction] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyTipificationInteraction = SaveReadyTipificationInteraction;
function GetReadyTipificationInteractions(executionCode) {
    let hash = 'ReadyTipificationInteraction:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then((result) => {
        let interactions = [];
        try {
            let interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (let key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[TipificationRedis::GetReadyTipificationInteraction] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch((err) => {
        logger.error('[TipificationRedis::GetReadyTipificationInteraction] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyTipificationInteractions = GetReadyTipificationInteractions;
function GetAllReadyTipificationInteractionStorages(executionCode) {
    let config = redis_config_1.REDIS_OPTIONS;
    let pattern = `${config.prefix}ReadyTipificationInteraction:*`;
    return redisClient.keysAsync([pattern])
        .then((result) => {
        if (result?.length > 1)
            logger.info(`[TipificationRedis::ReadyTipificationInteraction] Getting ${result.length} storages for execution ${executionCode}`);
        let executionCodes = [];
        result.forEach(r => {
            executionCodes.push(r.replace(`${config.prefix}ReadyTipificationInteraction:`, ''));
        });
        executionCodes = executionCodes.sort();
        return executionCodes;
    })
        .catch((err) => {
        logger.error('[TipificationRedis::ReadyTipificationInteraction] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetAllReadyTipificationInteractionStorages = GetAllReadyTipificationInteractionStorages;
