"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteRequestData = exports.GetRequestData = exports.SaveRequestData = exports.DeleteVCC = exports.GetVCC = exports.SaveVCC = exports.AddTaskCognitiveEngineMigration = exports.UpdateCognitiveEngineMigrationData = exports.GetCognitiveEngineMigrationData = exports.SaveCognitiveEngineMigrationModel = exports.SaveCognitiveEngineMigrationNewId = exports.GetCognitiveEngineMigrationNewId = exports.GetCognitiveEngineMigrationModel = exports.UpdateCognitiveEngineMigrationModel = exports.DeleteCognitiveEngineMigrationModel = exports.DeleteCognitiveEngineMigrationTask = exports.DeleteCognitiveEngineMigrationKey = exports.VccRedis = void 0;
const log4js = require("log4js");
const redis_config_1 = require("../../config/redis.config");
const promises_shared_1 = require("../../shared/promises.shared");
let logger = log4js.getLogger('Redis');
let redisClient;
function VccRedis(app) {
    redisClient = app.get('redisClient');
}
exports.VccRedis = VccRedis;
function VccRedisGenericErrorHandler(err) {
    let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
    logger.error(errorMsg);
    return Promise.reject(err);
}
function DeleteCognitiveEngineMigrationKey(key) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.delAsync(key);
    }).catch(VccRedisGenericErrorHandler);
}
exports.DeleteCognitiveEngineMigrationKey = DeleteCognitiveEngineMigrationKey;
function DeleteCognitiveEngineMigrationTask(key, id) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hdelAsync(key, id);
    }).catch(VccRedisGenericErrorHandler);
}
exports.DeleteCognitiveEngineMigrationTask = DeleteCognitiveEngineMigrationTask;
function DeleteCognitiveEngineMigrationModel(key, id) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hdelAsync(key, id);
    }).catch(VccRedisGenericErrorHandler);
}
exports.DeleteCognitiveEngineMigrationModel = DeleteCognitiveEngineMigrationModel;
function UpdateCognitiveEngineMigrationModel(key, id, data) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hsetAsync(key, id, JSON.stringify(data));
    }).
        then(() => {
        return true;
    })
        .catch(VccRedisGenericErrorHandler);
}
exports.UpdateCognitiveEngineMigrationModel = UpdateCognitiveEngineMigrationModel;
function GetCognitiveEngineMigrationModel(key, keys) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hmgetAsync(key, keys);
    }).catch(VccRedisGenericErrorHandler);
}
exports.GetCognitiveEngineMigrationModel = GetCognitiveEngineMigrationModel;
function GetCognitiveEngineMigrationNewId(key, oldAppId) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hgetAsync(key, oldAppId);
    }).catch(VccRedisGenericErrorHandler);
}
exports.GetCognitiveEngineMigrationNewId = GetCognitiveEngineMigrationNewId;
function SaveCognitiveEngineMigrationNewId(key, oldAppId, newAppId) {
    return promises_shared_1.DummyPromise().then(_ => {
        return redisClient.hsetAsync(key, oldAppId, newAppId);
    })
        .then(_ => {
        return redisClient.expireAsync(key, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .catch(VccRedisGenericErrorHandler);
}
exports.SaveCognitiveEngineMigrationNewId = SaveCognitiveEngineMigrationNewId;
function SaveCognitiveEngineMigrationModel(key, id, rows) {
    return promises_shared_1.DummyPromise().then(_ => {
        let promises = [];
        let keys = [];
        for (let row of rows) {
            promises.push(redisClient.hsetAsync(key, row[id], JSON.stringify(row)));
            keys.push(row[id]);
        }
        promises.push(redisClient.lpushAsync(`${key}:Keys`, keys));
        return Promise.all(promises);
    })
        .then(_ => {
        return redisClient.expireAsync(key, redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(() => {
        return redisClient.expireAsync(`${key}:Keys`, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .catch(VccRedisGenericErrorHandler);
}
exports.SaveCognitiveEngineMigrationModel = SaveCognitiveEngineMigrationModel;
function GetCognitiveEngineMigrationData() {
    logger.info('[VccRedis::GetCognitiveEngineMigrationData] Registering CognitiveEngineMigration to redis');
    return new Promise((resolve, reject) => {
        return redisClient.hgetall('CognitiveEngineMigration', (err, result) => {
            if (err) {
                let errorMsg = 'Requesting error: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
                logger.error('[VccRedis::GetCognitiveEngineMigrationData] ' + errorMsg);
                reject(err);
            }
            else {
                resolve(result);
            }
        });
    });
}
exports.GetCognitiveEngineMigrationData = GetCognitiveEngineMigrationData;
function UpdateCognitiveEngineMigrationData(vcc, data) {
    return AddTaskCognitiveEngineMigration(vcc, data);
}
exports.UpdateCognitiveEngineMigrationData = UpdateCognitiveEngineMigrationData;
function AddTaskCognitiveEngineMigration(vcc, data) {
    return redisClient.hsetAsync('CognitiveEngineMigration', vcc, JSON.stringify(data))
        .then((_) => {
        return redisClient.expireAsync('CognitiveEngineMigration', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).catch(VccRedisGenericErrorHandler);
}
exports.AddTaskCognitiveEngineMigration = AddTaskCognitiveEngineMigration;
function SaveVCC(vcc, objVCC) {
    if (!objVCC || !objVCC.id)
        throw 'CANNOT_SAVE_WITH_NULL_ID';
    let vccaux = (objVCC || {});
    return redisClient.hsetAsync(vcc + ':VCC', objVCC.id, JSON.stringify(objVCC)).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        console.log(err);
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveVCC = SaveVCC;
function GetVCC(vcc, vccId) {
    if (!vccId)
        throw 'CANNOT_GET_WITH_NULL_ID';
    return redisClient.hgetAsync(vcc + ':VCC', vccId)
        .then((result) => {
        let vcc;
        try {
            vcc = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(vcc);
    }).catch((err) => {
        let errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetVCC = GetVCC;
function DeleteVCC(vcc) {
    let storage = vcc + ':VCC';
    if (!vcc)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, vcc)
        .then((result) => {
        return Promise.resolve(result ? true : false);
    }).catch((err) => {
        logger.error('[VccRedis::DeleteVCC] getting error:', err, 'with key:', vcc);
        return Promise.resolve(null);
    });
}
exports.DeleteVCC = DeleteVCC;
function SaveRequestData(vcc, objRequestData) {
    if (!objRequestData) {
        logger.warn('[VccRedis::SaveRequestData] Received RequestData is null');
        throw 'CANNOT_SAVE_WITH_NULL_ID';
    }
    logger.info('[VccRedis::SaveRequestData] Registering RequestData on redis');
    return redisClient.hsetAsync(vcc + ':RequestData', vcc, JSON.stringify(objRequestData))
        .then((_) => {
        logger.info('[VccRedis::SaveRequestData] Setting the expire time');
        return redisClient.expireAsync(vcc + ':RequestData', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then((result) => {
        return Promise.resolve(true);
    }).catch((err) => {
        let errorMsg = 'Error on: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[VccRedis::SaveRequestData] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRequestData = SaveRequestData;
function GetRequestData(vcc) {
    if (!vcc) {
        logger.warn('[VccRedis::GetRequestData] Received Vcc is null');
        throw 'CANNOT_GET_WITH_NULL_ID';
    }
    logger.info('[VccRedis::GetRequestData] Registering RequestData to redis');
    return redisClient.hgetAsync(vcc + ':RequestData', vcc)
        .then((result) => {
        let requestData;
        try {
            logger.info('[VccRedis::GetRequestData] Parsing received object from redis');
            requestData = JSON.parse(result);
        }
        catch (err) {
            logger.info('[VccRedis::GetRequestData] Cann\'t parse object from redis');
            return Promise.resolve(null);
        }
        return Promise.resolve(requestData);
    }).catch((err) => {
        let errorMsg = 'Requesting error: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[VccRedis::GetRequestData] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetRequestData = GetRequestData;
function DeleteRequestData(vcc) {
    let storage = vcc + ':RequestData';
    if (!vcc)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, vcc)
        .then((result) => {
        return Promise.resolve(result ? true : false);
    })
        .catch((err) => {
        logger.error('[VccRedis::DeleteRequestData] getting error:', err, 'with key:', vcc);
        return Promise.resolve(null);
    });
}
exports.DeleteRequestData = DeleteRequestData;
