"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioTransferImplementation = void 0;
const log4js = require("log4js");
const fs = require("fs");
const child_process = require("child_process");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const file_manager_1 = require("../shared/file-manager");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
let exec = child_process.exec;
const aws_s3_implementation_1 = require("./aws-s3.implementation");
const billing_implementation_1 = require("../implementation/billing.implementation");
const process_implementation_1 = require("./process.implementation");
const audio_service_1 = require("../services/audio.service");
const interaction_service_1 = require("../services/interaction.service");
const request_shared_1 = require("../shared/request.shared");
const arrays_shared_1 = require("../shared/arrays.shared");
const SSH2Promise = require("ssh2-promise");
const moment = require("moment");
const timers_1 = require("timers");
const disk_space_check_shared_1 = require("../shared/disk-space-check.shared");
const task_config_1 = require("../config/task.config");
let Client = require('ftp');
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'AudioTransferImplementation';
class AudioTransferImplementation {
    constructor() {
        this.sshConnections = [];
        this.environments = {};
    }
    Execute(executionCode, rateRecursive) {
        logger.info(`[AudioTransferImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let startTime = moment();
        let executeIndex = 1;
        let ratio = config_1.RATIO_AUDIO_TRANSFER;
        let executeResult = true;
        let currentBlockPromise;
        return promises_shared_1.SleepPromise(config_1.SLEEP_AUDIO_TRANSFER)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_CHECK_DISK_USAGE';
            return disk_space_check_shared_1.GetDiskUsage();
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_DISK_USAGE_FAILED';
            logger.info(`[AudioTransferImplementation::Execute] executionCode: ${executionCode} free: ${((result.free * 100) / result.size)}%`);
            if ((result.free / result.size) < task_config_1.MINIMUN_FREE_DISK_STORAGE)
                throw 'SERVER_INFO_NO_ENOUGHT_FREE_STORAGE';
            currentBlockPromise = 'BLOCK_02_GET_FROM_REDIS';
            return redis_service_1.Redis.LRange('Audios', 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_START_EXTERNAL_CONNECTIONS';
            ratio = result && result.length <= ratio ? result.length : 0;
            let promiseArray = [];
            arrays_shared_1.GetUniqueFromArray(result.map(r => {
                let items = r.split(':');
                return `${items[0]}:${items[1]}`;
            }))
                .forEach(r => {
                promiseArray.push(this.StartExternalConnection(r));
            });
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_04_LAUNCH_AUDIO_TRANSFER_IMPL';
            let promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(this.AudioTransferExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_05_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('Audios', 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_06_RELAUNCH_AUDIO_TRANSFER_IMPL';
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_07_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_AUDIO_TRANSFER)
                logger.info(`[AudioTransferImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive} after ${moment().diff(startTime, 'minutes')} minutes`);
            this.EndExternalConnection();
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[AudioTransferImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            this.EndExternalConnection();
            return Promise.resolve(false);
        });
    }
    AudioTransferExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction = null;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let currentBlockPromise = null;
        return redis_service_1.Redis.RPopLPush('Audios', 'AudiosExecute')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_01_GETTING_DATA_FROM_PROCESS_REDIS_LIST';
            if (queueId?.length) {
                currentQueueId = queueId;
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((processInteractionResult) => {
            currentBlockPromise = 'BLOCK_02_SEARCHING_JSON_FILE';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.isCancel)
                    return Promise.resolve(null);
                if (processInteraction.retranscription)
                    this.DeleteLocalTranscriptionJSON(vcc, interactionId, interactionSection, interactionPart);
                return this.FindLocalTranscriptionJSON(vcc, interactionId, interactionSection, interactionPart);
            }
            return Promise.resolve(false);
        })
            .then((foundLocalJSONTranscription) => {
            currentBlockPromise = 'BLOCK_03_RESOLVING_TRANSCRIPTION_STATUS';
            if (processInteraction?.isCancel)
                return Promise.resolve(null);
            let promises = [
                Promise.resolve(foundLocalJSONTranscription)
            ];
            if (foundLocalJSONTranscription) {
                promises.push(this.ResumeTranscription(storage, currentQueueId, processInteraction));
            }
            return Promise.all(promises);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_04_CREATING_TRANSCRIPTION_FOLDER';
            if (processInteraction?.isCancel)
                return Promise.resolve(null);
            let haveToResume = results[0];
            let hasResumed = results[1] ? true : false;
            if (haveToResume && hasResumed)
                throw new Error("RESUMING_TRANSCRIPTION");
            if (processInteraction) {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                let baseFolder = '';
                if (processInteraction.interactionType === 'CALL') {
                    baseFolder = ['ftp', 'sftp', 's3'].includes(processInteraction.environmentType) ? config_1.PATH_FOLDER_PROCESS_AUDIO : config_1.PATH_FOLDER_TEMP_AUDIO;
                }
                else {
                    baseFolder = config_1.TRANSCRIPTION_FOLDER;
                }
                return file_manager_1.CreateFolder(baseFolder + '/' + processInteraction.vcc);
            }
            else {
                if (currentQueueId)
                    logger.warn('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_05_EXECUTING_FUNCTION';
            if (processInteraction?.isCancel)
                return Promise.resolve(null);
            if (processInteraction) {
                let promise = null;
                if (processInteraction.interactionType === 'CALL') {
                    switch (processInteraction.environmentType) {
                        case 'allegro':
                        case 'i6':
                            promise = this.GetIntegratedAudioFile(processInteraction, currentQueueId, executionCode);
                            break;
                        case 'ftp':
                        case 'i-ftp':
                            promise = this.GetFTPAudioFile(processInteraction, currentQueueId, executionCode);
                            break;
                        case 'sftp':
                        case 'i-sftp':
                            promise = this.GetSFTPAudioFile(processInteraction, currentQueueId, executionCode);
                            break;
                        case 's3':
                        case 'i-s3':
                            promise = this.GetS3AudioFile(processInteraction, currentQueueId, executionCode);
                            break;
                    }
                }
                else
                    promise = this.GetI6TextFile(processInteraction, currentQueueId, executionCode);
                return promise;
            }
            return Promise.resolve(null);
        }).then(result => {
            currentBlockPromise = 'BLOCK_06_GETTING_AUDIO_INFO';
            if (processInteraction?.status === 1
                &&
                    processInteraction?.statusDescription === 'pendingAnalyze') {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TO_MEDIA_ANALIZE');
                return this.GetAudioInfo(processInteraction, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_07_SAVING_INTERACTION_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_08_FINISHING_QUEUE';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result && processInteraction && processInteraction.status === 1) {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                if (processInteraction.statusDescription === 'pendingUploadRawAudio') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('DiarizationConvert', currentQueueId));
                }
                else if (processInteraction.statusDescription === 'pendingUpload') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueR('TranscriptionsUpload', currentQueueId));
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.errorMessage === 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND') {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_AUDIO_TRANSFER;
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_AUDIO_TRANSFER) {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                this.CloseTranscriptionDetailWithMissingData(billing, interactionData);
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_09_UPDATING_PROGRESS';
            let promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_AUDIO_TRANSFER) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_10_RESOLVING';
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CloseTranscriptionDetailWithMissingData(billing, interactionData, implementation) {
        implementation ?? (implementation = 'AudioTransferImplementation');
        let batchTypeName = null;
        let batchTypeId = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcess(interactionData.vcc, interactionData.processId);
        })
            .then(result => {
            if (result.type === 'SPEECH') {
                batchTypeName = 'TRANSCRIPTION';
                batchTypeId = interactionData.processId;
                return Promise.resolve(null);
            }
            else {
                batchTypeName = 'EVALUATION_PLAN';
                batchTypeId = result.description;
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`[${implementation}::CloseTranscriptionDetailWithMissingData] getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    FindLocalTranscriptionJSON(vcc, id, section, part) {
        let path = `${config_1.TRANSCRIPTION_FOLDER}/${vcc}/${id}_${section}_${part}.json`;
        return promises_shared_1.DummyPromise().then(_ => {
            logger.info('[AudioTransferImplementation::FindLocalTranscriptionJSON] file:', path);
            return file_manager_1.ExistInFileSystem(path);
        })
            .catch(err => {
            logger.debug('[AudioTransferImplementation::FindLocalTranscriptionJSON] Error:', err);
            return Promise.resolve(false);
        });
    }
    DeleteLocalTranscriptionJSON(vcc, id, section, part) {
        let path = `${config_1.TRANSCRIPTION_FOLDER}/${vcc}/${id}_${section}_${part}.json`;
        return promises_shared_1.DummyPromise().then(_ => {
            logger.info('[AudioTransferImplementation::DeleteLocalTranscriptionJSON] file: ', path);
            return file_manager_1.DeleteFile(path);
        })
            .catch(err => {
            logger.error('[AudioTransferImplementation::DeleteLocalTranscriptionJSON] Error:', err);
            return Promise.resolve(false);
        });
    }
    ResumeTranscription(storage, currentQueueId, pi) {
        let path = `${config_1.TRANSCRIPTION_FOLDER}/${pi.vcc}/${pi.id}_${pi.section}_${pi.part}.json`;
        let sentimentNlu;
        let resumeResult = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (config_1.USAGE_SENTIMENT_ANALYSIS)
                return mssql_service_1.MSSql.GetNLU(pi.vcc, 'sentiment-' + pi.language);
            return null;
        })
            .then(result => {
            if (result)
                sentimentNlu = result;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            if (result) {
                try {
                    let data = JSON.parse(result);
                    let validChar = 0;
                    if (data.results && data.results.length > 0) {
                        data.results.map(r => {
                            if (r.alternatives && r.alternatives.length > 0) {
                                validChar += r.alternatives[0].content ? r.alternatives[0].content.length + 1 : 0;
                            }
                        });
                    }
                    if (!pi['fileName'])
                        pi['fileName'] = `${pi.id}_${pi.section}_${pi.part}.${pi.format || 'wav'}`;
                    pi['durationLinguisticTime'] = data?.job?.duration ? data.job.duration : null;
                    pi['validChar'] = validChar;
                    pi['validWords'] = data && data.results ? data.results.length : 0;
                    pi['transcriptionApi'] = data.transcriptionApi;
                    pi['status'] = 3;
                    pi['statusDescription'] = pi.isDiariced === true ? 'pendingIdentifyAgent' : config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                    pi['retry'] = 0;
                    pi['uploadedDate'] = moment().format('x');
                    pi['errorFunction'] = null;
                    pi['errorMessage'] = null;
                    pi['errorDetail'] = null;
                    return Promise.resolve(true);
                }
                catch (err) {
                    logger.error('[AudioTransferImplementation::ResumeTranscription] Error: ', err);
                    throw 'CANNOT_PARSE_JSON_TRANSCRIPTION';
                }
            }
            return Promise.resolve(false);
        })
            .then((result) => {
            if (result) {
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, pi);
            }
            return Promise.resolve(false);
        })
            .then((result) => {
            if (result) {
                let queueTarget = null;
                switch (pi['statusDescription']) {
                    case 'pendingAnalysis':
                        queueTarget = 'TranscriptionsDownload';
                        break;
                    case 'pendingIdentifyAgent':
                        queueTarget = 'DiarizationAnalysis';
                        break;
                    case 'pendingSentimentAnalysis':
                        queueTarget = 'SentimentAnalysis';
                        break;
                }
                logger.debug(`[AudioTransferImplementation::ResumeTranscription] Skipping ${pi.id}_${pi.section}_${pi.part} => ${pi['statusDescription']}`);
                return redis_service_1.Redis.DeleteInsertOnList('AudiosExecute', queueTarget, currentQueueId);
            }
            throw 'CANNOT_SAVE_PROCESS_INTERACTION_TO_REDIS';
        })
            .then(result => {
            if (result) {
                resumeResult = true;
                let promiseArray = [];
                let sentimentNluToRegister = config_1.USAGE_SENTIMENT_ANALYSIS ? [`sentiment-${pi.language}`] : [];
                let diarizacionNluToRegister = [];
                if (pi.isDiariced)
                    diarizacionNluToRegister.push(`diarization-${pi.language}`);
                let process = new process_implementation_1.ProcessImplementation();
                if (sentimentNluToRegister.length)
                    promiseArray.push(process.RunCurrentBatchesRasaModels(sentimentNlu.vcc, 'TRANSCRIPTION', pi.processId, sentimentNluToRegister));
                if (diarizacionNluToRegister.length)
                    promiseArray.push(process.RunCurrentBatchesRasaModels(pi.vcc, 'TRANSCRIPTION', pi.processId, diarizacionNluToRegister));
                if (promiseArray.length)
                    return Promise.all(promiseArray);
            }
            return Promise.resolve(result);
        })
            .then(_ => {
            return Promise.resolve(resumeResult);
        })
            .catch(err => {
            logger.error(`[AudioTransferImplementation::ResumeTranscription] Error resuming ${pi.id}_${pi.section}_${pi.part}:`, err);
            return Promise.resolve(false);
        });
    }
    GetIntegratedAudioFile(interaction, currentQueueId, executionCode) {
        let localFilePath = '';
        let localFileName = '';
        let timer;
        timer = timers_1.setInterval(() => {
            logger.info(`[AudioTransferImplementation::GetIntegratedAudioFile] executionCode: ${executionCode} waiting for ${currentQueueId}`);
        }, 60 * 1000);
        return audio_service_1.GetAudioFileFromIntegrated(interaction.vcc, interaction.environmentType, interaction.id, interaction.section, interaction.part, null, interaction.format, 'REMOTE')
            .catch((err) => {
            throw err && err.extraContent ? err.extraContent : 'GET_AUDIO_REQUEST_FAILED';
        })
            .then(result => {
            localFileName += result.audioFile.split('/').pop();
            localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise((resolve, reject) => {
                fs.open(localFilePath, 'w', (err, fd) => {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        return reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    if (!Buffer.isBuffer(result.buffer)) {
                        logger.error(`[AudioTransferImplementation::GetIntegratedAudioFile] executionCode: ${executionCode} getting error, the result is not a buffer object`);
                        return reject('SERVER_ERROR_CANNOT_GET_AUDIO_FILE_BUFFER');
                    }
                    else {
                        fs.write(fd, result.buffer, 0, result.buffer.length, null, (err) => {
                            if (err) {
                                logger.info('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                                return reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                            }
                            fs.close(fd, () => {
                                return resolve('OK');
                            });
                        });
                    }
                });
            });
        })
            .then((result) => {
            timers_1.clearInterval(timer);
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['retry'] = 0;
                interaction['filePath'] = localFilePath;
                interaction['fileName'] = localFileName;
            }
            else {
                logger.error('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetIntegratedAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            timers_1.clearInterval(timer);
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetIntegratedAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    GetI6TextFile(interaction, currentQueueId, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return interaction_service_1.GetJsonDataFromI6Data(interaction);
        })
            .then(result => {
            if (result) {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetI6TextFile';
                interaction['errorMessage'] = 'FILES_DOWNLOAD_FAILED';
                interaction['errorDetail'] = 'FILES_DOWNLOAD_FAILED';
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[GetI6TextFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetI6TextFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    GetFileNamePathFromInteraction(interaction, fileType, message) {
        let fileNamePath = '';
        let localPath = '';
        localPath = config_1.TRANSCRIPTION_FOLDER;
        localPath += '/';
        localPath += interaction.vcc;
        localPath += '/';
        fileNamePath = localPath;
        fileNamePath += this.GetFileNameFromInteraction(interaction, fileType, message);
        return fileNamePath;
    }
    GetFileNameFromInteraction(interaction, fileType, message) {
        let interactionId = interaction.id ? interaction.id : interaction.interactionId;
        let fileName = '';
        let processInteractionId = JSON.parse(JSON.stringify(interactionId));
        let cleanedProcessInteractionId = processInteractionId.split(':').join('_');
        let localViewFileName = '';
        let localJsonFileName = '';
        localViewFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId;
        localJsonFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId + '.json';
        if (interaction.interactionType === 'MAIL') {
            if (message && !isNaN(message))
                localViewFileName += `_${message}`;
            else
                localViewFileName += `_${interaction.totalMessage}`;
            localViewFileName += '.eml';
        }
        else
            localViewFileName += '.html';
        if (fileType === 'view')
            fileName = localViewFileName;
        if (fileType === 'json')
            fileName = localJsonFileName;
        return fileName;
    }
    GetI6TextFileFromUrl(executionCode, currentQueueId, localI6TextFileName, remoteI6TextFilePath) {
        logger.info(`[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode: ${executionCode} getting queueId: ${currentQueueId} url: ${remoteI6TextFilePath}`);
        let file;
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let getOptions = {
                uri: remoteI6TextFilePath,
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded'
                },
                retryProtocol: false,
                timeout: 30000
            };
            return request_shared_1.SimpleGetRequest(getOptions);
        })
            .then(result => {
            logger.debug(`[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode: ${executionCode} getting queueId: ${currentQueueId} result: ${result}`);
            if (!result)
                throw 'SERVER_ERROR_CONTENT_DOWNLOAD_FAILED';
            if (typeof result === 'object')
                return file_manager_1.WriteFileContent(localI6TextFileName, JSON.stringify(result), 'utf-8', true);
            else
                throw 'SERVER_ERROR_GET_TEXT_REQUEST_FAILED';
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SAVE_TEXT_CONTENT_FAILED';
            genericResult.status = true;
            genericResult.result = 'DOWNLOAD_SUCCESSFULL';
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : err.message;
            logger.error('[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'cannot download audio from: ', remoteI6TextFilePath, 'getting errorMessage:', errorMessage);
            genericResult.status = false;
            genericResult.error = err.message;
            return Promise.resolve(genericResult);
        });
    }
    GetFTPAudioFile(interaction, currentQueueId, executionCode) {
        let localFilePath = '';
        let localFileName = '';
        let startTime = moment();
        let timer;
        let minutesInterval = 1 * 60 * 1000;
        let maxWaitingMinutes = 5;
        timer = timers_1.setInterval(() => {
            let currentTime = moment();
            let minutes = currentTime.diff(startTime, 'minutes');
            logger.info(`[AudioTransferImplementation::GetFTPAudioFile] executionCode: ${executionCode} waiting ${minutes} minutes for ${currentQueueId}`);
            if (minutes >= maxWaitingMinutes) {
                try {
                    logger.info(`[AudioTransferImplementation::GetFTPAudioFile] executionCode: ${executionCode} closing FTP client after ${maxWaitingMinutes} minutos for ${currentQueueId}`);
                    this.environments[`${interaction.vcc}:${interaction.name}`].end();
                }
                catch (e) {
                    logger.error(`[AudioTransferImplementation::GetFTPAudioFile] executionCode: ${executionCode} failed closing FTP client for ${currentQueueId}`, e);
                }
            }
        }, minutesInterval);
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (interaction.id && interaction.id.length > 0) {
                localFileName = interaction.id;
                localFileName += '_';
                localFileName += interaction.section && interaction.section > 0 ? interaction.section : '0';
                localFileName += '_';
                localFileName += interaction.part && interaction.part > 0 ? interaction.part : '0';
            }
            let audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
                relativeFileName: interaction.relativeFileName,
                section: interaction.section,
                part: interaction.part
            };
            let environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.bucket = interaction.bucket;
            environment.ftpDirectory = interaction.ftpDirectory;
            environment.type = interaction.environmentType;
            return audio_service_1.GetAudioFileFromFtp(interaction.vcc, audioData, environment, this.environments[`${interaction.vcc}:${interaction.name}`])
                .catch((err) => {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_FTP';
            });
        })
            .then(result => {
            if (!Buffer.isBuffer(result.buffer)) {
                logger.error(`[AudioTransferImplementation::GetFTPAudioFile] executionCode: ${executionCode} getting error, the result is not a buffer object`);
                throw ('SERVER_ERROR_CANNOT_GET_AUDIO_FILE_BUFFER');
            }
            localFileName += '.' + result.audioFile.split('.').pop();
            if (['ftp', 'sftp', 's3'].includes(interaction.environmentType))
                localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            else
                localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise((resolve, reject) => {
                fs.open(localFilePath, 'w', (err, fd) => {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, (err) => {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, () => {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then((result) => {
            timers_1.clearInterval(timer);
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            timers_1.clearInterval(timer);
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetFTPAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    GetSFTPAudioFile(interaction, currentQueueId, executionCode) {
        let localFilePath = '';
        let localFileName = '';
        return promises_shared_1.DummyPromise()
            .then((result) => {
            if (interaction.id && interaction.id.length > 0) {
                localFileName = interaction.id;
                localFileName += '_';
                localFileName += interaction.section && interaction.section > 0 ? interaction.section : '0';
                localFileName += '_';
                localFileName += interaction.part && interaction.part > 0 ? interaction.part : '0';
            }
            let audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
                relativeFileName: interaction.relativeFileName,
                section: interaction.section,
                part: interaction.part
            };
            let environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.ftpDirectory = interaction.ftpDirectory;
            environment.type = interaction.environmentType;
            return audio_service_1.GetAudioFileFromSftp(interaction.vcc, audioData, environment, this.environments[`${interaction.vcc}:${interaction.name}`])
                .catch((err) => {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_SFTP';
            });
        })
            .then(result => {
            localFileName += '.' + result.audioFile.split('.').pop();
            if (['ftp', 'sftp', 's3'].includes(interaction.environmentType))
                localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            else
                localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise((resolve, reject) => {
                fs.open(localFilePath, 'w', (err, fd) => {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        return reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, (err) => {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            return reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, () => {
                            return resolve('OK');
                        });
                    });
                });
            });
        })
            .then((result) => {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetSFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetSFTPAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    GetS3AudioFile(interaction, currentQueueId, executionCode) {
        let localFilePath = '';
        let localFileName = '';
        if (interaction.id?.length)
            localFileName = `${interaction.id}_${interaction.section || 0}_${interaction.part || 0}`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
                relativeFileName: interaction.relativeFileName,
                section: interaction.section,
                part: interaction.part
            };
            let environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.bucket = interaction.bucket;
            environment.ftpDirectory = interaction.ftpDirectory;
            environment.type = interaction.environmentType;
            return audio_service_1.GetAudioFileFromAwsS3(interaction.vcc, audioData, environment, this.environments[`${interaction.vcc}:${interaction.name}`])
                .catch((err) => {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_S3';
            });
        })
            .then(result => {
            localFileName += '.' + result.audioFile.split('.').pop();
            if (interaction.environmentType === 's3')
                localFilePath = `${config_1.PATH_FOLDER_PROCESS_AUDIO}/${interaction.vcc}/${localFileName}`;
            else
                localFilePath = `${config_1.PATH_FOLDER_TEMP_AUDIO}/${interaction.vcc}/${localFileName}`;
            return new Promise((resolve, reject) => {
                fs.open(localFilePath, 'w', (err, fd) => {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetS3AudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        return reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, (err) => {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetS3AudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            return reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, () => {
                            return resolve('OK');
                        });
                    });
                });
            });
        })
            .then((result) => {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetS3AudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetSFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetS3AudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetS3AudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    GetAudioInfo(interaction, currentQueueId, executionCode) {
        let needToDiarizate = false;
        let tmpPath = '';
        let isOgg = false;
        let currentFilePathOgg = '';
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let cmd = '"' + config_1.MEDIAINFO_FOLDER_PATH + 'mediainfo" --Output=JSON "' + interaction.filePath + '"';
            logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd1:', cmd);
            return new Promise((resolve, reject) => {
                try {
                    exec(cmd, (err, stdout, stderr) => {
                        logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting stderr:', stderr);
                        if (err) {
                            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', err);
                            reject(err);
                        }
                        else {
                            let res = typeof stdout === 'string' ? JSON.parse(stdout) : stdout;
                            return resolve(res.media);
                        }
                    });
                }
                catch (e) {
                    reject(e);
                }
            });
        }).then((result) => {
            if (result && result.track) {
                let generalInfo = result.track.find(t => t['@type'] === 'General');
                let audioInfo = result.track.find(t => t['@type'] === 'Audio');
                let minDurationToDiarizate = 60;
                let maxDurationToDiarizate = 1800;
                if (generalInfo && audioInfo) {
                    interaction.format = generalInfo.FileExtension;
                    interaction.channels = Number(audioInfo.Channels) === 1 ? 'mono' : 'stereo';
                    interaction.mediaInfoTime = audioInfo.Duration;
                    if (task_config_1.TRANSCRIPTION_ENGINE === 'SPEECHMATICS'
                        &&
                            ['wav', 'ogg'].includes(generalInfo.FileExtension)
                        &&
                            Number(audioInfo.Channels) === 1
                        &&
                            Number(audioInfo.Duration) >= minDurationToDiarizate
                        &&
                            Number(audioInfo.Duration) <= maxDurationToDiarizate
                        &&
                            interaction.diarization) {
                        needToDiarizate = true;
                        if (generalInfo.FileExtension === 'ogg') {
                            isOgg = true;
                            const newFileName = `${interaction.fileName.split('.')[0]}.wav`;
                            const arrayUrl = interaction.filePath.split('/');
                            arrayUrl.splice(arrayUrl.length - 1, 1);
                            let cmd = config_1.FFMPEG_FOLDER_PATH + 'ffmpeg -y -i "' + interaction.filePath + '" -vn -ar 8000 -ac 1 -acodec pcm_s16le -f wav -fflags +bitexact -flags:a +bitexact -map_metadata -1 "' + arrayUrl.join('/') + '/' + newFileName + '"';
                            logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd convert ogg:', cmd);
                            return new Promise((resolve, reject) => {
                                try {
                                    exec(cmd, function (err, stdout, stderr) {
                                        if (err) {
                                            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage :', err);
                                            return reject(err);
                                        }
                                        else {
                                            currentFilePathOgg = interaction.filePath;
                                            interaction.fileName = newFileName;
                                            interaction.filePath = `${arrayUrl.join('/')}/${newFileName}`;
                                            interaction.format = 'wav';
                                            return resolve(true);
                                        }
                                    });
                                }
                                catch (e) {
                                    logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'convert ogg getting cmd2:', cmd);
                                    reject(e);
                                }
                            });
                        }
                        else {
                            return Promise.resolve(true);
                        }
                    }
                    else {
                        return Promise.resolve(true);
                    }
                }
                else {
                    interaction['retry'] = config_1.RETRY_AUDIO_TRANSFER;
                    throw 'SERVER_ERROR_CANNOT_GET_AUDIO_MEDIAINFO_ANALYSIS';
                }
            }
            else {
                interaction['retry'] = config_1.RETRY_AUDIO_TRANSFER;
                throw 'SERVER_ERROR_CANNOT_GET_AUDIO_MEDIAINFO_RESPONSE';
            }
        }).then(_ => {
            if (needToDiarizate && !isOgg) {
                tmpPath = interaction.filePath + '-TMP';
                return file_manager_1.RenameFile(interaction.filePath, tmpPath);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(result => {
            if (needToDiarizate) {
                return mssql_service_1.MSSql.GetNLU(interaction.vcc, 'diarization-' + interaction.language);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then(result => {
            if (needToDiarizate && !isOgg) {
                if (!result
                    ||
                        (result && !result.lastTrainedDateTime))
                    throw 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND';
                let cmd = config_1.FFMPEG_FOLDER_PATH + 'ffmpeg -y -i "' + tmpPath + '" -vn -ar 8000 -ac 1 -acodec pcm_s16le -f wav -fflags +bitexact -flags:a +bitexact -map_metadata -1 "' + interaction.filePath + '"';
                logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd2:', cmd);
                return new Promise((resolve, reject) => {
                    try {
                        exec(cmd, function (err, stdout, stderr) {
                            if (err) {
                                return reject(err);
                            }
                            else {
                                return resolve(stdout.media);
                            }
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            else {
                return Promise.resolve(null);
            }
        }).then(result => {
            if (needToDiarizate) {
                return file_manager_1.DeleteFile(isOgg ? currentFilePathOgg : tmpPath);
            }
            else {
                return Promise.resolve(null);
            }
        }).then(result => {
            if (needToDiarizate) {
                interaction['statusDescription'] = 'pendingUploadRawAudio';
            }
            else {
                interaction['statusDescription'] = 'pendingUpload';
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetAudioInfo';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            if (errorMessage === 'AUDIO_NEED_SENTIMENT_ANALYSIS_BUT_PHRASES_WERE_NOT_FOUND' || 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND')
                file_manager_1.RenameFile(tmpPath, interaction.filePath);
            return Promise.resolve(interaction);
        });
    }
    StartExternalConnection(vccProcessId) {
        let currentEnvironment = null;
        let items = vccProcessId.split(':');
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetProcessEnvironment(vccProcessId);
        })
            .then((result) => {
            if (!result)
                throw 'SERVER_ERROR_NOT_FOUND_PROCESS_ENVIRONMENT_DATA';
            currentEnvironment = result;
            if (this.environments[`${currentEnvironment.vcc}:${currentEnvironment.name}`])
                throw 'SERVER_ERROR_PROCESS_ENVIRONMENT_ALREADY_ADDED';
            if (!currentEnvironment
                ||
                    !currentEnvironment.apiHost
                ||
                    !currentEnvironment.apiPort
                ||
                    !currentEnvironment.apiUser
                ||
                    !currentEnvironment.apiPassword)
                return mssql_service_1.MSSql.GetEnvironment(currentEnvironment.vcc, currentEnvironment.name);
            return Promise.resolve(currentEnvironment);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_ENVIRONMENT_DATA_NOT_FOUND';
            currentEnvironment = result;
            logger.info(`[AudioTransferImplementation::StartExternalConnection] Starting environment ${currentEnvironment.name}`);
            let promise = null;
            switch (currentEnvironment.type) {
                case 'ftp':
                case 'i-ftp':
                    promise = this.ConnectToFtp(currentEnvironment);
                    break;
                case 'i-sftp':
                case 'sftp':
                    promise = this.ConnectToSftp(currentEnvironment);
                    break;
                case 'i-s3':
                case 's3':
                    promise = this.ConnectToS3(currentEnvironment);
                    break;
            }
            return promise;
        })
            .then(result => {
            return redis_service_1.Redis.SaveProcessEnvironment(vccProcessId, currentEnvironment);
        })
            .catch(err => {
            logger.error('[AudioTransferImplementation::StartExternalConnection] Getting error:', err);
            return Promise.resolve(false);
        });
    }
    ConnectToFtp(environment) {
        let environmentConfig = {
            host: environment.apiHost,
            port: environment.apiPort,
            user: environment.apiUser,
            password: environment.apiPassword,
            connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
            pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT,
            aliveTimeout: config_1.FTP_CONNECTION_TIMEOUT
        };
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return new Promise((resolve, reject) => {
                let clientFtp = new Client();
                clientFtp.connect(environmentConfig);
                if (clientFtp) {
                    clientFtp.on('greeting', (message) => {
                        logger.info('[AudioTransferImplementation::ConnectToFtp] Greeting event message:', message);
                    });
                    clientFtp.on('ready', () => {
                        return resolve(clientFtp);
                    });
                    clientFtp.on('end', () => {
                        let eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[AudioTransferImplementation::ConnectToFtp] getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('close', (hadError) => {
                        let eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                        logger.warn('[AudioTransferImplementation::ConnectToFtp] hadError:', hadError);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('error', (err) => {
                        logger.error('[AudioTransferImplementation::ConnectToFtp] NOT_FTP_CLIENT', err);
                        return reject('NOT_FTP_CLIENT');
                    });
                }
                else {
                    logger.error('[AudioTransferImplementation::ConnectToFtp] Can not create FTP Client');
                    return reject('SERVER_ERROR_CANNOT_CREATE_FTP_CLIENT');
                }
            });
        })
            .then(result => {
            this.environments[`${environment.vcc}:${environment.name}`] = result;
            return Promise.resolve(true);
        })
            .catch(error => {
            logger.error('[AudioTransferImplementation::ConnectToFtp] Getting error:', error);
            return Promise.resolve(false);
        });
    }
    ConnectToSftp(environment) {
        let sshId = `${environment.vcc}:${environment.name}`;
        let environmentConfig = {
            host: environment.apiHost,
            port: environment.apiPort,
            username: environment.apiUser,
            password: environment.apiPassword,
            connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
            pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
        };
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let ssh2 = new SSH2Promise(environmentConfig);
            this.environments[sshId] = ssh2.sftp();
            this.sshConnections.push(ssh2);
            return Promise.resolve(true);
        })
            .catch(error => {
            logger.error('[AudioTransferImplementation::ConnectToSftp] Getting error:', error);
            return Promise.resolve(false);
        });
    }
    ConnectToS3(environment) {
        let awsS3 = new aws_s3_implementation_1.AwsS3Implementation(environment.apiHost, environment.apiUser, environment.apiPassword, environment.bucket);
        return promises_shared_1.DummyPromise()
            .then(_ => {
            this.environments[`${environment.vcc}:${environment.name}`] = awsS3;
            return Promise.resolve(true);
        })
            .catch(error => {
            logger.error('[AudioTransferImplementation::ConnectToS3] Getting error:', error);
            return Promise.resolve(false);
        });
    }
    EndExternalConnection() {
        let keys = Object.keys(this.environments);
        if (keys.length)
            logger.info(`[AudioTransferImplementation::EndExternalConnection] Found ${keys.length} external connections`);
        keys.forEach(k => {
            if (k) {
                logger.info(`[AudioTransferImplementation::EndExternalConnection] Closing ${k} as ftp`);
                try {
                    this.environments[k].end();
                }
                catch (error1) {
                    logger.warn(`[AudioTransferImplementation::EndExternalConnection] Failed after try closing as ftp, trying close ${k} as sftp`);
                    try {
                        this.environments[k].close();
                    }
                    catch (error2) {
                        logger.error(`[AudioTransferImplementation::EndExternalConnection] Failed after try closing as ftp, trying close ${k} as s3`);
                        this.environments[k].CloseConnection();
                    }
                    ;
                }
            }
        });
        if (this.sshConnections.length) {
            logger.info(`[AudioTransferImplementation::EndExternalConnection] Closing ${this.sshConnections.length} ssh connections`);
            this.sshConnections.forEach(sc => sc.close());
        }
    }
}
exports.AudioTransferImplementation = AudioTransferImplementation;
