"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AwsS3Implementation = void 0;
const client_s3_1 = require("@aws-sdk/client-s3");
const log4js = require("log4js");
let logger = log4js.getLogger('Task');
class AwsS3Implementation {
    constructor(region, accessKeyId, secretAccessKey, defaultBucket) {
        this.s3Client = null;
        this.defaultBucket = null;
        this.s3Client = new client_s3_1.S3Client({
            region: region,
            credentials: {
                accessKeyId: accessKeyId,
                secretAccessKey: secretAccessKey
            }
        });
        this.defaultBucket = defaultBucket;
    }
    ListObjects(bucket, config) {
        let input = {
            Bucket: bucket
        };
        if (config) {
            if (config.prefix)
                input.Prefix = config.prefix;
            if (config.maxKeys)
                input.MaxKeys = config.maxKeys;
        }
        const promiseAws = async () => {
            return (await this.s3Client.send(new client_s3_1.ListObjectsCommand(input)));
        };
        return promiseAws()
            .then(result => {
            logger.error('[AwsS3Implementation::TestPath] response:', result);
            return Promise.resolve(result.Contents);
        })
            .catch(err => {
            logger.error('[AwsS3Implementation::ListObjects] error:', err);
            return Promise.resolve(null);
        });
    }
    DownloadFile(path) {
        const promiseAws = async () => {
            return (await this.s3Client.send(new client_s3_1.GetObjectCommand({
                Bucket: this.defaultBucket,
                Key: this.CleanPath(path)
            }))).Body;
        };
        return promiseAws()
            .then(result => {
            logger.info('[AwsS3Implementation::DownloadFile] Succed');
            return Promise.resolve(result);
        })
            .catch(err => {
            logger.error('[AwsS3Implementation::DownloadFile] error:', err);
            return Promise.resolve(null);
        });
    }
    TestBucket(bucket) {
        const input = { Bucket: bucket };
        const command = new client_s3_1.HeadBucketCommand(input);
        const promiseAws = async () => {
            return (await this.s3Client.send(command));
        };
        return promiseAws()
            .then(result => {
            return Promise.resolve(result ? true : false);
        })
            .catch(err => {
            logger.error('[AwsS3Implementation::TestBucket] error:', err);
            if (err.name === '403')
                return Promise.reject('SERVER_ERROR_S3_NOT_CLIENT');
            return Promise.resolve(false);
        });
    }
    TestPath(bucket, path) {
        const input = { Bucket: bucket, Key: this.CleanPath(path) };
        const command = new client_s3_1.GetObjectCommand(input);
        const promiseAws = async () => {
            return (await this.s3Client.send(command)).Body;
        };
        return promiseAws()
            .catch(err => {
            logger.warn('[AwsS3Implementation::TestPath] fail search with the path, tring by prefix');
            let config = {
                prefix: this.CleanPath(path),
                maxKeys: 1
            };
            return this.ListObjects(bucket, config);
        })
            .then(result => {
            return Promise.resolve(result ? true : false);
        })
            .catch(err => {
            logger.error('[AwsS3Implementation::TestPath] error:', err);
            return Promise.resolve(false);
        });
    }
    CleanPath(path) {
        if (path.startsWith('/'))
            path = path.slice(1);
        if (!path.match(/\.(mp3|wav|ogg)$/g)
            &&
                !path.endsWith('/'))
            path += '/';
        return path;
    }
    CloseConnection() {
        this.s3Client.destroy();
    }
}
exports.AwsS3Implementation = AwsS3Implementation;
