"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BillingBatchImplementation = void 0;
const billing_implementation_1 = require("./billing.implementation");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const platform_service_1 = require("../services/platform.service");
let icModule = require('intermediate-component-sa');
let logger = log4js.getLogger('ServerScripts');
class BillingBatchImplementation {
    CancelBatchSummary(batchManage) {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::CancelBatchSummary] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {
                    batchManage: batchManage
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_SUMMARY_CANCEL', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::CancelBatchSummary] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    }
    GetCurrentBatches() {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::GetCurrentBatches] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {};
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_CURRENT_BATCHES_GET', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::GetCurrentBatches] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    }
    GetCancelledBatches() {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::GetCurrentBatches] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {};
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_CANCELLED_BATCHES_GET', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::GetCancelledBatches] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    }
    GetBatchBalanceDetail(batchDetail) {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::GetBatchBalanceDetail] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {
                    batchDetail: batchDetail
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_BALANCE_DETAIL', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::GetBatchBalanceDetail] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    }
    FindBatchId(billingBatch) {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::FindBatchId] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {
                    batch: billingBatch
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_ID_SEARCH', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.data && result.data.length > 0) {
                    genericResult.status = true;
                    genericResult.result = result.data;
                }
                else {
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::FindBatchId] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    }
    ResumeBatchSummary(batchManage) {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::ResumeBatchSummary] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {
                    batchManage: batchManage
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_SUMMARY_RESUME', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::ResumeBatchSummary] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    }
    SaveBatchSummary(billingBatchSummary) {
        let billing = new billing_implementation_1.BillingImplementation();
        let installationId = '';
        let systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::SaveBatchSummary] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return billing.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = billing.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                batch: billingBatchSummary
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_BATCH_SUMMARY_CREATE', body);
        })
            .then((result) => {
            return Promise.resolve(result);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::SaveBatchSummary] Getting error:', err);
            return Promise.resolve(err);
        });
    }
    CloseBatchSummary(billingBatchFinished) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingBatchImplementation::CloseBatchSummary] Getting result after GetVersion');
            if (result && result.installationId) {
                let body = {
                    batch: billingBatchFinished
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_SUMMARY_CLOSE', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result && result.data && result.data.result) {
                logger.debug('[BillingBatchImplementation::CloseBatchSummary] Finishing object:', JSON.stringify(billingBatchFinished));
                return mssql_service_1.MSSql.FinishedBatchClose(billingBatchFinished.vcc, billingBatchFinished.batchTypeName, billingBatchFinished.batchTypeId);
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            logger.error('[BillingBatchImplementation::CloseBatchSummary] Getting error:', err);
            return Promise.resolve(false);
        });
    }
}
exports.BillingBatchImplementation = BillingBatchImplementation;
