"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCognitiveService = exports.CognitiveAnalysisImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const moment = require("moment");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const file_manager_1 = require("../shared/file-manager");
const billing_implementation_1 = require("./billing.implementation");
const luis_service_implementation_1 = require("./luis-service.implementation");
const rasa_service_implementation_1 = require("./rasa-service.implementation");
const billing_batch_model_1 = require("../models/billing-batch.model");
const rasa_api_service_implementation_1 = require("./rasa-api-service.implementation");
const semantic_analysis_download_implementation_1 = require("./semantic-analysis-download.implementation");
let logger = log4js.getLogger('Task');
let cognitiveService;
const CURRENT_SYSTEM_SERVICE = 'COGNITIVE_ANALYSIS';
const CURRENT_IMPLEMENTATION = 'CognitiveAnalysisImplementation';
let useRasaApi = config_1.USE_RASA_API;
class CognitiveAnalysisImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_COGNITIVE_ANALYSIS;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        return promises_shared_1.SleepPromise(2)
            .then(result => {
            return redis_service_1.Redis.LRange('CognitiveAnalysis', 0, ratio - 1);
        })
            .then((result) => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[CognitiveAnalysisImplementation::Execute] CognitiveAnalysis lrange result: ' + result);
            result.map(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[7].toLowerCase())) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.CognitiveAnalysisExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('CognitiveAnalysis', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CognitiveAnalysisExecute(executionCode, availableBatch, cancelledBatch) {
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let type = '';
        let typeId = '';
        let currentQueueId = '';
        let interactionCognitive;
        let queueProgressStorage = '';
        let filePath = '';
        return redis_service_1.Redis.RemoveQueueL('CognitiveAnalysis')
            .then((result) => {
            let arrQueue = result.split(':');
            let promiseArray = [];
            if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[8].toLowerCase())) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysisExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[8], 'COGNITIVE_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            let queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[CognitiveQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = currentQueueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    type = item[7] ? item[7] : null;
                    typeId = item[8] ? item[8] : null;
                    if (type && typeId)
                        queueProgressStorage = vcc + ':' + processId + ':' + type + ':' + typeId;
                    if (type && !typeId)
                        queueProgressStorage = vcc + ':' + processId + ':' + type;
                    logger.info('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return redis_service_1.Redis.GetModelInteraction(currentQueueId);
                }
                else {
                    logger.error('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then((result) => {
            if (!result)
                logger.warn('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: NOT_FOUND_INTERACTION_COGNITIVE');
            if (result) {
                if (cancelledBatch.find(b => b.VCC === result.vcc && b.BatchTypeId.toLowerCase() === (result.studyId ? result.studyId : result.evaluationPlanId).toLowerCase())) {
                    result.status = -2;
                    result.statusDescription = 'REMOVED_BY_CANCELLED';
                    result.retry = config_1.RETRY_COGNITIVE_ANALYSIS;
                    result.errorFunction = 'CognitiveAnalysisExecute';
                    result.errorMessage = 'REMOVED_BY_CANCELLED';
                    result.errorDetail = 'REMOVED_BY_CANCELLED';
                }
            }
            interactionCognitive = result;
            return file_manager_1.CreateFolder(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + vcc);
        }).then((result) => {
            if (!result)
                throw 'ERROR_CREATE_COGNITIVE_ANALYSIS_FOLDER';
            if (interactionCognitive && interactionCognitive.statusDescription === 'pendingPhrases') {
                logger.info('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingPhrases');
                return this.GetPhrasesFromIdathaJson(interactionCognitive, currentQueueId, executionCode);
            }
            else if (interactionCognitive && interactionCognitive.statusDescription === 'pendingNLU') {
                logger.info('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingNLU');
                return this.GetCognitive(interactionCognitive, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        }).then((resultInteraction) => {
            if (interactionCognitive)
                return redis_service_1.Redis.SaveModelInteraction(currentQueueId, interactionCognitive);
            return Promise.resolve(null);
        }).then((result) => {
            let promiseArray = [];
            if (result && interactionCognitive?.status === 6 && interactionCognitive?.statusDescription === 'ready') {
                logger.info('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'READY');
                this.CloseCognitiveModel(currentQueueId, true, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
                if (type === 'tipification') {
                    interactionCognitive.statusDescription = 'pendingAnalysis';
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentQueueId));
                    promiseArray.push(redis_service_1.Redis.SaveTipificationInteraction(currentQueueId, interactionCognitive));
                }
            }
            else if (result && interactionCognitive?.status === 6 && interactionCognitive?.statusDescription === 'pendingNLU' && interactionCognitive.retry < config_1.RETRY_COGNITIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive && interactionCognitive.status === -2 && interactionCognitive.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                this.CloseCognitiveModel(currentQueueId, false, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive && interactionCognitive.status === -6 && interactionCognitive.statusDescription === 'ERROR_FILE_JSON') {
                this.CloseCognitiveModel(currentQueueId, false, 'JSON_PROBLEM');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (interactionCognitive && interactionCognitive.retry >= config_1.RETRY_COGNITIVE_ANALYSIS) {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                this.CloseCognitiveModel(currentQueueId, false, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionCognitive) {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                this.CloseCognitiveModel(currentQueueId, false, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('CognitiveAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            let promiseArray = [];
            if (interactionCognitive
                && interactionCognitive.status === 6
                && interactionCognitive.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteModelInteraction(currentQueueId));
                if (['study', 'template'].includes(type)) {
                    logger.debug('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for study:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyModelInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else if (['ability', 'tipification'].includes(type)) {
                    logger.debug('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyModelAbility(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
            }
            else if (interactionCognitive && interactionCognitive.retry >= config_1.RETRY_COGNITIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteModelInteraction(currentQueueId));
                if (['study', 'template'].includes(type)) {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for study:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyModelInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else if (['ability', 'tipification'].includes(type)) {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyModelAbility(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[CognitiveAnalysisImplementation::CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, interactionCognitive.id, interactionCognitive.modelId, this.errorData.errorFunction, this.errorData.errorMessage, this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetPhrasesFromIdathaJson(interaction, currentQueueId, executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetPhrasesFromIdathaJson.name}] [interactionId => ${interaction.id}]: `;
        let existsDocument = false;
        let totalPhrases = 0;
        let semanticAnalyisDownloadImpl = new semantic_analysis_download_implementation_1.SemanticAnalysisDownloadImplementation();
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return GetCognitiveService(interaction.vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            return semanticAnalyisDownloadImpl.GetJsonForModelAnalysis(interaction, 'COGNITIVE');
        })
            .then((result) => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result?.existsDocument) {
                existsDocument = result.existsDocument;
                totalPhrases = result.phrases.length;
                delete (result.existsDocument);
                return Promise.resolve(null);
            }
            return semanticAnalyisDownloadImpl.SaveJsonForModelAnalysis(interaction, result, 'COGNITIVE', cognitiveService);
        })
            .then(result => {
            if (result?.status || existsDocument) {
                interaction['status'] = 6;
                interaction['totalPhrases'] = interaction['totalPhrases'] ? interaction['totalPhrases'] : totalPhrases;
                interaction['statusDescription'] = 'pendingNLU';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            else {
                logger.warn(`${logLabel}executionCode:`, executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_PHASES');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'GetPhrasesFromIdathaJson';
                interaction['errorMessage'] = 'NOT_FOUND_PHASES';
                this.errorData.SetErrorData('GetPhrasesFromIdathaJson', interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}executionCode:`, executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('GetPhrasesFromIdathaJson', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetCognitive(interaction, currentQueueId, executionCode) {
        let item = currentQueueId.split(':');
        let type = item[7] ? item[7] : null;
        let path = '';
        let isLast = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return GetCognitiveService(interaction.vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            let regex = /\.(mp3|wav|ogg|json)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.COGNITIVE_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return this.GetNluResults(interaction, path, currentQueueId, executionCode);
        })
            .then(result => {
            if (!result || !result.status)
                throw 'ERROR_SAVE_DOCUMENT';
            isLast = result.isLast;
            if (isLast && type !== 'tipification')
                return this.EvaluateCognitive(path, interaction, executionCode, currentQueueId);
            return Promise.resolve(null);
        })
            .then((result) => {
            if (isLast) {
                interaction.succedItems = result ? result : [];
                if (!result)
                    logger.warn('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_SCORE');
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (result) {
                interaction['status'] = 6;
                interaction['statusDescription'] = isLast ? 'ready' : 'pendingNLU';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'NOT_FOUND_PHRASE') {
                logger.warn('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = 6;
                interaction['statusDescription'] = 'ready';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
            }
            else if (errorMessage === 'OUT_OF_VOLUME_QUOTA') {
                logger.error('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = config_1.RETRY_COGNITIVE_ANALYSIS;
            }
            else {
                logger.error('[CognitiveAnalysisImplementation::GetCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            }
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('GetCognitive', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetNluResults(interaction, path, currentQueueId, executionCode) {
        let index = 0;
        let isLast = false;
        let alreadyEvaluated = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            let jsonContent;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                this.errorData.SetErrorData('GetNluResultsFromCognitiveEngine', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(jsonContent);
        })
            .then(result => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result.phrases)
                alreadyEvaluated = !result.phrases.map(p => p[interaction.modelId] ? true : false).includes(false) ? true : false;
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            isLast = interaction.totalPhrases - 1 <= index ? true : false;
            return this.GetNluResultsFromCognitiveEngine(interaction, path, result, currentQueueId, executionCode);
        })
            .then(result => {
            return Promise.resolve({ status: result ? true : false, isLast });
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.info(`[CognitiveAnalysisImplementation::GetNluResults] executionCode: ${executionCode} getting queueId: ${currentQueueId} getting error: ${errorMessage}`);
            return Promise.reject(err);
        });
    }
    GetNluResultsFromCognitiveEngine(interaction, path, content, currentQueueId, executionCode) {
        let phrase = '';
        let index = 0;
        let currentNlu = {};
        let currentSubscriptionKey = {};
        let apiIdLabel = cognitiveService.GetCognitiveEngineName() === 'LUIS' ? 'LuisApi' : 'RasaApi';
        let billing = new billing_implementation_1.BillingImplementation();
        return promises_shared_1.DummyPromise()
            .then(_ => {
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            phrase = content.phrases[index]?.content ? content.phrases[index].content : null;
            if (!phrase)
                throw 'NOT_FOUND_PHRASE';
            return Promise.resolve(true);
        })
            .then(result => {
            return this.GetNlu(interaction.vcc, interaction.modelId, currentQueueId);
        })
            .then(result => {
            if (!result)
                throw 'NLU_MODEL_NOT_FOUND';
            currentNlu = result;
            if (apiIdLabel === 'LuisApi') {
                return this.GetAzureSubscriptionKey(interaction.vcc, currentNlu.subscriptionKey);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (apiIdLabel === 'LuisApi') {
                currentSubscriptionKey = result;
                if (!currentSubscriptionKey)
                    throw 'SUSCRIPTION_KEY_NOT_FOUND';
                billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', apiIdLabel, currentSubscriptionKey.subscriptionKey);
                billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', 'IsDemo', currentSubscriptionKey.isDemo);
            }
            else {
                currentSubscriptionKey.subscriptionRegion = null;
                currentSubscriptionKey.subscriptionKey = null;
                billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', apiIdLabel, config_1.RASA_IP + ':' + config_1.RASA_PORT);
                billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', 'IsDemo', false);
            }
            return this.ModelQuery(cognitiveService, currentSubscriptionKey.subscriptionRegion, currentSubscriptionKey.subscriptionKey, interaction.appId, phrase, interaction.modelId, executionCode, currentQueueId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_COGNITIVE_REQUEST_RESULT_NOT_FOUND';
            if (result && result.error)
                throw result.error;
            return this.SaveDocumentCognitive(index, path, result, interaction.modelId, executionCode, currentQueueId);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::GetNluResultsFromCognitiveEngine] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    GetNlu(vcc, name, queue) {
        logger.info('[CognitiveAnalysisImplementation::GetNLU] getting vcc:', vcc, 'name:', name);
        let cacheUpdateNeeded = false;
        let arrQueue = queue.split(':');
        let nluRedisKey = vcc + ":" + arrQueue[7] + ":" + arrQueue[8] + ":" + name;
        return redis_service_1.Redis.GetNlu(nluRedisKey)
            .then(result => {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetNLU(vcc, name);
            }
        })
            .then(result => {
            if (result) {
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveNLU(nluRedisKey, result);
                this.errorData.CleanErrorData();
                return Promise.resolve(result);
            }
            else {
                throw 'ERROR_NLU_MODEL_NOT_FOUND';
            }
        })
            .catch((err) => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CognitiveAnalysisImplementation::GetNLU] getting errorMsg:', errorMsg);
            this.errorData.SetErrorData('GetNLU', errorMsg, err);
            return Promise.resolve(null);
        });
    }
    GetAzureSubscriptionKey(vcc, azureSubscriptionKey) {
        logger.info('[CognitiveAnalysisImplementation::GetAzureSubscriptionKey] getting vcc: ', vcc, ' azureSubscriptionKey: ', azureSubscriptionKey);
        let cacheUpdateNeeded = false;
        let azureSubscriptionKeyRedisKey = vcc + ":" + azureSubscriptionKey;
        return redis_service_1.Redis.GetSubscriptionKey(azureSubscriptionKeyRedisKey)
            .then(result => {
            if (result) {
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, azureSubscriptionKey);
            }
        })
            .then(result => {
            if (result) {
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveSubscriptionKey(azureSubscriptionKeyRedisKey, result);
                this.errorData.CleanErrorData();
                return Promise.resolve(result);
            }
            else {
                throw 'ERROR_AZURE_SUBSCRIPTIONKEY_NOT_FOUND';
            }
        })
            .catch((err) => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CognitiveAnalysisImplementation::GetAzureSubscriptionKey] getting errorMsg:', errorMsg);
            this.errorData.SetErrorData('GetAzureSubscriptionKey', errorMsg, err);
            return Promise.resolve(null);
        });
    }
    SaveDocumentCognitive(index, path, data, modelId, executionCode, currentQueueId) {
        return promises_shared_1.DummyPromise()
            .then(() => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        }).then((resultFile) => {
            let jsonContent;
            let result = data ? data : [];
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path: ', path);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting index: ', index);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting model: ', modelId);
            logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting result: ', result);
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting jsonContent: ', jsonContent);
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases: ', jsonContent.phrases.length);
                    jsonContent['phrases'][index][modelId] = result;
                }
                else {
                    jsonContent = { phrases: [{}] };
                    logger.debug('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'creating item on jsonContent');
                    jsonContent['phrases'][index][modelId] = result;
                }
            }
            catch (ex) {
                logger.error('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                this.errorData.SetErrorData('SaveDocumentCognitive', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return file_manager_1.WriteFileContent(path, JSON.stringify(jsonContent), 'utf-8');
        }).then((result) => {
            this.errorData.CleanErrorData();
            return Promise.resolve(true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[CognitiveAnalysisImplementation::SaveDocumentCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('SaveDocumentCognitive', errorMessage, error);
            return Promise.resolve(false);
        });
    }
    ModelQuery(cognitiveService, subscriptionRegion, subscriptionKey, appId, query, nluName, executionCode, currentQueueId) {
        let billing = new billing_implementation_1.BillingImplementation();
        return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, query, null, null, null)
            .then((response) => {
            logger.info('[CognitiveAnalysisImplementation::ModelQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Request finished OK');
            logger.info('[CognitiveAnalysisImplementation::ModelQuery] Response: ' + JSON.stringify(response));
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', 'TotalValidRequest', 1);
            this.errorData.CleanErrorData();
            return Promise.resolve({
                intents: response.intents,
                entities: response.entities ? response.entities : null
            });
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::ModelQuery] Error 1: ' + JSON.stringify(err));
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', 'TotalInvalidRequest', 1);
            let responseError = {};
            if (typeof errorMessage === 'string') {
                responseError['error'] = err;
                if (errorMessage.indexOf('Out of call volume quota') > -1)
                    responseError['error'] = 'OUT_OF_VOLUME_QUOTA';
            }
            else
                responseError['error'] = 'SERVER_ERROR_MODEL_QUERY_UNKNOWN_ERROR';
            this.errorData.SetErrorData('ModelQuery', responseError && responseError['error'] ? responseError['error'] : errorMessage, err);
            return Promise.resolve(responseError);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CognitiveAnalysisImplementation::ModelQuery] Error 2: ' + JSON.stringify(err));
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'COGNITIVE', 'TotalInvalidRequest', 1);
            this.errorData.SetErrorData('ModelQuery', errorMessage, err);
            return Promise.resolve(null);
        });
    }
    EvaluateCognitive(path, interaction, executionCode, currentQueueId) {
        let studyTemplateThemes = [];
        let evaluationPlanAbilities = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                return redis_service_1.Redis.GetStudyTemplateThemes(interaction.vcc, interaction.studyId);
            else
                return redis_service_1.Redis.GetEvaluationPlanAbilities(interaction.vcc, interaction.evaluationPlanId);
        })
            .then(result => {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                studyTemplateThemes = result;
            else
                evaluationPlanAbilities = result;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then((resultFile) => {
            let jsonContent = null;
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                }
            }
            catch (ex) {
                logger.error('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                this.errorData.SetErrorData('EvaluateCognitive', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return jsonContent;
        })
            .then((jsonContent) => {
            if (!jsonContent)
                throw 'ERROR_FILE_JSON';
            let phrases = jsonContent['phrases'];
            let succedItems = [];
            if (phrases && phrases.length) {
                phrases.forEach((phrase) => {
                    if (phrase[interaction.modelId] && phrase[interaction.modelId].intents && phrase[interaction.modelId].intents.length) {
                        if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName)) {
                            if (!studyTemplateThemes.length)
                                logger.warn('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'studyTemplateThemes is empty, need items to get succedItems');
                            let templateThemes = interaction.templateThemes.split(',');
                            let succedTemplateThemeItems = [];
                            templateThemes.map(tt => {
                                let currentTemplateTheme = studyTemplateThemes.find(stt => stt.id === tt);
                                if (currentTemplateTheme) {
                                    let succedIntents = [];
                                    currentTemplateTheme.items.map(tti => {
                                        let intent = phrase[interaction.modelId].intents.find(i => i.intent === tti.id.split('|')[1]);
                                        if (!intent)
                                            intent = phrase[interaction.modelId].intents.find(i => i.intent === tti.query.split('|')[1]);
                                        if (intent) {
                                            if (intent.score >= tti.acceptancePercentage) {
                                                if (tti.entities && tti.entities.length > 0) {
                                                    let succeedEntities = [];
                                                    tti.entities.map(e => {
                                                        let entities = phrase[interaction.modelId].entities.filter(en => en.type === e.entityName && en.score >= e.acceptancePercentage);
                                                        if (entities && entities.length) {
                                                            succeedEntities.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities.length > 0) {
                                                        intent.entities = succeedEntities;
                                                        succedIntents.push(intent);
                                                        succedTemplateThemeItems.push(tti.id);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents.push(intent);
                                                    succedTemplateThemeItems.push(tti.id);
                                                }
                                            }
                                        }
                                    });
                                    logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedIntents result:', succedIntents);
                                    if (succedIntents.length > 0) {
                                        if (succedItems.find(stt => stt.id === tt)) {
                                            let succeedTemplateTheme = succedItems.find(stt => stt.id === tt);
                                            succeedTemplateTheme.intents.map(i => {
                                                let succeedIntent = succedIntents.find(si => si.intent === i.id.split('|')[1]);
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                templateThemeId: tt,
                                                templateThemeItems: succedTemplateThemeItems,
                                                intents: succedIntents
                                            });
                                        }
                                    }
                                }
                            });
                        }
                        else {
                            if (!evaluationPlanAbilities.length)
                                logger.warn('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'evaluationPlanAbilities is empty, need items to get succedItems');
                            let abilities = interaction.abilities.split(',');
                            abilities.map(a => {
                                let currentAbility = evaluationPlanAbilities.find(epa => epa.idAbility === a);
                                if (currentAbility) {
                                    let succedIntents = [];
                                    currentAbility.queries.map(q => {
                                        let intent = phrase[interaction.modelId].intents.find(i => i.intent === q.queryContent.split('|')[1]);
                                        if (!intent)
                                            intent = phrase[interaction.modelId].intents.find(i => i.intent === q.queryId);
                                        if (intent) {
                                            if (intent.score >= q.acceptancePercentage) {
                                                if (q.entities && q.entities.length > 0) {
                                                    let succeedEntities = [];
                                                    q.entities.map(e => {
                                                        let entities = phrase[interaction.modelId].entities.filter(en => en.type === e.entityName && en.score >= e.acceptancePercentage);
                                                        if (entities && entities.length) {
                                                            succeedEntities.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities.length > 0) {
                                                        intent.entities = succeedEntities;
                                                        succedIntents.push(intent);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents.push(intent);
                                                }
                                            }
                                        }
                                    });
                                    logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedIntents result:', succedIntents);
                                    if (succedIntents.length > 0) {
                                        if (succedItems.find(sa => sa.abilityId === a)) {
                                            let succeedAbility = succedItems.find(sa => sa.abilityId === a);
                                            succeedAbility.intents.map(i => {
                                                let succeedIntent = succedIntents.find(si => si.intent === i.intent);
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                abilityId: a,
                                                intents: succedIntents
                                            });
                                        }
                                    }
                                    else {
                                    }
                                }
                            });
                        }
                    }
                });
            }
            logger.info('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'succedItems result:', succedItems);
            this.errorData.CleanErrorData();
            return Promise.resolve(succedItems);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[CognitiveAnalysisImplementation::EvaluateCognitive] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('EvaluateCognitive', errorMessage, error);
            return Promise.resolve(null);
        });
    }
    CloseCognitiveModel(queueId, resultCode, resultDescription) {
        let billing = new billing_implementation_1.BillingImplementation();
        let myBatchName = '';
        let myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[7]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'tipification':
                myBatchName = 'EVALUATION_PLAN';
                break;
        }
        billing.AddCloseModelAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[6], myBatchName, myQueueIdArray[8], resultCode ? 1 : 0, resultDescription);
    }
}
exports.CognitiveAnalysisImplementation = CognitiveAnalysisImplementation;
function GetCognitiveService(vcc, cognitiveEngine) {
    let cognitiveService;
    vcc = vcc ? vcc : null;
    cognitiveEngine = cognitiveEngine ? cognitiveEngine : null;
    return promises_shared_1.DummyPromise()
        .then(result => {
        if (vcc || cognitiveEngine) {
            if (vcc)
                return redis_service_1.Redis.GetVCC(vcc, vcc);
            if (cognitiveEngine)
                return Promise.resolve(true);
            throw 'SERVER_ERROR_MISSING_PARAMETERS';
        }
        else
            throw 'SERVER_ERROR_MISSING_PARAMETERS';
    }).then(result => {
        if (vcc) {
            cognitiveEngine = result && result.cognitiveEngine ? result.cognitiveEngine : null;
        }
        switch (cognitiveEngine) {
            case 'LUIS':
                cognitiveService = new luis_service_implementation_1.LUISServiceImplementation();
                break;
            case 'RASA':
                if (useRasaApi)
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                else
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                break;
            case 'OLD_RASA_ENGINE':
                if (useRasaApi)
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                else
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                break;
            case 'NEW_RASA_ENGINE':
                if (useRasaApi)
                    cognitiveService = new rasa_api_service_implementation_1.RASAAPIServiceImplementation();
                else
                    cognitiveService = new rasa_service_implementation_1.RASAServiceImplementation();
                break;
            default:
                cognitiveService = null;
                logger.warn('[cognitive-analysis.impl::GetCognitiveService] Undefined engine: ' + cognitiveEngine);
                break;
        }
        return Promise.resolve(cognitiveService);
    }).catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[cognitive-analysis.impl::GetCognitiveService] Error: ' + error);
        return Promise.reject(errorMessage);
    });
}
exports.GetCognitiveService = GetCognitiveService;
