"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityMigrator = void 0;
const redis_service_1 = require("../../db/redis.service");
const log4js = require("log4js");
const mssql_service_1 = require("../../db/mssql.service");
const promises_shared_1 = require("../../shared/promises.shared");
const helper_1 = require("./helper");
const enum_1 = require("./enum");
const arrays_shared_1 = require("../../shared/arrays.shared");
const logger = log4js.getLogger('Task');
class EntityMigrator {
    MigrateEntities(vcc, data, start, end) {
        logger.info('[CognitiveEngineMigrateImplementation::MigrateEntities] Migrating entities for ', vcc, ' into ', data.newEngine, ' from ', start, ' and ', end);
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.LRange(`${enum_1.ROOT_KEY}:${vcc}:Entities:Keys`, start, end - 1);
        }).then((keys) => {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateEntities] Found keys for entities ', keys);
            if (keys && keys.length > 0) {
                return redis_service_1.Redis.GetCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Entities`, arrays_shared_1.GetUniqueFromArray(keys));
            }
            return [];
        }).then((entities) => {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateEntities] Found  entities ', entities);
            if (entities) {
                let entityList = [];
                for (let e of entities) {
                    let entity = JSON.parse(e);
                    if (entity) {
                        entityList.push(entity);
                    }
                }
                if (entityList.length > 0) {
                    return this.ProcessEntities(vcc, data, entityList);
                }
            }
            return false;
        }).then(_ => {
            if (end < data.totalEntity) {
                return this.MigrateEntities(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE);
            }
            return true;
        });
    }
    ProcessEntities(vcc, data, entities) {
        logger.info('[CognitiveEngineMigrateImplementation::ProcessEntities] Starting process entities:', entities.length);
        return promises_shared_1.DummyPromise().then(_ => {
            let promises = [];
            for (let entity of entities) {
                switch (entity.status) {
                    case 0:
                        promises.push(this.AddEntity(vcc, data, entity));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::ProcessEntities] Results for process entities:', results);
            if (results.length > 0) {
                let totalCreated = results.filter(r => r == true).length;
                let totalError = results.filter(r => !r).length;
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                    totalCreatedEntity: totalCreated + data.totalCreatedEntity,
                    totalErrorEntity: totalError + data.totalErrorEntity
                }));
            }
            return false;
        });
    }
    AddEntity(vcc, data, entity) {
        logger.info('[CognitiveEngineMigrateImplementation::AddEntity] Adding entity ', entity.name, ' into ', data.newEngine);
        let AppId = null;
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId(`${enum_1.ROOT_KEY}:${vcc}:AppIds`, entity.appId);
        }).
            then((appId) => {
            if (appId) {
                AppId = appId;
                return helper_1.GetServiceInstance(data.newEngine);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddEntity] Error on get new app id from redis for ', entity.key);
            return null;
        })
            .then((service) => {
            if (service && AppId) {
                return service.AddSimpleEntity(data.subscriptionRegion, data.subscriptionKey, AppId, entity);
            }
            return null;
        })
            .then((newEntityId) => {
            if (newEntityId) {
                let promises = [];
                promises.push(Promise.resolve(newEntityId));
                promises.push(mssql_service_1.MSSql.UpdateEntityId(vcc, AppId, entity.id, newEntityId));
                return Promise.all(promises);
            }
            logger.error(`[CognitiveEngineMigrateImplementation::AddEntity] Error on add entity into ${data.newEngine}:`, newEntityId);
            return [null];
        })
            .then((result) => {
            let newEntityId = result[0];
            if (newEntityId) {
                return mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'entity', newEntityId, 1, enum_1.AVAILABLE);
            }
            return false;
        })
            .then((result) => {
            logger.info('[CognitiveEngineMigrateImplementation::AddEntity] Updated new entity id into db:', result);
            if (result) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Entities`, entity.key);
            }
            return false;
        }).catch((err) => {
            logger.error(`[CognitiveEngineMigrateImplementation::AddEntity] Unknown error on add entity into ${data.newEngine}`, err);
            return false;
        });
    }
}
exports.EntityMigrator = EntityMigrator;
