"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IntentMigrator = void 0;
const redis_service_1 = require("../../db/redis.service");
const log4js = require("log4js");
const mssql_service_1 = require("../../db/mssql.service");
const promises_shared_1 = require("../../shared/promises.shared");
const helper_1 = require("./helper");
const enum_1 = require("./enum");
const arrays_shared_1 = require("../../shared/arrays.shared");
const logger = log4js.getLogger('Task');
class IntentMigrator {
    MigrateIntents(vcc, data, start, end) {
        logger.debug("[CognitiveEngineMigrateImplementation::MigrateIntents] Migrating intents for ", vcc, " into ", data.newEngine, " from ", start, " to ", end);
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.LRange(`${enum_1.ROOT_KEY}:${vcc}:Intents:Keys`, start, end - 1);
        }).then((intentKeys) => {
            logger.debug("[CognitiveEngineMigrateImplementation::MigrateIntents] Found intent keys:", intentKeys);
            if (!intentKeys || intentKeys.length == 0) {
                return false;
            }
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Intents`, arrays_shared_1.GetUniqueFromArray(intentKeys));
        }).then((intents) => {
            if (intents) {
                let intentList = [];
                for (let i of intents) {
                    let intent = JSON.parse(i);
                    if (intent) {
                        intentList.push(intent);
                    }
                }
                if (intentList.length > 0) {
                    return this.ProcessIntents(vcc, data, intentList);
                }
            }
            return false;
        }).then(_ => {
            if (end < data.totalIntent) {
                return this.MigrateIntents(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE);
            }
            return true;
        });
    }
    ProcessIntents(vcc, data, intents) {
        return promises_shared_1.DummyPromise().then(_ => {
            let promises = [];
            for (let intent of intents) {
                switch (intent.status) {
                    case 0:
                        promises.push(this.AddIntent(vcc, data, intent));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::ProcessIntents] Results for process intents:', results);
            if (results.length > 0) {
                let totalCreated = results.filter(r => r == true).length;
                let totalError = results.filter(r => !r).length;
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                    totalCreatedIntent: totalCreated + data.totalCreatedIntent,
                    totalErrorIntent: totalError + data.totalErrorIntent
                }));
            }
            return false;
        });
    }
    AddIntent(vcc, data, intent) {
        logger.info('[CognitiveEngineMigrateImplementation::AddIntent]  Adding intent ', intent.name, ' in ', data.newEngine);
        let AppId = null;
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId(`${enum_1.ROOT_KEY}:${vcc}:AppIds`, intent.appId);
        }).
            then((appId) => {
            if (appId) {
                AppId = appId;
                return helper_1.GetServiceInstance(data.newEngine);
            }
            logger.debug('[CognitiveEngineMigrateImplementation::AddIntent] Not found AppId for intent ', intent.name);
            return null;
        })
            .then((service) => {
            if (service && AppId) {
                return service.AddIntent(data.subscriptionRegion, data.subscriptionKey, AppId, intent.name);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddIntent] Error on get service and appId for ', intent.name);
            return null;
        }).then((newIntentId) => {
            if (newIntentId) {
                logger.info('[CognitiveEngineMigrateImplementation::AddIntent] Success new IntentId:', newIntentId, ' for intent:', intent.name);
                let promises = [];
                promises.push(Promise.resolve(newIntentId));
                promises.push(mssql_service_1.MSSql.UpdateIntentId(vcc, AppId, intent.intentId, newIntentId));
                promises.push(redis_service_1.Redis.SaveCognitiveEngineMigrationNewId(`${enum_1.ROOT_KEY}:${vcc}:IntentIds`, intent.key, newIntentId));
                return Promise.all(promises);
            }
            logger.error(`[CognitiveEngineMigrateImplementation::AddIntent] Error on add intent into ${data.newEngine}:`, newIntentId);
            return [null];
        })
            .then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::AddIntent] Results for Update new intent Id:', results);
            let newIntentId = results[0];
            if (newIntentId) {
                let promises = [];
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'intent', newIntentId, 1, enum_1.AVAILABLE));
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Intents`, intent.key));
                return Promise.all(promises);
            }
            return [false];
        })
            .then((results) => {
            return !results.includes(false);
        })
            .catch((err) => {
            logger.error(`[CognitiveEngineMigrateImplementation::AddIntent] Unknown error on add intent into ${data.newEngine}`, err);
            if (typeof err === 'string' && err == "SERVER_ERROR_INTENT_ALREADY_EXIST") {
                let promises = [];
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Intents`, intent.key, Object.assign(intent, {
                    status: 3
                })));
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'intent', intent.intentId, 3, 'ERROR_INTENT_EXISTS_IN_RASA'));
                return Promise.all(promises).then(() => {
                    return false;
                });
            }
            return false;
        });
    }
}
exports.IntentMigrator = IntentMigrator;
