"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiarizationAnalysisImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
const billing_implementation_1 = require("./billing.implementation");
const process_implementation_1 = require("./process.implementation");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'DiarizationAnalysisImplementation';
let withSentimentAnalysis = true;
class DiarizationAnalysisImplementation {
    Execute(executionCode, rateRecursive) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_DIARIZATION_ANALYSIS;
        let executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_DIARIZATION_ANALYSIS)
            .then(result => {
            return redis_service_1.Redis.LRange('DiarizationAnalysis', 0, ratio - 1);
        }).then(result => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(this.DiarizationConvertExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('DiarizationAnalysis', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[DiarizationAnalysisImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    }
    DiarizationConvertExecute(executionCode) {
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let isCancel = false;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let batchTypeName = '';
        let batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('DiarizationAnalysis', 'DiarizationAnalysisExecute')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_TO_GET_PROCESS_INTERACTION';
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then((processInteractionResult) => {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_MAIN_FUNCTION';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.statusDescription === 'pendingIdentifyAgent') {
                    return this.IdentifyAgent(processInteraction, currentQueueId, executionCode);
                }
                else if (processInteraction.statusDescription === 'pendingInvertChannels' || processInteraction.channelchange) {
                    return this.InvertChannels(processInteraction, currentQueueId, executionCode);
                }
                return null;
            }
            else {
                if (currentQueueId)
                    logger.warn('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_UPDATE_INTERACTION';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_FINISHING_FUNCTION';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 4
                && (processInteraction.statusDescription === 'pendingSentimentAnalysis' || processInteraction.statusDescription === 'pendingAnalysis')) {
                logger.info('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                if (processInteraction.statusDescription === 'pendingSentimentAnalysis')
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SentimentAnalysis', currentQueueId));
                if (processInteraction.statusDescription === 'pendingAnalysis')
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticAnalysisUpload', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && (processInteraction.status === 3
                    ||
                        (processInteraction.status === 4
                            &&
                                processInteraction.statusDescription !== 'pendingSentimentAnalysis'
                            &&
                                processInteraction.statusDescription !== 'pendingAnalysis'))
                && processInteraction.retry < config_1.RETRY_DIARIZATION_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('DiarizationAnalysis', currentQueueId, config_1.RATE_RECURSIVE_DIARIZATION_ANALYSIS * config_1.RATIO_DIARIZATION_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && processInteraction.status === -4
                && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_ANALYSIS) {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(true);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_UPDATE_REDIS';
            let promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchTypeId, batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_FINISH_EXECUTION';
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    IdentifyAgent(interaction, currentQueueId, executionCode) {
        let path = '';
        let jsonContent = null;
        let cognitiveService;
        let cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        let phrases = null;
        let agentResults = [];
        let clientResults = [];
        let agentEvaluationResult = [];
        let clientEvaluationResult = [];
        let needInvertChannels = false;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            let regex = /\.(mp3|ogg|wav)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.TRANSCRIPTION_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(result => {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                phrases = this.GeneratePhrases(jsonContent.results, config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE);
            }
            catch (ex) {
                jsonContent = '';
                throw 'SERVER_ERROR_GET_JSON_FAILED';
            }
            return Promise.resolve(phrases);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_PHRASES_NOT_FOUND';
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_COGNITIVE_SERVICE';
            cognitiveService = result;
            let promiseArray = [];
            phrases.agent.filter(ap => ap.trim() !== '').map((ap, i) => {
                let appId = interaction.vcc + '::diarization-' + interaction.language;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(() => {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_NLU_AGENT_EVALUATION_FAILED';
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting agent evaluation result:', result);
            let promiseArray = [];
            if (result.find(r => r.error === 'SERVER_ERROR_CANNOT_GET_MODEL_PORT')) {
                let process = new process_implementation_1.ProcessImplementation();
                promiseArray.push(process.RunCurrentBatchesRasaModels(interaction.vcc, 'TRANSCRIPTION', interaction.processId, ['diarization-' + interaction.language]));
            }
            agentEvaluationResult = result;
            phrases.client.filter(ap => ap.trim() !== '').map((ap, i) => {
                let appId = interaction.vcc + '::diarization-' + interaction.language;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(() => {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_NLU_CLIENT_EVALUATION_FAILED';
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting client evaluation result:', result);
            clientEvaluationResult = result;
            let maxAgentScore = [];
            let maxClientScore = [];
            let agentMaxScore = Math.max.apply(Math, agentEvaluationResult.map((aer, i) => {
                agentResults.push({
                    phrase: phrases.agent[i],
                    scores: aer.intents
                });
                let agentItem = aer.intents.find(i => i.intent === 'Agent');
                if (agentItem) {
                    maxAgentScore.push({
                        phrase: phrases.agent[i],
                        score: agentItem.score
                    });
                    return agentItem.score;
                }
                else
                    return 0;
            }));
            let clientMaxScore = Math.max.apply(Math, clientEvaluationResult.map((cer, i) => {
                clientResults.push({
                    phrase: phrases.client[i],
                    scores: cer.intents
                });
                let clientItem = cer.intents.find(i => i.intent === 'Agent');
                if (clientItem) {
                    maxClientScore.push({
                        phrase: phrases.client[i],
                        score: clientItem.score
                    });
                    return clientItem.score;
                }
                else
                    return 0;
            }));
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting max client evaluation result:', maxClientScore.filter(mcs => mcs.score === clientMaxScore));
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting max agent evaluation result:', maxAgentScore.filter(mas => mas.score === agentMaxScore));
            if (agentMaxScore > clientMaxScore) {
                needInvertChannels = false;
            }
            else if (agentMaxScore === clientMaxScore) {
                let promAgent = agentEvaluationResult.reduce((sum, current) => {
                    let agentItem = current.intents.find(i => i.intent === 'Agent');
                    return sum + agentItem ? agentItem.score : 0;
                }, 0) / agentEvaluationResult.length;
                let promclient = clientEvaluationResult.reduce((sum, current) => {
                    let clientItem = current.intents.find(i => i.intent === 'Agent');
                    return sum + clientItem ? clientItem.score : 0;
                }, 0) / clientEvaluationResult.length;
                if (promAgent > promclient)
                    needInvertChannels = false;
                else if (promAgent < promclient)
                    needInvertChannels = true;
            }
            else {
                needInvertChannels = true;
            }
            let resultsObject = {
                Model: interaction.vcc + '::diarization-' + interaction.language,
                AllPhrases: phrases.allPhrases,
                AgentPhrases: agentResults,
                ClientPhrases: clientResults
            };
            return this.SaveDiarizationResults(interaction, resultsObject, currentQueueId, executionCode);
        })
            .then((result) => {
            return Promise.resolve('OK');
        })
            .then((result) => {
            if (result === 'OK') {
                interaction['status'] = 4;
                interaction['statusDescription'] = needInvertChannels ? 'pendingInvertChannels' : config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -4;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'IdentifyAgent';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -4;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'IdentifyAgent';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    InvertChannels(interaction, currentQueueId, executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::InvertChannels] [vcc::processId::id => ${interaction.vcc}::${interaction.processId}::${interaction.id}]: `;
        let path = '';
        let jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            let regex = /\.(mp3|wav|ogg)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.TRANSCRIPTION_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(result => {
            jsonContent = result;
            let agentRegex = new RegExp(`(?<="channel"\\:)"Agent"`, 'gm');
            let clientRegex = new RegExp(`(?<="channel"\\:)"Caller"`, 'gm');
            jsonContent = jsonContent.replace(agentRegex, '"OldAgent"');
            jsonContent = jsonContent.replace(clientRegex, '"OldCaller"');
            let oldAgentRegex = new RegExp(`(?<="channel"\\:)"OldAgent"`, 'gm');
            let oldClientRegex = new RegExp(`(?<="channel"\\:)"OldCaller"`, 'gm');
            jsonContent = jsonContent.replace(oldAgentRegex, '"Caller"');
            jsonContent = jsonContent.replace(oldClientRegex, '"Agent"');
            return Promise.resolve(true);
        }).then((result) => {
            return file_manager_1.WriteFileContent(path, jsonContent, 'utf-8', true);
        }).then((result) => {
            return Promise.resolve('OK');
        }).then((result) => {
            if (result === 'OK') {
                logger.info(`${logLabel}executionCode: ${executionCode} getting queueId: ${currentQueueId} channels were inverted`);
                interaction['status'] = 4;
                interaction['statusDescription'] = config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                interaction['channelsChanged'] = true;
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
            }
            else {
                logger.info(`${logLabel}executionCode: ${executionCode} getting queueId: ${currentQueueId} getting errorMessage:`, result);
                interaction['status'] = -4;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'InvertChannels';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationAnalysisImplementation::InvertChannels] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -4;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'InvertChannels';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    SaveDiarizationResults(interaction, jsonContent, currentQueueId, executionCode) {
        let diarizationFolder = config_1.DIARIZATION_FOLDER + '/' + interaction.vcc;
        let regex = /\.(mp3|wav|ogg|json)$/;
        let jsonFileName = interaction.fileName.replace(regex, '.json');
        let jsonFilePath = null;
        jsonFilePath = diarizationFolder;
        jsonFilePath += '/';
        jsonFilePath += jsonFileName;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.CreateFolder(diarizationFolder);
        })
            .then(result => {
            logger.info('[DiarizationAnalysisImplementation::SaveDiarizationResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'saving results on:', jsonFilePath);
            return file_manager_1.WriteFileContent(jsonFilePath, JSON.stringify(jsonContent), 'utf-8');
        })
            .then((result) => {
            return Promise.resolve(true);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[DiarizationAnalysisImplementation::SaveDiarizationResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    }
    GeneratePhrases(jsonWords, charsPerPhrase) {
        let allPhrases = [];
        let agentPhrases = [];
        let clientPhrases = [];
        let currentEntity = '';
        let currentPhrase = '';
        let currentPhraseStartTime = 0;
        let currentPhraseEndTime = 0;
        jsonWords.sort((a, b) => (a.start_time > b.start_time) ? 1 : -1);
        jsonWords.map((jw, i) => {
            let currentContent = '';
            currentPhraseEndTime = jw.end_time;
            if (currentEntity === jw.channel) {
                currentContent += jw.type === 'word' ? ' ' : '';
            }
            else {
                if (currentEntity !== '') {
                    allPhrases.push({
                        entity: currentEntity,
                        phrase: currentPhrase,
                        startTime: currentPhraseStartTime,
                        endTime: currentPhraseEndTime
                    });
                    if (currentPhrase.length) {
                        if (currentEntity === 'Agent')
                            agentPhrases.push(currentPhrase);
                        if (currentEntity === 'Caller')
                            clientPhrases.push(currentPhrase);
                    }
                }
                currentEntity = jw.channel;
                currentPhrase = '';
                currentPhraseStartTime = jw.start_time;
            }
            currentContent += jw.alternatives[0].content;
            if ((currentPhrase.length + currentContent.length) > charsPerPhrase
                ||
                    (i + 1 === jsonWords.length)) {
                allPhrases.push({
                    entity: currentEntity,
                    phrase: currentPhrase,
                    startTime: currentPhraseStartTime,
                    endTime: currentPhraseEndTime
                });
                if (currentPhrase.length) {
                    if (currentEntity === 'Agent')
                        agentPhrases.push(currentPhrase);
                    if (currentEntity === 'Caller')
                        clientPhrases.push(currentPhrase);
                }
                currentPhrase = '';
            }
            else {
                currentPhrase += currentContent;
            }
        });
        return {
            agent: this.ConcatPhrasesWithMaxLength(agentPhrases, charsPerPhrase),
            client: this.ConcatPhrasesWithMaxLength(clientPhrases, charsPerPhrase),
            allPhrases: this.ConcatPhrasesWithMaxLength(allPhrases, charsPerPhrase)
        };
    }
    ConcatPhrasesWithMaxLength(phrases, maxLength) {
        let currentConcatenatedPhrase = '';
        const concatenatedPhrases = [];
        for (let i = 0; i < phrases.length; i++) {
            const currentPhrase = phrases[i];
            if (currentConcatenatedPhrase.length + currentPhrase.length <= maxLength) {
                currentConcatenatedPhrase += currentPhrase;
            }
            else {
                concatenatedPhrases.push(currentConcatenatedPhrase);
                currentConcatenatedPhrase = currentPhrase;
            }
        }
        concatenatedPhrases.push(currentConcatenatedPhrase);
        return concatenatedPhrases;
    }
}
exports.DiarizationAnalysisImplementation = DiarizationAnalysisImplementation;
