"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiarizationConvertImplementation = void 0;
const log4js = require("log4js");
const fs = require("fs");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const mssql_config_1 = require("../config/mssql.config");
const redis_service_1 = require("../db/redis.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const process_implementation_1 = require("../implementation/process.implementation");
const SSH2Promise = require("ssh2-promise");
const billing_implementation_1 = require("../implementation/billing.implementation");
let Client = require('ftp');
let net = require('net');
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'DiarizationConvertImplementation';
class DiarizationConvertImplementation {
    Execute(executionCode, rateRecursive) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_DIARIZATION_CONVERT;
        let executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_DIARIZATION_CONVERT)
            .then(result => {
            return redis_service_1.Redis.LRange('DiarizationConvert', 0, ratio - 1);
        }).then(result => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                let timeOut = (executeIndex - 1) * 8000;
                promiseArray.push(new Promise((resolve, reject) => {
                    setTimeout(() => {
                        return resolve(this.DiarizationConvertExecute(executionCode));
                    }, timeOut);
                }));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('DiarizationConvert', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[DiarizationConvertImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    }
    DiarizationConvertExecute(executionCode) {
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let isCancel = false;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let batchTypeName = '';
        let batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('DiarizationConvert', 'DiarizationConvertExecute')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_TO_GET_PROCESS_INTERACTION';
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then((processInteractionResult) => {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_MAIN_FUNCTION';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                return this.CheckConnection().then(result => {
                    if (result) {
                        if (processInteraction.statusDescription === 'pendingUploadRawAudio') {
                            return this.UploadRawAudio(processInteraction, currentQueueId, executionCode);
                        }
                        else if (processInteraction.statusDescription === 'pendingLaunchAudioConverter') {
                            return this.LaunchAudioConverter(processInteraction, currentQueueId, executionCode);
                        }
                        else if (processInteraction.statusDescription === 'pendingDownloadConvertedAudio') {
                            return this.DownloadConvertedAudio(processInteraction, currentQueueId, executionCode);
                        }
                        return null;
                    }
                    else {
                        logger.error('[DiarizationConvertImplementation::CheckConnection] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                        processInteraction['status'] = -2;
                        processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
                        processInteraction['errorFunction'] = 'CheckConnection';
                        processInteraction['errorMessage'] = 'SERVER_DIARIZATION_CONNECTION_ERROR';
                        processInteraction['errorDetail'] = result;
                        return processInteraction;
                    }
                });
            }
            else {
                if (currentQueueId)
                    logger.warn('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_UPDATE_INTERACTION';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_FINISHING_FUNCTION';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 2
                && processInteraction.statusDescription === 'pendingUpload') {
                logger.info('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TranscriptionsUpload', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
                let process = new process_implementation_1.ProcessImplementation();
                promiseArray.push(process.RunCurrentBatchesRasaModels(vcc, 'TRANSCRIPTION', processId, ['diarization-' + processInteraction.language]));
            }
            else if (result
                && processInteraction
                && (processInteraction.status === 1
                    ||
                        (processInteraction.status === 2 && processInteraction.statusDescription !== 'pendingUpload'))
                && processInteraction.retry < config_1.RETRY_DIARIZATION_CONVERT) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('DiarizationConvert', currentQueueId, config_1.RATE_RECURSIVE_DIARIZATION_CONVERT * config_1.RATIO_DIARIZATION_CONVERT * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && processInteraction.status === -2
                && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_CONVERT) {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, processId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(true);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_UPDATE_REDIS';
            let promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_CONVERT) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processId, processId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_FINISH_EXECUTION';
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    UploadRawAudio(interaction, currentQueueId, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return this.CheckFtpOrSftpType();
        }).then(result => {
            if (result === 'FTP OK') {
                return this.UploadFtpRawAudio(interaction, currentQueueId, executionCode);
            }
            else if (result === 'SFTP OK') {
                return this.UploadSftpRawAudio(interaction, currentQueueId, executionCode);
            }
            else {
                logger.error('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'CheckFtpOrSftpType';
                interaction['errorMessage'] = 'SERVER_DIARIZATION_ISNOT_FTP_NEITHER_SFTP';
                interaction['errorDetail'] = result;
                return interaction;
            }
        });
    }
    UploadFtpRawAudio(interaction, currentQueueId, executionCode) {
        let filePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + interaction.fileName;
        let uploadFtpFolder = config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        let uploadFtpPath = uploadFtpFolder + '/' + interaction.fileName;
        let downloadFtpFolder = config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.ExistInFileSystem(filePath);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_NOT_FOUND';
            logger.debug('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting result from ExistInFileSystme:', result);
            return new Promise((resolve, reject) => {
                let clientFtp = new Client();
                clientFtp.connect({
                    host: config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT,
                    port: config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT,
                    user: config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT,
                    password: config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
                });
                if (clientFtp) {
                    clientFtp.on('greeting', function (message) {
                        logger.info('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting on greeting event message:', message);
                    });
                    clientFtp.on('ready', function () {
                        clientFtp.mkdir(uploadFtpFolder, true, function (err) {
                            if (err) {
                                clientFtp.end();
                                logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_INPUT_FOLDER_CREATE_FAILED:', err);
                                return resolve('SERVER_ERROR_INPUT_FOLDER_CREATE_FAILED');
                            }
                            else {
                                logger.info('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_EVENT_INPUT_FOLDER_CREATE_SUCCED:', err);
                                clientFtp.put(filePath, uploadFtpPath, function (err) {
                                    if (err) {
                                        clientFtp.end();
                                        logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_AUDIO_UPLOAD_FAILED:', err);
                                        return resolve('SERVER_ERROR_AUDIO_UPLOAD_FAILED');
                                    }
                                    else {
                                        logger.info('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_EVENT_AUDIO_UPLOAD_SUCCED:', err);
                                        clientFtp.mkdir(uploadFtpFolder, true, function (err) {
                                            if (err) {
                                                clientFtp.end();
                                                logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_OUTPUT_FOLDER_CREATE_FAILED:', err);
                                                return resolve('SERVER_ERROR_INPUT_FOLDER_CREATE_FAILED');
                                            }
                                            else {
                                                logger.info('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_EVENT_OUTPUT_FOLDER_CREATE_SUCCED:', err);
                                                clientFtp.end();
                                                return resolve('OK');
                                            }
                                        });
                                    }
                                });
                            }
                        });
                    });
                    clientFtp.on('end', function () {
                        let eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('close', function (hadError) {
                        let eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                        logger.warn('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage, 'hadError:', hadError);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('error', function (err) {
                        logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_NOT_CLIENT', err);
                        return resolve('SERVER_ERROR_FTP_NOT_CLIENT');
                    });
                }
                else {
                    logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_NOT_CLIENT');
                    return resolve('SERVER_ERROR_FTP_NOT_CLIENT');
                }
            });
        }).then((result) => {
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingLaunchAudioConverter';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'UploadRawAudio';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::UploadFtpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'UploadRawAudio';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    UploadSftpRawAudio(interaction, currentQueueId, executionCode) {
        let filePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + interaction.fileName;
        let uploadFtpFolder = 'speech_analytics' + config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        let uploadFtpPath = uploadFtpFolder + '/' + interaction.fileName;
        const environmentConfig = {
            host: config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT,
            port: config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT,
            username: config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT,
            password: config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT,
            connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
            pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
        };
        let ssh2 = new SSH2Promise(environmentConfig);
        let sftp;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.ExistInFileSystem(filePath);
        }).then(result => {
            if (!result)
                throw new Error("SERVER_ERROR_AUDIO_NOT_FOUND");
            return new Promise((resolve, reject) => {
                if (ssh2) {
                    ssh2.exec('mkdir -p ' + uploadFtpFolder)
                        .then((result) => {
                        sftp = ssh2.sftp();
                        sftp.fastPut(filePath, uploadFtpPath)
                            .then(result => {
                            ssh2.close();
                            resolve('OK');
                        })
                            .catch(err => {
                            ssh2.close();
                            resolve(err);
                        });
                    })
                        .catch((err) => {
                        ssh2.close();
                        resolve(err);
                    });
                }
                else {
                    logger.error('[DiarizationConvertImplementation::UploadSftpRawAudio] executionCode:', 'getting queueId:', 'SERVER_ERROR_FTP_NOT_CLIENT');
                    throw 'SERVER_ERROR_NOT_SFTP_CLIENT';
                }
            }).then((result) => {
                if (result === 'OK') {
                    interaction['status'] = 2;
                    interaction['statusDescription'] = 'pendingLaunchAudioConverter';
                    interaction['retry'] = 0;
                    interaction['errorFunction'] = null;
                    interaction['errorMessage'] = null;
                    interaction['errorDetail'] = null;
                }
                else {
                    logger.error('[DiarizationConvertImplementation::UploadSftpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                    interaction['status'] = -2;
                    interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                    interaction['errorFunction'] = 'UploadRawAudio';
                    interaction['errorMessage'] = result;
                    interaction['errorDetail'] = result;
                }
                return Promise.resolve(interaction);
            }).catch((err) => {
                let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
                logger.error('[DiarizationConvertImplementation::UploadSftpRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'UploadRawAudio';
                interaction['errorMessage'] = errorMessage;
                interaction['errorDetail'] = JSON.stringify(err);
                return Promise.resolve(interaction);
            });
        });
    }
    LaunchAudioConverter(interaction, currentQueueId, executionCode) {
        let ftpConverterApplicationFolder = config_1.CONVERTER_FTP_CONVERTER_APP_PATH;
        let ftpInputFileFolder = config_1.CONVERTER_FTP_ROOT_FOLDER_DIARIZATION_CONVERT + config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        let ftpOutputFileFolder = config_1.CONVERTER_FTP_ROOT_FOLDER_DIARIZATION_CONVERT + config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        let remoteRepoDiarizationPath = (config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT !== mssql_config_1.MSSQL_HOST ? config_1.CONVERTER_FTP_ROOT_FOLDER_DIARIZATION_CONVERT + ' \\\\' + config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT + '\\speech_analytics ' + config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT + ' /USER:' + config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT : 'local');
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.LaunchAudioConverter(interaction.vcc, ftpConverterApplicationFolder, ftpInputFileFolder + '/' + interaction.fileName, ftpOutputFileFolder, remoteRepoDiarizationPath);
        }).then(result => {
            if (result && (result.errcode || result.errcode === 0)) {
                switch (result.errcode) {
                    case 0: return Promise.resolve('OK');
                    case -1: throw 'SERVER_ERROR_DIARIZATION_FAILED';
                    case -2: throw 'SERVER_ERROR_CHANNEL_SPLIT_FAILED';
                    case -3: throw 'SERVER_ERROR_FFMPEG_FUNCTION_FAILED';
                    case -100: throw 'SERVER_ERROR_FILE_NOT_FOUND';
                    default: throw 'SERVER_ERROR_UNKNOWN_ERROR';
                }
            }
            else {
                throw 'SERVER_ERROR_CONVERTER_AUDIO_SP_FAILED';
            }
        }).then(result => {
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingDownloadConvertedAudio';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[DiarizationConvertImplementation::LaunchAudioConverter] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'LaunchAudioConverter';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::LaunchAudioConverter] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'LaunchAudioConverter';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    DownloadConvertedAudio(interaction, currentQueueId, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return this.CheckFtpOrSftpType();
        }).then(result => {
            if (result === 'FTP OK') {
                return this.DownloadFtpConvertedAudio(interaction, currentQueueId, executionCode);
            }
            else if (result === 'SFTP OK') {
                return this.DownloadSftpConvertedAudio(interaction, currentQueueId, executionCode);
            }
            else {
                logger.error('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'CheckFtpOrSftpType';
                interaction['errorMessage'] = 'SERVER_DIARIZATION_ISNOT_FTP_NEITHER_SFTP';
                interaction['errorDetail'] = result;
                return interaction;
            }
        });
    }
    DownloadFtpConvertedAudio(interaction, currentQueueId, executionCode) {
        let filePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + interaction.fileName;
        let tmpPath = filePath + '-TMP';
        let uploadFtpFolder = config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        let uploadFtpPath = uploadFtpFolder + '/' + interaction.fileName;
        let downloadFtpFolder = config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        let downloadFtpPath = downloadFtpFolder + '/' + interaction.fileName;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.ExistInFileSystem(filePath);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_NOT_FOUND';
            return file_manager_1.RenameFile(filePath, tmpPath);
        }).then(result => {
            return new Promise((resolve, reject) => {
                let clientFtp = new Client();
                clientFtp.connect({
                    host: config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT,
                    port: config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT,
                    user: config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT,
                    password: config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
                });
                if (clientFtp) {
                    clientFtp.on('greeting', function (message) {
                        logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting on greeting event message:', message);
                    });
                    clientFtp.on('ready', function () {
                        clientFtp.list(downloadFtpFolder, function (err, list) {
                            if (err)
                                return resolve(err.message);
                            if (list.find(l => l && l.name && l.name === interaction.fileName)) {
                                clientFtp.get(downloadFtpPath, function (err, stream) {
                                    if (err) {
                                        clientFtp.end();
                                        logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_AUDIO_FILE_GET_FTP:', err);
                                        return resolve('SERVER_ERROR_AUDIO_FILE_GET_FTP');
                                    }
                                    else {
                                        let file = fs.createWriteStream(filePath);
                                        stream.pipe(fs.createWriteStream(filePath));
                                        stream.once('close', function () {
                                            logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DOWNLOADED_CONVERTED_AUDIO');
                                            clientFtp.delete(uploadFtpPath, (err) => {
                                                if (err) {
                                                    clientFtp.end();
                                                    logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_INPUT_AUDIO_DELETE_FAILED:', err);
                                                    return resolve('SERVER_ERROR_FTP_INPUT_AUDIO_DELETE_FAILED');
                                                }
                                                else {
                                                    logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_RAW_AUDIO');
                                                    clientFtp.delete(downloadFtpPath, (err) => {
                                                        if (err) {
                                                            clientFtp.end();
                                                            logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_OUTPUT_AUDIO_DELETE_FAILED:', err);
                                                            return resolve('SERVER_ERROR_FTP_OUTPUT_AUDIO_DELETE_FAILED');
                                                        }
                                                        else {
                                                            logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_CONVERTED_AUDIO');
                                                            clientFtp.end();
                                                            file.close();
                                                            return resolve('OK');
                                                        }
                                                    });
                                                }
                                            });
                                        });
                                    }
                                });
                            }
                            else {
                                return resolve('SERVER_ERROR_AUDIO_TO_DOWNLOAD_NOT_FOUND');
                            }
                        });
                    });
                    clientFtp.on('end', function () {
                        let eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('close', function (hadError) {
                        let eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                        logger.warn('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage, 'hadError:', hadError);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('error', function (err) {
                        logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_NOT_CLIENT', err);
                        return resolve('SERVER_ERROR_FTP_NOT_CLIENT');
                    });
                }
                else {
                    logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_NOT_CLIENT');
                    return resolve('SERVER_ERROR_FTP_NOT_CLIENT');
                }
            });
        }).then(result => {
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['isDiariced'] = true;
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
                return file_manager_1.DeleteFile(tmpPath);
            }
            else {
                logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'DownloadConvertedAudio';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
                return file_manager_1.RenameFile(tmpPath, filePath);
            }
        }).then(result => {
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'DownloadConvertedAudio';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    DownloadSftpConvertedAudio(interaction, currentQueueId, executionCode) {
        let filePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + interaction.fileName;
        let tmpPath = filePath + '-TMP';
        let uploadSftpFolder = 'speech_analytics' + config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        let uploadSftpPath = uploadSftpFolder + '/' + interaction.fileName;
        let downloadSftpFolder = 'speech_analytics' + config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        let downloadSftpPath = downloadSftpFolder + '/' + interaction.fileName;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.ExistInFileSystem(filePath);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_NOT_FOUND';
            return file_manager_1.RenameFile(filePath, tmpPath);
        }).then(result => {
            return new Promise((resolve, reject) => {
                const environmentConfig = {
                    host: config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT,
                    port: config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT,
                    username: config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT,
                    password: config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
                };
                let ssh2 = new SSH2Promise(environmentConfig);
                if (ssh2) {
                    let sftp = ssh2.sftp();
                    sftp.fastGet(downloadSftpPath, filePath, function (err, stream) {
                        if (err) {
                            sftp.end();
                            ssh2.close();
                            logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_AUDIO_FILE_GET_FTP:', err);
                            return resolve('SERVER_ERROR_AUDIO_FILE_GET_SFTP');
                        }
                        else {
                            logger.info('[DiarizationConvertImplementation::DownloadSftpConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DOWNLOADED_CONVERTED_SFTP_AUDIO');
                            ssh2.exec('rm ' + uploadSftpPath).then((data) => {
                                logger.info('[DiarizationConvertImplementation::DownloadSftpConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_SFTP_AUDIO_FROM_INPUT');
                                ssh2.exec('rm ' + downloadSftpPath).then((data) => {
                                    logger.info('[DiarizationConvertImplementation::DownloadSftpConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_SFTP_AUDIO_FROM_OUTPUT');
                                    resolve('OK');
                                }).catch((err) => {
                                    logger.info('[DiarizationConvertImplementation::DownloadSftpConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_SFTP_AUDIO_FROM_OUTPUT_ERROR');
                                    resolve('DELETED_SFTP_AUDIO_FROM_OUTPUT_ERROR');
                                });
                            }).catch((err) => {
                                logger.info('[DiarizationConvertImplementation::DownloadSftpConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_SFTP_AUDIO_FROM_INPUT_ERROR');
                                resolve('DELETED_SFTP_AUDIO_FROM_INPUT_ERROR');
                            });
                        }
                    });
                }
            });
        }).then(result => {
            logger.info('[DiarizationConvertImplementation::DownloadSftpConvertedAudio:] executionCode:', executionCode, 'getting queueId:', currentQueueId, result);
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['isDiariced'] = true;
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
                return file_manager_1.DeleteFile(tmpPath);
            }
            else {
                logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'DownloadSftpConvertedAudio';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
                return file_manager_1.RenameFile(tmpPath, filePath);
            }
        }).then(result => {
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'DownloadConvertedAudio';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    CheckConnection() {
        let host = config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT;
        let port = config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT;
        let timeout = config_1.FTP_CONNECTION_TIMEOUT;
        return promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[DiarizationConvertImplementation::CheckConnection] executionCode:', 'getting on greeting event message:', result);
            return new Promise(function (resolve, reject) {
                timeout = timeout || 10000;
                var timer = setTimeout(function () {
                    resolve(false);
                    socket.end();
                }, timeout);
                var socket = net.createConnection(port, host, function () {
                    clearTimeout(timer);
                    socket.destroy();
                    resolve(true);
                });
                socket.on('error', function (err) {
                    clearTimeout(timer);
                    resolve(false);
                });
            });
        });
    }
    CheckFtpOrSftpType() {
        let host = config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT;
        let port = config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT;
        let user = config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT;
        let password = config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT;
        let connTimeout = config_1.FTP_CONNECTION_TIMEOUT;
        let pasvTimeout = config_1.FTP_CONNECTION_TIMEOUT;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return new Promise((resolve, reject) => {
                const clientFtp = new Client();
                clientFtp.connect({
                    host,
                    port,
                    user,
                    password,
                    connTimeout,
                    pasvTimeout
                });
                if (clientFtp) {
                    clientFtp.on('greeting', function (message) {
                        logger.info('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', 'getting on greeting event message:', message);
                        clientFtp.end();
                        return resolve('FTP OK');
                    });
                    clientFtp.on('end', function () {
                        const eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', 'getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('error', function (err) {
                        logger.error('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', 'getting queueId:', 'SERVER_ERROR_FTP_NOT_CLIENT', err);
                        return resolve(err);
                    });
                    clientFtp.on('close', function (err) {
                        logger.error('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', 'getting queueId:', 'SERVER_ERROR_FTP_NOT_CLIENT', err);
                        return resolve(err);
                    });
                    setTimeout(() => {
                        clientFtp.destroy();
                        reject("TIMEOUT");
                    }, 5000);
                }
            });
        }).catch((reason => {
            logger.error('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', 'getting queueId:', 'SERVER_ERROR_FTP_NOT_CLIENT', reason);
            return new Promise((resolve, reject) => {
                const clientSftpt = {
                    host,
                    port,
                    username: user,
                    password,
                    connTimeout,
                    pasvTimeout,
                    reconnect: false
                };
                const ssh2 = new SSH2Promise(clientSftpt);
                if (ssh2) {
                    const sftp = ssh2.sftp();
                    sftp.stat("/").then((list) => {
                        ssh2.close();
                        return resolve('SFTP OK');
                    }).catch((errorSFTP) => {
                        logger.error('[DiarizationConvertImplementation::CheckFtpOrSftpType] executionCode:', 'getting queueId:', 'SERVER_ERROR_FTP_NOT_CLIENT', errorSFTP);
                        return resolve(errorSFTP);
                    });
                }
                else {
                    return resolve('SERVER_ERROR_NOT_SFTP_CLIENT');
                }
            });
        }));
    }
}
exports.DiarizationConvertImplementation = DiarizationConvertImplementation;
