"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationPlansImplementation = void 0;
const mssql_service_1 = require("../db/mssql.service");
const log4js = require("log4js");
let logger = log4js.getLogger('Task');
class EvaluationPlansImplementation {
    Execute(currentDate) {
        let promiseArray = [];
        let evaluationPlansExpired = [];
        return mssql_service_1.MSSql.GetEvaluationPlansExpired()
            .then(result => {
            logger.info('[EvaluationPlansImplementation::Execute] GetEvaluationPlansExpired', currentDate);
            if (result.length) {
                result.forEach(planExpired => {
                    const { vcc, id, idInteraction, idAuditor, lastModifiedByUserId } = planExpired;
                    evaluationPlansExpired.push({ vcc, id });
                    promiseArray.push(mssql_service_1.MSSql.SaveEvaluationStatus(vcc, id, idInteraction, idAuditor, false, true, lastModifiedByUserId));
                });
                return Promise.all(promiseArray);
            }
            return Promise.resolve([]);
        })
            .then(_ => {
            logger.info('[EvaluationPlansImplementation::Execute] SaveEvaluationStatus', currentDate);
            promiseArray = [];
            if (!evaluationPlansExpired.length)
                return Promise.resolve([]);
            evaluationPlansExpired = evaluationPlansExpired.filter((obj, index) => evaluationPlansExpired.findIndex(item => item.id === obj.id) === index);
            evaluationPlansExpired.forEach(planExpired => {
                const { vcc, id } = planExpired;
                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, id));
            });
            return Promise.all(promiseArray);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[EvaluationPlansImplementation::Execute] executionCode:', currentDate, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
}
exports.EvaluationPlansImplementation = EvaluationPlansImplementation;
