"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetQueryIdathaProcessV2 = exports.SemanticQueryImplementation = void 0;
const log4js = require("log4js");
const idatha_model_1 = require("../models/idatha.model");
const moment = require("moment");
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const config_2 = require("../config/config");
const file_manager_1 = require("../shared/file-manager");
const billing_implementation_1 = require("../implementation/billing.implementation");
const idatha_service_1 = require("../services/idatha.service");
const request_shared_1 = require("../shared/request.shared");
const billing_batch_model_1 = require("../models/billing-batch.model");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'SEMANTIC_ANALYSIS';
const CURRENT_IMPLEMENTATION = 'SemanticQueryImplementation';
class SemanticQueryImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_SEMANTIC_QUERY;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return redis_service_1.Redis.LRange('SemanticQuery', 0, ratio - 1);
        })
            .then((result) => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            result.map(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[4].toLowerCase())) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.SemanticQueryExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SemanticQuery', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SemanticQueryExecute(executionCode, availableBatch, cancelledBatch) {
        let currentQueueId = '';
        let query = {};
        let process = {};
        let queryId = '';
        let genericQueryId = '';
        let type = '';
        let typeId = '';
        let templateId = '';
        let currentUserAccessToken = {};
        let isError = false;
        let isComplete = false;
        let isCancel = false;
        let queueProgressStorage = '';
        let retry;
        let queryFilePath = '';
        return redis_service_1.Redis.RemoveQueueL('SemanticQuery')
            .then((result) => {
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Creating queues');
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current queue: ', result);
            let arrQueue = result.split(':');
            let promiseArray = [];
            if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[4].toLowerCase())) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQueryExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                logger.warn('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SEMANTIC_QUERY_ITEM_NOT_ALLOWED');
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current item was not found on allowed Batches: ', availableBatch);
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueR('SemanticQuery', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[4], 'SEMANTIC_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            let queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item && item.length) {
                    if (cancelledBatch.find(b => b.VCC === item[1] && b.BatchTypeId.toLowerCase() === item[4].toLowerCase())) {
                        logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' REMOVING CANCELLED BATCH');
                        throw 'REMOVED_BY_CANCELLED';
                    }
                    genericQueryId = item[2];
                    type = item[3] ? item[3] : null;
                    typeId = item[4] ? item[4] : null;
                    if (type && typeId)
                        queueProgressStorage = item[0] + ':' + item[1] + ':' + item[3] + ':' + item[4];
                    if (type && !typeId)
                        queueProgressStorage = item[0] + ':' + item[1] + ':' + item[3];
                    logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return mssql_service_1.MSSql.GetProcessTask(item[0], item[1]);
                }
                else {
                    logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        }).then((result) => {
            process = result;
            if (!process)
                throw 'PROCESS_NOT_FOUND';
            return redis_service_1.Redis.GetRetry(currentQueueId);
        }).then((result) => {
            retry = result && result.value ? result.value : 0;
            if (config_1.RETRY_SEMANTIC_QUERY <= retry)
                throw 'REMOVED_BY_RETRIES';
            logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Current retry :', retry);
            return mssql_service_1.MSSql.GetGenericQuery(process.vcc, genericQueryId);
        }).then((result) => {
            query = result;
            if (!query || (query && !query.query))
                throw 'QUERY_IS_NOT_FOUND';
            let key = process.vcc + ':' + process.vcc;
            return idatha_service_1.IdathaLogin(process.vcc);
        }).then((userAccessToken) => {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            currentUserAccessToken = userAccessToken;
            return this.GetIdathaQueryId(currentUserAccessToken.accessToken, process, executionCode, currentQueueId);
        }).then((resultQueryId) => {
            queryId = resultQueryId;
            if (!queryId)
                throw 'NOT_FOUND_QUERY_ID';
            if (['study', 'template'].includes(type) && typeId)
                return mssql_service_1.MSSql.GetStudy(process.vcc, typeId);
            return Promise.resolve(null);
        }).then((result) => {
            if (result && ['study', 'template'].includes(type))
                templateId = result.templateId;
            queryFilePath = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
            return file_manager_1.DeleteFile(queryFilePath, true);
        }).then((result) => {
            return this.GetIdathaQuery(currentUserAccessToken.accessToken, process.vcc, queryId, query.name, query.query, 0, 25, executionCode, currentQueueId, query.version, process);
        }).then((result) => {
            if (result === null)
                throw 'IDATHA_REQUEST_FAILED';
            if (typeof result === 'boolean' && result === false)
                throw 'NOT_FOUND_DATA';
            return this.GetDocumentQuery(executionCode, currentQueueId);
        }).then((results) => {
            if (['study', 'template'].includes(type) && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_STUDY');
                return mssql_service_1.MSSql.BulkInteractionQueryStudyTemp(process.vcc, process.name, typeId, templateId, genericQueryId, results[query.name]);
            }
            else if (type === 'evaluationPlan' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_EVALUATION_PLAN_CONTENT_SEARCH');
                return mssql_service_1.MSSql.BulkEvaluationPlanQueryTemp(process.vcc, process.name, typeId, genericQueryId, results[query.name]);
            }
            else if (type === 'ability' && typeId) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_ABILITY');
                return mssql_service_1.MSSql.BulkEvaluationPlanAbilityTemp(process.vcc, process.name, typeId, genericQueryId, results[query.name]);
            }
            else if (process && type === 'search') {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_AUDIO');
                return mssql_service_1.MSSql.BulkInteractionQueryTemp(process.vcc, process.name, genericQueryId, results[query.name]);
            }
            else if (results && results[query.name]) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BULK_NONE');
                return Promise.resolve(true);
            }
            return Promise.resolve(null);
        }).then((result) => {
            isComplete = result ? true : false;
            if (!isComplete) {
                retry++;
                return redis_service_1.Redis.UpdateRetry(currentQueueId);
            }
            return Promise.resolve(isComplete);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (!this.errorData.errorFunction)
                this.errorData.errorFunction = 'SemanticQueryExecute';
            if (!this.errorData.errorMessage)
                this.errorData.errorMessage = errorMessage ? errorMessage : 'ERROR';
            if (!this.errorData.errorDetail)
                this.errorData.errorDetail = err;
            if ([
                'PROCESS_NOT_FOUND',
                'QUERY_IS_NOT_FOUND',
                'NOT_FOUND_ACCESS_TOKEN',
                'NOT_FOUND_QUERY_ID',
                'PROCESS_FILTER_NOT_FOUND',
                'REMOVED_BY_CANCELLED',
                'REMOVED_BY_RETRIES'
            ].includes(errorMessage)) {
                isError = true;
            }
            else {
                isComplete = false;
                retry++;
                return redis_service_1.Redis.UpdateRetry(currentQueueId);
            }
            return Promise.resolve(false);
        }).then((result) => {
            if (isComplete) {
                logger.info('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                this.CloseSemanticQuery(currentQueueId, true, 'SUCCED');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else if (isError) {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_ERRORS_QUEUE_FINISHED');
                this.CloseSemanticQuery(currentQueueId, false, 'REMOVED_BY_ERRORS_QUEUE_FINISHED');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else if (config_1.RETRY_SEMANTIC_QUERY <= retry) {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRY');
                this.CloseSemanticQuery(currentQueueId, false, 'REMOVED_BY_RETRY');
                return redis_service_1.Redis.DeleteQueue('SemanticQueryExecute', currentQueueId);
            }
            else {
                logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(null);
        }).then((result) => {
            let promiseArray = [];
            if (isComplete) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteRetry(currentQueueId));
            }
            else if (isError
                ||
                    config_1.RETRY_SEMANTIC_QUERY <= retry) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteRetry(currentQueueId));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(process.vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, null, query.name, this.errorData.errorFunction, this.errorData.errorMessage, this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::SemanticQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetIdathaQueryId(accessToken, process, executionCode, currentQueueId) {
        let queryId = '';
        return promises_shared_1.DummyPromise()
            .then(result => {
            let currentDate = new Date();
            let queryDataIdDate = new Date(process.queryDataIdDate);
            let diffDays = Math.round(Math.abs((currentDate.getTime() - queryDataIdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (!process.queryDataId || diffDays >= config_1.IDATHA_QUERY_ID_EXPIRATION) {
                if (!process.processFilters)
                    throw 'PROCESS_FILTER_NOT_FOUND';
                let queryProcess = GetQueryIdathaProcessV2(process.processFilters);
                logger.debug('[SemanticQueryImplementation::GetIdathaQueryId] Result from GetQueryIdathaProcessV2:', queryProcess);
                let options = this.GetIdathaRequestOptions(process.vcc, accessToken, queryId, queryProcess, 0, 1, 2, process.processFilters);
                logger.info('[SemanticQueryImplementation::GetIdathaQueryId] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'option request:', options);
                return request_shared_1.SimplePostRequest(options.body);
            }
            queryId = process.queryDataId;
            return Promise.resolve(null);
        }).then((result) => {
            if (result) {
                queryId = result.query_id;
                return mssql_service_1.MSSql.SaveProcess(process.vcc, process.name, process.description, process.clientId, process.status, process.type, process.startDate, process.endDate, process.minDurationTime, process.maxDurationTime, process.language, process.dictionaryId, process.environment, process.queryId, queryId, moment().utc().toDate(), process.batchScheduleId, process.createdDate, process.createdByUserId, process.lastModifiedDate, process.lastModifiedByUserId);
            }
            return Promise.resolve(null);
        }).then((result) => {
            this.errorData.CleanErrorData();
            return Promise.resolve(queryId);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticQueryImplementation::GetIdathaQueryId] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('GetIdathaQueryId', errorMessage, err);
            return Promise.resolve(null);
        });
    }
    GetIdathaQueryType(queryString) {
        let queryObject = null;
        let queryType = 'REGULAR';
        try {
            queryObject = JSON.parse(queryString);
            if (typeof queryObject === 'object') {
                if (queryObject.filter.includes('silences:'))
                    queryType = 'SILENCES';
                if (queryObject.filter.includes('fillers'))
                    queryType = 'FILLERS';
            }
        }
        catch (e) {
            logger.info('[SemanticQueryImplementation::GetIdathaQueryType] Could not parse query:', queryString, ', is not silences query');
        }
        return {
            queryType,
            queryString,
            queryObject
        };
    }
    GetIdathaRequestOptions(vcc, accessToken, queryId, queryString, from, size, version = 1, processFilters) {
        let idatha = new idatha_model_1.IdathaConnection();
        let uri = null;
        let path = null;
        let headers = { 'Authorization': null, 'Content-Type': 'application/json' };
        let body = {};
        let bodyParsed = null;
        let idathaQueryType = this.GetIdathaQueryType(queryString);
        let queryType = idathaQueryType.queryType;
        let queryObject = idathaQueryType.queryObject;
        switch (queryType) {
            case 'SILENCES':
                headers['Authorization'] = 'Basic ZWxhc3RpYzpFbGFzdGlj';
                path = `${vcc}_entries/_search`;
                idatha.httpPort = '9200';
                let must = [];
                let silencesRange = { boost: 1.0 };
                let speaker = null;
                let startDate = processFilters?.find(pf => pf.key === 'startDate')?.value || '2021-06-01';
                let endDate = processFilters?.find(pf => pf.key === 'endDate')?.value || '2050-12-31';
                startDate = startDate.slice(0, 10);
                endDate = endDate.slice(0, 10);
                if (queryObject?.minType?.length) {
                    silencesRange[queryObject.minType.toLowerCase()] = queryObject.minValue;
                    silencesRange['include_lower'] = queryObject.minType.includes('e') ? true : false;
                }
                if (queryObject?.maxType?.length) {
                    silencesRange[queryObject.maxType.toLowerCase()] = queryObject.maxValue;
                    silencesRange['include_upper'] = queryObject.maxType.includes('e') ? true : false;
                }
                must.push({ range: { "words.silence": silencesRange } });
                if (queryObject.scope && queryObject.scope !== 'TEXT_ALL') {
                    if (queryObject.scope === 'in_sentences.')
                        speaker = 'in';
                    if (queryObject.scope === 'out_sentences.')
                        speaker = 'out';
                    if (speaker)
                        must.push({ match: { "words.speaker": speaker } });
                }
                bodyParsed = {
                    from: from,
                    size: size,
                    query: {
                        bool: {
                            filter: [
                                {
                                    bool: {
                                        must: [
                                            {
                                                nested: {
                                                    query: {
                                                        bool: {
                                                            must: must
                                                        }
                                                    },
                                                    path: "words",
                                                    ignore_unmapped: false,
                                                    score_mode: "none",
                                                    boost: 1.0,
                                                    inner_hits: {
                                                        ignore_unmapped: false,
                                                        from: 0,
                                                        size: 100,
                                                        version: false,
                                                        explain: false,
                                                        track_scores: false,
                                                        _source: {
                                                            includes: ["words.silence", "words.name", "words.speaker", "words.time"],
                                                            excludes: []
                                                        }
                                                    }
                                                }
                                            },
                                            {
                                                nested: {
                                                    query: {
                                                        bool: {
                                                            must: [
                                                                {
                                                                    range: {
                                                                        "filter_data.end_date": {
                                                                            gte: `${startDate}T00:00:00`,
                                                                            lte: `${endDate}T23:59:59`,
                                                                            include_lower: true,
                                                                            include_upper: true,
                                                                            boost: 1.0
                                                                        }
                                                                    }
                                                                }
                                                            ]
                                                        }
                                                    },
                                                    path: "filter_data",
                                                    ignore_unmapped: false,
                                                    score_mode: "none",
                                                    boost: 1.0,
                                                    inner_hits: {
                                                        ignore_unmapped: false,
                                                        from: 0,
                                                        size: 100,
                                                        version: false,
                                                        explain: false,
                                                        track_scores: false,
                                                        _source: {
                                                            includes: ["words.silence", "words.name", "words.speaker", "words.time"],
                                                            excludes: []
                                                        }
                                                    }
                                                }
                                            }
                                        ],
                                        adjust_pure_negative: true,
                                        boost: 1.0
                                    }
                                }
                            ],
                            adjust_pure_negative: true,
                            boost: 1.0
                        }
                    }
                };
                break;
            case 'FILLERS':
                headers['Authorization'] = 'Basic ZWxhc3RpYzpFbGFzdGlj';
                path = `${vcc}_entries/_search`;
                idatha.httpPort = '9200';
                let searchText = queryObject.value;
                let fromText = from;
                let sizeText = size;
                let scope = queryObject.scope;
                let stringQuery = '';
                switch (scope) {
                    case '':
                        let stringQuery_1 = '{"from":' + fromText + ',"size":' + sizeText + ',"query": {"bool": {"filter": [{"bool": {"should": [{"nested": {"query": {"bool": {"must": [{"match_phrase": {"in_sentences.text":';
                        let stringQuery_2 = '"' + searchText + '"}}]}},"path": "in_sentences","ignore_unmapped": false,"score_mode": "none","boost": 1.0}},{"nested": {"query": {"bool": {"must": [{"match_phrase": {"out_sentences.text":';
                        let stringQuery_3 = '"' + searchText + '"}}]}},"path": "out_sentences","ignore_unmapped": false,"score_mode": "none","boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}},"script_fields": {"ocurrencies_in": {"script": {"lang": "painless","source": "int count = 0; if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        let stringQuery_4 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},"ocurrencies_out": {"script": {"lang": "painless","source": "int count = 0; if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        let stringQuery_5 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},';
                        let stringQuery_6 = '"phrases_ocurrencies_in": {"script": {"lang": "painless","source": "def arr = []; if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        let stringQuery_7 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.in_sentences[i].text)}}} return arr;"}},';
                        let stringQuery_8 = '"phrases_ocurrencies_out": {"script": {"lang": "painless","source": "def arr = []; if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        let stringQuery_9 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.out_sentences[i].text)}}} return arr;"}},';
                        let stringQuery_10 = '"filter_data":{"script": {"source": "params._source.filter_data"}},"words":{"script":{"source": "params._source.words"}}}}';
                        stringQuery = stringQuery_1 + stringQuery_2 + stringQuery_3 + stringQuery_4 + stringQuery_5 + stringQuery_6 + stringQuery_7 + stringQuery_8 + stringQuery_9 + stringQuery_10;
                        break;
                    case 'in_sentences.':
                        let stringQueryIn_1 = '{"from": ' + fromText + ',"size":' + sizeText + ',"query": {"bool": {"filter": [{"bool": { "must": [ { "nested": { "query": { "bool": { "must": [ {"match_phrase":';
                        let stringQueryIn_2 = '{"in_sentences.text": "' + searchText + '"}}]}}, "path": "in_sentences", "ignore_unmapped": false, "score_mode": "none", "boost": 1.0}}], "adjust_pure_negative": true, "boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}},"script_fields": { "ocurrencies_in": {"script": {"lang": "painless","source": "int count = 0;if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        let stringQueryIn_3 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},';
                        let stringQueryIn_4 = '"phrases_ocurrencies_in": {"script": {"lang": "painless","source": "def arr = []; if(params._source.in_sentences != null) {for(int i=0; i<params._source.in_sentences.length; i++){String tmp = params._source.in_sentences[i].text;';
                        let stringQueryIn_5 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.in_sentences[i].text)}}} return arr;"}},';
                        let stringQueryIn_6 = '"filter_data":{ "script": {"source": "params._source.filter_data"}},"words":{"script":{"source": "params._source.words"}}}}';
                        stringQuery = stringQueryIn_1 + stringQueryIn_2 + stringQueryIn_3 + stringQueryIn_4 + stringQueryIn_5 + stringQueryIn_6;
                        break;
                    case 'out_sentences.':
                        let stringQueryOut_1 = '{"from": ' + fromText + ',"size":' + sizeText + ',"query": {"bool": {"filter": [{"bool": {"must": [{"nested": {"query": {"bool": {"must": [ {"match_phrase":';
                        let stringQueryOut_2 = '{"out_sentences.text": "' + searchText + '"}}]}}, "path": "out_sentences", "ignore_unmapped": false, "score_mode": "none", "boost": 1.0}}], "adjust_pure_negative": true, "boost": 1.0}}],"adjust_pure_negative": true,"boost": 1.0}},"script_fields": { "ocurrencies_out": {"script": {"lang": "painless","source": "int count = 0;if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        let stringQueryOut_3 = 'Matcher m = /(?i)\\\\b' + searchText + '\\\\b/.matcher(tmp); while (m.find()) { count++;}}} return count;"}},';
                        let stringQueryOut_4 = '"phrases_ocurrencies_out": {"script": {"lang": "painless","source": "def arr = []; if(params._source.out_sentences != null) {for(int i=0; i<params._source.out_sentences.length; i++){String tmp = params._source.out_sentences[i].text;';
                        let stringQueryOut_5 = 'Matcher m = /\\\\b' + searchText + '\\\\b/i.matcher(tmp); if(m.find()) { arr.add(params._source.out_sentences[i].text)}}} return arr;"}},';
                        let stringQueryOut_6 = '"filter_data":{ "script": {"source": "params._source.filter_data"}},"words":{"script":{"source": "params._source.words"}}}}';
                        stringQuery = stringQueryOut_1 + stringQueryOut_2 + stringQueryOut_3 + stringQueryOut_4 + stringQueryOut_5 + stringQueryOut_6;
                        break;
                }
                bodyParsed = JSON.parse(stringQuery);
                break;
            default:
                headers['Authorization'] = `Bearer ${accessToken}`;
                path = 'api/query';
                if (version === 2) {
                    let dataObject = {
                        version,
                        from,
                        size,
                        query: queryString
                    };
                    if (queryId)
                        dataObject.query_id = queryId;
                    let data = JSON.stringify(dataObject);
                    bodyParsed = JSON.parse(data.replace(/\\\\\\/g, "\\"));
                }
                else {
                    let stringQuery = '{"query_id": "' + queryId + '", "query": "' + queryString.replace(/\"/g, "\\\\\\\\\\\\\"") + '", "from": ' + from + ', "size": ' + size + ',"version":' + (version ? version : 1) + '}';
                    bodyParsed = JSON.parse(stringQuery);
                }
                break;
        }
        uri = `http://${idatha.ip}:${idatha.httpPort}/`;
        uri += path;
        body = {
            uri: uri,
            timeout: 60000,
            body: bodyParsed,
            headers,
            retryProtocol: false
        };
        return {
            queryType,
            body
        };
    }
    GetIdathaRequestResponse(queryType, queryString, rawResponse) {
        let response = {};
        switch (queryType) {
            case 'SILENCES':
                let queryObject = JSON.parse(queryString);
                response.total_results = rawResponse.hits.total.value;
                response.results = [];
                rawResponse.hits.hits.map(h => {
                    h._source.matches = h.inner_hits.words.hits.total.value;
                    h._source.matchesDetail = [];
                    h.inner_hits.words.hits.hits.map(hh => {
                        if (hh._source.words)
                            h._source.matchesDetail.push(hh._source.words);
                        else
                            h._source.matchesDetail.push(hh._source);
                    });
                    if (h._source.matches >= queryObject.minOcurrencies)
                        response.results.push(h._source);
                });
                break;
            case 'FILLERS':
                let queryFillersObject = JSON.parse(queryString);
                response.total_results = rawResponse.hits.total.value;
                response.results = [];
                rawResponse.hits.hits.map(h => {
                    let totalOcurrencies = 0;
                    let _source = {};
                    let scope = queryFillersObject.scope;
                    switch (scope) {
                        case '':
                            totalOcurrencies = h.fields.ocurrencies_out[0] + h.fields.ocurrencies_in[0];
                            break;
                        case 'in_sentences.':
                            totalOcurrencies = h.fields.ocurrencies_in[0];
                            break;
                        case 'out_sentences.':
                            totalOcurrencies = h.fields.ocurrencies_out[0];
                            break;
                    }
                    if (totalOcurrencies >= queryFillersObject.minOcurrencies) {
                        h._source = _source;
                        switch (scope) {
                            case '':
                                logger.debug("entraqui con ", totalOcurrencies);
                                logger.debug("entraqui con2 ", h.fields.phrases_ocurrencies_in);
                                h._source.matches = totalOcurrencies;
                                h._source.matchesDetail = [];
                                h.fields.phrases_ocurrencies_in?.map(hh => {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                h.fields.phrases_ocurrencies_out?.map(hh => {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                break;
                            case 'in_sentences.':
                                h._source.matches = totalOcurrencies;
                                h._source.matchesDetail = [];
                                h.fields.phrases_ocurrencies_in?.map(hh => {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                break;
                            case 'out_sentences.':
                                h._source.matches = totalOcurrencies;
                                h._source.matchesDetail = [];
                                h.fields.phrases_ocurrencies_out?.map(hh => {
                                    if (hh)
                                        h._source.matchesDetail.push(hh);
                                });
                                break;
                        }
                        h._source.id = h._id;
                        h._source.words = h.fields.words;
                        h._source.filter_data = h.fields.filter_data[0];
                        delete (h.fields);
                        response.results.push(h._source);
                    }
                });
                break;
            default:
                response = rawResponse;
                break;
        }
        return response;
    }
    GetIdathaQuery(accessToken, vcc, queryId, queryName, query, from, size, executionCode, currentQueueId, version = 1, process) {
        let isNext = true;
        let queryType = null;
        let queryResultTotal = 0;
        let queryResultPages = 0;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            let options = this.GetIdathaRequestOptions(vcc, accessToken, queryId, query, from, size, version, process.processFilters);
            queryType = options.queryType;
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'option request:', options);
            return request_shared_1.SimplePostRequest(options.body);
        })
            .then((result) => {
            let response = this.GetIdathaRequestResponse(queryType, query, result);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Items per page: ' + size);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Total results: ' + response.total_results);
            logger.info('[SemanticQueryImplementation::GetIdathaRegularQuery] Query: "' + query + '", Results for this request: ' + response.results.length);
            isNext = from + size <= (response.total_results * 1) ? true : false;
            queryResultTotal = response.total_results;
            queryResultPages = response.total_results > 0 ? Math.ceil(response.total_results / size) : 1;
            response.results = this.ReduceRequestResult(response, query);
            return this.SaveDocumentQuery(response.results, queryName, executionCode, currentQueueId);
        }).then((result) => {
            if (!result)
                throw 'ERROR_GET_IDATHA_INTERACTIONS';
            if (isNext)
                return this.GetIdathaQuery(accessToken, vcc, queryId, queryName, query, (from + size), size, executionCode, currentQueueId, version, process);
            return true;
        }).then((result) => {
            this.SaveSemanticQueryResult(currentQueueId, queryResultTotal, queryResultPages, size);
            this.errorData.CleanErrorData();
            return Promise.resolve(result);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.debug('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting rawError:', err);
            logger.error('[SemanticQueryImplementation::GetIdathaRegularQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('GetIdathaRegularQuery', errorMessage, err);
            if (err && err.statusCode && err.statusCode === 500) {
                return Promise.resolve(null);
            }
            else {
                return Promise.resolve(false);
            }
        });
    }
    ReduceRequestResult(response, query) {
        let pattern = /\in_sentences|\out_sentences/g;
        if (query.match(pattern)) {
            let orPattern1 = /((?!out_sentences|in_sentences)\w+\s(OR)\s.*(in_sentences|out_sentences)[^\s]+)/g;
            let orPattern2 = /((out_sentences|in_sentences)[^\s]+\s(OR)\s((?!out_sentences|in_sentences)\w+[^\s]))/g;
            if (query.match(orPattern1) || query.match(orPattern2)) {
            }
            else {
                response.results = response.results.filter(r => r && r.filter_data && r.filter_data.channel && r.filter_data.channel !== 'MAIL');
            }
        }
        if (response?.results?.length) {
            response.results.forEach((r, i) => {
                if (r) {
                    r.char = r.words.map(w => w.name.length).reduce((acc, curr) => acc + curr + 1, 0);
                    let inWords = r.words.filter(w => w.speaker === 'in');
                    let outWords = r.words.filter(w => w.speaker === 'out');
                    r.lenWordsIn = inWords.length || 0;
                    r.lenWordsOut = outWords.length || 0;
                    r.callDurationIn = 0;
                    r.callSilenceIn = 0;
                    r.callConfidenceIn = 0;
                    r.callDurationOut = 0;
                    r.callSilenceOut = 0;
                    r.callConfidenceOut = 0;
                    r.otherDurationIn = 0;
                    r.otherSilenceIn = 0;
                    r.otherConfidenceIn = 0;
                    r.otherDurationOut = 0;
                    r.otherSilenceOut = 0;
                    r.otherConfidenceOut = 0;
                    r.totalConfidenceIn = 0;
                    r.totalConfidenceOut = 0;
                    if (r.filter_data.channel === 'CALL') {
                        r.callDurationIn = inWords.map(iw => iw.duration).reduce((acc, curr) => acc + curr, 0);
                        r.callSilenceIn = inWords.filter(iw => iw.silence > 0).map(iw => iw.silence).reduce((acc, curr) => acc + curr, 0);
                        r.callConfidenceIn = inWords.map(iw => iw.confidence).reduce((acc, curr) => acc + curr, 0);
                        r.callDurationOut = outWords.map(ow => ow.duration).reduce((acc, curr) => acc + curr, 0);
                        r.callSilenceOut = outWords.filter(ow => ow.silence > 0).map(ow => ow.silence).reduce((acc, curr) => acc + curr, 0);
                        r.callConfidenceOut = outWords.map(ow => ow.confidence).reduce((acc, curr) => acc + curr, 0);
                    }
                    else {
                        r.otherDurationIn = inWords.map(iw => iw.duration).reduce((acc, curr) => acc + curr, 0);
                        r.otherSilenceIn = inWords.filter(iw => iw.silence > 0).map(iw => iw.silence).reduce((acc, curr) => acc + curr, 0);
                        r.otherConfidenceIn = inWords.map(iw => iw.confidence).reduce((acc, curr) => acc + curr, 0);
                        r.otherDurationOut = outWords.map(ow => ow.duration).reduce((acc, curr) => acc + curr, 0);
                        r.otherSilenceOut = outWords.filter(ow => ow.silence > 0).map(ow => ow.silence).reduce((acc, curr) => acc + curr, 0);
                        r.otherConfidenceOut = outWords.map(ow => ow.confidence).reduce((acc, curr) => acc + curr, 0);
                    }
                    if (r.lenWordsIn)
                        r.totalConfidenceIn = ((r.callConfidenceIn + r.otherConfidenceIn) / r.lenWordsIn);
                    if (r.lenWordsOut)
                        r.totalConfidenceOut = ((r.callConfidenceOut + r.otherConfidenceOut) / r.lenWordsOut);
                    if (r.words)
                        delete (r.words);
                    if (r.in_sentences)
                        delete (r.in_sentences);
                    if (r.out_sentences)
                        delete (r.out_sentences);
                }
            });
        }
        return response.results;
    }
    SaveDocumentQuery(data, id, executionCode, currentQueueId) {
        let fileName = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
        return promises_shared_1.DummyPromise()
            .then(() => {
            return file_manager_1.CreateFolder(config_2.QUERY_FOLDER);
        }).then(() => {
            return file_manager_1.ReadFileContent(fileName, 'utf-8', true);
        }).then((resultFile) => {
            let jsonContent;
            let results = data ? data : [];
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                    jsonContent[id] = jsonContent[id] ? jsonContent[id].concat(results) : [];
                }
                else {
                    jsonContent = {};
                    jsonContent[id] = results;
                }
            }
            catch (ex) {
                logger.error('[SemanticQueryImplementation::SaveDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', ex);
                throw 'ERROR_FILE_JSON';
            }
            return file_manager_1.WriteFileContent(fileName, JSON.stringify(jsonContent), 'utf-8', true);
        }).then((result) => {
            this.errorData.CleanErrorData();
            return Promise.resolve(true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticQueryImplementation::SaveDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('SaveDocumentQuery', errorMessage, error);
            return Promise.resolve(false);
        });
    }
    GetDocumentQuery(executionCode, currentQueueId) {
        let fileName = config_2.QUERY_FOLDER + '/' + currentQueueId.replace(/:/g, '=') + '.json';
        return promises_shared_1.DummyPromise()
            .then(() => {
            return file_manager_1.ReadFileContent(fileName, 'utf-8', true);
        }).then((resultFile) => {
            let jsonContent;
            try {
                jsonContent = JSON.parse(resultFile);
            }
            catch (ex) {
                jsonContent = null;
                throw 'ERROR_FILE_JSON';
            }
            this.errorData.CleanErrorData();
            return Promise.resolve(jsonContent);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticQueryImplementation::GetDocumentQuery] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('GetDocumentQuery', errorMessage, error);
            return Promise.resolve(null);
        });
    }
    SaveSemanticQueryResult(queueId, totalResults, totalPages, totalItemsPerPage) {
        let billing = new billing_implementation_1.BillingImplementation();
        let myBatchName = '';
        let myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[3]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'evaluationPlan':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'search':
                myBatchName = '';
                break;
        }
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalResults', totalResults);
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalPages', totalPages);
        billing.AddUpdateSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], 'TotalItemsPerPage', totalItemsPerPage);
    }
    CloseSemanticQuery(queueId, resultCode, resultDescription) {
        let billing = new billing_implementation_1.BillingImplementation();
        let myBatchName = '';
        let myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[3]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'evaluationPlan':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'search':
                myBatchName = '';
                break;
        }
        billing.AddCloseSemanticAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[2], myBatchName, myQueueIdArray[4], resultCode ? 1 : 0, resultDescription);
    }
}
exports.SemanticQueryImplementation = SemanticQueryImplementation;
function GetQueryIdathaProcessV2(processFilters) {
    let queries = [];
    let campaigns = [];
    let agents = [];
    let channels = [];
    let startDate = '';
    let endDate = '';
    for (let p of processFilters) {
        if (p.value) {
            if (p.key == 'agents') {
                agents.push(`filter_data:(agent:"${p.value}")`);
            }
            if (p.key == 'endDate') {
                endDate = p.value;
            }
            if (p.key == 'startDate') {
                startDate = p.value;
            }
            if (p.key == 'types') {
                channels.push(`filter_data:(channel:"${p.value}")`);
            }
        }
    }
    if (startDate != '' && endDate != '') {
        startDate = startDate.slice(0, 10);
        endDate = endDate.slice(0, 10);
        queries.push(`filter_data:(end_date:[${startDate} TO ${endDate}])`);
    }
    if (agents.length > 0) {
        queries.push(`(${agents.join(' OR ')})`);
    }
    if (campaigns.length > 0) {
        queries.push(`(${campaigns.join(' OR ')})`);
    }
    if (channels.length > 0) {
        queries.push(`(${channels.join(' OR ')})`);
    }
    return queries.join(' AND ');
}
exports.GetQueryIdathaProcessV2 = GetQueryIdathaProcessV2;
