"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SupervisorImplementation = void 0;
const log4js = require("log4js");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const billing_implementation_1 = require("./billing.implementation");
const billing_batch_implementation_1 = require("./billing-batch.implementation");
const idatha_service_1 = require("../services/idatha.service");
const platform_service_1 = require("../services/platform.service");
let logger = log4js.getLogger('Task');
class SupervisorImplementation {
    Execute(executionCode) {
        let queueItems = ['Process', 'I6Update', 'Audios', 'DiarizationConvert', 'TranscriptionsUpload', 'TranscriptionsStandby', 'TranscriptionsDownload', 'DiarizationAnalysis', 'SentimentAnalysis', 'SemanticAnalysisUpload', 'SemanticAnalysisDownload', 'SemanticAnalysisConvertAudio', 'SemanticQuery', 'CognitiveAnalysis', 'GenerativeAnalysis', 'TipificationAnalysis', 'BatchScheduleList'];
        let processTail = queueItems[0];
        let executeIndex = 0;
        let ratio = config_1.RATIO_SUPERVISOR;
        let executeResult = true;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            billing.SplitGeneralFileToSegmentedFiles();
            promiseArray.push(billing.SegmentedFilesUpload(result.installationId));
            if (!result.isRegistered)
                promiseArray.push(billing.SavePlatformVersion());
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return mssql_service_1.MSSql.GetFinishedBatch();
        })
            .then(result => {
            if (result && result.length > 0) {
                let promiseArray = [];
                let billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
                result.map(b => {
                    promiseArray.push(billingBatch.CloseBatchSummary(b));
                });
                return Promise.all(promiseArray);
            }
            else {
                return Promise.resolve([true]);
            }
        })
            .then(result => {
            ratio = queueItems.length;
            let promiseArray = [];
            while (ratio > executeIndex) {
                promiseArray.push(this.RestoreQueue(queueItems[executeIndex], executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return Promise.resolve(executeResult);
        }).then((result) => {
            return redis_service_1.Redis.LRange(processTail, 0, -1);
        })
            .then((result) => {
            let vccs = result.map((x) => {
                return x.split(':')[0];
            });
            let uniqueVCCs = this.UniqueVCC(vccs);
            ratio = uniqueVCCs.length;
            executeIndex = 0;
            let promiseArray = [];
            while (ratio > executeIndex) {
                promiseArray.push(this.IdathaLogin(uniqueVCCs[executeIndex], executionCode));
                promiseArray.push(this.AzureSuscription(uniqueVCCs[executeIndex], executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return Promise.resolve(executeResult);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    RestoreQueue(queueId, executionCode) {
        let source = queueId;
        let executionTail = queueId;
        executionTail += 'Execute';
        let executeResult = true;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.LRange(executionTail, 0, -1);
        })
            .then((result) => {
            if (result?.length === 0)
                throw 'SERVER_INFO_REDIS_KEYS_NOT_FOUND';
            logger.info('[SupervisorImplementation::RestoreQueue] executionCode:', executionCode, `getting result for ${queueId}: ${result.length}`);
            let lenItems = result && result.length;
            let executeIndex = 1;
            let promiseArray = [];
            while (lenItems >= executeIndex) {
                promiseArray.push(redis_service_1.Redis.RPopLPush(executionTail, source));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return Promise.resolve(executeResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info(`[SupervisorImplementation::RestoreQueue] executionCode: ${executionCode}, Queue: ${source}, getting info:`, errorMessage);
            else
                logger.error(`[SupervisorImplementation::RestoreQueue] executionCode: ${executionCode}, Queue: ${source}, getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    IdathaLogin(vcc, executionCode) {
        logger.info('[SupervisorImplementation::IdathaLogin] executionCode: ', executionCode, 'vcc: ', vcc);
        let userAcessToken = {};
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return idatha_service_1.IdathaLogin(vcc);
        }).then(result => {
            return Promise.resolve(result ? true : false);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SupervisorImplementation::IdathaLogin] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    }
    AzureSuscription(vcc, executionCode) {
        let isNew = false;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return redis_service_1.Redis.GetSubscriptionKey(vcc);
        }).then((result) => {
            if (!result) {
                isNew = true;
                return mssql_service_1.MSSql.GetSubscriptionKey(vcc);
            }
            return result;
        }).then((result) => {
            if (result && isNew) {
                return redis_service_1.Redis.SaveSubscriptionKey(vcc, result);
            }
            else if (result) {
                return true;
            }
            return false;
        }).then(result => {
            return Promise.resolve(result ? true : false);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SupervisorImplementation::AzureSuscription] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    }
    UniqueVCC(vccs) {
        let result = [];
        if (vccs && vccs.length) {
            result = vccs.filter((vcc, pos) => {
                return vccs.indexOf(vcc) === pos;
            });
        }
        return result;
    }
}
exports.SupervisorImplementation = SupervisorImplementation;
