"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailDisconect = exports.SendAttach = exports.Send = exports.PrepareEmailTemplate = exports.SmtpService = void 0;
const log4js = require("log4js");
const mail_config_1 = require("../config/mail.config");
const server_config_1 = require("../config/server.config");
const file_manager_1 = require("../shared/file-manager");
const templates_config_1 = require("../config/templates.config");
const redis_service_1 = require("../db/redis.service");
const mssql_service_1 = require("../db/mssql.service");
let logger = log4js.getLogger('ServerScripts');
let hbs = require('hbs');
let mailServer;
function SmtpService(app) {
    mailServer = app.get('mailserver');
}
exports.SmtpService = SmtpService;
function PrepareEmailTemplate(instance, userName, description, title, template) {
    let assetsUrl;
    return GetMailAssetsDomain(instance)
        .then((result) => {
        assetsUrl = result;
        return file_manager_1.ReadFileContent(file_manager_1.JoinPath(templates_config_1.TEMPLATE_MAIL_PATH, template + '.hbs'), 'utf8');
    }).then(content => {
        let theme = hbs.compile(content);
        let resultContent = '';
        switch (template) {
            case 'notification':
                resultContent = theme({
                    'userName': userName,
                    'description': description,
                    'title': title,
                    'assetsUrl': assetsUrl
                });
                break;
            default:
                resultContent = '';
                break;
        }
        return resultContent;
    }).catch(err => {
        logger.error('PrepareEmailTemplate: ' + err);
    });
}
exports.PrepareEmailTemplate = PrepareEmailTemplate;
function Send(receiver, subject, content) {
    var mailOptions = {
        from: mail_config_1.MAILSERVER_CONFIG.from,
        to: receiver,
        subject: subject,
        text: content,
        html: content
    };
    return new Promise((resolve, reject) => {
        mailServer.sendMail(mailOptions, function (error, info) {
            if (error) {
                console.log('Error al intentar enviar por MailService: \'' + error + '\'');
                logger.error('Error al intentar enviar por MailService: \'' + error + '\'');
                reject(error);
            }
            else {
                logger.info('Mensaje enviado por MailService: \'' + info.response + '\'');
                resolve(true);
            }
        });
    });
}
exports.Send = Send;
function SendAttach(receiver, subject, content, attachments) {
    var mailOptions = {
        from: mail_config_1.MAILSERVER_CONFIG.from,
        to: receiver,
        subject: subject,
        attachments: attachments,
        text: content,
        html: content
    };
    return new Promise((resolve, reject) => {
        mailServer.sendMail(mailOptions, function (error, info) {
            if (error) {
                console.log('Error al intentar enviar por MailService: \'' + error + '\'');
                logger.error('Error al intentar enviar por MailService: \'' + error + '\'');
                reject(error);
            }
            else {
                logger.info('Mensaje enviado por MailService: \'' + info.response + '\'');
                resolve(true);
            }
        });
    });
}
exports.SendAttach = SendAttach;
function MailDisconect() {
    return new Promise((resolve, reject) => {
        if (mailServer) {
            logger.info('Cerrando conexión a webmail \'' + mail_config_1.MAILSERVER_CONFIG.server + '\'');
            console.log('Cerrando conexión a webmail \'' + mail_config_1.MAILSERVER_CONFIG.server + '\'');
            mailServer.close();
        }
        resolve(true);
    });
}
exports.MailDisconect = MailDisconect;
function GetMailAssetsDomain(instance) {
    let mailAssetsUrl = (server_config_1.HTTPS_ENABLED ? 'https://' : 'http://') + '127.0.0.1' +
        (server_config_1.HTTPS_ENABLED && server_config_1.HTTPS_BINDING_PORT && server_config_1.HTTPS_BINDING_PORT !== 443 ? ':' + server_config_1.HTTPS_BINDING_PORT.toString() :
            (!server_config_1.HTTPS_ENABLED && server_config_1.HTTP_BINDING_PORT && server_config_1.HTTP_BINDING_PORT !== 80 ? ':' + server_config_1.HTTP_BINDING_PORT.toString() : '')) + '/client/assets/img/';
    let cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetMailingDomainByInstance(instance)
        .then((mailingDomain) => {
        if (mailingDomain || JSON.stringify(mailingDomain) === JSON.stringify({})) {
            return Promise.resolve(mailingDomain);
        }
        else {
            logger.info('El dato de GetMailingDomainByInstance para \'' + instance + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetDomains(instance, 'domain', '');
        }
    }).then((resultDomains) => {
        if (typeof resultDomains !== 'string') {
            resultDomains = resultDomains || [];
            let selectedDomain = resultDomains.find(domain => domain.isMainDomain);
            if (!selectedDomain)
                selectedDomain = resultDomains.find(domain => !domain.isContentDomain);
            if (selectedDomain) {
                let protocol = (server_config_1.HTTPS_ENABLED ? 'https://' : 'http://');
                let domain = selectedDomain.domain || '127.0.0.1';
                let port = (((!server_config_1.HTTPS_ENABLED && selectedDomain.contentPort && selectedDomain.contentPort !== '80') || (server_config_1.HTTPS_ENABLED && selectedDomain.contentPort && selectedDomain.contentPort !== '443')) ? ':' + selectedDomain.contentPort : '');
                mailAssetsUrl = protocol + domain + port + '/assets/img/';
            }
        }
        else {
            mailAssetsUrl = resultDomains;
        }
        logger.info('Devuelve la ruta para assets de mailing de la instancia  \'' + instance + ' con valor \'' + mailAssetsUrl + '\'');
        if (cacheUpdateNeeded)
            redis_service_1.Redis.SaveMailingDomainByInstance(instance, mailAssetsUrl);
        if (mailAssetsUrl || JSON.stringify(mailAssetsUrl) === JSON.stringify({})) {
            return Promise.resolve(mailAssetsUrl);
        }
        else {
            throw 'DOES_NOT_EXIST_IN_DATABASE';
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetMailAssetsDomain: ' + errMsg);
    });
}
