"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerativeIntentItem = exports.GenerativeSave = exports.GenerativeCreate = exports.GenerativeGetAppById = exports.GenerativeGetAllService = exports.GenerativeExample = exports.GenerativeIntent = exports.GenerativeExtended = exports.Generative = void 0;
const moment = require("moment");
class Generative {
    constructor() {
        this.id = null;
        this.name = null;
        this.description = null;
        this.language = null;
        this.isPublished = false;
        this.intents = [];
        this.status = null;
    }
}
exports.Generative = Generative;
class GenerativeExtended extends Generative {
    constructor() {
        super(...arguments);
        this.needPublish = false;
        this.lastPublishedDate = null;
        this.createdByUserId = null;
        this.createdDate = null;
        this.lastModifiedByUserId = null;
        this.lastModifiedDate = null;
    }
}
exports.GenerativeExtended = GenerativeExtended;
class GenerativeIntent {
    constructor() {
        this.appId = null;
        this.id = null;
        this.name = null;
        this.description = null;
        this.examples = [];
    }
}
exports.GenerativeIntent = GenerativeIntent;
class GenerativeExample {
    constructor() {
        this.appId = null;
        this.intentId = null;
        this.id = null;
        this.text = null;
    }
}
exports.GenerativeExample = GenerativeExample;
class GenerativeGetAllService {
    constructor() {
        this.id = null;
        this.name = null;
        this.description = null;
        this.language = null;
        this.active = 0;
        this.published = 0;
        this.createdAt = null;
        this.updateAt = null;
    }
    ConvertToGenerative() {
        let generative = new Generative();
        generative.id = this.id;
        generative.name = this.name;
        generative.description = this.description;
        generative.isPublished = this.published ? true : false;
        return generative;
    }
}
exports.GenerativeGetAllService = GenerativeGetAllService;
class GenerativeGetAppById {
    constructor() {
        this.id = null;
        this.name = null;
        this.description = null;
        this.language = null;
        this.published = false;
        this.versionA = null;
        this.versionB = null;
    }
    ConvertToGenerative() {
        let generative = new Generative();
        generative.id = this.id;
        generative.name = this.name;
        generative.description = this.description;
        generative.language = this.language;
        generative.isPublished = this.published ? true : false;
        let currentVersion = null;
        if (!this.versionB)
            currentVersion = this.versionA;
        else {
            let versionACreatedDate = moment(this.versionA.createdAt);
            let versionBCreatedDate = moment(this.versionB.createdAt);
            if (versionACreatedDate.isAfter(versionBCreatedDate))
                currentVersion = this.versionA;
            else
                currentVersion = this.versionB;
        }
        generative.intents = currentVersion.intents.map(i => {
            let intent = new GenerativeIntent();
            intent = Object.assign(intent, i);
            intent.appId = generative.id;
            intent.examples = i.examples.map(e => {
                let example = new GenerativeExample();
                example.appId = generative.id;
                example.intentId = intent.id;
                example.id = e.id;
                example.text = e.name;
                return example;
            });
            return intent;
        });
        return generative;
    }
}
exports.GenerativeGetAppById = GenerativeGetAppById;
class GenerativeCreate {
    constructor(data) {
        this.name = null;
        this.description = null;
        this.language = null;
        this.versionA = null;
        this.name = data.name;
        this.description = data.description;
        this.language = data.language;
        this.versionA = {
            intents: data.intents.map(i => {
                return {
                    name: i.name,
                    description: i.description,
                    examples: i.examples.map(e => {
                        return {
                            name: e.text
                        };
                    })
                };
            })
        };
    }
}
exports.GenerativeCreate = GenerativeCreate;
class GenerativeSave {
    constructor(data) {
        this.intents = [];
        this.intents = data.intents.map(i => {
            return {
                name: i.name,
                description: i.description,
                examples: i.examples.map(e => {
                    return {
                        name: e.text
                    };
                })
            };
        });
    }
}
exports.GenerativeSave = GenerativeSave;
class GenerativeIntentItem {
    constructor() {
        this.appId = null;
        this.appName = null;
        this.culture = null;
        this.id = null;
        this.name = null;
    }
}
exports.GenerativeIntentItem = GenerativeIntentItem;
