"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccountService = void 0;
const log4js = require("log4js");
const promises_shared_1 = require("../shared/promises.shared");
const billing_implementation_1 = require("../implementation/billing.implementation");
const billing_account_implementation_1 = require("../implementation/billing-account.implementation");
let logger = log4js.getLogger('ServerScripts');
function AccountService(app) {
    app.post('/api/account/get_account_status', (req, res) => {
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetAccountStatusByVCC(vcc);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            if (result) {
                let accountStatus = {};
                if (result.status && result.status === true) {
                    accountStatus.enabled = result.status;
                    accountStatus.description = result.description;
                }
                else {
                    accountStatus.enabled = result.status;
                    accountStatus.description = result.error;
                }
                res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: accountStatus });
            }
            else
                throw 'CANNOT_PROCESS_REQUEST';
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AccountService::get_account_status] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_accounts_list', (req, res) => {
        let vcc = res.locals.vcc;
        let updated = req.body.updated || false;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let billing = new billing_implementation_1.BillingImplementation();
            return billing.GetAccountsSubAccounts(vcc, updated);
        }).then((result) => {
            return Promise.resolve(result);
        }).then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AccountService::/api/account/get_accounts_list] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_vccs_by_account', (req, res, next) => {
        let vcc = res.locals.vcc;
        let accountId = req.body.accountId;
        let type = req.body.type;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetVCCsByAccount(accountId, type);
        }).then((result) => {
            return Promise.resolve(result);
        }).then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AccountService::/api/account/get_vccs_by_account] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_movements', (req, res, next) => {
        let vcc = res.locals.vcc;
        let billingMovement = (req.body || {});
        if (res.locals.vcc === 'system'
            && res.locals.userId === 'admin') {
        }
        else
            billingMovement.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetMovements(billingMovement);
        }).then((result) => {
            return Promise.resolve(result);
        }).then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::/api/account/get_movements] Getting error:', errorMsg, 'params:', billingMovement);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_movement_currencies', (req, res, next) => {
        let vcc = res.locals.vcc;
        let billingMovement = (req.body || {});
        if (res.locals.vcc === 'system'
            && res.locals.userId === 'admin') {
        }
        else
            billingMovement.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetMovementCurrencies(billingMovement);
        })
            .then((result) => {
            return Promise.resolve(result);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::/api/account/get_movements] Getting error:', errorMsg, 'params:', billingMovement);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_costs', (req, res, next) => {
        let vcc = res.locals.vcc;
        let billingMovement = (req.body || {});
        if (res.locals.vcc === 'system'
            && res.locals.userId === 'admin') {
        }
        else
            billingMovement.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then((result) => {
            let billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetCosts(billingMovement);
        }).then((result) => {
            return Promise.resolve(result);
        }).then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::/api/account/get_cost] Getting error:', errorMsg, 'params:', billingMovement);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AccountService = AccountService;
