"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AgentsService = void 0;
const log4js = require("log4js");
const promises_shared_1 = require("../shared/promises.shared");
const mssql_service_1 = require("../db/mssql.service");
const allegro_service_1 = require("./allegro.service");
const role_service_1 = require("./role.service");
let logger = log4js.getLogger('ServerScripts');
function AgentsService(app) {
    app.post('/api/agent/get_agents?', (req, res, next) => {
        logger.info('[AgentsService::get_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let userRole = res.locals.user.role;
        let agents = [];
        let agentStr = req.query.agents ? req.query.agents.toString() : '';
        let campaigns = req.body.campaigns ? req.body.campaigns : [];
        let userHasOnlyAgentFunctions = false;
        logger.info('[AgentsService::get_agents] Received vcc::agents::campaigns => ' + vcc, '::', agentStr, '::', campaigns);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, userId, userRole);
        })
            .then(result => {
            if (result
                &&
                    result.data
                &&
                    (!result.data.hasAdminFunctions
                        &&
                            !result.data.hasAuditorFunctions
                        &&
                            result.data.hasAgentFunctions))
                userHasOnlyAgentFunctions = true;
            if (userHasOnlyAgentFunctions) {
                logger.info('[AgentsService::get_agents] Current user has only agent function.');
                agentStr = userId;
            }
            return allegro_service_1.GetAgents(vcc, type, campaigns);
        })
            .then(result => {
            agents = result;
            if (userHasOnlyAgentFunctions)
                agents = agents.filter(a => a.userId === agentStr);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: agents });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AgentsService::get_agents] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/agent/evaluation', (req, res, next) => {
        logger.info('[AgentsService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let referrer = res.locals.pageReferrer.split('/').pop();
        let searchType = null;
        let campaigns = req.body.campaigns ? req.body.campaigns : [];
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetAgentsFromEvaluation(vcc, userId, campaigns, searchType);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_AGENTS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AgentsService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/agent/get_agent_qualification_average/:agentId', (req, res, next) => {
        logger.info('[AgentsService::get_agent_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let agentStr = req.params.agentId || '';
        logger.info('[AgentsService::get_agent_qualification_average] Received instance ' + instance);
        logger.info('[AgentsService::get_agent_qualification_average] Received agent ' + agentStr);
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[AgentsService::get_agent_qualification_average] Calling GetAgentQualificationAverage SP');
            return mssql_service_1.MSSql.GetAgentQualificationAverage(instance, agentStr);
        }).then(result => {
            if (res.headersSent)
                return;
            logger.debug('[AgentsService::get_agent_qualification_average] Received result from GetAgentQualificationAverage SP: ' + JSON.stringify(result));
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[AgentsService::get_agent_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/agent/interaction/', (req, res) => {
        logger.info('[NLUService::agent_interaction] Receives ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise().then(_ => {
            return mssql_service_1.MSSql.GetInteractionAgents(vcc);
        }).then(agents => {
            res.json({ status: true, description: 'ok', data: agents });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::agent_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AgentsService = AgentsService;
