"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAgents = exports.AllegroService = void 0;
const log4js = require("log4js");
const redis_service_1 = require("../db/redis.service");
const interaction_model_1 = require("../models/interaction.model");
const mssql_config_1 = require("../config/mssql.config");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const integration_config_1 = require("../config/integration.config");
const http_request_occ_1 = require("../shared/http-request-occ");
const interaction_service_1 = require("./interaction.service");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
let logger = log4js.getLogger('ServerScripts');
function AllegroService(app) {
    app.get('/api/process/allegro/get_allegro_campaigns', (req, res, next) => {
        logger.info('[AllegroService::get_allegro_campaigns] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let campaigns = [];
        let allegroCampaign = [];
        promises_shared_1.DummyPromise()
            .then(result => {
            let promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, vcc, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OCC':
                    let flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, vcc, flagOCC));
                    if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                        promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    }
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                    campaigns = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
                    campaigns = result[0].body;
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_CAMPAIGNS_FAILED';
                    let allegroCampaigns = result[0] ? result[0] : [];
                    let campaignsApi;
                    if (result[1] && result[1].status === true) {
                        campaignsApi = result[1] && result[1].status ? result[1].body : [];
                        campaigns = allegroCampaigns.filter(c => campaignsApi.find(ca => ca.id === c.id));
                    }
                    else {
                        campaigns = allegroCampaigns;
                    }
                    break;
                case 'OTHER':
                    break;
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: campaigns });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AllegroService::get_allegro_campaigns] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/allegro/get_allegro_supervisors', (req, res, next) => {
        logger.info('[SupervisorService::get_supervisors] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let vccType = res.locals.type;
        let campaigns = req.body.campaigns ? req.body.campaigns : [];
        let source = req.body.source ? req.body.source : 'INTEGRATED';
        let supervisors = [];
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (source === 'CSV')
                vccType = 'OTHER';
            return GetSupervisors(vcc, vccType, campaigns);
        })
            .then(result => {
            supervisors = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: supervisors });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[SupervisorService::get_supervisors] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/allegro/get_allegro_agents', (req, res, next) => {
        logger.info('[AllegroService::get_allegro_agents] Receives ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let agents = [];
        let campaigns = req.body.campaigns || [];
        promises_shared_1.DummyPromise()
            .then(result => {
            return GetAgents(vcc, type, campaigns);
        })
            .then(result => {
            agents = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: agents });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AllegroService::get_allegro_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/allegro/get_allegro_campaign_agents', (req, res, next) => {
        logger.info('[AllegroService::get_allegro_agents] Receives ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let campaigns = req.body.campaigns || [];
        let campaignsMembers = [];
        promises_shared_1.DummyPromise()
            .then(_ => {
            return http_request_occ_1.ApiI6GetCampaignMembers(vcc, campaigns);
        })
            .then(_ => {
            let promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaignMembers(integration_config_1.IP_BD, vcc, campaigns, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6GetCampaignMembers(vcc, campaigns));
                    break;
                case 'OCC':
                    let flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaignMembers(integration_config_1.IP_BD, vcc, campaigns, flagOCC));
                    if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                        promiseArray.push(http_request_occ_1.ApiI6GetCampaignMembers(vcc, campaigns));
                    }
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                    campaignsMembers = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
                    campaignsMembers = result[0].body;
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_CAMPAIGNS_FAILED';
                    let allegroCampaignsMembers = result[0] ? result[0] : [];
                    let campaignsMemebersApi;
                    if (result[1] && result[1].status === true) {
                        campaignsMemebersApi = result[1] && result[1].status ? result[1].body : [];
                        campaignsMembers = allegroCampaignsMembers.concat(campaignsMemebersApi);
                    }
                    else {
                        campaignsMembers = allegroCampaignsMembers;
                    }
                    break;
                case 'OTHER':
                    break;
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: campaignsMembers });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AllegroService::get_allegro_campaign_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/process/allegro/get_allegro_management_results', (req, res, next) => {
        logger.info('[AllegroService::get_allegro_management_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let allegroManagementResult = [];
        promises_shared_1.DummyPromise()
            .then(result => {
            let promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroManagementResults(integration_config_1.IP_BD, vcc, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OCC':
                    let flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroManagementResults(integration_config_1.IP_BD, vcc, flagOCC));
                    if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                        promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    }
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_DISPOSITION_CODES_FAILED';
                    allegroManagementResult = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_DISPOSITION_CODES_FAILED';
                    let campaignsApiI6 = result[0].body;
                    campaignsApiI6.map(c => {
                        if (c.dispositionCodes) {
                            c.dispositionCodes.map(dc => {
                                let dispositionCode = {};
                                dispositionCode.campaignId = c.id;
                                dispositionCode.description = dc.parent ? dc.parent : null;
                                dispositionCode.description2 = dc.child ? dc.child : null;
                                dispositionCode.description3 = dc.child2 ? dc.child2 : null;
                                dispositionCode.description4 = dc.child3 ? dc.child3 : null;
                                allegroManagementResult.push(dispositionCode);
                            });
                        }
                    });
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_DISPOSITION_CODES_FAILED';
                    allegroManagementResult = result[0];
                    let campaignsApi = result[1] && result[1].status ? result[1].body : [];
                    campaignsApi.map(c => {
                        if (c.dispositionCodes) {
                            c.dispositionCodes.map(dc => {
                                let dispositionCode = {};
                                dispositionCode.vcc = vcc;
                                dispositionCode.code = null;
                                dispositionCode.campaignId = c.id;
                                dispositionCode.description = dc.parent ? dc.parent : null;
                                dispositionCode.description2 = dc.child ? dc.child : null;
                                dispositionCode.description3 = dc.child2 ? dc.child2 : null;
                                dispositionCode.description4 = dc.child3 ? dc.child3 : null;
                                allegroManagementResult.push(dispositionCode);
                            });
                        }
                    });
                    break;
                case 'OTHER':
                    break;
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: allegroManagementResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/allegro/get_allegro_summary_interactions', (req, res, next) => {
        logger.info('[AllegroService::get_allegro_summary_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let params = new interaction_model_1.InteractionSearch;
        params = interaction_service_1.GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body, false);
        params.searchType = 'TO_SEARCH';
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params?.environmentType)) {
                return Promise.resolve(true);
            }
            else {
                let ftpCSVRedisKey;
                if (params.evaluationFormatId)
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                else
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
            }
        })
            .then(result => {
            if (['ftp', 'sftp', 's3'].includes(params.environmentType))
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            let promiseArray = [];
            promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, type, true, 'SUMMARIZED', params, result.data);
            return Promise.all(promiseArray);
        })
            .then(result => {
            let arrResult = [];
            arrResult = result.filter(r => r);
            arrResult = arrResult.filter(ar => ar.totalInteractions);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: arrResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/interaction/allegro/get_allegro_billing_interactions', (req, res, next) => {
        logger.info('[AllegroService::get_billing_interactions] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let processId = req.body.processId || null;
        let isScheduledBatch = req.body.isScheduledBatch || false;
        let modelSecondsPerRequest = req.body.secondsPerRequestRate || 0;
        let params = new interaction_model_1.InteractionSearch;
        let retranscription = req.body.retranscription;
        let isExistingTranscription = req.body.isExistingTranscription;
        let ftpCSVRedisKey;
        let cognitiveService;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            if (processId || isScheduledBatch) {
                return Promise.resolve([true]);
            }
            else {
                params = interaction_service_1.GetInteractionSearchFromRequest(req.body.evaluationFormatId, req.body.filters, true);
                params.searchType = 'TO_OPERATE';
                let promiseArray = [];
                if (params.evaluationFormatId && params.campaigns && params.campaigns.length === 0) {
                    logger.info('[AllegroService::get_billing_interactions] Searching evaluation format\'s campaigns');
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                    promiseArray.push(mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId));
                }
                else {
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                    promiseArray.push(Promise.resolve([true]));
                }
                if (['ftp', 'sftp', 's3'].includes(params.environmentType)) {
                    promiseArray.push(redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId));
                }
                else {
                    promiseArray.push(Promise.resolve([true]));
                }
                return Promise.all(promiseArray);
            }
        })
            .then(result => {
            let promise = null;
            if (processId || isScheduledBatch || isExistingTranscription) {
                if (processId)
                    promise = mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, processId, modelSecondsPerRequest);
            }
            else {
                logger.info('[AllegroService::get_billing_interactions] Requesting not evaluated interactions from HistoricalData');
                if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType)) {
                    if (params.campaigns && params.campaigns.length === 0) {
                        let evaluationFormatCampaigns = [];
                        evaluationFormatCampaigns = result[0].map(r => r.id);
                        params.campaigns = evaluationFormatCampaigns;
                    }
                }
                else {
                    if (result[1] === null)
                        throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
                }
                promise = interaction_service_1.GetChannelOldInteractionSummaryPromises(vcc, type, modelSecondsPerRequest, cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 100, params, result[1].data ? result[1].data : [], retranscription);
            }
            return promise;
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AllegroService::get_billing_interactions] Error on: \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AllegroService = AllegroService;
function GetSupervisors(vcc, vccType, campaigns) {
    let supervisors = [];
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let promiseArray = [];
        switch (vccType) {
            case 'ALLEGRO':
                let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? true : false;
                promiseArray.push(mssql_service_1.MSSql.GetAllegroSupervisors(integration_config_1.IP_BD, vcc, flagAllegro));
                break;
            case 'I6':
                promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, null, 'supervisor'));
                break;
            case 'OCC':
                let flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? true : false;
                promiseArray.push(mssql_service_1.MSSql.GetAllegroSupervisors(integration_config_1.IP_BD, vcc, flagOCC));
                if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                    promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, null, "supervisor"));
                }
                break;
            case 'OTHER':
                promiseArray.push(mssql_service_1.MSSql.GetSupervisors(vcc));
                break;
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        switch (vccType) {
            case 'ALLEGRO':
                if (!result[0])
                    throw 'SERVER_ERROR_GET_SQL_SUPERVISORS_FAILED';
                supervisors = result[0] ? result[0].filter(r => r.campaigns.filter(c => campaigns.includes(c.campaignId))) : [];
                break;
            case 'I6':
                if ((!result[0]) || (result[0] && result[0].status === false))
                    throw 'SERVER_ERROR_GET_API_I6_SUPERVISORS_FAILED';
                supervisors = result[0].body;
                break;
            case 'OCC':
                if (!result[0]
                    &&
                        ((!result[1]) || (result[1] && result[1].status === false))) {
                    supervisors = [];
                }
                else {
                    let allegroSupervisors = result[0] ? result[0].filter(r => r.campaigns.filter(c => campaigns.includes(c.campaignId)).length) : [];
                    allegroSupervisors.map(as => as.userName = as.userName.replace('  ', ' '));
                    let apiSupervisors = [];
                    if (result[1] && result[1].status === true) {
                        apiSupervisors = result[1] && result[1].status ? result[1].body : [];
                        supervisors = allegroSupervisors.concat(apiSupervisors);
                    }
                    else {
                        supervisors = allegroSupervisors;
                    }
                }
                break;
            case 'OTHER':
                if (!result[0])
                    throw 'SERVER_ERROR_GET_SQL_SUPERVISORS_FAILED';
                supervisors = result[0];
                break;
        }
        return Promise.resolve(supervisors);
    })
        .catch(err => {
        logger.error('[SupervisorService::GetSupervisors] Error al procesar: ', err);
        return Promise.reject(err);
    });
}
function GetAgents(vcc, vccType, campaigns = []) {
    if (!campaigns)
        campaigns = [];
    let agents = [];
    return promises_shared_1.DummyPromise()
        .then(result => {
        let promiseArray = [];
        switch (vccType) {
            case 'ALLEGRO':
                let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                promiseArray.push(mssql_service_1.MSSql.GetAllegroAgents(integration_config_1.IP_BD, vcc, flagAllegro, campaigns.toString()));
                break;
            case 'I6':
                promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, campaigns));
                break;
            case 'OCC':
                logger.debug('[AllegroService::GetAgents] Calling SQL & ApiI6: ', campaigns);
                let flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                promiseArray.push(mssql_service_1.MSSql.GetAllegroAgents(integration_config_1.IP_BD, vcc, flagOCC, campaigns.toString()));
                if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                    promiseArray.push(http_request_occ_1.ApiI6SearchUsers(vcc, campaigns));
                }
                break;
            case 'OTHER':
                break;
        }
        return Promise.all(promiseArray);
    })
        .then(result => {
        switch (vccType) {
            case 'ALLEGRO':
                if (!result[0])
                    throw 'SERVER_ERROR_GET_SQL_AGENTS_FAILED';
                agents = result[0];
                break;
            case 'I6':
                if ((!result[0]) || (result[0] && result[0].status === false))
                    throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
                agents = result[0].body;
                break;
            case 'OCC':
                if (!result[0]
                    &&
                        ((!result[1]) || (result[1] && result[1].status === false)))
                    throw 'SERVER_ERROR_GET_AGENTS_FAILED';
                let allegroAgents = result[0] ? result[0] : [];
                allegroAgents.map(aa => aa.userName = aa.userName.replace('  ', ' '));
                let agentsApi;
                if (result[1] && result[1].status === true) {
                    agentsApi = result[1] && result[1].status ? result[1].body : [];
                    agents = allegroAgents.concat(agentsApi);
                }
                else {
                    agents = allegroAgents;
                }
                break;
            case 'OTHER':
                break;
        }
        return Promise.resolve(agents);
    })
        .catch(err => {
        logger.error('[AllegroService::GetAgents] Error al procesar: ', err);
        return Promise.reject(err);
    });
}
exports.GetAgents = GetAgents;
