"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompareLicenses = exports.AuthService = void 0;
const moment = require("moment");
const jwt = require("jsonwebtoken");
const log4js = require("log4js");
const jwt_config_1 = require("../config/jwt.config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const user_service_1 = require("./user.service");
const role_service_1 = require("./role.service");
const promises_shared_1 = require("../shared/promises.shared");
const billing_implementation_1 = require("../implementation/billing.implementation");
const http_shared_1 = require("../shared/http.shared");
let logger = log4js.getLogger('ServerScripts');
function AuthService(app) {
    app.post('/auth/login', (req, res) => {
        let vcc = res.locals.vcc;
        let nowUtc = moment().utc().toDate();
        let credential = req.body.userId;
        let credentials = credential.split('@');
        let userId = credentials.length > 0 ? credentials[0] : '';
        vcc = credentials[1];
        let password = req.body.password;
        if (credential === 'admin')
            vcc = 'system';
        let session;
        let user;
        let role;
        let vccSpeech = [];
        let vccCenter = {};
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!userId || !password)
                throw 'AUTH_SERVER_ERROR_MISSING_USER_OR_PASSWORD';
            userId = userId.toLowerCase();
            return mssql_service_1.MSSql.ValidateUserLogin(vcc, userId, password);
        }).then(resultValidateUserLogin => {
            if (!resultValidateUserLogin)
                throw 'AUTH_SERVER_ERROR_WRONG_USER_OR_PASSWORD';
            return mssql_service_1.MSSql.GetVCCLicense(vcc);
        }).then(result => {
            if (vcc === 'system')
                return Promise.resolve(null);
            else {
                vccSpeech = result;
                let billing = new billing_implementation_1.BillingImplementation();
                return billing.GetVCCStatusDetail(vcc);
            }
        }).then(result => {
            let compare;
            if (vcc !== 'system') {
                if (typeof result == 'string') {
                    logger.info('[AuthService::/auth/login] Result afet Billing GetVCCStatus' + result);
                    throw result;
                }
                if (result.error && result.message.includes('ECONNREFUSED')) {
                    throw 'ERROR_FROM_EXTERNAL_SERVER';
                }
                if (result && !result.error && typeof result != 'string') {
                    logger.info('[AuthService::login] Result afet Billing GetVCCStatus' + result);
                    vccCenter = result;
                    compare = CompareLicenses(vccSpeech, vccCenter, userId, nowUtc);
                    if (!result.products.find(rp => rp.valid === 1))
                        throw 'LICENSE_EXPIRED';
                }
                if (!result.vcc.isActivated && result.vcc.desactivationReason && result.vcc.desactivationReason !== '')
                    throw 'AUTH_SERVER_ERROR_PLATFORM_WAS_DESACTIVATED_BY_ADMINSITRATOR';
            }
            logger.info('[AuthService::/auth/login] Compare VCC Licenses between SPEECH DB and BILLING DB , match : ' + compare);
            session = jwt.sign({ userId: userId, vcc: vcc }, jwt_config_1.JWT_SECRET, { expiresIn: jwt_config_1.JWT_EXPIRATION_TIME });
            return redis_service_1.Redis.SaveSession(vcc, userId, session);
        }).then(resultSaveSession => {
            if (!resultSaveSession)
                throw 'AUTH_SERVER_ERROR_CANNOT_SAVE_SESSION';
            return user_service_1.GetUser(vcc, userId);
        }).then(resultUser => {
            if (!resultUser)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
            user = resultUser;
            return mssql_service_1.MSSql.InsertUserHistory(vcc, userId, user.role, 'Login', nowUtc, session);
        }).then(resultInsertUserHistory => {
            if (!resultInsertUserHistory)
                throw 'AUTH_SERVER_ERROR_CANNOT_SAVE_SESSION';
            return role_service_1.GetRole(vcc, user.role);
        }).then(resultRole => {
            if (!resultRole)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
            role = resultRole;
            if (!session || !user || !role)
                throw 'AUTH_SERVER_ERROR_ON_START_SESSION';
            let data = {
                session: session,
                loggedUser: user,
                loggedUserRole: role
            };
            res.cookie('Session', session, { httpOnly: true });
            logger.info('[AuthService::/auth/login] Se inicio sesion ' + session + ' para el usuario ' + userId + '@' + vcc);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'Login failed');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AuthService::/auth/login] Error logging for user ' + userId + ' : ' + errorMsg);
            if (res.headersSent)
                return;
            res.status(401).json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/auth/logout', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let user = res.locals.user;
        let nowUtc = moment().utc().toDate();
        let session = req.body.session || res.locals.session || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return redis_service_1.Redis.DeleteSession(vcc, userId);
        }).then(result => {
            return redis_service_1.Redis.DeleteUserById(vcc, userId);
        }).then(result => {
            return mssql_service_1.MSSql.InsertUserHistory(vcc, userId, user.role, 'Logout', nowUtc, session);
        }).then(result => {
            res.clearCookie('Session');
            logger.info('Se finalizo sesion ' + session + ' para el usuario ' + userId + '@' + vcc);
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'AUTH_SERVER_ERROR_ON_END_SESSION'), data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'Logout failed');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al finalizar sesion para el usuario ' + userId + ' : ' + errorMsg);
            if (res.headersSent)
                return;
            res.status(401).json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/auth/get_user_session_data', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vccName = res.locals.vcc;
        let userId = res.locals.userId;
        userId = (userId || '').toLowerCase();
        let vcc;
        let session;
        let user;
        let role;
        promises_shared_1.DummyPromise()
            .then(result => {
            return http_shared_1.GetVCC(vccName);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_VCC_NOT_FOUND';
            vcc = result;
            return redis_service_1.Redis.GetSession(vccName, userId);
        }).then(resultSession => {
            if (!resultSession)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
            session = resultSession;
            return user_service_1.GetUser(vcc.id, userId);
        }).then(resultUser => {
            if (!resultUser)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_USER_DATA';
            user = resultUser;
            return role_service_1.GetRole(vcc.id, user.role);
        }).then(resultRole => {
            if (!resultRole)
                throw 'AUTH_SERVER_ERROR_CANNOT_GET_USER_DATA';
            role = resultRole;
            if (role.categories)
                delete role.categories;
            if (!session || !user || !role)
                throw 'AUTH_SERVER_ERROR_ON_START_SESSION';
            let data = {
                session: session,
                loggedUser: user,
                loggedUserRole: role,
                vcc: vcc.vcc,
                cognitiveEngine: vcc.cognitiveEngine,
                enableGenerativeEngine: vcc.enableGenerativeEngine
            };
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AuthService = AuthService;
function CompareLicenses(locals, external, user, date) {
    let status = true;
    if (external.products.length > 0) {
        locals.forEach(x => {
            external.products.forEach(y => {
                if (x.product === y.product) {
                    if (x.status != y.valid) {
                        status = false;
                        mssql_service_1.MSSql.SaveVCCLicense(x.product, y.valid, x.vcc, x.createdDate, x.createdByUserId, date, user);
                    }
                }
            });
        });
    }
    else {
        locals.forEach(x => {
            mssql_service_1.MSSql.SaveVCCLicense(x.product, 0, x.vcc, x.createdDate, x.createdByUserId, date, user);
        });
    }
    return status;
}
exports.CompareLicenses = CompareLicenses;
