"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateNewBatcScheduleExecutions = exports.SaveEvaluationPlanBatchSchedule = exports.SaveTemplateBatchSchedule = exports.SaveProcessBatchSchedule = exports.BatchScheduleService = void 0;
const log4js = require("log4js");
const moment = require("moment");
const mssql_service_1 = require("../db/mssql.service");
const schedule_model_1 = require("../models/schedule.model");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
const batch_schedule_implementation_1 = require("../implementation/batch-schedule.implementation");
const template_service_1 = require("./template.service");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('ServerScripts');
function BatchScheduleService(app) {
    app.post('/api/schedule/get_schedule', (req, res, next) => {
        logger.info('[BatchScheduleService::get_schedule] Received ' + req.method + ' methos on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let bodyData = (req.body || {});
        let batchScheduleId = bodyData.scheduleId;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchScheduleId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedule] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_transcription_schedules', (req, res, next) => {
        logger.info('[BatchScheduleService::get_transcription_schedules] Received ' + req.method + ' methos on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let bodyData = (req.body || {});
        let searchCriteria = bodyData.searchCriteria;
        let searchExpression = bodyData.searchExpression;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression, 'TRANSCRIPTION');
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_transcription_schedules] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/get_template_schedules', (req, res, next) => {
        logger.info('[BatchScheduleService::get_template_schedules] Received ' + req.method + ' methos on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let bodyData = (req.body || {});
        let searchCriteria = bodyData.searchCriteria;
        let searchExpression = bodyData.searchExpression;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression, 'TEMPLATE');
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_template_schedules] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/get_evaluation_plan_schedules', (req, res, next) => {
        logger.info('[BatchScheduleService::get_evaluation_plan_schedules] Received ' + req.method + ' methos on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let bodyData = (req.body || {});
        let searchCriteria = bodyData.searchCriteria;
        let searchExpression = bodyData.searchExpression;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression, 'EVALUATION_PLAN');
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_evaluation_plan_schedules] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/get_schedule_executions', (req, res, next) => {
        logger.info('[BatchScheduleService::get_schedule_executions] Received ' + req.method + ' methos on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let bodyData = (req.body || {});
        let batchScheduleId = bodyData.scheduleId;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByBatchType(vcc, batchScheduleId, null);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedule_executions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/save_schedule_status', (req, res, next) => {
        logger.info('[BatchScheduleService::save_schedule_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let bodyData = (req.body || {});
        let batchScheduleId = bodyData.scheduleId;
        let newStatus = bodyData.status;
        let scheduleData = null;
        let saveResult = false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchScheduleId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULE_NOT_FOUND';
            scheduleData = result;
            switch (newStatus) {
                case 'PAUSE':
                    scheduleData.status = 'PAUSED';
                    break;
                case 'RESUME':
                    scheduleData.status = 'RUNNING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchSchedule(vcc, scheduleData, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULE_STATUS_UPDATE_FAILED';
            saveResult = result;
            let promiseArray = [];
            switch (newStatus) {
                case 'PAUSE':
                    promiseArray.push(DeleteBatchsScheduleExecutionsData(vcc, scheduleData));
                    break;
                case 'RESUME':
                    promiseArray.push(CreateBatchScheduleExecutionsData(vcc, scheduleData));
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(_ => {
            if (res.headersSent)
                return;
            res.json({ status: saveResult, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: saveResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::save_schedule_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/delete_schedule', (req, res, next) => {
        logger.info('[BatchScheduleService::delete_schedule] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let batchSchedule = req.body.schedule;
        logger.info(`[BatchScheduleService::delete_schedule] Deleting schedule ${batchSchedule.id} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!batchSchedule)
                throw 'SERVER_ERROR_MISSING_SCHEDULE_ID';
            return mssql_service_1.MSSql.DeleteBatchSchedule(vcc, batchSchedule.batchType, batchSchedule.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SCHEDULE_DELETE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_SCHEDULE_DELETE_FAILED'), data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::delete_schedule] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_scheduled_template_results_summary', (req, res, next) => {
        logger.info('[BatchScheduleService::get_scheduled_template_results_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let templateColors = [
            { color: '#FF864B', font: '#FFFFFF' },
            { color: '#155979', font: '#FFFFFF' },
            { color: '#82B7E8', font: '#202121' },
            { color: '#A45C6F', font: '#FFFFFF' },
            { color: '#E57B80', font: '#FFFFFF' },
            { color: '#FC9C51', font: '#FFFFFF' },
            { color: '#C56B77', font: '#FFFFFF' },
            { color: '#FF864B', font: '#FFFFFF' },
            { color: '#155979', font: '#FFFFFF' },
            { color: '#82B7E8', font: '#202121' },
            { color: '#A45C6F', font: '#FFFFFF' },
            { color: '#E57B80', font: '#FFFFFF' },
            { color: '#FC9C51', font: '#FFFFFF' },
            { color: '#C56B77', font: '#FFFFFF' }
        ];
        let filters = req.body || {};
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchScheduledTemplateResults(vcc, filters.scheduledTemplates, {
                startDate: moment(filters.dateIni).toDate(),
                endDate: moment(filters.dateFin).toDate()
            });
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULED_TEMPLATE_RESULTS_NOT_FOUND';
            let resultSummary = result;
            let latestResults = result.filter(r => r.latestOcurrencies === 1);
            let data = {
                children: [],
                total: 0
            };
            let scheduledTemplates = arrays_shared_1.GetUniqueFromObjectArray(resultSummary, 'batchScheduleId');
            scheduledTemplates.map((st, i) => {
                let currentTemplate = {
                    batchScheduledTemplate: st.batchScheduleId,
                    level: 1,
                    isFirstLevel: true,
                    isFinalLevel: false,
                    name: st.batchScheduleId,
                    items: GetBatchScheduledTemplateData(resultSummary.filter(r => r.batchScheduleId === st.batchScheduleId), latestResults)
                };
                currentTemplate.value = arrays_shared_1.GetSumFromObjectArray(currentTemplate.items, 'total');
                currentTemplate.total = currentTemplate.value;
                currentTemplate.latestResults = latestResults.filter(lr => lr.batchScheduleId === st.batchScheduleId).length || 0;
                data.children.push(currentTemplate);
                data.total += currentTemplate.total;
            });
            data.children = arrays_shared_1.SortObjectArrayByValue(data.children, 'total', 'D');
            if (data.batchScheduledTemplate) {
                data.color = templateColors[0].color;
                data.items.map((c, i) => c.color = data.color);
            }
            else {
                data.children.map((c, i) => c.color = templateColors[(i % templateColors.length)].color);
            }
            return Promise.resolve(data);
        })
            .then(result => {
            result.children = result.children.map(item => {
                const dataItems = item.items.map(children => { return { ...children, totalTemplate: item.total }; });
                return { ...item, items: dataItems };
            });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_scheduled_template_results_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_scheduled_template_item_results_detailed', (req, res, next) => {
        logger.info('[BatchScheduleService::get_scheduled_template_item_results_detailed] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let filters = req.body || {};
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchScheduledTemplateItemResults(vcc, {
                batchScheduledId: filters.batchScheduledTemplateId,
                startDate: moment(filters.startDate).toDate(),
                endDate: moment(filters.endDate).toDate(),
                category1: filters.category1,
                category2: filters.category2,
                description: filters.description
            });
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_scheduled_template_results_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.BatchScheduleService = BatchScheduleService;
function SaveBatchSchedule(vcc, scheduleData, userId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetBatchScheduleById(vcc, scheduleData.id);
    })
        .then(result => {
        if (result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_ALREADY_EXIST';
        scheduleData.id = scheduleData.id.toLowerCase();
        return mssql_service_1.MSSql.SaveBatchSchedule(vcc, scheduleData, userId);
    })
        .then(_ => {
        if (['TRANSCRIPTION', 'EVALUATION_PLAN'].includes(scheduleData.batchType))
            return CreateBatchScheduleExecutionsData(vcc, scheduleData);
        else
            return Promise.resolve(true);
    })
        .then(_ => {
        logger.info('[BatchScheduleService::SaveBatchSchedule] Finishing function');
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[BatchScheduleService::SaveBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
function CreateBatchScheduleExecutionsData(vcc, scheduleData) {
    let executionCode = moment().toDate().toString();
    let batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return batchScheduleImplementation.GenerateNewScheduleBatchExecution(executionCode, scheduleData.id);
    })
        .then(_ => {
        return batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode, scheduleData.id);
    })
        .then(_ => {
        logger.info('[BatchScheduleService::CreateBatchScheduleExecutionsData] Finishing function');
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[BatchScheduleService::CreateBatchScheduleExecutionsData] Function failed');
        return Promise.reject(err);
    });
}
function DeleteBatchsScheduleExecutionsData(vcc, scheduleData) {
    let executionCode = moment().toDate().toString();
    let batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return batchScheduleImplementation.RemoveScheduleBatchExecutions(vcc, scheduleData, executionCode);
    })
        .then(_ => {
    })
        .then(_ => {
        logger.info('[BatchScheduleService::DeleteBatchsScheduleExecutionsData] Finishing function');
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[BatchScheduleService::DeleteBatchsScheduleExecutionsData] Function failed');
        return Promise.reject(err);
    });
}
function GetBatchScheduledTemplateData(data, latestResults) {
    let firstLevel = [];
    let uniqueCategory1Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(r => r.category1), 'category1');
    let nullCategory1Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(r => !r.category1), 'description');
    uniqueCategory1Objects.map(uc1o => {
        let currentSecondLevel = [];
        let uniqueCategory2Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(r => r.category1 === uc1o.category1 && r.category2 !== null), 'category2');
        let nullCategory2Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(r => r.category1 === uc1o.category1 && r.category2 === null), 'description');
        uniqueCategory2Objects.map(uc2o => {
            let currentThirdLevel = [];
            let uniqueDescriptionObjects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(r => r.category1 === uc1o.category1 && r.category2 === uc2o.category2), 'description');
            uniqueDescriptionObjects.map(udo => {
                let itemL3 = {
                    isFinalLevel: true,
                    name: udo.description,
                    batchScheduledTemplate: udo.batchScheduleId,
                    colname: `${udo.batchScheduleId}-L4`,
                    level: 4,
                    value: data.filter(r => r.category1 === uc2o.category1 && r.category2 === uc2o.category2 && r.description === udo.description).length,
                    total: data.filter(r => r.category1 === uc2o.category1 && r.category2 === uc2o.category2 && r.description === udo.description).length,
                    category1: udo.category1,
                    category2: udo.category2,
                    description: udo.description
                };
                currentThirdLevel.push(itemL3);
            });
            let itemL2 = {
                isFinalLevel: false,
                name: uc2o.category2,
                batchScheduledTemplate: uc2o.batchScheduleId,
                colname: `${uc2o.batchScheduleId}-L3`,
                level: 3,
                items: currentThirdLevel,
                value: arrays_shared_1.GetSumFromObjectArray(currentThirdLevel, 'total'),
                category1: uc2o.category1,
                category2: null,
                description: uc2o.category2
            };
            itemL2.total = itemL2.value;
            currentSecondLevel.push(itemL2);
        });
        nullCategory2Objects.map(nc2o => {
            let itemL2 = {
                isFinalLevel: true,
                name: nc2o.description,
                batchScheduledTemplate: nc2o.batchScheduleId,
                colname: `${nc2o.batchScheduleId}-L3`,
                level: 3,
                value: data.filter(r => r.category1 === nc2o.category1 && r.category2 === null && r.description === nc2o.description).length,
                total: data.filter(r => r.category1 === nc2o.category1 && r.category2 === null && r.description === nc2o.description).length,
                category1: nc2o.category1,
                category2: null,
                description: nc2o.description
            };
            currentSecondLevel.push(itemL2);
        });
        let itemL1 = {
            isFinalLevel: false,
            name: uc1o.category1,
            batchScheduledTemplate: uc1o.batchScheduleId,
            colname: `${uc1o.batchScheduleId}-L2`,
            level: 2,
            items: currentSecondLevel,
            value: arrays_shared_1.GetSumFromObjectArray(currentSecondLevel, 'total'),
            category1: null,
            category2: null,
            description: uc1o.category1
        };
        itemL1.total = itemL1.value;
        if (currentSecondLevel.length === 0) {
            itemL1.isFinalLevel = true;
            itemL1.value = data.filter(r => r.category1 === uc1o.category1).length;
            itemL1.total = data.filter(r => r.category1 === uc1o.category1).length;
        }
        firstLevel.push(itemL1);
    });
    nullCategory1Objects.map(nc1o => {
        let itemL1 = {
            isFinalLevel: true,
            name: nc1o.description,
            batchScheduledTemplate: nc1o.batchScheduleId,
            colname: `${nc1o.batchScheduleId}-L2`,
            level: 2,
            value: data.filter(r => r.category1 === null && r.description === nc1o.description).length,
            total: data.filter(r => r.category1 === null && r.description === nc1o.description).length,
            category1: null,
            category2: null,
            description: nc1o.description
        };
        firstLevel.push(itemL1);
    });
    return firstLevel;
}
function SaveProcessBatchSchedule(vcc, vccType, process, userId) {
    let res = false;
    process.scheduleData.firstExecutionDate = moment(process.scheduleData.firstExecutionDate).utc().toDate();
    process.scheduleData.startDate = moment(process.scheduleData.startDate).utc().toDate();
    process.scheduleData.endDate = process.scheduleData.endDate ? moment(process.scheduleData.endDate).utc().toDate() : null;
    let executionDelay = 0;
    process.scheduleData.batchData = {
        clientId: process.clientId,
        description: process.description,
        type: process.type,
        language: process.language,
        dictionaryId: process.dictionaryId,
        environment: process.environment,
        environmentType: process.environmentType,
        executionDelay: executionDelay,
        samplePercentage: process.percentage,
        vccType: vccType
    };
    process.scheduleData.executionDelay = executionDelay;
    let executionList = [];
    process.scheduleData.executionList.forEach((el, i) => {
        let execution = new schedule_model_1.ScheduleExecution();
        execution.batchScheduleId = process.scheduleData.id;
        execution.order = i + 1,
            execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = moment(el).toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = userId;
        executionList.push(execution);
    });
    process.scheduleData.executionList = executionList;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return SaveBatchSchedule(vcc, process.scheduleData, userId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_SAVE_FAILED';
        let promiseArray = [];
        process.processFilters.forEach(pf => {
            promiseArray.push(mssql_service_1.MSSql.SaveProcessFilter(vcc, pf.processId, pf.key, pf.value, moment().utc().toDate(), pf.createdByUserId, moment().utc().toDate(), pf.lastModifiedByUserId));
        });
        return Promise.all(promiseArray);
    })
        .then(results => {
        logger.info('[BatchScheduleService::SaveProcessBatchSchedule] Finishing function');
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(err => {
        logger.error('[BatchScheduleService::SaveProcessBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveProcessBatchSchedule = SaveProcessBatchSchedule;
function SaveTemplateBatchSchedule(vcc, template, userId) {
    template.scheduleData.id = template.name;
    template.scheduleData.batchData = template.template;
    template.scheduleData.processFiltersId = utilities_1.guid();
    template.scheduleData.startDate = moment(template.scheduleData.startDate).utc().toDate();
    template.scheduleData.firstExecutionDate = moment(template.scheduleData.firstExecutionDate).utc().toDate();
    template.scheduleData.startDate = moment(template.scheduleData.firstExecutionDate).utc().toDate();
    if (template.scheduleData.endDate)
        template.scheduleData.endDate = moment(template.scheduleData.endDate).utc().toDate();
    template.scheduleData.daysInterval = null;
    template.scheduleData.executionHour = null;
    template.scheduleData.hoursInterval = null;
    template.scheduleData.monthDay = null;
    template.scheduleData.weekDay = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return SaveBatchSchedule(vcc, template.scheduleData, userId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_SAVE_FAILED';
        if (template.scheduleData.includeHistoricalData) {
        }
        template.name = template.scheduleData.processFiltersId;
        return template_service_1.CreateTemplate(vcc, template, userId);
    })
        .then(_ => {
        logger.info('[BatchScheduleService::SaveTemplateBatchSchedule] Finishing function');
        return Promise.resolve(null);
    })
        .catch(err => {
        logger.error('[BatchScheduleService::SaveTemplateBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveTemplateBatchSchedule = SaveTemplateBatchSchedule;
function SaveEvaluationPlanBatchSchedule(vcc, vccType, evaluationPlan, process, userId) {
    evaluationPlan.scheduleData.id = evaluationPlan.id;
    evaluationPlan.scheduleData.firstExecutionDate = moment(evaluationPlan.scheduleData.firstExecutionDate).utc().toDate();
    evaluationPlan.scheduleData.startDate = moment(evaluationPlan.scheduleData.startDate).utc().toDate();
    evaluationPlan.scheduleData.endDate = evaluationPlan.scheduleData.endDate ? moment(evaluationPlan.scheduleData.endDate).utc().toDate() : null;
    let executionDelay = 0;
    let processData = null;
    if (process?.type) {
        processData = {
            clientId: process.clientId,
            description: process.description,
            type: process.type,
            language: process.language,
            dictionaryId: process.dictionaryId,
            environment: process.environment,
            environmentType: process.environmentType,
            executionDelay: executionDelay,
            samplePercentage: process.percentage
        };
    }
    let evaluationPlanData = {
        description: evaluationPlan.description,
        modality: evaluationPlan.modality,
        idEvaluationFormat: evaluationPlan.idEvaluationFormat,
        templateId: evaluationPlan.templateId,
        transcriptionLanguage: evaluationPlan.transcriptionLanguage,
        environmentId: evaluationPlan.environmentId,
        enabled: evaluationPlan.enabled,
        evaluation_mode: evaluationPlan.evaluation_mode,
        filterByContent: evaluationPlan.filterByContent,
        filterByContentPercentage: evaluationPlan.filterByContentPercentage,
        filterByContentQueryName: evaluationPlan.filterByContentQueryName,
        filterByContentQuery: evaluationPlan.filterByContentQuery,
        evaluatePercentage: evaluationPlan.evaluatePercentage,
        auditors: evaluationPlan.auditors.map((a) => { return { auditorId: a.idAuditor, percentage: a.numberInteractionsPercentage }; }),
        vccType: vccType
    };
    evaluationPlan.scheduleData.batchData = {
        process: processData,
        evaluationPlan: evaluationPlanData
    };
    evaluationPlan.scheduleData.executionDelay = executionDelay;
    let executionList = [];
    evaluationPlan.scheduleData.executionList.forEach((el, i) => {
        let execution = new schedule_model_1.ScheduleExecution();
        execution.batchScheduleId = evaluationPlan.scheduleData.id;
        execution.order = i + 1,
            execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = moment(el).toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = userId;
        executionList.push(execution);
    });
    evaluationPlan.scheduleData.executionList = executionList;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return SaveBatchSchedule(vcc, evaluationPlan.scheduleData, userId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_SAVE_FAILED';
        let promiseArray = [];
        if (process) {
            process.processFilters.forEach(pf => {
                promiseArray.push(mssql_service_1.MSSql.SaveProcessFilter(vcc, pf.processId, pf.key, pf.value, moment().utc().toDate(), pf.createdByUserId, moment().utc().toDate(), pf.lastModifiedByUserId));
            });
        }
        if (!promiseArray.length)
            promiseArray.push(Promise.resolve(true));
        return Promise.all(promiseArray);
    })
        .then(results => {
        logger.info('[BatchScheduleService::SaveEvaluationPlanBatchSchedule] Finishing function');
        if (results.filter(r => { return (r !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(err => {
        logger.error('[BatchScheduleService::SaveEvaluationPlanBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveEvaluationPlanBatchSchedule = SaveEvaluationPlanBatchSchedule;
function GenerateNewBatcScheduleExecutions(schedule, range, lastExecutionOrder) {
    if (!range)
        range = 7;
    if (!lastExecutionOrder)
        lastExecutionOrder = 0;
    let lastExecutionDate = moment(schedule.lastExecutionDate);
    let maxLastExecutionDate = moment(schedule.endDate ? schedule.endDate : moment().add(range, 'days'));
    let executionList = [];
    while (lastExecutionDate.isSameOrBefore(maxLastExecutionDate)) {
        switch (schedule.frequency) {
            case 'daily':
                lastExecutionDate.add(1, 'days');
                break;
            case 'weekly':
                lastExecutionDate.add(1, 'weeks');
                break;
            case 'monthly':
                lastExecutionDate.add(1, 'months');
                break;
            case 'custom_per_days':
                lastExecutionDate.add(schedule.daysInterval, 'days');
                break;
            case 'custom_per_hours':
                lastExecutionDate.add(schedule.hoursInterval, 'hours');
                break;
            default: break;
        }
        let execution = new schedule_model_1.ScheduleExecution();
        execution.vcc = schedule.vcc;
        execution.batchScheduleId = schedule.id;
        execution.order = ++lastExecutionOrder;
        execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = lastExecutionDate.toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = schedule.createdByUserId;
        console.log('RG-LOG: fecha de ejecución', lastExecutionDate.toDate());
        if (lastExecutionDate.isSameOrAfter(moment()) && lastExecutionDate.isBefore(maxLastExecutionDate))
            executionList.push(execution);
    }
    console.log(executionList);
    return executionList;
}
exports.GenerateNewBatcScheduleExecutions = GenerateNewBatcScheduleExecutions;
