"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const billing_batch_model_1 = require("../models/billing-batch.model");
let logger = log4js.getLogger('ServerScripts');
function BatchService(app) {
    app.post('/api/batch/get_batch_status', (req, res, next) => {
        logger.info('[BatchService::get_batch_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let batchTypeName = req.body.batchTypeName;
        let batchTypeId = req.body.batchTypeId;
        let batchDetail = {};
        let genericResult = {};
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(resultUser => {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            return mssql_service_1.MSSql.GetBatchDetail(vcc, batchTypeName, batchTypeId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_DETAIL_NOT_FOUND';
            batchDetail = result;
            let billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.GetBatchBalanceDetail(batchDetail);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_BALANCE_DETAIL_NOT_FOUND';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = { batchDetail: batchDetail, batchBalanceDetail: result.data };
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::get_batch_status] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/cancel_batch_system_service', (req, res, next) => {
        logger.info('[BatchService::cancel_batch_system_service] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let batchTypeName = req.body.batchTypeName;
        let batchTypeId = req.body.batchTypeId;
        let systemService = req.body.systemService;
        let genericResult = {};
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(resultUser => {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            let batchManage = new billing_batch_model_1.BatchManage;
            batchManage.vcc = vcc;
            batchManage.batchTypeName = batchTypeName;
            batchManage.batchTypeId = batchTypeId;
            batchManage.systemService = systemService;
            batchManage.user = userId;
            let billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.CancelBatchSummary(batchManage);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_CANCEL_FAILED';
            let batchRunningStatus = null;
            switch (batchTypeName) {
                case 'TRANSCRIPTION':
                    batchRunningStatus = 'CANCELLING';
                    break;
                case 'STUDY':
                    batchRunningStatus = 'CANCELLING';
                    break;
                case 'EVALUATION_PLAN':
                    batchRunningStatus = 'CANCELLING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchStatus(vcc, batchTypeId, systemService, batchRunningStatus);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_STATUS_UPDATE_FAILED';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result.data;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::cancel_batch_system_service] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/resume_batch_system_service', (req, res, next) => {
        logger.info('[BatchService::resume_batch_system_service] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let batchTypeName = req.body.batchTypeName;
        let batchTypeId = req.body.batchTypeId;
        let systemService = req.body.systemService;
        let genericResult = {};
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(resultUser => {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            let batchManage = new billing_batch_model_1.BatchManage;
            batchManage.vcc = vcc;
            batchManage.batchTypeName = batchTypeName;
            batchManage.batchTypeId = batchTypeId;
            batchManage.systemService = systemService;
            batchManage.user = userId;
            let billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
            return billingBatch.ResumeBatchSummary(batchManage);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_RESUME_FAILED';
            let batchRunningStatus = null;
            switch (batchTypeName) {
                case 'TRANSCRIPTION':
                    batchRunningStatus = 'INPROCESS_TRANSCRIPTION';
                    break;
                case 'STUDY':
                    batchRunningStatus = 'RUNNING';
                    break;
                case 'EVALUATION_PLAN':
                    batchRunningStatus = 'RUNNING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchStatus(vcc, batchTypeId, systemService, batchRunningStatus);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BATCH_STATUS_UPDATE_FAILED';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result.data;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::resume_batch_system_service] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/get_batch_errors', (req, res, next) => {
        logger.info('[BatchService::get_batch_errors] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let batchType = req.body.batchType;
        let batchTypeId = req.body.batchTypeId;
        let genericResult = {};
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetBatchErrors(vcc, batchType, batchTypeId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_BATCH_ERRORS_FAILED';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[BatchService::get_batch_errors] Error on \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/batch/check_batch_name', (req, res, next) => {
        let vcc = res.locals.vcc;
        let currentUserId = res.locals.userId;
        let batchType = req.body.type;
        let batchId = req.body.name;
        let genericResult = {};
        promises_shared_1.DummyPromise()
            .then(_ => {
            switch (batchType) {
                case 'SCHEDULE': return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchId);
                case 'TRANSCRIPTION': return mssql_service_1.MSSql.GetProcessByName(vcc, batchId);
                default: throw 'SERVER_ERROR_UNKNOWN_BATCH_TYPE';
            }
        })
            .then(result => {
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.data = result ? true : false;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchService::check_batch_name] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
}
exports.BatchService = BatchService;
