"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CampainService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const integration_config_1 = require("../config/integration.config");
const mssql_config_1 = require("../config/mssql.config");
const http_request_occ_1 = require("../shared/http-request-occ");
let logger = log4js.getLogger('ServerScripts');
function CampainService(app) {
    app.get('/api/campaign/get_campaigns/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[CampainService::get_campaigns] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let type = res.locals.type;
        let campaigns = [];
        let allegroCampaign = [];
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        type = search_criteria === 'HISTORIC' ? search_criteria : type;
        promises_shared_1.DummyPromise()
            .then(result => {
            let promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    let flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, instance, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(instance));
                    break;
                case 'OCC':
                    let flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroCampaigns(integration_config_1.IP_BD, instance, flagOCC));
                    if (!integration_config_1.USE_OCC_WITH_ALLEGRO) {
                        promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(instance));
                    }
                    break;
                case 'OTHER':
                    break;
                case 'HISTORIC':
                    let excludesworkgroupsup = search_expression == "ADD" ? true : false;
                    promiseArray.push(mssql_service_1.MSSql.GetHistoricCampaigns(instance, excludesworkgroupsup));
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (search_criteria == 'HISTORIC') {
                if (!result[0])
                    throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                campaigns = result[0];
            }
            else {
                switch (type) {
                    case 'ALLEGRO':
                        if (!result[0])
                            throw 'SERVER_ERROR_GET_SQL_CAMPAIGNS_FAILED';
                        campaigns = result[0];
                        break;
                    case 'I6':
                        if ((!result[0]) || (result[0] && result[0].status === false))
                            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
                        campaigns = result[0].body;
                        break;
                    case 'OCC':
                        if (!result[0]
                            &&
                                ((!result[1]) || (result[1] && result[1].status === false)))
                            throw 'SERVER_ERROR_GET_CAMPAIGNS_FAILED';
                        let allegroCampaigns = result[0] ? result[0] : [];
                        let campaignsApi;
                        if (result[1] && result[1].status === true) {
                            campaignsApi = result[1] && result[1].status ? result[1].body : [];
                            campaigns = allegroCampaigns.filter(c => campaignsApi.find(ca => ca.id === c.id));
                        }
                        else {
                            campaigns = allegroCampaigns;
                        }
                        break;
                    case 'OTHER':
                        break;
                }
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: campaigns });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/campaign/evaluation', (req, res, next) => {
        logger.info('[CampainService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let referrer = res.locals.pageReferrer.split('/').pop();
        let searchType = null;
        let campaigns = req.body.campaigns ? req.body.campaigns : [];
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCampaignsFromEvaluation(vcc, userId, searchType);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_CAMPAIGNS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CampainService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/campaign/get_campaigns_evaluation_format/:idEvaluationFormat', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let idEvaluationFormat = req.params.idEvaluationFormat || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(instance, idEvaluationFormat);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let campainMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: campainMsg, error: errorMsg });
        });
    });
    app.get('/api/campaign/get_campaign_qualification_average/:campaignId', (req, res, next) => {
        logger.info('[CampainService::get_campaign_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let campaignStr = req.params.campaignId || '';
        logger.info('[CampainService::get_campaign_qualification_average] Received instance: ' + instance);
        logger.info('[CampainService::get_campaign_qualification_average] Received campaignId: ' + campaignStr);
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[CampainService::get_campaign_qualification_average] Calling GetCampaignQualificationAverage SP');
            return mssql_service_1.MSSql.GetCampaignQualificationAverage(instance, campaignStr);
        }).then(result => {
            logger.info('[CampainService::get_campaign_qualification_average] Received result from GetCampaignQualificationAverage SP: ' + JSON.stringify(result));
            if (!result) { }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[CampainService::get_campaign_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/campaign/interaction/', (req, res) => {
        logger.info('[NLUService::campaigns_interaction] Receives ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise().then(_ => {
            return mssql_service_1.MSSql.GetInteractionCampaigns(vcc);
        }).then(campaigns => {
            res.json({ status: true, description: 'ok', data: campaigns });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::campaigns_interaction] Error on \'' + req.path + '\': ' + errorMsg);
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.CampainService = CampainService;
