"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetClient = exports.ClientService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function ClientService(app) {
    app.get('/api/client/get_clients/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetClients(vcc, search_criteria, search_expression);
        }).then(clients => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: clients });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/client/get_client/:client_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let clientId = req.params.client_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!clientId)
                throw 'CLIENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetClient(vcc, clientId);
        }).then(resultClient => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultClient });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/create_client', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newClient = (req.body || {});
        newClient.createdDate = nowUtc;
        newClient.createdByUserId = userId;
        newClient.lastModifiedDate = nowUtc;
        newClient.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetClient(vcc, newClient.id);
        }).then(resultClient => {
            if (resultClient)
                throw 'CLIENT_SERVER_ERROR_EXISTS';
            return SaveClient(vcc, newClient);
        }).then(resultSaveClient => {
            if (!resultSaveClient)
                throw 'CLIENT_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSaveClient => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveClient, description: (resultSaveClient ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_SAVE'), data: resultSaveClient });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/save_client', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newClient = (req.body || {});
        newClient.createdDate = null;
        newClient.createdByUserId = null;
        newClient.lastModifiedDate = nowUtc;
        newClient.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(resultDeleteClient => {
            if (!resultDeleteClient)
                throw 'CLIENT_SERVER_ERROR_ON_SAVE';
            return SaveClient(vcc, newClient);
        }).then(resultSaveClient => {
            if (!resultSaveClient)
                throw 'CLIENT_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSaveClient => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveClient, description: (resultSaveClient ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_SAVE'), data: resultSaveClient });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/delete_client', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let client = req.body.client;
        let clientId = client.id;
        logger.info(`[ClientService::delete_client] Deleting client ${clientId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!client)
                throw 'CLIENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteClient(vcc, clientId);
        }).then(resultDeleteClient => {
            if (!resultDeleteClient)
                throw 'CLIENT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteClient => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteClient, description: (resultDeleteClient ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_DELETE'), data: resultDeleteClient });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/client/delete_clients', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let clients = req.body.clients || [];
        logger.info(`[ClientService::delete_clients] Deleting clients ${clients.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!clients)
                throw 'CLIENT_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            clients.forEach(client => {
                promiseArray.push(mssql_service_1.MSSql.DeleteClient(vcc, client.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteClients => {
            let result;
            if (resultDeleteClients.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Clientes eliminados correctamente');
            }
            return result;
        }).then(resultDeleteClients => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteClients, description: (resultDeleteClients ? 'SERVER_SUCCESS_MESSAGE' : 'CLIENT_SERVER_ERROR_ON_MULTIPLE_DELETE'), data: resultDeleteClients });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ClientService = ClientService;
function GetClient(vcc, id) {
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetClient(vcc, id);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetClient: ' + errMsg);
    });
}
exports.GetClient = GetClient;
function SaveClient(vcc, client) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveClient(vcc, client.id, client.description, client.country, client.contact, client.createdDate, client.createdByUserId, client.lastModifiedDate, client.lastModifiedByUserId));
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
