"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
let logger = log4js.getLogger('ServerScripts');
function EntityService(app) {
    let cognitiveService;
    app.get('/api/entity/get_entities/:app_id/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[EntityService::get_entities] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        logger.info('[EntityService::get_entities] getting vcc:', vcc, 'azure:', azure);
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let appId = req.params.app_id || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            return cognitiveService.GetEntitiesList(azure.subscriptionRegion, azure.subscriptionKey, appId, vcc);
        })
            .then((result) => {
            logger.info('[EntityService::get_entities] getting', (result ? result.length : 0), 'items from cognitiveService.GetEntities');
            let idArray = result.map(a => a.id);
            logger.info('[EntityService::get_entities] getting  stringId:', idArray.join('\',\''));
            return mssql_service_1.MSSql.GetEntities(vcc, appId, search_criteria, search_expression, idArray.join('\',\''));
        })
            .then((entities) => {
            logger.info('[EntityService::get_entities] getting intents:', (entities ? entities.length : 0), 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: entities });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[EntityService::get_entities] getting error:', err, 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/entity/get_entity/:customfield_id', (req, res, next) => {
    });
    app.post('/api/entity/create_entity', (req, res, next) => {
        logger.info('[EntityService::create_entity] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let azure = res.locals.azure;
        let entityType = 'SIMPLE';
        let newEntity = null;
        switch (entityType) {
            case 'SIMPLE':
                newEntity = newEntity;
                newEntity = (req.body || {});
                break;
        }
        newEntity.type = entityType;
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (!newEntity.name)
                throw 'SERVER_ERROR_ENTITY_NAME_NOT_FOUND';
            return mssql_service_1.MSSql.GetEntity(vcc, newEntity.appId, null, newEntity.name);
        })
            .then(result => {
            if (result)
                throw 'SERVER_ERROR_ENTITY_ALREADY_EXISTS';
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            let promise = null;
            switch (entityType) {
                case 'SIMPLE':
                    promise = cognitiveService.AddSimpleEntity(azure.subscriptionRegion, azure.subscriptionKey, newEntity.appId, newEntity);
                    break;
                default:
                    promise = Promise.resolve(null);
                    break;
            }
            return promise;
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_ENTITY_CREATE_FALED';
            newEntity.id = result;
            return mssql_service_1.MSSql.SaveEntity(vcc, newEntity.appId, newEntity.id, newEntity.name, newEntity.type, userId, null);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_ENTITY_SAVE_FALED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EntityService::create_entity] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/entity/delete_entity', (req, res, next) => {
        logger.info('[EntityService::delete_entity] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let entity = (req.body.entity || {});
        let appId = entity.appId;
        logger.info(`[EntityService::delete_entity] Deleting entity ${entity.id} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            if (!entity)
                throw 'SERVER_ERROR_ENTITY_NOT_FOUND';
            return cognitiveService.DeleteAppEntity(azure.subscriptionRegion, azure.subscriptionKey, entity.appId, entity.id);
        })
            .then(result => {
            logger.info('[EntityService::delete_entity] getting result:', result, 'after of cognitiveService.DeleteAppEntity');
            return mssql_service_1.MSSql.DeleteEntity(vcc, appId, entity.id);
        })
            .then(result => {
            logger.info('[EntityService::delete_entity] getting result:', result, 'after of MSSql.DeleteEntity');
            if (!result)
                throw 'SERVER_ERROR_DELELE_ENTITY_FROM_DATABASE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[EntityService::delete_entity] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/entity/get_nlus_entities', (req, res, next) => {
        logger.info('[EntityService::get_nlus_entities] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        logger.info('[EntityService::get_nlus_entities] getting vcc:', vcc, 'azure:', azure);
        let appIds = req.body.appIds || [];
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            let promiseArray = [];
            let timeOut = 0;
            appIds.map((appId, index) => {
                if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                    timeOut = Math.floor(index / 5) * 1500;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(() => {
                            return resolve(cognitiveService.GetEntitiesList(azure.subscriptionRegion, azure.subscriptionKey, appId, vcc));
                        }, timeOut);
                    }));
                }
                else {
                    timeOut = index * 1000;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(() => {
                            return resolve(cognitiveService.GetEntitiesList(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, appId, vcc));
                        }, timeOut);
                    }));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            let modelEntities = [];
            appIds.map((appId, i) => {
                modelEntities.push({
                    appId: appId,
                    entities: result[i] || []
                });
            });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: modelEntities });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[EntityService::get_nlus_entities] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    function CreateSimpleEntity(vcc, entity) {
        return Promise.resolve()
            .then(result => {
            return cognitiveService.GetIntentsList(null, null, entity.appId);
        })
            .then(() => {
            return mssql_service_1.MSSql.GetIntent(vcc, name);
        }).catch((err) => {
            let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al ejecutar GetIntent: ' + errMsg);
        });
    }
}
exports.EntityService = EntityService;
