"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExceptionService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
const exception_mssql_1 = require("../db/mssql/exception.mssql");
let logger = log4js.getLogger('ServerScripts');
function ExceptionService(app) {
    app.get('/api/exception/get_exceptions/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetExceptions(vcc, search_criteria, search_expression);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.get('/api/exception/get_exception/:exception_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let exceptionId = req.params.exception_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!exceptionId)
                throw 'EXCEPTION_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetException(vcc, exceptionId);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/create_exception', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        let vcc = res.locals.vcc;
        logger.info('[ExceptionService::create_exception] params: ' + vcc);
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newException = (req.body || {});
        newException.createdDate = nowUtc;
        newException.createdByUserId = userId;
        newException.lastModifiedDate = nowUtc;
        newException.lastModifiedByUserId = userId;
        newException.description = newException.description.trim();
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetException(vcc, null, newException.description);
        }).then(result => {
            if (result)
                throw 'EXCEPTION_SERVER_ERROR_EXISTS';
            newException.id = utilities_1.guid();
            return exception_mssql_1.SaveException(vcc, newException);
        }).then(result => {
            if (!result)
                'EXCEPTION_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(newException.id);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EXCEPTION_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/save_exception', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newException = (req.body || {});
        newException.createdDate = null;
        newException.createdByUserId = null;
        newException.lastModifiedDate = nowUtc;
        newException.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetException(vcc, null, newException.description);
        }).then(result => {
            let exceptionDesc = {};
            exceptionDesc = result || exceptionDesc;
            if (result && exceptionDesc.id !== newException.id)
                throw 'EXCEPTION_SERVER_ERROR_EXISTS';
            return exception_mssql_1.SaveException(vcc, newException);
        }).then(resultSave => {
            if (!resultSave)
                throw 'EXCEPTION_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EXCEPTION_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/delete_exception', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let exception = req.body.exception;
        let exceptionId = exception.id;
        logger.info(`[ExceptionService::delete_exception] Deleting exception ${exceptionId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!exception)
                throw 'EXCEPTION_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteException(vcc, exceptionId);
        }).then(resultDeleteException => {
            if (!resultDeleteException)
                throw 'EXCEPTION_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteException => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteException, description: (resultDeleteException ? 'SERVER_SUCCESS_MESSAGE' : 'EXCEPTION_SERVER_ERROR_ON_DELETE'), data: resultDeleteException });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/delete_exceptions', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let exceptions = req.body.exceptions || [];
        logger.info(`[ExceptionService::delete_exceptions] Deleting exceptions ${exceptions.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!exceptions)
                throw 'EXCEPTION_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            exceptions.forEach(exception => {
                promiseArray.push(mssql_service_1.MSSql.DeleteException(vcc, exception.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteExceptions => {
            let result;
            if (resultDeleteExceptions.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Excepciones eliminados correctamente');
            }
            return result;
        }).then(resultDeleteExceptions => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteExceptions, description: (resultDeleteExceptions ? 'SERVER_SUCCESS_MESSAGE' : 'EXCEPTION_SERVER_ERROR_ON_DELETE'), data: resultDeleteExceptions });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ExceptionService = ExceptionService;
