"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveFeelings = exports.FeelingService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function FeelingService(app) {
    app.get('/api/feeling/get_feelings/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetFeelings(vcc, search_criteria, search_expression);
        }).then(feelings => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: feelings });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.FeelingService = FeelingService;
function SaveFeelings(vcc, feeling) {
    let promiseArray = [];
    if (feeling) {
        feeling.forEach(feelings => {
            promiseArray.push(mssql_service_1.MSSql.SaveFeeling(vcc, feelings.name, feelings.description, feelings.createdDate, feelings.createdByUserId, feelings.lastModifiedDate, feelings.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveFeelings = SaveFeelings;
