"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveGenericQuerys = exports.SaveGenericQuery = exports.GenericQueryService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const idatha_service_1 = require("./idatha.service");
const request_shared_1 = require("../shared/request.shared");
const semantic_query_implementation_1 = require("../implementation/semantic-query.implementation");
let logger = log4js.getLogger('ServerScripts');
function GenericQueryService(app) {
    app.get('/api/generic_query/get_generic_queries/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[GenericQueryService::/api/generic_query/get_generic_queries] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        logger.info(`[GenericQueryService::get_generic_queries] Received vcc: ${vcc}, search_criteria: ${search_criteria}, search_expression: ${search_expression}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info('[GenericQueryService::get_generic_queries] Calling GetGenericQueries SP');
            return mssql_service_1.MSSql.GetGenericQueries(vcc, search_criteria, search_expression);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            logger.info(`[GenericQueryService::get_generic_queries] Received result from GetGenericQueries SP: ${result && result.length ? result.length : 0}`);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::get_generic_queries] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_query/get_generic_query/:name', (req, res, next) => {
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let name = req.params.name;
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received vcc: ' + vcc);
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received query name: ' + name);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!name)
                throw 'MISSING_NAME_PARAMETER';
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Calling GetGenericQueryValidate SP');
            return mssql_service_1.MSSql.GetGenericQueryValidate(vcc, name);
        }).then(resultGenericQuery => {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query] Received result from GetGenericQueryValidate SP: ' + resultGenericQuery.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGenericQuery });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/get_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/generic_query/get_generic_query_usage/:name', (req, res, next) => {
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let name = req.params.name;
        logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Received vcc: ' + vcc, 'name:', name);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!name)
                throw 'MISSING_NAME_PARAMETER';
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Calling GetGenericQueryUsage SP');
            return mssql_service_1.MSSql.GetGenericQueryUsage(vcc, name);
        }).then((result) => {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Received result from GetGenericQueryUsage SP: ' + result);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/get_generic_query_usage/] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_query/create_generic_query', (req, res, next) => {
        logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newGenericQuery = (req.body || {});
        newGenericQuery.createdDate = nowUtc;
        newGenericQuery.createdByUserId = userId;
        newGenericQuery.lastModifiedDate = nowUtc;
        newGenericQuery.lastModifiedByUserId = userId;
        logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Received vcc: ' + vcc);
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Calling GetGenericQuery SP for an existing query with the name: ' + newGenericQuery.name);
            return mssql_service_1.MSSql.GetGenericQuery(vcc, newGenericQuery.name);
        }).then(resultGenericQuery => {
            if (resultGenericQuery)
                throw 'GENERIC_QUERY_ALREADY_EXISTS \'' + newGenericQuery.name + '\'.';
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Calling SaveGenericQuery function');
            return SaveGenericQuery(vcc, newGenericQuery);
        }).then(resultSaveGenericQuery => {
            if (!resultSaveGenericQuery)
                throw 'ERROR_ON_SAVE';
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Succed');
            return Promise.resolve(true);
        }).then(resultSaveGenericQuery => {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/create_generic_query] Sending response to client');
            res.json({ status: resultSaveGenericQuery, description: (resultSaveGenericQuery ? 'SERVER_SUCCESS_MESSAGE' : 'GENERIC_QUERY_SAVE_FAILED'), data: resultSaveGenericQuery });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/create_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_query/save_generic_query', (req, res, next) => {
        logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newGenericQuery = (req.body || {});
        newGenericQuery.createdDate = null;
        newGenericQuery.createdByUserId = null;
        newGenericQuery.lastModifiedDate = nowUtc;
        newGenericQuery.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Calling SaveGenericQuery function');
            return SaveGenericQuery(vcc, newGenericQuery);
        }).then(resultSaveGenericQuery => {
            if (!resultSaveGenericQuery)
                throw 'ERROR_ON_SAVE';
            logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Succed');
            return Promise.resolve(true);
        }).then(resultSaveGenericQuery => {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/save_generic_query] Sending response to client');
            res.json({ status: resultSaveGenericQuery, description: (resultSaveGenericQuery ? 'SERVER_SUCCESS_MESSAGE' : 'GENERIC_QUERY_SAVE_FAILED'), data: resultSaveGenericQuery });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/save_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generic_query/test_generic_query', (req, res) => {
        logger.info('[GenericQueryService::/api/generic_query/test_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let semanticQuery = new semantic_query_implementation_1.SemanticQueryImplementation();
        let query = req.body.query || '';
        let queryType = null;
        let data = JSON.stringify({
            version: 2,
            from: 0,
            size: 1,
            query,
        });
        let bodyParsed = JSON.parse(data.replace(/\\\\\\/g, "\\"));
        promises_shared_1.DummyPromise().then(_ => {
            return idatha_service_1.IdathaLogin(config_1.IDATHA_TEST_REALM, true);
        }).then(token => {
            let tk = typeof token === 'string' ? token : token.accessToken;
            let options = semanticQuery.GetIdathaRequestOptions(vcc, tk, null, query, 0, 1, 2);
            queryType = options.queryType;
            logger.info('[GenericQueryService::/api/generic_query/test_generic_query] data :' + JSON.stringify(options));
            return request_shared_1.SimplePostRequest(options.body);
        }).then(rawResult => {
            let result = semanticQuery.GetIdathaRequestResponse(queryType, query, rawResult);
            res.json({ status: true, data: result, description: 'QUERY_SYNTAX_OK' });
        }).catch(err => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/test_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            let errorCode = '';
            if (errorMsg.includes("Syntax")) {
                errorCode = 'QUERY_SYNTAX_ERROR';
            }
            else if (errorMsg.includes("Internal") || errorMsg.includes("500")) {
                errorCode = 'QUERY_INTERNAL_SERVER_ERROR';
            }
            else if (errorMsg.includes("Bad") || errorMsg.includes("401")) {
                errorCode = 'QUERY_BAD_USERNAME_OR_PASSWORD';
            }
            else {
                errorCode = 'CANNOT_PROCESS_REQUEST';
            }
            res.json({ status: false, description: errorCode, error: errorMsg });
        });
    });
    app.post('/api/generic_query/delete_generic_query', (req, res, next) => {
        logger.info('[GenericQueryService::/api/generic_query/delete_generic_query] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let genericQuery = req.body.genericQuery;
        let name = genericQuery.name;
        logger.info(`[GenericQueryService::delete_generic_query] Deleting genericQuery ${name} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!name)
                throw 'MISSING_GENERIC_QUERY_NAME_INPUT_PARAMETER';
            logger.info('[GenericQueryService::/api/generic_query/delete_generic_query] Calling DeleteGenericQuery function');
            return mssql_service_1.MSSql.DeleteGenericQuery(vcc, name);
        }).then(resultDeleteGenericQuery => {
            if (!resultDeleteGenericQuery)
                throw 'ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteGenericQuery => {
            if (res.headersSent)
                return;
            logger.info('[GenericQueryService::/api/generic_query/delete_generic_query] Sending response to client');
            res.json({ status: resultDeleteGenericQuery, description: (resultDeleteGenericQuery ? 'SERVER_SUCCESS_MESSAGE' : 'GENERIC_QUERY_DELETE_FAILED'), data: resultDeleteGenericQuery });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[GenericQueryService::/api/generic_query/delete_generic_query] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GenericQueryService = GenericQueryService;
function SaveGenericQuery(vcc, genericQuery) {
    logger.info('[GenericQueryService::/api/generic_query/SaveGenericQuery] Staring function: ' + JSON.stringify(genericQuery));
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGenericQuery(vcc, genericQuery.name, genericQuery.description, genericQuery.type, genericQuery.query, genericQuery.rawData, genericQuery.createdDate, genericQuery.createdByUserId, genericQuery.lastModifiedDate, genericQuery.lastModifiedByUserId, genericQuery.version));
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveGenericQuery = SaveGenericQuery;
function SaveGenericQuerys(vcc, genericQuerys) {
    logger.info('[GenericQueryService::/api/generic_query/SaveGenericQuery] Staring function: ' + JSON.stringify(genericQuerys));
    let promiseArray = [];
    genericQuerys.forEach(genericQuery => {
        promiseArray.push(mssql_service_1.MSSql.SaveGenericQuery(vcc, genericQuery.name, genericQuery.description, genericQuery.type, genericQuery.query, genericQuery.rawData, genericQuery.createdDate, genericQuery.createdByUserId, genericQuery.lastModifiedDate, genericQuery.lastModifiedByUserId, genericQuery.version));
    });
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveGenericQuerys = SaveGenericQuerys;
