"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GroupService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
let logger = log4js.getLogger('ServerScripts');
function GroupService(app) {
    app.get('/api/group/get_groups/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[GroupService::get_groups] Received ' + req.method + ' on \'' + req.path + '\'' + ' | ' + res.locals.instance);
        let instance = res.locals.instance;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGroups(instance, search_criteria, search_expression, disabled);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.post('/api/group/create_group', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        let instance = res.locals.instance;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newGroup = (req.body || {});
        newGroup.createdDate = nowUtc;
        newGroup.createdByUserId = userId;
        newGroup.lastModifiedDate = nowUtc;
        newGroup.lastModifiedByUserId = userId;
        newGroup.description = newGroup.description.trim();
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGroupDescription(instance, newGroup.description);
        }).then(result => {
            if (result)
                throw 'GROUP_SERVER_ERROR_EXISTS';
            newGroup.id = utilities_1.guid();
            return SaveGroup(instance, newGroup);
        }).then(result => {
            if (!result)
                throw 'GROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(newGroup.id);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'GROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/group/get_group/:group_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let groupId = req.params.group_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!groupId)
                throw 'GROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetGroup(instance, groupId);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/save_group', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        let instance = res.locals.instance;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newGroup = (req.body || {});
        newGroup.createdDate = null;
        newGroup.createdByUserId = null;
        newGroup.lastModifiedDate = nowUtc;
        newGroup.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetGroupDescription(instance, newGroup.description);
        }).then(result => {
            let grupDesc = {};
            grupDesc = result || grupDesc;
            if (result && grupDesc.id !== newGroup.id)
                throw 'GROUP_SERVER_ERROR_EXISTS';
            return SaveGroup(instance, newGroup);
        }).then(resultSave => {
            if (!resultSave)
                throw 'GROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'GROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/delete_group', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let group = (req.body || {});
        let groupId = group.id;
        logger.info(`[GroupService::delete_group] Deleting group ${groupId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!group)
                throw 'GROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteGroup(instance, groupId);
        }).then(resultDelete => {
            if (!resultDelete)
                throw 'GROUP_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDelete => {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'SERVER_SUCCESS_MESSAGE' : 'GROUP_SERVER_ERROR_ON_DELETE'), data: resultDelete });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/delete_groups', (req, res, next) => {
        logger.info('[GroupService::delete_groups] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let groups = req.body.groups || [];
        let ids = groups.map(item => item.id);
        logger.info(`[GroupService::delete_groups] Deleting groups ${ids.join(',')} by the user ${res.locals.userId}`);
        let response;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetGroupsUsage(vcc, ids);
        })
            .then(resultNoDeleted => {
            let deleteData = ids;
            let noDelete = [];
            if (resultNoDeleted.length > 0) {
                deleteData = [];
                groups.forEach(item => {
                    if (!resultNoDeleted.find(noDeleted => noDeleted.idGroup === item.id)) {
                        deleteData.push(item.id);
                    }
                    else {
                        noDelete.push(item.description);
                    }
                });
            }
            return Promise.resolve({ noDelete, deleteData });
        })
            .then(resultDelete => {
            response = resultDelete;
            if (resultDelete.deleteData.length)
                return mssql_service_1.MSSql.DeleteGroups(vcc, resultDelete.deleteData);
            return Promise.resolve(true);
        })
            .then(resultDelete => {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[GroupService::delete_groups] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GroupService = GroupService;
function SaveGroup(instance, group) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGroup(instance, group.id, group.description, group.createdDate, group.createdByUserId, group.lastModifiedDate, group.lastModifiedByUserId, group.enabled));
    return Promise.all(promiseArray)
        .then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
