"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPercentage = exports.getUniqueFinalResults = exports.processFinalResults = exports.saveStudyResultSummaryDetail = exports.getTotalPages = exports.RecursiveIdathaRequest = exports.RecursiveRequestQueryFilter = exports.UpdateStudyStatus = exports.IdathaStatus = exports.IdathaDocumentQueryFilter = exports.IdathaDeletePnl = exports.IdathaPnl = exports.IdathaDocumentWords = exports.IdathaPopularWords = exports.IdathaLogin = exports.IdathaCreateFolder = exports.IdathaRegister = exports.CreateIdathaRealmDirs = exports.CreateIdathaRealm = exports.IdathaRegisterCaller = exports.IdathaService = void 0;
const moment = require("moment");
const { NodeSSH } = require('node-ssh');
const log4js = require("log4js");
let scp2 = require('scp2');
const idatha_model_1 = require("../models/idatha.model");
const mssql_service_1 = require("../db/mssql.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const semantic_query_implementation_1 = require("../implementation/semantic-query.implementation");
const request_shared_1 = require("../shared/request.shared");
let logger = log4js.getLogger('ServerScripts');
function IdathaService(app) {
    let cognitiveService;
    app.post('/api/idatha_auth/get_idatha_auth_by_realm', (req, res, next) => {
        req.setTimeout(7.2e+6, () => { });
        logger.info('[IdathaService::get_idatha_auth_by_realm] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let objData = (req.body || {});
        let userName = '';
        let password = '';
        let accessToken = '';
        let visual = [];
        let auditive = [];
        let kinestesic = [];
        let words = objData.words;
        let wordsDeleted = objData.wordsDeleted;
        let processId = objData.id;
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (!objData.name)
                throw 'IDATHA_SERVER_ERROR_MISSING_REALM';
            if (objData.name === 'init')
                return Promise.resolve({ name: objData.name, isSystem: true });
            return mssql_service_1.MSSql.GetIdathaAuthByRealm(vcc, vcc);
        }).then(resultIdathaAuthByRealm => {
            if (resultIdathaAuthByRealm && resultIdathaAuthByRealm.isSystem) {
                return Promise.resolve({ name: objData.name, isSystem: true });
            }
            else if (resultIdathaAuthByRealm && resultIdathaAuthByRealm.userName) {
                userName = resultIdathaAuthByRealm.userName;
                password = resultIdathaAuthByRealm.password;
                return mssql_service_1.MSSql.GetUserAccessTokenByRealm(vcc, vcc);
            }
            else {
                userName = 'user_speech';
                password = 'inc0nc3rt2019';
                let idathaAuth = { vcc: vcc, userName: userName, password: password, realm: vcc };
                return IdathaRegister(idathaAuth);
            }
        })
            .then((resultUserAccessTokenByRealm) => {
            if (resultUserAccessTokenByRealm.accessToken) {
                accessToken = resultUserAccessTokenByRealm.accessToken;
                let currentDate = new Date(), createdDate = new Date(resultUserAccessTokenByRealm.createdDate), diffDays = Math.round(Math.abs((currentDate.getTime() - createdDate.getTime()) / (24 * 60 * 60 * 1000)));
                if (diffDays >= 7) {
                    return IdathaLogin(vcc)
                        .then((result) => {
                        objData.accessToken = result;
                        return objData;
                    })
                        .catch((err) => {
                        return Promise.reject(err);
                    });
                }
                else {
                    objData.accessToken = resultUserAccessTokenByRealm.accessToken;
                    return objData;
                }
            }
            else if (resultUserAccessTokenByRealm.isSystem) {
                return Promise.resolve({ name: objData.name, isSystem: true });
            }
            else if (resultUserAccessTokenByRealm.length > 0) {
                if (objData.type === 'wordcloud') {
                    return IdathaPopularWords(vcc, objData.name, resultUserAccessTokenByRealm, objData.origin);
                }
                else if (objData.type === 'document') {
                    let fileName = '';
                    fileName = config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER;
                    fileName += '/';
                    fileName += vcc;
                    fileName += '/';
                    fileName += objData.id;
                    if ((objData.section && objData.section >= 0) && (objData.part && objData.part > 0)) {
                        fileName += '_';
                        fileName += objData.section;
                        fileName += '_';
                        fileName += objData.part;
                    }
                    fileName += '.json';
                    logger.info('ExistInFileSystem: ', fileName);
                    return file_manager_1.ExistInFileSystem(fileName)
                        .then(resultExistInFileSystem => {
                        if (resultExistInFileSystem)
                            return Promise.resolve(true);
                        return IdathaDocumentWords(resultUserAccessTokenByRealm, objData.id, objData.section, objData.part);
                    }).then((resultIdathaDocumentWords) => {
                        let regex = /\bNaN\b/g;
                        let data;
                        data = JSON.stringify(resultIdathaDocumentWords.response);
                        if (regex.test(resultIdathaDocumentWords.response))
                            data = resultIdathaDocumentWords.response.replace(regex, 'null');
                        if (resultIdathaDocumentWords.response)
                            return file_manager_1.WriteFileContent(fileName, data, 'utf-8');
                        return Promise.resolve(true);
                    }).catch((err) => {
                        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                        logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
                    });
                }
                else if (objData.type === 'createProcess') {
                    return mssql_service_1.MSSql.GetWords(vcc, 'NeuroLinguisticProgrammingId', '', 'init', objData.language)
                        .then((result) => {
                        result.filter((word) => word.neuroLinguisticProgrammingId === 'visual').map((x) => visual.push(x.name));
                        result.filter((word) => word.neuroLinguisticProgrammingId === 'auditive').map((x) => auditive.push(x.name));
                        result.filter((word) => word.neuroLinguisticProgrammingId === 'kinestesic').map((x) => kinestesic.push(x.name));
                        return Promise.resolve(true);
                    }).then((result) => {
                        return IdathaPnl(resultUserAccessTokenByRealm, 'visual', visual);
                    }).then((result) => {
                        return IdathaPnl(resultUserAccessTokenByRealm, 'auditive', auditive);
                    }).then((result) => {
                        return IdathaPnl(resultUserAccessTokenByRealm, 'kinestesic', kinestesic);
                    }).then((result) => {
                        return Promise.resolve(true);
                    });
                }
                else if (objData.type === 'query') {
                    return mssql_service_1.MSSql.GetProcess(vcc, objData.name)
                        .then(data => {
                        return IdathaDocumentQueryFilter(resultUserAccessTokenByRealm.accessToken, objData.query, objData.from, objData.totalItems, data, objData.version);
                    }).then(resultIdathaDocumentQuery => {
                        return Promise.resolve(resultIdathaDocumentQuery);
                    }).then(result => {
                        let audios = {}, objAudios = {};
                        audios.id = [];
                        if (result.response.total_results > 0) {
                            audios.total = result.response.total_results;
                            for (let row of result.response.results) {
                                audios.id.push(row.id);
                            }
                            objAudios = { ids: audios, documents: result.response };
                        }
                        else if (result.response.total_results === 0) {
                            audios.total = result.response.total_results;
                            objAudios = { ids: audios, documents: result.response };
                        }
                        return Promise.resolve(objAudios);
                    });
                }
            }
            else {
                return Promise.resolve(true);
            }
        }).then((objData) => {
            if (objData.type === 'wordcloud') {
                return IdathaPopularWords(vcc, objData.name, typeof objData.accessToken === 'string' ? objData.accessToken : objData.accessToken.accessToken, objData.origin);
            }
            else if (objData.type === 'document') {
                let fileName = '';
                fileName = config_1.SEMANCTIC_ANALYSIS_FOLDER;
                fileName += '/';
                fileName += vcc;
                fileName += '/';
                fileName += objData.id;
                if (objData.section >= 0 && objData.part > 0) {
                    fileName += '_';
                    fileName += objData.section;
                    fileName += '_';
                    fileName += objData.part;
                }
                fileName += '.json';
                logger.info('ExistInFileSystem:', fileName);
                return file_manager_1.ExistInFileSystem(fileName)
                    .then(resultExistInFileSystem => {
                    if (resultExistInFileSystem)
                        return Promise.resolve(true);
                    return IdathaDocumentWords(objData.accessToken, objData.id, objData.section, objData.part);
                }).then((resultIdathaDocumentWords) => {
                    logger.info('resultIdathaDocumentWords: ', resultIdathaDocumentWords);
                    let regex = /\bNaN\b/g;
                    let data;
                    data = JSON.stringify(resultIdathaDocumentWords.response);
                    if (regex.test(resultIdathaDocumentWords.response))
                        data = resultIdathaDocumentWords.response.replace(regex, 'null');
                    if (resultIdathaDocumentWords.response)
                        return file_manager_1.WriteFileContent(fileName, data, 'utf-8');
                    return Promise.resolve(true);
                }).catch((err) => {
                    let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                    logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
                });
            }
            else if (objData.type === 'pnl') {
                let wordsSend = [];
                words.map((x) => wordsSend.push(x.name));
                let wordsDeletedSend = [];
                wordsDeleted.map((x) => wordsDeletedSend.push(x.name));
                let wordsTotal = [];
                wordsTotal = wordsSend.concat(wordsDeletedSend);
                return IdathaDeletePnl(objData.accessToken, wordsTotal)
                    .then(result => {
                    return IdathaPnl(objData.accessToken, objData.neuroLinguisticProgrammingId, wordsSend);
                }).then(result => {
                    return Promise.resolve(true);
                });
            }
            else if (objData.type === 'query') {
                return mssql_service_1.MSSql.GetProcess(vcc, objData.name)
                    .then(data => {
                    return IdathaDocumentQueryFilter(objData.accessToken, objData.query, objData.from, objData.totalItems, data);
                }).then(resultIdathaDocumentQuery => {
                    return Promise.resolve(resultIdathaDocumentQuery);
                }).then(result => {
                    let audios = {}, objAudios = {};
                    audios.id = [];
                    if (result.response.total_results > 0) {
                        audios.total = result.response.total_results;
                        for (let row of result.response.results) {
                            audios.id.push(row.id);
                        }
                        objAudios = { ids: audios, documents: result.response };
                    }
                    else if (result.response.total_results === 0) {
                        audios.total = result.response.total_results;
                        objAudios = { ids: audios, documents: result.response };
                    }
                    return Promise.resolve(objAudios);
                });
            }
            else if (objData.type === 'study') {
                (function () {
                    let vccTemp = '';
                    let themeItems = null;
                    let themeModels = null;
                    let processAudios = null;
                    logger.info('objData: ', objData);
                    return mssql_service_1.MSSql.GetTemplateThemeItems('system', objData.study.templateId, null)
                        .then((result) => {
                        logger.info('MSSql.GetTemplateThemeItems: ', result);
                        if (result && result.length > 0) {
                            vccTemp = 'system';
                            return result;
                        }
                        vccTemp = vcc;
                        return mssql_service_1.MSSql.GetTemplateThemeItems(vcc, objData.study.templateId, null);
                    })
                        .then((resultGetTemplateThemeItems) => {
                        logger.info('resultGetTemplateThemeItems: ', resultGetTemplateThemeItems);
                        if (resultGetTemplateThemeItems && resultGetTemplateThemeItems.length > 0) {
                            themeItems = resultGetTemplateThemeItems;
                            let index = 0, from = 0, size = 15, times = 0, page = 1;
                            objData.template = objData.study.templateId;
                            return mssql_service_1.MSSql.GetProcess(vcc, objData.name)
                                .then(result => {
                                logger.info('MSSql.GetProcess - result: ', result);
                                return Promise.resolve(exports.RecursiveRequestQueryFilter(vcc, objData, userId, objData.accessToken, themeItems, index, from, size, times, [], page, result));
                            });
                        }
                        else {
                            let result = { status: false, content: 'NO_QUERY_FOR_STUDY_TEMPLATE' };
                            return Promise.resolve(result);
                        }
                    })
                        .then((studyResults) => {
                        logger.info('studyResults: ', studyResults);
                        if (studyResults.status === false) {
                            return Promise.resolve(studyResults);
                        }
                        else {
                            var len = studyResults.idathaArray.length, obj, themeItem, finalResult, lenResults, objResult, i, j, finalResults = [], SQL = '';
                            for (i = 0; i < len; i++) {
                                obj = studyResults.idathaArray[i];
                                themeItem = themeItems.find((x) => {
                                    return x.id === obj.id;
                                });
                                if (themeItem) {
                                    finalResult = finalResults.find((x) => {
                                        return x.templateThemeItemId === themeItem.id;
                                    });
                                    if (finalResult) {
                                        lenResults = obj.result.response.results.length;
                                        for (j = 0; j < lenResults; j++) {
                                            objResult = obj.result.response.results[j];
                                            finalResult.results.push(objResult);
                                        }
                                    }
                                    else {
                                        finalResults.push({
                                            templateId: obj.templateId,
                                            templateThemeId: obj.templateThemeId,
                                            templateThemeItemId: obj.id,
                                            results: obj.result.response.results
                                        });
                                    }
                                }
                            }
                            len = finalResults.length;
                            for (i = 0; i < len; i++) {
                                obj = finalResults[i];
                                SQL += saveStudyResultSummaryDetail(vcc, studyResults.studyId, userId, obj);
                            }
                            finalResults = getUniqueFinalResults(finalResults);
                            len = finalResults.length;
                            for (i = 0; i < len; i++) {
                                obj = finalResults[i];
                                SQL += processFinalResults(vcc, studyResults.studyId, userId, obj);
                            }
                            return Promise.resolve(SQL);
                        }
                    })
                        .then((sqlResult) => {
                        if (sqlResult.status === false) {
                            return Promise.resolve(sqlResult);
                        }
                        else {
                            return mssql_service_1.MSSql.SaveSQLStudyDetail(vcc, objData.study.studyId, objData.study.templateId, sqlResult, objData.study.restart);
                        }
                    })
                        .then((result) => {
                        logger.info('MSSql.SaveSQLStudyDetail - result: ', vcc, objData.study.studyId, objData.study.templateId);
                        let vccTemp = '';
                        let themeItems = null;
                        return mssql_service_1.MSSql.GetTemplateThemeModels('system', objData.study.templateId, null)
                            .then((result) => {
                            logger.info('MSSql.GetTemplateThemeModels - result - system : ', result);
                            if (result && result.length > 0) {
                                vccTemp = 'system';
                                return result;
                            }
                            vccTemp = vcc;
                            return mssql_service_1.MSSql.GetTemplateThemeModels(vcc, objData.study.templateId, null);
                        });
                    }).then(resultGetTemplateThemeModels => {
                        themeModels = resultGetTemplateThemeModels;
                        logger.info('themeModels', JSON.stringify(themeModels));
                        if (themeModels && themeModels.length > 0) {
                            return mssql_service_1.MSSql.GetProcessAudios(vcc, objData.study.processId)
                                .then(resultGetProcessAudios => {
                                let promiseArray = [];
                                logger.info('resultGetProcessAudios: ', resultGetProcessAudios);
                                if (resultGetProcessAudios && resultGetProcessAudios.length) {
                                    let results = [];
                                    return resultGetProcessAudios.reduce((promise, resultGetProcessAudio) => {
                                        return promise.then((result) => {
                                            return GetQueryModel(vcc, cognitiveService, objData.accessToken, resultGetProcessAudio, themeModels, azure.subscriptionRegion, azure.subscriptionKey).then(result => results.push(result));
                                        });
                                    }, Promise.resolve())
                                        .then(() => {
                                        return results;
                                    })
                                        .then(results => {
                                        logger.info('GetQueryModel - QueryLUIS: ', JSON.stringify(results));
                                        logger.info('themeModels: ', JSON.stringify(themeModels));
                                        var flatteningDataResult = nlu.flatteningData(results, themeModels);
                                        logger.info('flatteningDataResult: ', JSON.stringify(flatteningDataResult));
                                        return Promise.resolve(flatteningDataResult);
                                    })
                                        .then((flatteningDataResult) => {
                                        if (flatteningDataResult && flatteningDataResult.length > 0) {
                                            var SQL = nlu.setInput(vcc, objData.study.studyId, userId, flatteningDataResult, themeModels);
                                            logger.info('SQL Input Study: ', SQL);
                                            return Promise.resolve(SQL);
                                        }
                                        else {
                                            return Promise.resolve('NO_DATA_STUDY_NLU');
                                        }
                                    });
                                }
                                else {
                                    return null;
                                }
                            })
                                .then((sqlResult) => {
                                logger.info('sqlResult SQL Input Study: ', sqlResult);
                                if (sqlResult !== 'NO_DATA_STUDY_NLU') {
                                    return mssql_service_1.MSSql.SaveSQLStudyDetail(vcc, objData.study.studyId, objData.study.templateId, sqlResult, objData.study.restart);
                                }
                                else {
                                    return Promise.resolve(sqlResult);
                                }
                            })
                                .then((sqlResult) => {
                                logger.info('sqlResult: ', sqlResult);
                                if (sqlResult === 'NO_DATA_STUDY_NLU') {
                                    return Promise.resolve(sqlResult);
                                }
                                else {
                                    return Promise.resolve(true);
                                }
                            }).catch((err) => {
                                let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
                                let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                                res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
                            });
                        }
                        else {
                            return Promise.resolve(true);
                        }
                    });
                }());
                return Promise.resolve(true);
            }
            else if (objData.type === 'status') {
                return IdathaStatus(objData.accessToken)
                    .then((result) => {
                    return Promise.resolve(result);
                })
                    .catch((err) => {
                    let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
                    let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                    res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
                });
            }
            else if (objData.type === 'createProcess') {
                return mssql_service_1.MSSql.GetWords(vcc, 'NeuroLinguisticProgrammingId', '', 'init', objData.language)
                    .then((result) => {
                    result.filter((word) => word.neuroLinguisticProgrammingId === 'visual').map((x) => visual.push(x.name));
                    result.filter((word) => word.neuroLinguisticProgrammingId === 'auditive').map((x) => auditive.push(x.name));
                    result.filter((word) => word.neuroLinguisticProgrammingId === 'kinestesic').map((x) => kinestesic.push(x.name));
                    return Promise.resolve(true);
                }).then((result) => {
                    return IdathaPnl(objData.accessToken, 'visual', visual);
                }).then((result) => {
                    return IdathaPnl(objData.accessToken, 'auditive', auditive);
                }).then((result) => {
                    return IdathaPnl(objData.accessToken, 'kinestesic', kinestesic);
                }).then((result) => {
                    return Promise.resolve(true);
                })
                    .then((result) => {
                    console.log(425, result);
                });
            }
            else if (objData) {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.IdathaService = IdathaService;
async function IdathaRegisterCaller(vcc, idathaIP, idathaPort) {
    let auth = {
        userName: config_1.REALM_USER_NAME,
        password: config_1.REALM_PASSWORD,
        realm: vcc,
        vcc
    };
    return CreateIdathaRealm(auth, idathaIP, idathaPort);
}
exports.IdathaRegisterCaller = IdathaRegisterCaller;
function CreateIdathaRealm(idathaAuth, idathaIP, idathaPort) {
    let userSystem = 'system';
    return promises_shared_1.DummyPromise().then(_ => {
        return mssql_service_1.MSSql.GetIdathaAuthByRealm(userSystem, userSystem);
    }).then(({ password }) => {
        return mssql_service_1.MSSql.GetUserAccessTokenByRealm(userSystem, userSystem);
    }).then(result => {
        console.log(result);
        if (result && result.accessToken) {
            let currentDate = new Date(), createdDate = new Date(result.createdDate);
            let diffDays = Math.round(Math.abs((currentDate.getTime() - createdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (diffDays >= 7) {
                return IdathaLogin(userSystem);
            }
            return result.accessToken;
        }
        return '';
    }).then(accessToken => {
        let options = {
            uri: 'http://' + idathaIP + ':' + idathaPort + '/api/auth/register',
            headers: {
                'Authorization': 'Bearer ' + accessToken
            },
            body: idathaAuth
        };
        return request_shared_1.SimplePostRequest(options);
    }).then(r => {
        console.log("req", r);
        return mssql_service_1.MSSql.SaveIdathaAuth(idathaAuth.vcc, idathaAuth.userName, idathaAuth.password, idathaAuth.realm);
    }).then(r => {
        console.log("r", r);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al procesar \'' + err + '\': ' + errorMsg);
        if (errorMsg === '500 - {"message":"Username already exists"}') {
            mssql_service_1.MSSql.SaveIdathaAuth(idathaAuth.vcc, idathaAuth.userName, idathaAuth.password, idathaAuth.realm);
        }
    });
}
exports.CreateIdathaRealm = CreateIdathaRealm;
function CreateIdathaRealmDirs(realm, vcc) {
    console.log('Envía a la creación de la carpeta de acuerdo al nombre.');
    return IdathaCreateFolder(realm).then((result) => {
        console.log('Obtiene resultado luego de la creación de la carpeta: ', result);
        return IdathaLogin(vcc);
    });
}
exports.CreateIdathaRealmDirs = CreateIdathaRealmDirs;
function IdathaRegister(idathaAuth) {
    let idatha = new idatha_model_1.IdathaConnection();
    var userSystem = 'system', password = '';
    return mssql_service_1.MSSql.GetIdathaAuthByRealm(userSystem, userSystem)
        .then((result) => {
        password = result.password;
        return mssql_service_1.MSSql.GetUserAccessTokenByRealm(userSystem, userSystem);
    })
        .then((resultUserAccessTokenByRealm) => {
        let accessToken = {};
        if (resultUserAccessTokenByRealm?.accessToken) {
            accessToken = resultUserAccessTokenByRealm;
            let currentDate = new Date(), createdDate = new Date(accessToken.createdDate), diffDays = Math.round(Math.abs((currentDate.getTime() - createdDate.getTime()) / (24 * 60 * 60 * 1000)));
            if (diffDays >= 7) {
                return IdathaLogin(userSystem);
            }
        }
        return Promise.resolve(accessToken);
    })
        .then((accessToken) => {
        let options = {
            uri: 'http://' + idatha.ip + ':' + idatha.httpPort + '/api/auth/register',
            headers: {
                'Authorization': 'Bearer ' + accessToken.accessToken
            },
            body: {
                username: idathaAuth.userName,
                password: idathaAuth.password,
                realm: idathaAuth.realm
            }
        };
        return request_shared_1.SimplePostRequest(options);
    })
        .then((response) => {
        return mssql_service_1.MSSql.SaveIdathaAuth(idathaAuth.vcc, idathaAuth.userName, idathaAuth.password, idathaAuth.realm);
    })
        .then((response) => {
        return IdathaCreateFolder(idathaAuth.realm);
    })
        .then((result) => {
        return IdathaLogin(idathaAuth.vcc);
    })
        .catch((err) => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al procesar \'' + err + '\': ' + errorMsg);
        if (errorMsg === '500 - {"message":"Username already exists"}') {
            return mssql_service_1.MSSql.SaveIdathaAuth(idathaAuth.vcc, idathaAuth.userName, idathaAuth.password, idathaAuth.realm)
                .then((response) => {
                return IdathaCreateFolder(idathaAuth.realm).then((result) => {
                    return IdathaLogin(idathaAuth.vcc);
                });
            })
                .catch((err) => {
                return Promise.reject(err);
            });
        }
        return Promise.resolve(false);
    });
}
exports.IdathaRegister = IdathaRegister;
function IdathaCreateFolder(folder) {
    logger.info('[IdathaService::IdathaCreateFolder] Se envía a crear una carpeta con nombre ' + folder + ' en un directorio del servidor de IDATHA.');
    let idatha = new idatha_model_1.IdathaConnection();
    var ssh = new NodeSSH();
    return ssh.connect({
        host: idatha.ip,
        username: idatha.user,
        password: idatha.password,
        port: idatha.sshPort,
        readyTimeout: 99999
    }).then(() => {
        let cmd = 'mkdir -p ';
        cmd += folder.toLowerCase();
        cmd += '/default';
        ssh.execCommand(cmd, { cwd: '/home/admon/in_concert_speech_analytics/input' }).then((x) => {
            console.log(984, x);
        });
    }).catch((error) => {
        console.log(error);
        logger.error('[IdathaService::IdathaCreateFolder] Ha sucedido el error: ' + error.message);
    });
}
exports.IdathaCreateFolder = IdathaCreateFolder;
function IdathaLogin(vcc, devMode = false) {
    let idatha = new idatha_model_1.IdathaConnection();
    let userAccessToken = {};
    let idathaAuth = null;
    let needToCreateRealm = false;
    let gotUserAccessToken = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let key = `${vcc}:${vcc}`;
        return redis_service_1.Redis.GetUserAccessTokenByRealm(key);
    })
        .then((result) => {
        if (!result
            ||
                (result
                    &&
                        !result.accessToken))
            return mssql_service_1.MSSql.GetUserAccessTokenByRealm(vcc, vcc);
        else
            return Promise.resolve(result);
    })
        .then((result) => {
        if (result) {
            userAccessToken = result;
            let currentDate = parseInt(moment().format('x'));
            let createdDate = parseInt(moment(typeof (userAccessToken.createdDate) === 'string' ? parseInt(userAccessToken.createdDate) : userAccessToken.createdDate).format('x'));
            let diffDays = Math.round(Math.abs((currentDate - createdDate) / (24 * 60 * 60 * 1000)));
            if (diffDays >= 6) {
                if (devMode)
                    return new idatha_model_1.IdathaAuth(vcc, config_1.REALM_USER_NAME, config_1.REALM_PASSWORD, vcc);
                return mssql_service_1.MSSql.GetIdathaAuthByRealm(vcc, vcc);
            }
            else {
                gotUserAccessToken = true;
                return Promise.resolve(null);
            }
        }
        else {
            if (devMode)
                return new idatha_model_1.IdathaAuth(vcc, config_1.REALM_USER_NAME, config_1.REALM_PASSWORD, vcc);
            return mssql_service_1.MSSql.GetIdathaAuthByRealm(vcc, vcc);
        }
    })
        .then((auth) => {
        if (auth && !gotUserAccessToken) {
            idathaAuth = auth;
            let options = {
                uri: 'http://' + idatha.ip + ':' + idatha.httpPort + '/api/auth/login',
                body: {
                    username: idathaAuth.userName,
                    password: idathaAuth.password,
                    realm: vcc === 'system' ? '' : idathaAuth.realm
                }
            };
            return request_shared_1.SimplePostRequest(options);
        }
        else {
            if (gotUserAccessToken)
                return Promise.resolve(null);
            else {
                logger.info(`[IdathaService::IdathaLogin] Registering realm for ${vcc}`);
                needToCreateRealm = true;
                idathaAuth = new idatha_model_1.IdathaAuth(vcc, config_1.REALM_USER_NAME, config_1.REALM_PASSWORD, vcc);
                return IdathaRegister(idathaAuth);
            }
        }
    })
        .then(result => {
        logger.info(`[IdathaService::IdathaLogin] Getting userAccessToken for ${vcc}:`, result);
        if (needToCreateRealm) {
            userAccessToken = result;
        }
        else if (result?.access_token) {
            let promiseArray = [];
            let nowUtc = moment().utc().toDate();
            userAccessToken.accessToken = result.access_token;
            userAccessToken.createdDate = moment().format('x');
            promiseArray.push(mssql_service_1.MSSql.SaveUserAccessToken(vcc, idathaAuth.userName, userAccessToken.accessToken, idathaAuth.realm, nowUtc));
            let key = idathaAuth.vcc + ':' + idathaAuth.vcc;
            promiseArray.push(redis_service_1.Redis.SaveUserAccessTokenByRealm(key, userAccessToken));
            return Promise.all(promiseArray);
        }
        return Promise.resolve(null);
    })
        .then(_ => {
        return Promise.resolve(userAccessToken);
    })
        .catch((err) => {
        return Promise.reject(err);
    });
}
exports.IdathaLogin = IdathaLogin;
function GetIdathaPopularWords(wordcloud, accessToken) {
    let idatha = new idatha_model_1.IdathaConnection();
    let options = {
        uri: `http://${idatha.ip}:${idatha.httpPort}/api/query/conversation/${wordcloud}`,
        headers: { 'Authorization': 'Bearer ' + accessToken }
    };
    return request_shared_1.SimpleGetRequest(options);
}
function IdathaPopularWords(vcc, processId, accessToken, origin) {
    let idatha = new idatha_model_1.IdathaConnection();
    let promiseArray = [];
    promiseArray.push(GetIdathaPopularWords('wordcloud', accessToken));
    promiseArray.push(GetIdathaPopularWords('wordcloud_in', accessToken));
    promiseArray.push(GetIdathaPopularWords('wordcloud_out', accessToken));
    return Promise.all(promiseArray)
        .then((response) => {
        let type = 'all';
        let tags = [];
        response.forEach((wordcloud, index) => {
            if (index == 1)
                type = 'in';
            else if (index === 2)
                type = 'out';
            tags.push({
                tags: wordcloud.wordcloud.map(item => {
                    return {
                        key: item.key,
                        value: item.doc_count
                    };
                }),
                type,
            });
        });
        let content = { type: 'popularwords' };
        return mssql_service_1.MSSql.GetExceptions(vcc, null, null).then(exceptions => {
            if (exceptions.length > 0) {
                tags = tags.map((x) => {
                    return {
                        tags: (function (x) {
                            return x.tags = x.tags.filter(function (y) {
                                return !exceptions.find((exception) => {
                                    return y.key.toLowerCase() === exception.description.toLowerCase();
                                });
                            });
                        }(x)),
                        type: x.type
                    };
                });
            }
            tags = tags.map((x) => {
                return (function () {
                    x.tags = x.tags.sort(function (a, b) {
                        return b.value - a.value;
                    });
                    x.tags.length = (x.tags.length < 30 ? x.tags.length : 30);
                    return x;
                }());
            });
            content.tags = tags;
            return content.tags;
        });
    })
        .catch((err) => {
        logger.error('[IdathaService::IdathaPopularWords] Getting error:', err, 'idatha:', idatha, 'processId:', processId, 'accessToken:', accessToken);
        return Promise.reject(err);
    });
}
exports.IdathaPopularWords = IdathaPopularWords;
function IdathaDocumentWords(accessToken, audioId, section, part) {
    let idatha = new idatha_model_1.IdathaConnection();
    let url = `http://${idatha.ip}:${idatha.httpPort}/api/query/document/${audioId}`;
    if ((section && section >= 0) && (part && part > 0)) {
        url += '_';
        url += section;
        url += '_';
        url += part;
    }
    let options = {
        method: 'GET',
        uri: url,
        headers: {
            'Authorization': 'Bearer ' + accessToken
        },
    };
    logger.info('IdathaDocumentWords - options: ', options);
    return request_shared_1.SimpleGetRequest(options)
        .then((response) => {
        let content = { response: response, type: 'document' };
        logger.info('content: ', content);
        return Promise.resolve(content);
    }).catch((err) => {
        options = {
            method: 'GET',
            uri: `http://${idatha.ip}:${idatha.httpPort}/api/query/document/${audioId}`,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            },
        };
        logger.info('IdathaDocumentWords - options - catch: ', options);
        return request_shared_1.SimpleGetRequest(options)
            .then((response) => {
            let content = { response: response, type: 'document' };
            return Promise.resolve(content);
        }).catch((err) => {
            logger.error('Ha sucedido el error: ' + err.message);
            return Promise.reject(err);
        });
    });
}
exports.IdathaDocumentWords = IdathaDocumentWords;
function IdathaPnl(accessToken, kind, words) {
    let idatha = new idatha_model_1.IdathaConnection();
    let options = {
        uri: `http://${idatha.ip}:${idatha.httpPort}/api/pnl/words`,
        body: {
            kind: kind,
            words: words
        },
        headers: {
            'Authorization': 'Bearer ' + accessToken
        }
    };
    return request_shared_1.SimplePostRequest(options)
        .then((response) => {
        let content = { response: response, type: 'pnl' };
        return Promise.resolve(content);
    }).catch((err) => {
        return Promise.reject(err);
    });
}
exports.IdathaPnl = IdathaPnl;
function IdathaDeletePnl(accessToken, words) {
    let idatha = new idatha_model_1.IdathaConnection();
    let options = {
        uri: `http://${idatha.ip}:${idatha.httpPort}/api/pnl/words`,
        body: {
            words: words
        },
        headers: {
            'Authorization': 'Bearer ' + accessToken
        }
    };
    return request_shared_1.SimpleDeleteRequest(options)
        .then((response) => {
        let content = { response: response, type: 'deletePnl' };
        return Promise.resolve(content);
    }).catch((err) => {
        return Promise.reject(err);
    });
}
exports.IdathaDeletePnl = IdathaDeletePnl;
function IdathaDocumentQueryFilter(accessToken, query, from, size, process, version = 1) {
    let idatha = new idatha_model_1.IdathaConnection();
    let body = null;
    let queryDataId = '';
    let currentDate = new Date();
    let queryDataIdDate = new Date(process.queryDataIdDate);
    let diffDays = Math.round(Math.abs((currentDate.getTime() - queryDataIdDate.getTime()) / (24 * 60 * 60 * 1000)));
    logger.info('diffDays: ', diffDays);
    if (diffDays >= config_1.IDATHA_QUERY_ID_EXPIRATION) {
        let queryProcess = semantic_query_implementation_1.GetQueryIdathaProcessV2(process.processFilters);
        logger.info('(IdathaDocumentQueryFilter) query_data_id caducó. Se envía consulta con filtros del proceso ' + process.name + ' ' + queryProcess);
        body = {
            query: queryProcess,
            from: from,
            size: 2,
            version: 2
        };
        logger.info('request con filter_data: ', body);
    }
    else {
        logger.info('(IdathaDocumentQueryFilter) query_data_id vigente. Hacemos la consulta con query_data_id : ' + process.name + ' ' + query);
        body = {
            query_id: process.queryDataId,
            query: query,
            from: from,
            size: size,
            version
        };
        logger.info('request con query_id: ', body);
    }
    let options = {
        method: 'POST',
        uri: `http://${idatha.ip}:${idatha.httpPort}/api/query`,
        body: body,
        headers: {
            'Authorization': 'Bearer ' + accessToken
        },
    };
    return request_shared_1.SimplePostRequest(options)
        .then((response) => {
        logger.info('total resultados filterdata: ', response.total_results);
        let content = { response: response, type: 'documentQuery' };
        if (diffDays >= config_1.IDATHA_QUERY_ID_EXPIRATION) {
            queryDataId = response.query_id;
            return mssql_service_1.MSSql.SaveProcess(process.vcc, process.name, process.description, process.clientId, process.status, process.type, process.startDate, process.endDate, process.minDurationTime, process.maxDurationTime, process.language, process.dictionaryId, process.environment, process.queryId, queryDataId, moment().utc().toDate(), process.batchScheduleId, process.createdDate, process.createdByUserId, process.lastModifiedDate, process.lastModifiedByUserId).then(result => {
                let body = {
                    query_id: queryDataId,
                    query: query,
                    from: from,
                    size: size
                };
                let options = {
                    method: 'POST',
                    uri: `http://${idatha.ip}:${idatha.httpPort}/api/query`,
                    body: body,
                    headers: {
                        'Authorization': 'Bearer ' + accessToken
                    }
                };
                logger.info('Proceso ' + process.name + ' ');
                logger.info('request con query_id: ', body);
                return request_shared_1.SimplePostRequest(options);
            }).then(response => {
                logger.info('total resultados filterdata con query: ', response.total_results);
                let content = { response: response, type: 'documentQuery' };
                return Promise.resolve(content);
            }).catch((err) => {
                console.log('Error: ', err);
                let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                logger.error('Error en IdathaDocumentQueryFilter: ' + process.name + ' ' + query + ' ' + errorMsg);
                return Promise.reject(err);
            });
        }
        else {
            logger.info('Se envía petición ' + options.method + ' a ' + options.uri + ' con query: ' + query + ' desde ' + from + ' hasta ' + size);
            return Promise.resolve(content);
        }
    }).catch((err) => {
        console.log("Falló consulta idatha:", body);
        console.log('Error: ', err);
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error en IdathaDocumentQueryFilter: ' + process.name + ' ' + query + ' ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.IdathaDocumentQueryFilter = IdathaDocumentQueryFilter;
function IdathaStatus(accessToken) {
    let idatha = new idatha_model_1.IdathaConnection();
    let options = {
        uri: 'http://' + idatha.ip + ':' + idatha.httpPort + '/api/admin/status',
        headers: {
            'Authorization': 'Bearer ' + accessToken
        }
    };
    return request_shared_1.SimpleGetRequest(options)
        .then((response) => {
        let content = { response: response, type: 'status' };
        return Promise.resolve(content);
    }).catch((err) => {
        console.log('Error: ', err);
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error en la consulta IDATHA: ' + '\': ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.IdathaStatus = IdathaStatus;
function UpdateStudyStatus(vcc, objData, status, id, errorMessage, lastModifiedDate, userId) {
    return mssql_service_1.MSSql.SaveStudyStatus(vcc, objData.study.processId, objData.template, objData.study.studyId, status.percentage, id, errorMessage, lastModifiedDate, userId)
        .then((resultSaveStudyStatus) => {
        console.log('Respuesta correcta, se actualizó...', 'resultSaveStudyStatus', resultSaveStudyStatus);
        return Promise.resolve(resultSaveStudyStatus);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al intentar actualizar el estado del estudio: ' + '\': ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateStudyStatus = UpdateStudyStatus;
function GetQueryModel(vcc, cognitiveService, accessToken, processAudios, themeModels, subscriptionRegion, subscriptionKey) {
    let arrayIntents = [];
    let modelsLUIS = [];
    let arrayQueries = [];
    let section = processAudios.section;
    section = section * 1;
    let part = processAudios.part;
    part = part * 1;
    return IdathaDocumentWords(accessToken, processAudios.audioId, section, part)
        .then(result => {
        let transcription = SortJson(result.response);
        logger.info('transcription: ', transcription);
        let phrases = nlu.validate(transcription);
        logger.info('phrases: ', phrases);
        return Promise.resolve(phrases);
    }).then(resultArray500 => {
        arrayQueries = resultArray500;
        logger.info('arrayQueries: ', arrayQueries);
        let models = [];
        for (let row of themeModels) {
            models.push(row.query.split('|')[0]);
        }
        models = uniqueNLU(models);
        logger.info('models: ', models);
        return mssql_service_1.MSSql.GetNLUsStudy(vcc, null, null, models.join('\',\''));
    }).then(result => {
        modelsLUIS = result;
        console.log('modelsLUIS', JSON.stringify(modelsLUIS));
        let promiseArray = [];
        if (arrayQueries && arrayQueries.length) {
            let timeOut = 0;
            arrayQueries.forEach((query, index) => {
                timeOut = index * Math.ceil(modelsLUIS.length / 5);
                logger.info('QueryLUIS: ', timeOut, index, query.content, processAudios.audioId, processAudios.section, processAudios.part);
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(QueryLUIS, timeOut, cognitiveService, query.content, modelsLUIS, subscriptionRegion, subscriptionKey, processAudios, resolve);
                }));
            });
        }
        return Promise.all(promiseArray)
            .then(results => {
            logger.info('QueryLUIS - promiseArray - results: ', JSON.stringify(results));
            return Promise.resolve(results);
        })
            .then((results) => {
            logger.info('QueryLUIS - promiseArray - results- GetQueryModel: ', JSON.stringify(results));
            return Promise.resolve(results);
        })
            .catch((err) => {
            logger.error('QueryLUIS - promiseArray - error- GetQueryModel: ', err);
            return Promise.reject(err);
        });
    }).then(results => {
        logger.info('result - 1226: ', JSON.stringify(results));
        return Promise.resolve(results);
    }).catch((err) => {
        logger.error('GetQueryModel - err: ' + err.message);
        return Promise.resolve(err);
    });
}
function uniqueNLU(models) {
    var result = [], i, len = models.length, model, index = -1;
    for (i = 0; i < len; i++) {
        model = models[i];
        index = result.findIndex(function (x) {
            return x === model;
        });
        if (index === -1) {
            result.push(model);
        }
    }
    return result;
}
function QueryLUIS(cognitiveService, query, nlus, subscriptionRegion, subscriptionKey, processAudios, resolve) {
    let promiseArray = [];
    if (nlus && nlus.length) {
        let timeOut = 0;
        logger.info('nlus: ', nlus);
        nlus.forEach((nlu, index) => {
            timeOut = index % 3 === 0 ? index * 1000 : timeOut;
            logger.info('timeOut: ', timeOut, index, nlu.appId, query, processAudios.audioId, processAudios.section, processAudios.part);
            promiseArray.push(new Promise(function (resolve, reject) {
                setTimeout(cognitiveService.ModelQuery, timeOut, subscriptionRegion, subscriptionKey, nlu.appId, query, processAudios, nlu.name, resolve);
            }));
        });
    }
    return Promise.all(promiseArray).then(results => {
        logger.info('cognitiveService.ModelQuery - results: ', JSON.stringify(results));
        console.log('result - luis: \n');
        return Promise.resolve(results);
    })
        .then((maxIntentData) => {
        logger.info('QueryLUIS - maxIntentData - Promise.all: ', JSON.stringify(maxIntentData));
        return resolve ? resolve(maxIntentData) : Promise.resolve(maxIntentData);
    })
        .catch((err) => {
        logger.info('QueryLUIS - error - Promise.all: ', err);
        return Promise.reject(err);
    });
}
function SortJson(jsonobj) {
    let outlength;
    let inlength;
    let i;
    let audioInit;
    let obj = {};
    obj.transcription = [];
    obj.summarySpeechData = {};
    obj.confidence = {};
    obj.init = '';
    let json = jsonobj;
    audioInit = json.words[0].speaker;
    obj.init = audioInit;
    obj.nlp = {};
    outlength = json.out_sentences.length;
    inlength = json.in_sentences.length;
    if (outlength >= inlength) {
        for (i in json.out_sentences) {
            if (i < inlength && inlength > 0) {
                if (audioInit === 'out' && inlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && inlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                        in: ''
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: '',
                        out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                    });
                }
            }
        }
    }
    else if (outlength <= inlength) {
        for (i in json.in_sentences) {
            if (i < outlength && outlength > 0) {
                if (audioInit === 'out' && outlength > 0) {
                    obj.transcription.push({
                        out: json.out_sentences[i].text,
                        in: json.in_sentences[i].text
                    });
                }
                else if (audioInit === 'in' && outlength > 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i].text,
                        out: json.out_sentences[i].text
                    });
                }
            }
            else {
                if (audioInit === 'out' && inlength >= 0) {
                    obj.transcription.push({
                        out: '',
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                    });
                }
                else if (audioInit === 'in' && inlength >= 0) {
                    obj.transcription.push({
                        in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        out: ''
                    });
                }
            }
        }
    }
    return obj;
}
let RecursiveRequestQueryFilter = (vcc, objData, userId, accessToken, themeItems, themeItemIndex, from, size, times, idathaArray = [], page, process) => {
    let objResult = {
        studyId: objData.study.studyId,
        idathaArray: idathaArray
    };
    let lastModifiedDate = moment().utc().toDate();
    let status = getPercentage(themeItems, themeItems[themeItemIndex].id);
    return UpdateStudyStatus(vcc, objData, status, themeItems[themeItemIndex].id, '', lastModifiedDate, userId)
        .then((resultUpdateStudyStatus) => {
        return IdathaDocumentQueryFilter(accessToken, themeItems[themeItemIndex].query, from, size, process).then((result) => {
            if (result.response.total_results > 0) {
                times = getTotalPages(result.response.total_results, size);
                console.log('RecursiveRequestQueryFilter:::: ', 'INDEX: ', themeItemIndex, themeItems[themeItemIndex].id, 'times: ', times, 'from:::: ', from, themeItems[themeItemIndex].query, result.response.total_results);
                idathaArray.push({
                    templateId: themeItems[themeItemIndex].templateId,
                    templateThemeId: themeItems[themeItemIndex].templateThemeId,
                    id: themeItems[themeItemIndex].id,
                    index: themeItemIndex,
                    query: themeItems[themeItemIndex].query,
                    from: from,
                    result: result
                });
                if (page === times) {
                    themeItemIndex++;
                    page = 1;
                    from = 0;
                }
                else {
                    from += size;
                    page++;
                }
            }
            else {
                themeItemIndex++;
                page = 1;
                from = 0;
            }
            times = 0;
            return (themeItemIndex < (themeItems && themeItems.length)) ? exports.RecursiveRequestQueryFilter(vcc, objData, userId, accessToken, themeItems, themeItemIndex, from, size, times, idathaArray, page, process) : Promise.resolve(objResult);
        }).catch(err => {
            console.log('Error: ', err);
            logger.error('Ha sucedido el error: ' + err.message);
            UpdateStudyStatus(vcc, objData, status, themeItems[themeItemIndex].id, err.message, lastModifiedDate, userId)
                .then((result) => {
                logger.info('UpdateStudyStatus Error: ', result);
                return Promise.resolve(result);
            })
                .catch((err) => {
                return Promise.reject(err);
            });
            return Promise.reject(err);
        });
    })
        .catch((err) => {
        logger.error('UpdateStudyStatus: ' + err.message);
        return Promise.reject(err);
    });
};
exports.RecursiveRequestQueryFilter = RecursiveRequestQueryFilter;
function RecursiveIdathaRequest(req, res, accessToken, inserted, pending) {
    logger.info('RecursiveIdathaRequest - inserted: ', inserted, 'pending: ', pending);
    return IdathaStatus(accessToken)
        .then((idathaResult) => {
        if (idathaResult.status === false) {
            console.log('Error');
            return Promise.resolve(0);
        }
        else {
            let status = 0;
            inserted = idathaResult.response.inserted_documents;
            pending = idathaResult.response.pending_documents;
            status = (inserted / (pending + inserted)) || 0;
            if (status < 1) {
                return RecursiveIdathaRequest(req, res, accessToken, inserted, pending);
            }
            else {
                return Promise.resolve(status);
            }
        }
    })
        .catch(err => {
        console.log('Error: ', err);
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al procesar IdathaStatus:' + errorMsg);
    });
}
exports.RecursiveIdathaRequest = RecursiveIdathaRequest;
function getTotalPages(totalItems, itemsPerPage) {
    var totalPages = itemsPerPage < 1 ? 1 : Math.ceil(totalItems / itemsPerPage);
    return Math.max(totalPages || 0, 1);
}
exports.getTotalPages = getTotalPages;
function saveStudyResultSummaryDetail(vcc, studyId, userId, idatha) {
    var len = idatha.results.length, obj, SQL = '', i;
    for (i = 0; i < len; i++) {
        obj = idatha.results[i];
        SQL += 'INSERT INTO [dbo].[StudyResultSummaryDetail]([VCC],[StudyId],[TemplateThemeId],[TemplateThemeItemId],[TranscriptionId],[CreatedDate],[CreatedByUserId],[TimeStamp])';
        SQL += 'VALUES(\'';
        SQL += vcc;
        SQL += '\', \'';
        SQL += studyId;
        SQL += '\', \'';
        SQL += idatha.templateThemeId;
        SQL += '\', \'';
        SQL += idatha.templateThemeItemId;
        SQL += '\', \'';
        SQL += obj.id;
        SQL += '\', GETUTCDATE(), \'';
        if (userId) {
            SQL += userId;
        }
        SQL += '\', GETUTCDATE()); ';
        SQL += '\n\n';
    }
    return SQL;
}
exports.saveStudyResultSummaryDetail = saveStudyResultSummaryDetail;
function processFinalResults(vcc, studyId, userId, idatha) {
    console.log(1183, idatha.results);
    var SQL = '', lenResults = idatha.results.length, lenWords, result, word, durationIn = 0, silenceIn = 0, durationOut = 0, silenceOut = 0, confidenceIn = 0, confidenceOut = 0, arrayConfidenceIn = [], arrayConfidenceOut = [], lenWordsIn = 0, lenWordsOut = 0, i, j;
    for (i = 0; i < lenResults; i++) {
        result = idatha.results[i];
        if (result && result.words && result.words.length > 0) {
            lenWords = result.words.length;
            for (j = 0; j < lenWords; j++) {
                word = result.words[j];
                if (word.speaker === 'in') {
                    durationIn += word.duration;
                    silenceIn += word.silence < 0 ? 0 : word.silence;
                    confidenceIn += word.confidence;
                }
                else {
                    durationOut += word.duration;
                    silenceOut += word.silence < 0 ? 0 : word.silence;
                    confidenceOut += word.confidence;
                }
            }
            lenWordsIn = result.words.filter(function (x) {
                return x.speaker === 'in';
            }).length;
            lenWordsOut = result.words.filter(function (x) {
                return x.speaker === 'out';
            }).length;
            if (lenWordsIn > 0) {
                confidenceIn = (confidenceIn / lenWordsIn);
                arrayConfidenceIn.push(confidenceIn);
            }
            if (lenWordsOut > 0) {
                confidenceOut = (confidenceOut / lenWordsOut);
                arrayConfidenceOut.push(confidenceOut);
            }
            SQL += 'INSERT INTO [dbo].[StudyDetail]([VCC],[StudyId],[TemplateThemeId],[TemplateThemeItemId],[TranscriptionId],[Total],[Duration],[Silence],[Confidence],[CreatedDate],[CreatedByUserId],[TimeStamp])';
            SQL += 'VALUES(\'';
            SQL += vcc;
            SQL += '\', \'';
            SQL += studyId;
            SQL += '\', \'';
            SQL += idatha.templateThemeId;
            SQL += '\', \'';
            SQL += idatha.templateThemeItemId;
            SQL += '\', \'';
            SQL += result.id;
            SQL += '\', 1, ';
            SQL += (durationIn + durationOut);
            SQL += ', ';
            SQL += (silenceIn + silenceOut);
            SQL += ', ';
            SQL += ((confidenceIn + confidenceOut) / 2);
            SQL += ', GETUTCDATE(), \'';
            if (userId) {
                SQL += userId;
            }
            SQL += '\', GETUTCDATE()); ';
            SQL += '\n\n';
            durationIn = 0;
            silenceIn = 0;
            durationOut = 0;
            silenceOut = 0;
        }
    }
    return SQL;
}
exports.processFinalResults = processFinalResults;
function getUniqueFinalResults(data) {
    function findDuplicate(x) {
        return x.templateThemeId === obj.templateThemeId;
    }
    var result = [], len = data.length, obj, item, i;
    for (i = 0; i < len; i++) {
        obj = data[i];
        item = result.find(findDuplicate);
        if (item) {
            item.results = item.results.concat(obj.results);
            item.results = item.results.filter(function (value, index, self) {
                return self.map(function (e) { return e.id; }).indexOf(value.id) === index;
            });
        }
        else {
            result.push(obj);
        }
    }
    return result;
}
exports.getUniqueFinalResults = getUniqueFinalResults;
function getPercentage(items, currentItem) {
    let result = null, index = items.findIndex(function (x) {
        return x.id === currentItem;
    });
    index += 1;
    result = (index * 1) / items.length;
    return {
        index: index,
        percentage: result
    };
}
exports.getPercentage = getPercentage;
var nlu = (function (o) {
    o.existInTemplateThemeItems = function (templateThemeItems, nlu, intents) {
        function filterIntents(x) {
            return x.id === [nlu, intent.intent].join('|');
        }
        var i, len = intents.length, intent, result, validIntents = [];
        for (i = 0; i < len; i++) {
            intent = intents[i];
            result = templateThemeItems.filter(filterIntents);
            if (result && result.length > 0) {
                validIntents.push(intent);
            }
        }
        return validIntents;
    };
    o.buildPhrases = function (words) {
        var i, len = words.length, word, phrase = '', phrasesArray = [], previousContent = '';
        for (i = 0; i < len; i++) {
            word = words[i];
            if (previousContent.length > 0) {
                phrase += previousContent;
                phrase += ' ';
            }
            phrase += word.name;
            phrase += ' ';
            if (phrase.length <= 500) {
                previousContent = '';
            }
            else {
                phrase = phrase.replace(word.name, '');
                previousContent = word.name;
                phrasesArray.push({ phrase: phrase, length: phrase.length, lastWord: word.name });
                phrase = '';
            }
        }
        phrasesArray.push({ phrase: phrase, length: phrase.length, lastWord: word.name });
        return phrasesArray;
    };
    o.validate = function (data) {
        var string_500 = [], phrases = '', len = data.transcription.length, item, phrasesResult = '', total = 0;
        for (var i = 0; i < len; i++) {
            item = data.transcription[i];
            if (data.init === 'out') {
                phrases += item.out;
                phrases += ' ';
                phrases += item.in;
            }
            else {
                phrases += item.in;
                phrases += ' ';
                phrases += item.out;
            }
            total = phrasesResult.length + phrases.length;
            if (total > 500) {
                string_500.push({ content: '', length: 0 });
                phrases = '';
                phrasesResult = '';
            }
            else {
                phrasesResult += phrases;
                if (string_500.length === 0) {
                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                }
                else {
                    string_500[string_500.length - 1].content += phrases;
                    string_500[string_500.length - 1].length = string_500[string_500.length - 1].content.length;
                }
            }
            phrases = '';
        }
        return string_500;
    };
    o.finalData = [];
    o.getData = function (intents) {
        function findIntent(x) {
            return x.intent === intentItem.intent;
        }
        if (intents && intents.length) {
            intents = intents.filter(function (x) {
                return x.intent !== 'None';
            });
        }
        else {
            return o.finalData;
        }
        if (o.finalData.length === 0) {
            o.finalData = intents;
        }
        else {
            var i, len = o.finalData.length, intentItem, foundIntent;
            for (i = 0; i < len; i++) {
                intentItem = o.finalData[i];
                foundIntent = intents.find(findIntent);
                if (foundIntent) {
                    intentItem.score = foundIntent.score > intentItem.score ? foundIntent.score : intentItem.score;
                }
            }
        }
        return o.finalData;
    };
    o.threshold = 0.00000005;
    o.getDataByThresholdMax = function (intents) {
        var result = [];
        result = intents.filter(function (x) {
            return x.score > o.threshold;
        });
        return result;
    };
    o.flatteningData = function (data, templateThemeItems) {
        function findInteraction(x) {
            return x.id === interactionItem.id && x.section === interactionItem.section && x.part === interactionItem.part;
        }
        function filterIntents(x) {
            return x.nlu === phraseItem.nlu;
        }
        function findIndex(x) {
            return x.nlu === phraseItem.nlu;
        }
        function mapPhrasesMaxIntents(x) {
            o.finalData = [];
            return {
                nlu: x.nlu,
                intents: x.intents.map(function (y) {
                    return o.getData(y);
                })[0]
            };
        }
        function mapPhrasesThresholdMax(x) {
            return {
                nlu: x.nlu,
                intents: o.getDataByThresholdMax(x.intents)
            };
        }
        function mapPhrasesExistInTemplateThemeItems(x) {
            return {
                nlu: x.nlu,
                intents: o.existInTemplateThemeItems(templateThemeItems, x.nlu, x.intents)
            };
        }
        var interactions = [], i, j, leni = data.length, lenj, interaction = null, interactionItem, index = -1, phrases;
        for (i = 0; i < leni; i++) {
            interaction = data[i];
            interaction = [].concat.apply([], interaction);
            lenj = interaction.length;
            for (j = 0; j < lenj; j++) {
                interactionItem = interaction[j];
                index = interactions.findIndex(findInteraction);
                if (index === -1) {
                    interactions.push({
                        id: interactionItem.id,
                        section: interactionItem.section,
                        part: interactionItem.part
                    });
                }
            }
        }
        leni = interactions.length;
        interactionItem = null;
        phrases = [];
        index = -1;
        var phraseItem, nlus = [];
        for (i = 0; i < leni; i++) {
            interactionItem = interactions[i];
            data[i] = [].concat.apply([], data[i]);
            phrases = data[i];
            nlus = [];
            if (phrases && phrases.length > 0) {
                lenj = phrases.length;
                for (j = 0; j < lenj; j++) {
                    phraseItem = phrases[j];
                    var intents = phrases.filter(filterIntents);
                    index = -1;
                    index = nlus.findIndex(findIndex);
                    if (index === -1) {
                        nlus.push({
                            nlu: intents[0].nlu,
                            intents: intents.map(function (x) {
                                return x.intents;
                            })
                        });
                    }
                }
                interactionItem.phrases = nlus;
                interactionItem.phrases = interactionItem.phrases.map(mapPhrasesMaxIntents);
                interactionItem.phrases = interactionItem.phrases.map(mapPhrasesThresholdMax);
                interactionItem.phrases = interactionItem.phrases.map(mapPhrasesExistInTemplateThemeItems);
            }
        }
        console.log(JSON.stringify(interactions));
        return interactions;
    };
    o.setInput = function (vcc, studyId, userId, interactions, templateThemeItems) {
        function filterTemplateThemeItems(x) {
            return x.id === [nluItem.nlu, intentItem.intent].join('|');
        }
        function findIndex(x) {
            return x.id === [interaction.id, interaction.section, interaction.part].join('_') && x.templateThemeId === themeItem.templateThemeId && x.templateThemeItemId === themeItem.templateThemeItemId;
        }
        var SQLResult = '';
        if (interactions && interactions.length > 0 && templateThemeItems && templateThemeItems.length > 0) {
            var idatha = { results: [] }, i, j, k, l, leni = interactions.length, lenj, lenk, lenl, interaction, nluItem, intentItem, themeItems = [], themeItem, index = -1;
            for (i = 0; i < leni; i++) {
                interaction = interactions[i];
                if (interaction.phrases && interaction.phrases.length > 0) {
                    lenj = interaction.phrases.length;
                    for (j = 0; j < lenj; j++) {
                        nluItem = interaction.phrases[j];
                        if (nluItem.intents && nluItem.intents.length > 0) {
                            lenk = nluItem.intents.length;
                            for (k = 0; k < lenk; k++) {
                                intentItem = nluItem.intents[k];
                                themeItems = templateThemeItems.filter(filterTemplateThemeItems);
                                if (themeItems && themeItems.length > 0) {
                                    lenl = themeItems.length;
                                    for (l = 0; l < lenl; l++) {
                                        themeItem = themeItems[l];
                                        index = idatha.results.findIndex(findIndex);
                                        if (index === -1) {
                                            idatha.results.push({
                                                id: [interaction.id, interaction.section, interaction.part].join('_'),
                                                templateThemeId: themeItem.templateThemeId,
                                                templateThemeItemId: themeItem.id
                                            });
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            SQLResult = o.saveStudyResultSummaryDetail(vcc, studyId, userId, idatha);
            SQLResult += o.processFinalResults(vcc, studyId, userId, idatha);
        }
        else {
            SQLResult = 'NO_INTERACTIONS_NO_TEMPLATE_THEME_ITEMS';
        }
        return SQLResult;
    };
    o.saveStudyResultSummaryDetail = function (vcc, studyId, userId, idatha) {
        var len = idatha.results.length, obj, SQL = '', i;
        for (i = 0; i < len; i++) {
            obj = idatha.results[i];
            SQL += 'INSERT INTO [dbo].[StudyResultSummaryDetail]([VCC],[StudyId],[TemplateThemeId],[TemplateThemeItemId],[TranscriptionId],[CreatedDate],[CreatedByUserId],[TimeStamp])';
            SQL += 'VALUES(\'';
            SQL += vcc;
            SQL += '\', \'';
            SQL += studyId;
            SQL += '\', \'';
            SQL += obj.templateThemeId;
            SQL += '\', \'';
            SQL += obj.templateThemeItemId;
            SQL += '\', \'';
            SQL += obj.id;
            SQL += '\', GETUTCDATE(), \'';
            if (userId) {
                SQL += userId;
            }
            SQL += '\', GETUTCDATE()); ';
            SQL += '\n\n';
        }
        return SQL;
    };
    o.processFinalResults = function (vcc, studyId, userId, idatha) {
        var SQL = '', lenResults = idatha.results.length, result, durationIn = 0, silenceIn = 0, durationOut = 0, silenceOut = 0, confidenceIn = 0, confidenceOut = 0, i;
        for (i = 0; i < lenResults; i++) {
            result = idatha.results[i];
            SQL += 'INSERT INTO [dbo].[StudyDetail]([VCC],[StudyId],[TemplateThemeId],[TemplateThemeItemId],[TranscriptionId],[Total],[Duration],[Silence],[Confidence],[CreatedDate],[CreatedByUserId],[TimeStamp])';
            SQL += 'VALUES(\'';
            SQL += vcc;
            SQL += '\', \'';
            SQL += studyId;
            SQL += '\', \'';
            SQL += result.templateThemeId;
            SQL += '\', \'';
            SQL += result.templateThemeItemId;
            SQL += '\', \'';
            SQL += result.id;
            SQL += '\', 1, ';
            SQL += (durationIn + durationOut) || 0;
            SQL += ', ';
            SQL += (silenceIn + silenceOut) || 0;
            SQL += ', ';
            SQL += ((confidenceIn + confidenceOut) / 2) || 0;
            SQL += ', GETUTCDATE(), \'';
            if (userId) {
                SQL += userId;
            }
            SQL += '\', GETUTCDATE()); ';
            SQL += '\n\n';
        }
        return SQL;
    };
    return o;
}(nlu || {}));
