"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveIntent = exports.GetIntent = exports.IntentService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const config_1 = require("../config/config");
const utterance_service_1 = require("../services/utterance.service");
const nlu_service_1 = require("../services/nlu.service");
let logger = log4js.getLogger('ServerScripts');
function IntentService(app) {
    let cognitiveService;
    app.get('/api/intent/get_intents/:app_id/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[IntentService::get_intents] Received ' + req.method + ' method on \'' + req.path + '\'');
        logger.info('[IntentService::/api/intent/get_intents/] getting app_id:', req.params.app_id, 'search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        logger.info('[IntentService::/api/intent/get_intents/] getting vcc:', vcc, 'azure:', azure);
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let app_id = req.params.app_id || '';
        let none;
        let arrayId = [];
        let stringId;
        let isSentimentVccSystem;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, app_id);
        })
            .then(result => {
            vcc = result.vcc;
            if (result.isSystemApp)
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            return cognitiveService.GetIntentsList(azure.subscriptionRegion, azure.subscriptionKey, app_id, vcc);
        })
            .then((result) => {
            logger.info('[IntentService::/api/intent/get_intents/] getting result:', result, 'of cognitiveService.GetIntentsList');
            none = result.filter(x => x.name = 'None');
            arrayId = result.map(a => a.id);
            stringId = arrayId.join('\',\'');
            logger.info('[IntentService::/api/intent/get_intents/] getting arrayId:', arrayId, 'stringId:', stringId);
            return mssql_service_1.MSSql.GetIntents(vcc, app_id, search_criteria, search_expression, stringId);
        })
            .then((intents) => {
            logger.info('[IntentService::/api/intent/get_intents/] getting intents:', intents, 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: intents });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/get_intents/] getting error:', err, 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlus_intents/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[IntentService::get_nlus_intents] Received ' + req.method + ' on \'' + req.path + '\'');
        logger.info('[IntentService::get_nlus_intents] getting search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        logger.info('[IntentService::get_nlus_intents] getting vcc:', vcc, 'azure:', azure);
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let apps_list;
        let nlus = [];
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_COGNITIVE_SERVICE_NOT_FOUND';
            cognitiveService = result;
            logger.info('[IntentService::get_nlus_intents] Calling GetAppsList from cognitive service', result);
            return cognitiveService.GetAppsList(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null);
        }).then((result) => {
            logger.info('[IntentService::get_nlus_intents] getting ' + result.length + ' result from cognitiveService.GetIntentsList');
            let arrayId = [], stringId = '';
            if (result && result.length) {
                apps_list = result;
                arrayId = result.map(a => a.id);
                stringId = arrayId.join('\',\'');
            }
            let isSystemApp = false;
            if (cognitiveService.GetCognitiveEngineName() === 'RASA' || cognitiveService.GetCognitiveEngineName() === 'RASAAPI')
                return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp, true, config_1.USE_RASA_API);
            return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp);
        }).then((nlusResult) => {
            nlus = nlusResult ? nlusResult.filter(nr => nr && nr.isPublished) : [];
            logger.info('[IntentService::get_nlus_intents] getting ' + nlus.length + ' nlus from MSSql.GetNLUs');
            let arrayAppId = nlus.map(a => a.appId);
            logger.info('[IntentService::get_nlus_intents] getting arrayAppId: ' + arrayAppId.length);
            let promiseArray = [];
            let timeOut = 0;
            arrayAppId.forEach((app_id, index) => {
                if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                    timeOut = Math.floor(index / 5) * 1500;
                }
                else {
                    timeOut = 0;
                }
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(() => {
                        return resolve(cognitiveService.GetIntentsListDemo(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, app_id, null, null, vcc));
                    }, timeOut);
                }));
            });
            return Promise.all(promiseArray);
        }).then(results => {
            logger.info('[IntentService::get_nlus_intents] getting results: ' + results.length + 'after Promise.all');
            let intentArray = [];
            results.forEach(function (object, index) {
                object.forEach(element => {
                    if (element.name !== 'None') {
                        element.appId = nlus[index].appId;
                        element.appName = nlus[index].name;
                        element.culture = nlus[index].culture;
                        intentArray.push(element);
                    }
                });
            });
            logger.info('[IntentService::get_nlus_intents] getting results:', results.length, 'after results.forEach');
            return Promise.resolve(intentArray);
        }).then((intentArray) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: intentArray });
        }).catch(err => {
            if (typeof err === 'string' && err === 'SERVER_ERROR_COGNITIVE_SERVICE_NOT_FOUND') {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: [] });
            }
            else {
                let clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
                let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
                logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
                logger.error('[NLUService::get_nlus_intents] getting errorMsg:', errorMsg);
                if (res.headersSent)
                    return;
                res.json({ status: false, description: clientMsg, error: clientMsg });
            }
        });
    });
    app.get('/api/intent/get_intent/:intent_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        logger.info('[IntentService::/api/intent/get_intent/] getting intent_id:', req.params.intent_id);
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        logger.info('[IntentService::/api/intent/get_intent/] getting vcc:', vcc, 'azure', azure);
        let intentId = req.params.intent_id;
        let intent;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!intentId)
                throw 'MISSING_INTENT_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetIntent(vcc, intentId);
        })
            .then((resultIntent) => {
            logger.info('[IntentService::/api/intent/get_intent/] getting resultIntent:', resultIntent, 'after of MSSql.GetIntent');
            intent = resultIntent;
            return Promise.resolve(true);
        })
            .then(result => {
            return intent;
        })
            .then(intent => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: intent });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/get_intent/] getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/create_intent', (req, res, next) => {
        logger.info('[IntentService::create_intent] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let isSystemApp;
        let newIntent = (req.body || {});
        newIntent.createdDate = nowUtc;
        newIntent.createdByUserId = userId;
        newIntent.lastModifiedDate = nowUtc;
        newIntent.lastModifiedByUserId = userId;
        logger.info('[IntentService::create_intent] getting newIntent:', newIntent);
        let cognitiveEngineName = '';
        let nlu = null;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, newIntent.appId);
        })
            .then(result => {
            nlu = result;
            if (result.vcc === 'system') {
                nlu = result;
                nlu.vcc = vcc;
                nlu.appId = vcc + '::' + nlu.name;
            }
            isSystemApp = result.isSystemApp;
            if (isSystemApp)
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            if (nlu.isSystemApp) {
                if (nlu.cognitiveEngine === 'RASA')
                    return utterance_service_1.CreateSystemAppIfNotExists(cognitiveService, nlu.vcc, nlu);
                return utterance_service_1.CreateRasaApiSystemAppIfNotExists(cognitiveService, nlu.vcc, nlu);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            if (typeof result === "string") {
                nlu.appId = result;
                newIntent.appId = result;
            }
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            return nlu_service_1.SaveNLU(nlu.vcc, nlu);
        })
            .then(result => {
            return mssql_service_1.MSSql.GetIntentByAppId(nlu.vcc, newIntent.appId, newIntent.name);
        })
            .then((resultIntent) => {
            logger.info('[IntentService::create_intent] getting result:', resultIntent, 'of MSSql.GetIntentByAppId');
            if (resultIntent)
                throw 'INTENT_ALREADY_EXIST ' + resultIntent.name;
            return cognitiveService.AddIntent(azure.subscriptionRegion, azure.subscriptionKey, newIntent.appId, newIntent.name);
        })
            .then((result) => {
            logger.info('[IntentService::create_intent] getting result:', result, 'after of cognitiveService.AddIntent');
            newIntent.intentId = result;
            if (!isSystemApp)
                newIntent.color = utilities_1.getRandomHexColor();
            return SaveIntent(nlu.vcc, newIntent);
        })
            .then((resultSaveIntent) => {
            logger.info('[IntentService::create_intent] getting resultSaveIntent:', resultSaveIntent, 'after of SaveIntent');
            if (!resultSaveIntent)
                throw 'ERROR_ON_SAVE';
            return resultSaveIntent;
        })
            .then(resultSaveIntent => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveIntent ? 'SERVER_SUCCESS_MESSAGE' : 'INTENT_SAVE_FAILED'), data: resultSaveIntent });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::create_intent] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/save_intent', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newIntent = (req.body || {});
        newIntent.createdDate = null;
        newIntent.createdByUserId = null;
        newIntent.lastModifiedDate = nowUtc;
        newIntent.lastModifiedByUserId = userId;
        logger.info('[IntentService::/api/intent/save_intent] getting newIntent:', newIntent);
        promises_shared_1.DummyPromise()
            .then(result => {
            return SaveIntent(vcc, newIntent);
        })
            .then(resultSaveIntent => {
            logger.info('[IntentService::/api/intent/save_intent] getting resultSaveIntent:', resultSaveIntent, 'after of SaveIntent');
            if (!resultSaveIntent)
                throw 'ERROR_ON_SAVE';
            return Promise.resolve(true);
        })
            .then(resultSaveIntent => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveIntent, description: (resultSaveIntent ? 'SERVER_SUCCESS_MESSAGE' : 'INTENT_SAVE_FAILED'), data: resultSaveIntent });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/save_intent] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/delete_intent', (req, res, next) => {
        logger.info('[IntentService::delete_intent] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let intent = req.body.intent;
        let appId = intent.appId;
        let intentId = intent.intentId;
        let appIdRasaApi = appId;
        logger.info(`[IntentService::delete_intent] Deleting intent ${intentId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(result => {
            vcc = result.vcc;
            if (result.isSystemApp)
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (!intent)
                throw 'MISSING_INTENT_INPUT_PARAMETER';
            return cognitiveService.DeleteAppIntent(azure.subscriptionRegion, azure.subscriptionKey, intent.appId, intent.intentId, vcc);
        })
            .then(resultDeleteIntent => {
            logger.info('[IntentService::delete_intent] getting resultDeleteIntent:', resultDeleteIntent, 'after of cognitiveService.DeleteAppIntent');
            if (!intent)
                throw 'MISSING_INTENT_INPUT_PARAMETER';
            return mssql_service_1.MSSql.DeleteIntent(vcc, appId, intentId);
        })
            .then(resultDeleteIntent => {
            logger.info('[IntentService::delete_intent] getting resultDeleteIntent:', resultDeleteIntent, 'after of MSSql.DeleteIntent');
            if (!resultDeleteIntent)
                throw 'INTENT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        })
            .then(resultDeleteIntent => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteIntent, description: (resultDeleteIntent ? 'SERVER_SUCCESS_MESSAGE' : 'INTENT_SERVER_ERROR_ON_DELETE'), data: resultDeleteIntent });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::delete_intent] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/intent/delete_intents', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let intents = req.body.intents || [];
        logger.info(`[IntentService::delete_intents] Deleting intents ${intents.map(x => x.name).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!intents)
                throw 'MISSING_INTENTS_INPUT_PARAMETER';
            let promiseArray = [];
            intents.forEach(intent => {
                promiseArray.push(mssql_service_1.MSSql.DeleteIntent(vcc, intent.name));
            });
            return Promise.all(promiseArray);
        })
            .then(resultDeleteIntents => {
            logger.info('[IntentService::/api/intent/delete_intents] getting resultDeleteIntents:', resultDeleteIntents);
            let result;
            if (resultDeleteIntents.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('[IntentService::/api/intent/delete_intents] could not delete all requested items');
            }
            else {
                result = true;
                logger.info('[IntentService::/api/intent/delete_intents] intents removed successfully');
            }
            return result;
        })
            .then(resultDeleteIntents => {
            logger.error('[IntentService::/api/intent/delete_intents] getting resultDeleteIntents:', resultDeleteIntents);
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteIntents, description: (resultDeleteIntents ? 'SERVER_SUCCESS_MESSAGE' : 'INTENTS_DELETE_FAILED'), data: resultDeleteIntents });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[IntentService::/api/intent/delete_intents] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.IntentService = IntentService;
function GetIntent(vcc, name) {
    logger.info('[IntentService::GetIntent] getting vcc:', vcc, 'name:', name);
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetIntent(vcc, name);
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
        logger.error('[IntentService::GetIntent] getting errMsg:', errMsg);
    });
}
exports.GetIntent = GetIntent;
function SaveIntent(vcc, intent) {
    logger.info('[IntentService::SaveIntent] getting vcc:', vcc, 'intent:', intent);
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveIntent(vcc, intent.name, intent.intentId, intent.appId, intent.color, intent.createdDate, intent.createdByUserId, intent.lastModifiedDate, intent.lastModifiedByUserId));
    return Promise.all(promiseArray)
        .then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveIntent = SaveIntent;
