"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionTypeService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const vcc_service_1 = require("./vcc.service");
const promises_shared_1 = require("../shared/promises.shared");
const http_request_occ_1 = require("../shared/http-request-occ");
const redis_service_1 = require("../db/redis.service");
let logger = log4js.getLogger('ServerScripts');
function InteractionTypeService(app) {
    app.post('/api/interaciontype/get_interactiontypes', (req, res, next) => {
        logger.info('[InteractionTypeService::get_interactiontypes] Received ' + req.method + ' method for \'' + req.path + '\'');
        let { vcc, type } = res.locals;
        let campaigns = req.body.campaigns || [];
        let channels = [];
        let isI6OrOcc = type === 'I6' || type === 'OCC' ? true : false;
        let dataRedisVcc;
        logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes for vcc: ' + vcc);
        logger.info('[InteractionTypeService::get_interactiontypes] Requesting interactionTypes for campaigns: ' + campaigns);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetVCC(vcc, vcc);
        })
            .then((redisVCC) => {
            dataRedisVcc = redisVCC;
            if (dataRedisVcc?.channels?.length) {
                channels = redisVCC.channels;
                return Promise.resolve([]);
            }
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetInteractionTypes(vcc, null, null));
            if (isI6OrOcc)
                promiseArray.push(http_request_occ_1.ApiI6ConfiguredChannels(vcc, campaigns));
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (channels.length)
                return Promise.resolve(true);
            channels = result[0]?.length ? result[0] : [];
            if (isI6OrOcc && Array.isArray(result[1].body)) {
                let arrayInteractionTypeAdd = [];
                result[1].body.forEach(insApi => {
                    if (!channels.find(insDb => insDb.id === insApi.id)) {
                        arrayInteractionTypeAdd.push(insApi);
                        channels.push(insApi);
                    }
                });
                return vcc_service_1.SaveInteractionTypes(arrayInteractionTypeAdd, vcc);
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            if (dataRedisVcc?.channels?.length) {
                return Promise.resolve([]);
            }
            dataRedisVcc.channels = channels;
            return redis_service_1.Redis.SaveVCC(vcc, dataRedisVcc);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[InteractionTypeService::get_interactiontypes] Received interactionTypes for instance: ' + vcc);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: channels });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[InteractionTypeService::get_interactiontypes] Error \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.InteractionTypeService = InteractionTypeService;
