"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportAgents = exports.GetInteractionsSample = exports.BeginProcess = exports.StartProcess = exports.jsonToQuery = exports.getInteractionsByTemplateNewStructure = exports.getInteractionsByTemplate = exports.LinguisticService = void 0;
const moment = require("moment");
const log4js = require("log4js");
let alasql = require('alasql');
const idatha_service_1 = require("./idatha.service");
const process_service_1 = require("./process.service");
const mssql_service_1 = require("../db/mssql.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const redis_service_1 = require("../db/redis.service");
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const interaction_service_1 = require("./interaction.service");
const platform_service_1 = require("./platform.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const allegro_service_1 = require("./allegro.service");
const http_shared_1 = require("../shared/http.shared");
let logger = log4js.getLogger('ServerScripts');
let objLinguisticProcessData;
function LinguisticService(app) {
    app.post('/api/linguistic/begin_process', (req, res, next) => {
        logger.info('[LinguisticService::begin_process] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let param = (req.body || {});
        let currentQueueId = '';
        logger.info('[LinguisticService::begin_process] getting vcc:', vcc, 'userId:', userId, 'nowUtc:', nowUtc, 'param:', param, 'currentQueueId:', currentQueueId);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!param.processId)
                throw 'MISSING_PROCESS_INPUT_PARAMTER';
            return StartProcess(vcc, param.processId, userId);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[LinguisticService::begin_process] with param:', param, 'an error has occured in:', req.path, 'with error meesage:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/linguistic/begin_query', (req, res, next) => {
        logger.info('[LinguisticService::begin_query] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let param = (req.body || {});
        let currentQueueId = '';
        logger.info('[LinguisticService::begin_query] getting vcc:', vcc, 'userId:', userId, 'nowUtc:', nowUtc, 'param:', param, 'currentQueueId:', currentQueueId);
        promises_shared_1.DummyPromise()
            .then((result) => {
            currentQueueId = vcc;
            currentQueueId += ':';
            currentQueueId += param.processId;
            currentQueueId += ':';
            currentQueueId += param.queryId;
            return redis_service_1.Redis.CreateQueueL('SemanticQuery', currentQueueId);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[LinguisticService::/api/linguistic/begin_query with param:', param, 'an error has occured in:', req.path, 'with error meesage:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/linguistic/create_linguistic_process_simple', (req, res, next) => {
        logger.info('[LinguisticService::create_linguistic_process_simple] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let type = res.locals.type;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let objLinguisticProcess = (req.body || {});
        let processId = objLinguisticProcess.processId;
        objLinguisticProcessData = objLinguisticProcess;
        let objProcess = ({});
        objProcess.vcc = vcc;
        objProcess.name = objLinguisticProcess.processId;
        objProcess.retranscription = objLinguisticProcess.retranscription;
        objProcess.diarization = objLinguisticProcess.diarization;
        objProcess.channelchange = objLinguisticProcess.channelchange;
        objProcess.percentage = objLinguisticProcess.evaluatePercentage;
        if (objProcess.percentage > 100)
            objProcess.percentage = objProcess.percentage % 100;
        if (objProcess.percentage > 0)
            objProcess.percentage = objProcess.percentage / 100;
        objProcess.createdDate = nowUtc;
        objProcess.createdByUserId = userId;
        let channelInteractions = [];
        let totalSyncInteraction = 0;
        let audios = [];
        let textos = [];
        let asyncInteractions = [];
        let totalAsyncInteraction = 0;
        let totalAsyncDetail = 0;
        objProcess.queryDataId = null;
        objProcess.queryDataIdDate = null;
        objProcess.minDurationTime = 0;
        objProcess.maxDurationTime = 0;
        objProcess.type = objLinguisticProcess.type.toUpperCase();
        objProcess.language = objLinguisticProcess.language;
        objProcess.dictionaryId = objLinguisticProcess.dictionaryId;
        objProcess.environment = objLinguisticProcess.environmentId ? objLinguisticProcess.environmentId : null;
        objProcess.environmentType = objLinguisticProcess.environmentType ? objLinguisticProcess.environmentType : null;
        if (objLinguisticProcess.queryId)
            objProcess.queryId = objLinguisticProcess.queryId;
        let params = {};
        params = interaction_service_1.GetInteractionSearchFromRequest(objLinguisticProcess.evaluationFormatId, req.body.interactionSearch, false);
        params.searchType = 'TO_OPERATE';
        let totalInteractionsToSave = 0;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetProcessByName(vcc, objProcess.name);
        })
            .then(result => {
            if (result)
                throw 'PROCESS_ALREADY_EXIST \'' + objProcess.name + '\'';
            if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                logger.info('[LinguisticService::create_linguistic_process_simple] Searching evaluation format\'s campaigns');
                return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType)) {
                if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                    if (!result || result.length === 0)
                        throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                    params.campaigns = result.map(r => r.id);
                }
            }
            return Promise.resolve(true);
        }).then(result => {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(params.environmentType)) {
                return Promise.resolve(true);
            }
            else {
                let ftpCSVRedisKey;
                if (params.evaluationFormatId)
                    ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                else
                    ftpCSVRedisKey = 'FTP_PROCESS:';
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
            }
        }).then(result => {
            if (['ftp', 'sftp', 's3'].includes(params.environmentType))
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            let promiseArray = [];
            logger.info('[LinguisticService::create_linguistic_process_simple] Requesting not evaluated interactions from HistoricalData');
            promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, type, false, 'DETAILED', params, result.data);
            return Promise.all(promiseArray);
        })
            .then(result => {
            logger.info('[LinguisticService::create_linguistic_process_simple] Getting result for ' + result.length + ' channels');
            if (!result || result.length === 0)
                throw 'SERVER_ERROR_NO_INTEGRATED_DATA_FOUND';
            let processAgentIds = [];
            channelInteractions = result;
            channelInteractions.map(ci => {
                if (ci) {
                    if (ci.interactionType === 'CALL') {
                        totalInteractionsToSave += ci.totalInteractions;
                        ci.interactions.map(a => {
                            if (a.Agent)
                                processAgentIds.push(a.Agent);
                        });
                    }
                    else {
                        totalInteractionsToSave += ci.totalInteractions;
                        ci.interactions.map(a => {
                            if (a.firstTakedAgent)
                                processAgentIds.push(a.firstTakedAgent);
                            if (a.lastTakedAgent)
                                processAgentIds.push(a.lastTakedAgent);
                        });
                    }
                }
            });
            totalInteractionsToSave = Math.round(totalInteractionsToSave * objProcess.percentage);
            processAgentIds = arrays_shared_1.GetUniqueFromArray(processAgentIds);
            if (type !== 'OTHER')
                return ImportAgents(vcc, type, processAgentIds);
            else
                return Promise.resolve(true);
        })
            .then(result => {
            logger.info('[ProcessService::create_process] Getting result:', result, 'after bulk agents');
            let interactionBulkPromiseArray = [];
            let max = null;
            let min = null;
            let hasInteractionIds = params.interactionIds.length > 0;
            if (hasInteractionIds) {
                params.hourIni = "00:00:00";
                params.hourFin = "23:59:59";
            }
            channelInteractions.map(ci => {
                if (ci && ci.totalInteractions) {
                    if (ci.interactionType === 'CALL') {
                        let iv = {};
                        iv = ci;
                        let idTitle = 'InteractionId';
                        let sampleResult = GetInteractionsSample(iv.interactions, objProcess.percentage, idTitle, totalInteractionsToSave);
                        totalSyncInteraction = sampleResult.sampleInteractionIds.length;
                        audios = sampleResult.sampleInteractionDetails;
                        totalInteractionsToSave -= sampleResult.sampleInteractionIds.length;
                        interactionBulkPromiseArray.push(mssql_service_1.MSSql.BulkAudioTemp(vcc, objProcess, audios, userId));
                        if (hasInteractionIds) {
                            if (!max) {
                                max = new Date(audios[0].EndDate.substring(0, 10));
                                min = max;
                            }
                            for (let a of audios) {
                                let d = new Date(a.EndDate.substring(0, 10));
                                if (d > max)
                                    max = d;
                                if (d < min)
                                    min = d;
                            }
                        }
                    }
                    else {
                        let aiv = {};
                        aiv = ci;
                        asyncInteractions = [];
                        let sampleResult = GetInteractionsSample(aiv.interactions, objProcess.percentage, 'id', totalInteractionsToSave);
                        totalAsyncInteraction += sampleResult.sampleInteractionIds.length;
                        totalAsyncDetail += sampleResult.sampleInteractionDetails.length;
                        asyncInteractions = sampleResult.sampleInteractionDetails;
                        totalInteractionsToSave -= sampleResult.sampleInteractionIds.length;
                        interactionBulkPromiseArray.push(mssql_service_1.MSSql.BulkAsyncInteractionTemp(vcc, objProcess.name, asyncInteractions, userId));
                        if (hasInteractionIds) {
                            if (!max) {
                                if (typeof asyncInteractions[0].endDate === 'string')
                                    asyncInteractions[0].endDate = new Date(asyncInteractions[0].endDate);
                                max = new Date(asyncInteractions[0].endDate.toISOString().substring(0, 10));
                                min = max;
                            }
                            for (let a of asyncInteractions) {
                                if (typeof a.endDate === 'string')
                                    a.endDate = new Date(a.endDate);
                                let d = new Date(a.endDate.toISOString().substring(0, 10));
                                if (d > max)
                                    max = d;
                                if (d < min)
                                    min = d;
                            }
                        }
                    }
                }
            });
            if (hasInteractionIds) {
                params.dateFin = moment(`${max.toISOString().substring(0, 10)} ${params.hourFin}`).utc().toDate();
                params.dateIni = moment(`${min.toISOString().substring(0, 10)} ${params.hourIni}`).utc().toDate();
            }
            return Promise.all(interactionBulkPromiseArray);
        })
            .then(result => {
            logger.info('[ProcessService::create_process] Getting result:', result, 'after BulkAudioTemp and BulkAsyncInteractionTemp');
            let summary = [];
            let audioSummary = alasql('SELECT Category1, Category2, Category3, Description, COUNT(*) AS Total FROM ? WHERE Description <> \'\' GROUP BY Category1, Category2, Category3, Description', [audios]);
            return mssql_service_1.MSSql.BulkAudioSummaryTemp(vcc, objProcess, audioSummary, userId);
        })
            .then(result => {
            logger.info('[LinguisticService::create_linguistic_process_simple] Getting result:', result, 'after MSSql.BulkAudioSummaryTemp');
            objProcess.processFilters = interaction_service_1.GetProcessFilterFromInteractionSearch(objProcess.name, params, userId);
            return process_service_1.SaveProcess(vcc, objProcess);
        })
            .then((result) => {
            if (result) {
                logger.info('[LinguisticService::create_linguistic_process_simple] Adding into process queue.');
                return Promise.resolve(true);
            }
            else {
                return Promise.resolve(false);
            }
        })
            .then((result) => {
            logger.info('[LinguisticService::create_linguistic_process_simple]  getting result:', result, 'after SaveProcess');
            if (!result)
                throw 'CANNOT_PROCESS_BEGIN';
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'PROCESS_SAVE_FAILED'), data: result });
        }).then(result => {
            if (res.headersSent)
                return;
            res.json(result);
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[LinguisticService::create_linguistic_process_simple] Error al procesar \'' + req.path + '\':' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/linguistic/update_linguistic_process', (req, res, next) => {
        req.setTimeout(7.2e+6, () => {
        });
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let status = '';
        let accessToken = '';
        let oUpdateLinguisticProcess = (req.body || {});
        logger.info('Parámetro oUpdateLinguisticProcess:', oUpdateLinguisticProcess);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!oUpdateLinguisticProcess.processId)
                throw 'MISSING_PROCESSID_PARAMETER';
            if (!oUpdateLinguisticProcess.interactionIds)
                throw 'MISSING_INTERACTIONIDS_PARAMETER';
            return mssql_service_1.MSSql.DeleteProcessAudioByProcessIdInteractionId(vcc, oUpdateLinguisticProcess.processId, oUpdateLinguisticProcess.interactionIds.map((x) => {
                return x.id;
            }).join(','), '1')
                .then((result) => {
                return Promise.resolve(true);
            })
                .catch((err) => {
                logger.error('DeleteProcessAudioByProcessIdInteractionId error:', err);
                return Promise.reject(err);
            });
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\':' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/linguistic/get_linguistic_process_result/:process_id', (req, res, next) => {
        req.setTimeout(7.2e+6, () => {
        });
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let processId = req.params.process_id;
        let userName = '';
        let password = '';
        let message = '';
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!processId)
                throw 'MISSING_PROCESSID_PARAMETER';
            return mssql_service_1.MSSql.GetLinguisticProcessStatus(vcc, processId);
        }).then((result) => {
            if (!result)
                throw 'PROCESS_NOT_FOUND';
            let processStatus = result.status;
            if (processStatus === 'COMPLETE') {
                let queryId = result.queryId;
                if (queryId) {
                    let query = '';
                    return mssql_service_1.MSSql.GetIdathaAuthByRealm(vcc, vcc)
                        .then((result) => {
                        if (result && result.userName) {
                            userName = result.userName;
                            password = result.password;
                            return mssql_service_1.MSSql.GetUserAccessTokenByRealm(vcc, vcc)
                                .then((result) => {
                                if (result.accessToken) {
                                    let accessToken = result.accessToken;
                                    let currentDate = new Date(), createdDate = new Date(result.createdDate), diffDays = Math.round(Math.abs((currentDate.getTime() - createdDate.getTime()) / (24 * 60 * 60 * 1000)));
                                    if (diffDays >= 7) {
                                        return idatha_service_1.IdathaLogin(vcc)
                                            .then((result) => {
                                            accessToken = result;
                                            return accessToken;
                                        })
                                            .catch((err) => {
                                            return Promise.reject(err);
                                        });
                                    }
                                    else {
                                        return accessToken;
                                    }
                                }
                                else {
                                    return Promise.resolve(false);
                                }
                            }).then((accessToken) => {
                                if (accessToken) {
                                    return mssql_service_1.MSSql.GetGenericQuery(vcc, queryId)
                                        .then((result) => {
                                        if (!result || result === null) {
                                            message = 'QUERY_NOT_FOUND';
                                            return mssql_service_1.MSSql.GetAudios(vcc, processId)
                                                .then((result) => {
                                                let audios = [], len = result.length, obj;
                                                for (let c = 0; c < len; c++) {
                                                    obj = result[c];
                                                    audios.push({
                                                        id: obj.id,
                                                        section: obj.section,
                                                        part: obj.part
                                                    });
                                                }
                                                return Promise.resolve({ interactions: audios });
                                            });
                                        }
                                        else {
                                            message = 'QUERY_EXECUTED';
                                            query = result.query;
                                            let from = 0, size = 15, times = 0, page = 1;
                                            return Promise.resolve(RecursiveRequestGetLinguisticProcessResult(accessToken, query, from, size, times, [], page, null));
                                        }
                                    }).then((result) => {
                                        return Promise.resolve(result);
                                    }).catch((err) => {
                                        return Promise.reject(err);
                                    });
                                }
                                else {
                                    return Promise.reject('IDATHA_ACCESS_TOKEN_ERROR.');
                                }
                            }).catch((err) => {
                                return Promise.reject(err);
                            });
                        }
                    });
                }
                else {
                    return mssql_service_1.MSSql.GetAudios(vcc, processId)
                        .then((result) => {
                        let audios = [], len = result.length, obj;
                        for (let c = 0; c < len; c++) {
                            obj = result[c];
                            audios.push({
                                id: obj.id,
                                section: obj.section,
                                part: obj.part
                            });
                        }
                        return Promise.resolve({ interactions: audios });
                    });
                }
            }
            else {
                return Promise.reject('INCOMPLETE_PROCESS');
            }
        }).then(resultGetLinguisticProcessResult => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', message: message, data: resultGetLinguisticProcessResult });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\':' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/linguistic/get_linguistic_process_status/:process_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let processId = req.params.process_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!processId)
                throw 'MISSING_PROCESSID_PARAMETER';
            return mssql_service_1.MSSql.GetLinguisticProcessStatus(vcc, processId);
        }).then((process) => {
            if (process) {
                return Promise.resolve(process);
            }
            else {
                return Promise.reject('PROCESS_NOT_FOUND');
            }
        }).then(resultGetLinguisticProcessStatus => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGetLinguisticProcessStatus });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\':' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/linguistic/get_execute_template_process_status', (req, res, next) => {
        req.setTimeout(7.2e+6, () => {
        });
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let objData = (req.body || {});
        let studyId;
        studyId = 'Quality_';
        studyId += objData.processId;
        studyId += '_';
        studyId += objData.templateId;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!objData.processId)
                throw 'MISSING_PROCESSID_PARAMETER';
            if (!objData.templateId)
                throw 'MISSING_TEMPLATEID_PARAMETER';
            return mssql_service_1.MSSql.GetLinguisticProcessStatus(vcc, objData.processId);
        })
            .then((result) => {
            if (!result)
                throw 'PROCESS_NOT_FOUND';
        })
            .then((result) => {
            return mssql_service_1.MSSql.GetTemplate('system', objData.templateId);
        }).then(result => {
            if (result)
                return result;
            return mssql_service_1.MSSql.GetTemplate(vcc, objData.templateId);
        })
            .then((result) => {
            if (!result)
                throw 'TEMPLATE_NOT_FOUND';
        })
            .then((result) => {
            return mssql_service_1.MSSql.GetStudy(vcc, studyId);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            if (result) {
                result = {
                    processId: result.processId,
                    templateId: result.templateId,
                    status: result.status,
                    lastQueryId: result.lastQueryId,
                    errorMessage: result.errorMessage
                };
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\':' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.LinguisticService = LinguisticService;
function SaveSpeechMaticsJSONFile(fileName, fileNameString, data, vcc, processId, userId) {
    logger.info('SaveSpeechMaticsJSONFile - fileName:', fileName);
    return file_manager_1.WriteFileContent(fileName, data, 'utf-8', true)
        .then((result) => {
        logger.info('SaveSpeechMaticsJSONFile - result:', result);
        if (result) {
            data = JSON.parse(data);
            let durationLinguisticTime = data.job.duration;
            let isTranscribed = 1;
            logger.info('fileNameString:', fileNameString);
            let array = fileNameString.split('_');
            logger.info('array____:', array);
            if (array && array.length > 0) {
                let id = array[0];
                let section = array[1];
                let part = array[2].replace('.json', '');
                let lastModifiedDate = moment().utc().toDate();
                let lastModifiedByUserId = userId;
                logger.info('MSSql.SaveAudioLinguisticProcess:', vcc, id, section, part, processId, durationLinguisticTime, isTranscribed);
                return mssql_service_1.MSSql.SaveAudioLinguisticProcess(vcc, id, section, part, processId, durationLinguisticTime, isTranscribed, lastModifiedDate, lastModifiedByUserId);
            }
        }
    })
        .then((result) => {
        return Promise.resolve(result);
    })
        .catch((err) => {
        logger.error('SaveSpeechMaticsJSONFile:', err);
        return Promise.reject(err);
    });
}
let RecursiveRequestGetLinguisticProcessResult = (accessToken, query, from, size, times, interactions = [], page, objProcess) => {
    let objResult = {
        interactions: interactions
    };
    return idatha_service_1.IdathaDocumentQueryFilter(accessToken, query, from, size, objProcess).then((result) => {
        if (result.response.total_results > 0) {
            logger.info('RecursiveRequestGetLinguisticProcessResult: Se encontró al menos un resultado Idatha para proceso ' + objProcess.name);
            times = idatha_service_1.getTotalPages(result.response.total_results, size);
            let len = result.response.results.length, interaction;
            for (let i = 0; i < len; i++) {
                interaction = result.response.results[i];
                interactions.push({
                    id: interaction.id
                });
                logger.info('InteractionId: ' + interaction.id);
            }
            if (page === times) {
                page = 1;
                from = 0;
            }
            else {
                from += size;
                page++;
            }
        }
        else {
            logger.info('RecursiveRequestGetLinguisticProcessResult: No se encontró resultados Idatha para proceso ' + objProcess.name);
        }
        return (page < times ? RecursiveRequestGetLinguisticProcessResult(accessToken, query, from, size, times, interactions, page, objProcess) : Promise.resolve(objResult));
    }).catch(err => {
        return Promise.reject(err);
    });
};
var studyExecute = (function (o) {
    o.getTemplateThemeItems = function (data) {
        function findIndex(x) {
            return x.templateThemeItemId === obj.templateThemeItemId;
        }
        var result = [], len = data.length, obj, index, c;
        for (c = 0; c < len; c++) {
            obj = data[c];
            index = result.findIndex(findIndex);
            if (index === -1) {
                result.push({ templateThemeItemId: obj.templateThemeItemId, query: obj.query });
            }
        }
        return result;
    };
    o.getInteractions = function (data) {
        var result = [], len = data.length, obj, c;
        for (c = 0; c < len; c++) {
            obj = data[c];
            result.push({
                transcriptionId: obj.interactionId,
                section: obj.section,
                part: obj.part
            });
        }
        return result;
    };
    o.checkCoincidence = function (interaction, templateThemeItems, resultQuery) {
        function filter(x) {
            return x.templateThemeItemId === obj.templateThemeItemId && [x.transcriptionId, x.section, x.part].join('_') === [interaction.transcriptionId, interaction.section, interaction.part].join('_');
        }
        var resultInteraction = resultQuery.filter(function (x) {
            return [x.transcriptionId, x.section, x.part].join('_') === [interaction.transcriptionId, interaction.section, interaction.part].join('_');
        });
        var result = [], templateThemeItemsLen = templateThemeItems.length, obj, interactionInResultQuery, c;
        for (c = 0; c < templateThemeItemsLen; c++) {
            obj = templateThemeItems[c];
            interactionInResultQuery = resultQuery.filter(filter);
            result.push({
                queryId: obj.templateThemeItemId,
                queryText: obj.query,
                status: resultInteraction && resultInteraction.length === 0 ? false : interactionInResultQuery.length > 0 ? true : false
            });
        }
        return result;
    };
    o.getInteractionsByTemplateThemeItems = function (interactions, templateThemeItems, resultQuery) {
        var result = [], interactionsLen = interactions.length, obj;
        for (var c = 0; c < interactionsLen; c++) {
            obj = interactions[c];
            result.push({
                id: obj.transcriptionId,
                section: obj.section,
                part: obj.part,
                result: o.checkCoincidence(obj, templateThemeItems, resultQuery)
            });
        }
        return result;
    };
    return o;
}(studyExecute || {}));
var studyExecuteNew = (function (o) {
    o.getInteractions = function (templateThemes, templateThemeItems, data) {
        var levels = o.getLevels(templateThemes, templateThemeItems, data.studyResultSummaryDetailViewModel);
        function findIndex(x) {
            return x.transcriptionId === obj.transcriptionId;
        }
        var result = [], c, len = data.studyResultSummaryDetailViewModel.length, obj, index;
        for (c = 0; c < len; c++) {
            obj = data.studyResultSummaryDetailViewModel[c];
            index = result.findIndex(findIndex);
            if (index === -1) {
                result.push({
                    id: obj.transcriptionId,
                    section: obj.transcriptionId.split('_')[1] || obj.section,
                    part: obj.transcriptionId.split('_')[2] || obj.part,
                    result: o.getResults(levels, obj, data.studyResultSummaryDetailViewModel, templateThemeItems)
                });
            }
        }
        return result;
    };
    o.getStatusByItem = function (obj, interaction, interactions, templateThemeItems) {
        var result = [];
        result = interactions.filter(function (x) {
            return x.templateThemeId === obj.metadata.templateThemeId;
        });
        result = result.filter(function (x) {
            return x.transcriptionId === interaction.transcriptionId;
        });
        return result.length > 0;
    };
    o.getQueryByData = function (obj, templateThemeItems) {
        var result = null;
        result = templateThemeItems.find(function (x) {
            return obj.metadata.templateThemeId === x.templateThemeId;
        });
        return result;
    };
    o.getResults = function (data, interaction, interactions, templateThemeItems) {
        var i, len = data.length, obj, ability, abilities = [], subCategory, subCategories = [], category, categories = [], query;
        function find(x) {
            return x.id === obj.parentId;
        }
        for (i = 0; i < len; i++) {
            obj = data[i];
            query = o.getQueryByData(obj, templateThemeItems);
            if (obj.id.indexOf('description') > -1) {
                ability = new o.Ability({
                    id: obj.id,
                    parentId: obj.parentId,
                    name: obj.metadata.description ? obj.metadata.description : obj.metadata.category2 ? obj.metadata.category2 : obj.metadata.category1 ? obj.metadata.category1 : '',
                    query: query ? new o.Query({
                        id: query.id,
                        text: query.query,
                        status: o.getStatusByItem(obj, interaction, interactions, templateThemeItems)
                    }) : null
                });
                abilities.push(ability);
            }
            if (obj.id.indexOf('category2') > -1) {
                subCategory = new o.SubCategory({
                    id: obj.id,
                    parentId: obj.parentId,
                    name: obj.metadata.description ? obj.metadata.description : obj.metadata.category2 ? obj.metadata.category2 : obj.metadata.category1 ? obj.metadata.category1 : '',
                    query: query ? new o.Query({
                        id: query.id,
                        text: query.query,
                        status: o.getStatusByItem(obj, interaction, interactions, templateThemeItems)
                    }) : null
                });
                subCategories.push(subCategory);
            }
            if (obj.id.indexOf('category1') > -1) {
                category = new o.Category({
                    id: obj.id,
                    parentId: obj.parentId,
                    name: obj.metadata.description ? obj.metadata.description : obj.metadata.category2 ? obj.metadata.category2 : obj.metadata.category1 ? obj.metadata.category1 : '',
                    query: query ? new o.Query({
                        id: query.id,
                        text: query.query,
                        status: o.getStatusByItem(obj, interaction, interactions, templateThemeItems)
                    }) : null
                });
                categories.push(category);
            }
        }
        len = abilities.length;
        var findSubCategory, findCategory;
        for (i = 0; i < len; i++) {
            obj = abilities[i];
            if (subCategories && subCategories.length > 0) {
                findSubCategory = subCategories.find(find);
                if (findSubCategory) {
                    findSubCategory.abilities.push(obj);
                }
            }
        }
        len = subCategories.length;
        for (i = 0; i < len; i++) {
            obj = subCategories[i];
            if (categories && categories.length > 0) {
                findCategory = categories.find(find);
                if (findCategory) {
                    findCategory.category.push(obj);
                }
            }
        }
        return o.cleanObjects(categories);
    };
    o.cleanObjects = function (data) {
        var i, j, k, leni = data.length, lenj, lenk, obji, objj, objk;
        for (i = 0; i < leni; i++) {
            obji = data[i];
            delete obji.id;
            delete obji.parentId;
            lenj = obji.category.length;
            for (j = 0; j < lenj; j++) {
                objj = obji.category[j];
                delete objj.id;
                delete objj.parentId;
                lenk = objj.abilities.length;
                for (k = 0; k < lenk; k++) {
                    objk = objj.abilities[k];
                    delete objk.id;
                    delete objk.parentId;
                }
            }
        }
        return data;
    };
    o.getLevels = function (data, interactionsData) {
        var items = [], i, len = data.length, obj, array, id, parentId, index, interactions = [];
        function findIndexId(x) {
            return x.id === id && x.parentId === parentId;
        }
        function findIndex(x) {
            return x.templateThemeId === obj.id;
        }
        for (i = 0; i < len; i++) {
            obj = data[i];
            array = obj.id.split('#');
            id = array[1];
            parentId = array[2];
            index = items.findIndex(findIndexId);
            if (index === -1) {
                interactions = interactionsData.filter(findIndex);
                items.push({
                    id: id,
                    parentId: parentId,
                    metadata: {
                        category1: obj.category1,
                        category2: obj.category2,
                        description: obj.description,
                        templateThemeId: obj.id
                    }
                });
            }
        }
        return items;
    };
    o.Query = function (data) {
        var t = this;
        t.id = data.id;
        t.text = data.text;
        t.status = data.status;
        return t;
    };
    o.Ability = function (data) {
        var t = this;
        t.id = data.id;
        t.parentId = data.parentId;
        t.name = data.name;
        t.query = data.query;
        return t;
    };
    o.SubCategory = function (data) {
        var t = this;
        t.id = data.id;
        t.parentId = data.parentId;
        t.name = data.name;
        t.query = data.query;
        t.abilities = [];
        return t;
    };
    o.Category = function (data) {
        var t = this;
        t.id = data.id;
        t.parentId = data.parentId;
        t.name = data.name;
        t.query = data.query;
        t.category = [];
        return t;
    };
    return o;
}(studyExecuteNew || {}));
function getInteractionsByTemplate(data) {
    let result, templateThemeItems = studyExecute.getTemplateThemeItems(data.studyResultSummaryDetailViewModel), interactions = studyExecute.getInteractions(data.processAudioStudy), resultQuery = data.studyResultSummaryDetailViewModel, interactionsByTemplateThemeItems = studyExecute.getInteractionsByTemplateThemeItems(interactions, templateThemeItems, resultQuery);
    result = interactionsByTemplateThemeItems;
    return result;
}
exports.getInteractionsByTemplate = getInteractionsByTemplate;
function getInteractionsByTemplateNewStructure(templateThemes, templateThemeItems, data) {
    let result = studyExecuteNew.getInteractions(templateThemes, templateThemeItems, data);
    return result;
}
exports.getInteractionsByTemplateNewStructure = getInteractionsByTemplateNewStructure;
function jsonToQuery(paramater) {
    var result = '', obj = null, campaigns, agents, objCampaign, dispositions, objDisposition, objAgent, origin, i, iLen = 0, j, jLen = 0, k, kLen = 0;
    obj = paramater.filterdata;
    result = '(filter_data.';
    for (var key in obj) {
        if (obj.hasOwnProperty(key)) {
            if (key === 'start_date') {
                result += 'end_date';
                result += ':[';
                result += obj[key];
            }
            if (key === 'end_date') {
                result += ' TO ';
                result += obj[key];
                result += '])';
            }
            if (key === 'campaign') {
                result += ' AND ';
                campaigns = obj[key];
                if (campaigns && campaigns.length > 0) {
                    iLen = campaigns.length;
                    for (i = 0; i < iLen; i++) {
                        objCampaign = campaigns[i];
                        result += '(filter_data.campaign:';
                        result += objCampaign.name;
                        dispositions = objCampaign.disposition;
                        if (dispositions && dispositions.length > 0) {
                            result += ' AND ';
                            jLen = dispositions.length;
                            result += '(filter_data.disposition:';
                            for (j = 0; j < jLen; j++) {
                                objDisposition = dispositions[j];
                                result += objDisposition;
                                result += ' OR ';
                            }
                            result = result.substring(0, result.length - 4);
                            result += ')) OR ';
                        }
                        else {
                            result += ') OR ';
                        }
                    }
                }
                result = result.substring(0, result.length - 4);
            }
            if (key === 'agent') {
                agents = obj[key];
                if (agents && agents.length > 0) {
                    kLen = agents.length;
                    result += ' OR (filter_data.agent:';
                    for (k = 0; k < kLen; k++) {
                        objAgent = agents[k];
                        result += objAgent;
                        result += ' OR ';
                    }
                    result = result.substring(0, result.length - 4);
                    result += ')';
                }
            }
            if (key === 'origin') {
                origin = obj[key];
                result += ' OR ';
                result += '(filter_data.origin:';
                result += origin;
                result += ')';
            }
        }
    }
    return result;
}
exports.jsonToQuery = jsonToQuery;
function StartProcess(vcc, processId, userId) {
    let process = null;
    let nowUtc = moment().utc().toDate();
    let totalAsyncInteraction = 0;
    let totalAsyncDetail = 0;
    let totalSyncInteraction = 0;
    let totalSyncDetail = 0;
    let totalSyncDetailDurationTime = 0;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetProcess(vcc, processId);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_PROCESS_NOT_FOUND';
        process = result;
        return mssql_service_1.MSSql.GetProcessChannelByProcessId(vcc, processId);
    })
        .then(result => {
        if (result === null)
            throw 'SERVER_ERROR_GET_PROCESS_CHANNELS_FAILED';
        let promiseArray = [];
        result.map(r => promiseArray.push(mssql_service_1.MSSql.GetInteractionsResult(vcc, processId, r.Value)));
        return Promise.all(promiseArray);
    })
        .then(result => {
        result.map(r => {
            if (r) {
                if (r.interactionType === 'CALL') {
                    totalSyncInteraction = r.totalSyncInteractions;
                    totalSyncDetail = r.quantity;
                    totalSyncDetailDurationTime = r.timeQuantity;
                }
                else {
                    totalAsyncInteraction += r.totalInteractions;
                    totalAsyncDetail += r.totalInteractions;
                }
            }
        });
        let promiseArray = [];
        promiseArray.push(platform_service_1.GetPlatformVersion());
        promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, processId, 1));
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result === null)
            throw 'SERVER_ERROR_GET_PROCESS_INFORMATION_FAILED';
        let batch = {};
        batch.vcc = vcc;
        batch.installationId = result[0].installationId;
        batch.totalInteraction = totalSyncInteraction + totalAsyncInteraction;
        batch.totalAsync = totalAsyncInteraction;
        batch.totalAsyncDetail = totalAsyncDetail;
        batch.totalSync = totalSyncInteraction;
        batch.totalAudio = totalSyncDetail;
        batch.durationTime = totalSyncDetailDurationTime;
        batch.roundedTime = 0;
        batch.inProcessAudio = result[1].inProcessAudio ? result[1].inProcessAudio : 0;
        batch.inProcessDurationTime = result[1].inProcessDurationTime ? result[1].inProcessDurationTime : 0;
        batch.alreadyTranscribedAudio = result[1].alreadyTranscribedAudio ? result[1].alreadyTranscribedAudio : 0;
        batch.alreadyDurationTime = result[1].alreadyDurationTime ? result[1].alreadyDurationTime : 0;
        batch.alreadyRoundedDurationTime = result[1].alreadyTranscribedRoundedDurationTime ? result[1].alreadyTranscribedRoundedDurationTime : 0;
        batch.alreadyTranscribedDurationTime = result[1].alreadyTranscribedValidDurationTime ? result[1].alreadyTranscribedValidDurationTime : 0;
        batch.alreadyTranscribedRoundedDurationTime = result[1].alreadyTranscribedRoundedValidDurationTime ? result[1].alreadyTranscribedRoundedValidDurationTime : 0;
        batch.alreadyCallPhrases = 0;
        batch.alreadyTextPhrases = 0;
        result[1].map(pih => {
            if (pih.interactionType === 'CALL') {
                batch.alreadyCallPhrases = pih.totalCallPhrases || 0;
                batch.inProcessAudio = pih.inProcessAudio || 0;
                batch.inProcessDurationTime = pih.inProcessDurationTime || 0;
                batch.alreadyTranscribedAudio = pih.alreadyTranscribedAudio || 0;
                batch.alreadyDurationTime = pih.alreadyTranscribedDurationTime || 0;
                batch.alreadyTranscribedDurationTime = pih.alreadyTranscribedValidDurationTime || 0;
                batch.alreadyTranscribedRoundedDurationTime = pih.alreadyTranscribedRoundedDurationTime || 0;
            }
            else {
                batch.alreadyTextPhrases += pih.totalTextPhrases || 0;
            }
        });
        batch.batchScheduleId = process.batchScheduleId;
        batch.batchTypeName = 'TRANSCRIPTION';
        batch.batchTypeId = processId;
        batch.processId = processId;
        batch.userApi = userId;
        batch.startDate = nowUtc;
        let billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.SaveBatchSummary(batch);
    })
        .then((result) => {
        if (result && !result.status)
            throw result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
        return BeginProcess(vcc, processId, process);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[LinguisticService::StartProcess] Error on:', err);
        return Promise.reject(errorMsg);
    });
}
exports.StartProcess = StartProcess;
function BeginProcess(vccId, processId, processData) {
    let currentQueueId = '';
    let vcc = null;
    let entityProcess = null;
    let warningMessage = '';
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return http_shared_1.GetVCC(vccId);
    })
        .then(result => {
        vcc = result;
        if (!processData)
            return mssql_service_1.MSSql.GetProcess(vccId, processId);
        else
            return Promise.resolve(processData);
    })
        .then((result) => {
        entityProcess = result;
        if (entityProcess) {
            logger.info(`[LinguisticService::BeginProcess(${processId})] getting process:`, entityProcess, 'after MSSql.GetProcess vcc:', vccId, 'processId:', processId);
            if (['INPROCESS_TRANSCRIPTION', 'INPROCESS_SEMANTIC_ANALYSIS', 'COMPLETE', 'FAILED'].includes(entityProcess.status))
                return Promise.resolve(false);
            return Promise.resolve(true);
        }
        else {
            throw 'PROCESS_NOT_FOUND';
        }
    })
        .then((result) => {
        logger.info(`[LinguisticService::BeginProcess(${processId})] getting result:`, result, 'after process validation vcc:', vccId, 'processId:', processId);
        let promiseArray = [];
        if (result) {
            currentQueueId = vccId;
            currentQueueId += ':';
            currentQueueId += entityProcess.name;
            if (['OCC', 'I6'].includes(vcc.type) && entityProcess.needI6Update)
                promiseArray.push(redis_service_1.Redis.CreateQueueL('Process', `${currentQueueId}:i6update`));
            else
                promiseArray.push(redis_service_1.Redis.CreateQueueL('Process', currentQueueId));
        }
        else {
            warningMessage = 'PROCESS_ALREADY_STARTED';
            promiseArray.push(Promise.resolve(warningMessage));
        }
        return Promise.all(promiseArray);
    })
        .then((result) => {
        logger.info(`[LinguisticService::BeginProcess(${processId})] getting result:` + result.length + ' after Redis.CreateQueueL currentQueueId: ' + currentQueueId);
        entityProcess.status = 'INPROCESS_TRANSCRIPTION';
        return mssql_service_1.MSSql.SaveProcessStatus(entityProcess.vcc, entityProcess.name, entityProcess.status, entityProcess.lastModifiedDate, entityProcess.lastModifiedByUserId);
    })
        .then((result) => {
        logger.info(`[LinguisticService::BeginProcess(${processId})] getting result:`, result, 'after MSSql.SaveProcess currentQueueId:', currentQueueId);
        if (warningMessage?.length) {
            return Promise.resolve(warningMessage);
        }
        return Promise.resolve(result);
    })
        .catch(err => {
        let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error(`[LinguisticService::BeginProcess(${processId})] Error on:`, err);
        return Promise.reject(errorMsg);
    });
}
exports.BeginProcess = BeginProcess;
function GetInteractionsSample(interactions, samplePercentage, idField, maxSampleSize) {
    if (!interactions || interactions.length === 0)
        throw 'SERVER_ERROR_NO_ITEMS_RECEIVED';
    if (samplePercentage === null || isNaN(samplePercentage))
        throw 'SERVER_ERROR_INVALID_SAMPLE_PERCENTAGE';
    idField = idField ? idField : 'id';
    samplePercentage = samplePercentage % 100;
    if (samplePercentage > 1)
        samplePercentage = samplePercentage / 100;
    if (samplePercentage === 0)
        samplePercentage = 1;
    if (!(maxSampleSize || maxSampleSize === 0))
        maxSampleSize = interactions.length;
    let resultArray = [];
    let interactionIds = interactions.map(item => item[idField]).filter((value, index, self) => self.indexOf(value) === index);
    let sampleInteractionSize = Math.round(interactionIds.length * samplePercentage);
    if (sampleInteractionSize > maxSampleSize)
        sampleInteractionSize = maxSampleSize;
    let sampleInteractionIds = [];
    for (let i = 0; i < sampleInteractionSize; i++) {
        let currentRandomId = interactionIds[Math.floor(Math.random() * interactionIds.length)];
        sampleInteractionIds.push(currentRandomId);
        interactionIds = interactionIds.filter(ii => ii !== currentRandomId);
    }
    sampleInteractionIds.map(sii => {
        let currentSampleInteractionIdDetails = interactions.filter(i => i[idField] === sii);
        resultArray = resultArray.concat(currentSampleInteractionIdDetails);
    });
    let result = {
        sampleInteractionIds: sampleInteractionIds,
        sampleInteractionDetails: resultArray
    };
    return result;
}
exports.GetInteractionsSample = GetInteractionsSample;
function ImportAgents(vcc, vccType, processAgentIds) {
    processAgentIds = arrays_shared_1.GetUniqueFromArray(processAgentIds);
    let dbAgents = [];
    let externalAgents = [];
    let newAgentIds = [];
    let newExternalAgents = [];
    return promises_shared_1.DummyPromise()
        .then(result => {
        return mssql_service_1.MSSql.GetAgents(vcc, processAgentIds.join(','));
    })
        .then(result => {
        dbAgents = result ? result : [];
        newAgentIds = processAgentIds.filter(pai => !dbAgents.find(dba => dba.agentId === pai));
        return allegro_service_1.GetAgents(vcc, vccType, null);
    })
        .then(result => {
        externalAgents = result ? arrays_shared_1.GetUniqueFromObjectArray(result, 'userId') : [];
        newExternalAgents = externalAgents.filter(ea => newAgentIds.includes(ea.userId));
        if (newExternalAgents.length)
            return mssql_service_1.MSSql.BulkAgent(vcc, newExternalAgents);
        else
            return Promise.resolve(true);
    })
        .then(result => {
        return Promise.resolve(true);
    })
        .catch(err => {
        logger.error('[LinguisticService::ImportAgents] Error:', err);
        return Promise.resolve(false);
    });
}
exports.ImportAgents = ImportAgents;
