"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetNLUToTrainDueToRasaUpdate = exports.GetNLUToUtterancesBulk = exports.SaveNLU = exports.GetNLU = exports.FormatWithIntensDB = exports.NLUService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const file_manager_1 = require("../shared/file-manager");
const utilities_1 = require("../shared/utilities");
const intent_service_1 = require("./intent.service");
const cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
const validate_shared_1 = require("../shared/validate.shared");
const enum_1 = require("../implementation/cognitive-engine-migration/enum");
const platform_redis_1 = require("../db/redis/platform.redis");
let logger = log4js.getLogger('ServerScripts');
let useRasaApi = config_1.USE_RASA_API;
function NLUService(app) {
    let cognitiveService;
    app.post('/api/nlu/validate_azure_credentials', (req, res, nex) => {
        logger.info('[NLUService::validate_azure_credentials] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azureCredentials = req.body.azureSubscription;
        let azureSubscriptionKey = req.body.azureSubscriptionKey;
        let tenantVCC = {};
        let authObject;
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'LUIS');
        }).then(result => {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetTenantVCCByVCC(vcc);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_TENANT_NOT_FOUND';
            tenantVCC = result;
            let info = {
                tenant_id: tenantVCC.tenantId,
                client_id: tenantVCC.clientId,
                client_secret: tenantVCC.clientSecret,
                resource: tenantVCC.resource
            };
            return cognitiveService.GetManagerToken(info);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_TOKEN_GENERATOR_FAILED';
            authObject = result;
            return cognitiveService.LoadAzureAccounts(azureSubscriptionKey.subscriptionRegion, azureSubscriptionKey.subscriptionKey, authObject);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_INVALID_SUBSCRIPTIONKEY';
            if (result && result.length === 0)
                throw 'SERVER_ERROR_NLU_ACCOUNTS_NOT_FOUND';
            let isValid = false;
            let resultDescription;
            let validResult = result.filter(r => r.AzureSubscriptionId === azureCredentials.azureSubscription);
            if (validResult && validResult.length > 0) {
                validResult = result.filter(r => r.ResourceGroup === azureCredentials.resourceGroup);
                if (validResult && validResult.length > 0) {
                    validResult = result.filter(r => r.AccountName === azureCredentials.accountName);
                    if (validResult && validResult.length > 0) {
                        resultDescription = 'OK';
                    }
                    else {
                        resultDescription = 'SERVER_ERROR_NLU_ACCOUNTNAME_NOT_FOUND';
                    }
                }
                else {
                    resultDescription = 'SERVER_ERROR_NLU_RESOURCEGROUP_NOT_FOUND';
                }
            }
            else {
                resultDescription = 'SERVER_ERROR_NLU_SUBSCRIPTION_NOT_FOUND';
            }
            if (validResult && validResult.length > 0)
                isValid = true;
            if (res.headersSent)
                return;
            res.json({ status: true, description: (isValid ? 'SERVER_SUCCESS_MESSAGE' : 'VALIDATE_AZURE_ACCOUNTS_FAILED'), data: { result: isValid, description: resultDescription } });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_VALIDATE_AZURE_CREDENTIALS');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::validate_azure_credentials] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/nlu/get_nlus/', (req, res, next) => {
        logger.info('[NLUService::get_nlus] Received ' + req.method + ' method on \'' + req.path + '\'');
        logger.info('[NLUService::get_nlus] getting search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        logger.info('[NLUService::get_nlus] getting vcc:', vcc, 'azure:', azure);
        let search_criteria = req.body.searchCriteria || '';
        let search_expression = req.body.searchExpression || '';
        let isSystemApp = req.body.isSystemApp || false;
        let arrayId = [];
        let stringId = '';
        promises_shared_1.DummyPromise()
            .then(result => {
            if (isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        }).then(result => {
            if (result) {
                cognitiveService = result;
                return cognitiveService.GetAppsList(azure.subscriptionRegion, azure.subscriptionKey);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (result?.length) {
                arrayId = result.map(a => a.id);
                logger.info('[NLUService::get_nlus] getting arrayId:', arrayId.length);
                stringId = arrayId.join('\',\'');
                logger.info('[NLUService::get_nlus] getting stringId:', stringId);
            }
            if (cognitiveService.GetCognitiveEngineName() === 'RASA' || cognitiveService.GetCognitiveEngineName() === 'RASAAPI')
                return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp, true, useRasaApi);
            else
                return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp, true);
        })
            .then((results) => {
            logger.info('[NLUService::get_nlus] getting nlus:', results.length);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: results });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_nlus] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/nlu/get_cultures', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/get_cultures] getting vcc:', vcc);
        let azure = res.locals.azure;
        logger.info('[NLUService::/api/nlu/get_cultures] getting azure:', azure);
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (cognitiveService) {
                return cognitiveService.GetCultures(azure.subscriptionRegion, azure.subscriptionKey);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then((cultures) => {
            logger.info('[NLUService::/api/nlu/get_cultures] getting cultures:', cultures, 'after cognitiveService.GetCultures');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: cultures });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/get_cultures] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlu_by_name/:name', (req, res, next) => {
        logger.info('[NLUService::get_nlu_by_name] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let nluName = req.params.name;
        let nlu = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetNLU(vcc, nluName);
        })
            .then((resultNLU) => {
            logger.info('[NLUService::get_nlu_by_name] getting resultNLU:', resultNLU, 'after MSSql.GetNLU');
            nlu = resultNLU;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: nlu });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::get_nlu_by_name] getting errorMsg:', errorMsg, 'nluname:', nlu.name);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlu/:nlu_id', (req, res, next) => {
        logger.info('[NLUService::get_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let nluId = req.params.nlu_id;
        let nlu = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, nluId);
        })
            .then((resultNLU) => {
            logger.info('[NLUService::get_nlu] getting resultNLU:', resultNLU, 'after MSSql.GetNLU');
            nlu = resultNLU;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(result => {
            cognitiveService = result;
            if (!nluId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return cognitiveService.GetAppVersion(azure.subscriptionRegion, azure.subscriptionKey, nlu.appId, vcc);
        })
            .then((result) => {
            logger.info('[NLUService::get_nlu] getting result:', result, 'after cognitiveService.GetAppVersion');
            nlu.version = result.version;
            nlu.createdDateTime = result.createdDateTime;
            nlu.lastModifiedDateTime = result.lastModifiedDateTime;
            nlu.lastTrainedDateTime = result.lastTrainedDateTime;
            nlu.lastPublishedDateTime = result.lastPublishedDateTime;
            nlu.endpointUrl = result.endpointUrl;
            nlu.assignedEndpointKey = result.assignedEndpointKey;
            nlu.externalApiKeys = result.externalApiKeys;
            nlu.intentsCount = result.intentsCount;
            nlu.entitiesCount = result.entitiesCount;
            nlu.endpointHitsCount = result.endpointHitsCount;
            nlu.trainingStatus = result.trainingStatus;
            return nlu;
        })
            .then((nluModel) => {
            logger.info('[NLUService::get_nlu] getting nluModel:', nluModel);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: nluModel });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::get_nlu] getting errorMsg:', errorMsg, 'nluId:', nluId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/create_nlu', (req, res, next) => {
        req.setTimeout(300000, () => { });
        let vcc = res.locals.vcc;
        let azure = res.locals.azure;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newNLU = (req.body || {});
        let saveNewNLUResult = null;
        newNLU.vcc = vcc;
        newNLU.createdDate = nowUtc;
        newNLU.createdByUserId = userId;
        newNLU.lastModifiedDate = nowUtc;
        newNLU.lastModifiedByUserId = userId;
        let cognitiveEngineName = '';
        logger.info('[NLUService::create_nlu] getting newNLU:', newNLU);
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            cognitiveEngineName = cognitiveService.GetCognitiveEngineName();
            switch (cognitiveEngineName) {
                case 'LUIS': return mssql_service_1.MSSql.GetNLUByName(null, newNLU.name);
                case 'RASA': return mssql_service_1.MSSql.GetNLUByName(vcc, newNLU.name);
                case 'RASAAPI': return mssql_service_1.MSSql.GetNLUByName(vcc, newNLU.name);
                default: throw 'SERVER_ERROR_UNKNOWN_COGNITIVE_ENGINE';
            }
        })
            .then((resultNLU) => {
            logger.info('[NLUService::create_nlu] getting resultNLU:', resultNLU, 'after MSSql.GetNLUByName');
            if (resultNLU?.length > 0)
                throw 'MODEL_NLU_ALREADY_EXIST \'' + newNLU.name + '\'';
            return cognitiveService.AddApp(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, newNLU);
        })
            .then((result) => {
            logger.info('[NLUService::create_nlu] getting result:', result, 'after cognitiveService.AddApp');
            newNLU.appId = result;
            return SaveNLU(vcc, newNLU);
        })
            .then((resultSaveNLU) => {
            logger.info('[NLUService::create_nlu] getting resultSaveNLU:', resultSaveNLU, 'after SaveNLU');
            if (!resultSaveNLU)
                throw 'ERROR_SAVING_MODEL_NLU';
            saveNewNLUResult = resultSaveNLU;
            if (newNLU.importNLU) {
                return ImportNLUContent(vcc, newNLU, cognitiveService);
            }
            else {
                return saveNewNLUResult;
            }
        })
            .then(resultSaveNLU => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (saveNewNLUResult ? 'SERVER_SUCCESS_MESSAGE' : 'MODEL_SAVE_FAILED'), data: saveNewNLUResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            if (err.error && err.error.error && err.error.error.message) {
                let message = '';
                let message2 = message.concat('An application with the same name (', newNLU.name, ') already exists.');
                if (err.error.error.message === message2) {
                    clientMsg = 'MODEL_NLU_INVALID_NAME';
                    errorMsg = 'MODEL_NLU_INVALID_NAME';
                }
            }
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::create_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/save_nlu', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/save_nlu] getting vcc:', vcc);
        let userId = res.locals.userId;
        logger.info('[NLUService::/api/nlu/save_nlu] getting userId:', userId);
        let nowUtc = moment().utc().toDate();
        logger.info('[NLUService::/api/nlu/save_nlu] getting nowUtc:', nowUtc);
        let newNLU = (req.body || {});
        newNLU.createdDate = null;
        newNLU.createdByUserId = null;
        newNLU.lastModifiedDate = nowUtc;
        newNLU.lastModifiedByUserId = userId;
        logger.info('[NLUService::/api/nlu/save_nlu] getting newNLU:', newNLU);
        promises_shared_1.DummyPromise()
            .then(result => {
            return SaveNLU(vcc, newNLU);
        })
            .then(resultSaveNLU => {
            logger.info('[NLUService::/api/nlu/save_nlu] getting resultSaveNLU:', resultSaveNLU);
            if (!resultSaveNLU)
                throw 'ERROR_SAVING_MODEL_NLU';
            return Promise.resolve(true);
        })
            .then(resultSaveNLU => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveNLU, description: (resultSaveNLU ? 'SERVER_SUCCESS_MESSAGE' : 'MODEL_SAVE_FAILED'), data: resultSaveNLU });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/save_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_nlu', (req, res, next) => {
        logger.info('[NLUService::delete_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[NLUService::delete_nlu] getting vcc:', vcc);
        let azure = res.locals.azure;
        logger.info('[NLUService::delete_nlu] getting azure:', azure);
        let nlu = req.body.nlu;
        logger.info('[NLUService::delete_nlu] getting nlu:', nlu);
        let nluId = nlu.name;
        logger.info(`[NluService::delete_nlu] Deleting nlu ${nluId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetNlusUsage(vcc, nluId);
        })
            .then((inUse) => {
            if (inUse) {
                throw 'CANNOT_DELETE_NLU_IN_USE';
            }
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (!nlu)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            nlu.vcc = vcc;
            return cognitiveService.DeleteApp(azure.subscriptionRegion, azure.subscriptionKey, nlu);
        }).then(result => {
            return mssql_service_1.MSSql.DeleteNLU(vcc, nluId);
        }).then((resultDeleteNLU) => {
            logger.info('[NLUService::delete_nlu] getting resultDeleteNLU:', resultDeleteNLU);
            if (!resultDeleteNLU)
                throw 'ERROR_DELETING_MODEL_NLU';
            return Promise.resolve(true);
        })
            .then(resultDeleteNLU => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteNLU, description: (resultDeleteNLU ? 'SERVER_SUCCESS_MESSAGE' : 'MODEL_DELETE_FAILED'), data: resultDeleteNLU });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::delete_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_nlus', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/delete_nlus] getting vcc:', vcc);
        let nlus = req.body.nlus || [];
        logger.info('[NLUService::/api/nlu/delete_nlus] getting nlus:', nlus);
        logger.info(`[NluService::delete_nlus] Deleting nlus ${nlus.map(x => x.name).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!nlus)
                throw 'MISSING_MODELS_NLU_INPUT_PARAMETER';
            let promiseArray = [];
            nlus.forEach(nlu => {
                promiseArray.push(mssql_service_1.MSSql.DeleteNLU(vcc, nlu.name));
            });
            return Promise.all(promiseArray);
        })
            .then((resultDeleteNLUs) => {
            logger.info('[NLUService::/api/nlu/delete_nlus] getting resultDeleteNLUs:', resultDeleteNLUs);
            let result;
            if (resultDeleteNLUs.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('[NLUService::/api/nlu/delete_nlus] could not delete all requested items');
            }
            else {
                result = true;
                logger.info('[NLUService::/api/nlu/delete_nlus] models deleted successfully');
            }
            return result;
        })
            .then(resultDeleteNLUs => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteNLUs, description: (resultDeleteNLUs ? 'SERVER_SUCCESS_MESSAGE' : 'MODELS_DELETE_FAILED'), data: resultDeleteNLUs });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/delete_nlus] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_subscriptionkey', (req, res, next) => {
        logger.info('[NLUService::get_subscriptionkey] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let azureSubscriptionKey = req.body;
        logger.info('[NLUService::get_subscriptionkey] getting vcc:', vcc);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (azureSubscriptionKey.subscriptionName !== null)
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByName(vcc, azureSubscriptionKey.subscriptionName);
            else if (azureSubscriptionKey.subscriptionKey !== null)
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, azureSubscriptionKey.subscriptionKey);
            else
                throw 'SERVER_ERROR_MISSING_PARAMETERS_FOR_SEARCH';
        })
            .then(result => {
            logger.info('[NLUService::get_subscriptionkey] getting keys:', result, 'after MSSql.GetSubscriptionKeys');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_subscriptionkeys] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_subscriptionkeys', (req, res, next) => {
        logger.info('[NLUService::get_subscriptionkeys] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[NLUService::get_subscriptionkeys] getting vcc:', vcc);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(vcc);
        })
            .then(result => {
            logger.info('[NLUService::get_subscriptionkeys] getting keys:', result.length, 'after MSSql.GetSubscriptionKeys');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_subscriptionkeys] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/upload_file/:batch_type_name?/:upload_dest?', (req, res, next) => {
        logger.info('[NLUService::upload_file] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let cognitiveEngineSource = req.params.batch_type_name ? req.params.batch_type_name : null;
        let nowUtc = moment().utc().toDate();
        let uploadToFolder = req.params.upload_dest || '';
        let nluFileRedisKey = 'NLU_FILE:' + userId;
        let nluFileContent = null;
        let resultUpload = false;
        let tempFolderName = (uploadToFolder ? uploadToFolder : (config_1.NLU_TEMP_PREFIX + '-' + vcc + '-' + userId).toLowerCase() + '-' + nowUtc.valueOf());
        let tempZipFileName = tempFolderName + '.json';
        let tempUploadPath = (uploadToFolder ? uploadToFolder : config_1.NLU_FOLDER_TEMP_PATH);
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (!nluFileRedisKey)
                throw 'SERVER_ERROR_NLU_TARGET_NOT_FOUND';
            logger.info('[NLUService::upload_file] Getting tempFolderName:', tempFolderName, 'with tempZipFileName:', tempZipFileName, 'with tempUploadPath:', tempUploadPath);
            return file_manager_1.ExistInFileSystem(tempUploadPath);
        }).then((result) => {
            if (result === false) {
                return file_manager_1.CreateFolder(tempUploadPath);
            }
            else {
                return true;
            }
        }).then((result) => {
            if (result) {
                return file_manager_1.UploadFileToServer(tempZipFileName, tempUploadPath, req, res, '(md|json)');
            }
            else {
                return false;
            }
        }).then((result) => {
            resultUpload = result;
            logger.info('[NLUServiceService::upload_file] Getting resultUpload:', resultUpload, 'after UploadFileToServer');
            if (!resultUpload)
                throw 'SERVER_ERROR_COULD_NOT_UPLOAD_FILE';
            let fileName = `${tempUploadPath}/${tempZipFileName}`;
            return file_manager_1.ReadFileContent(fileName, 'utf-8');
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_READ_FILE';
            let breakLineRegex = new RegExp(`(\r\n|\r)`, 'gm');
            nluFileContent = result.replace(breakLineRegex, '\n');
            if (nluFileContent && nluFileContent.length > 0) {
                return redis_service_1.Redis.SaveFileNLU(vcc, nluFileRedisKey, nluFileContent);
            }
            else
                throw 'SERVER_ERROR_UPLOADED_FILE_EMPTY';
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_COULDNOT_SAVE_FILE_CONTENT';
            logger.info('[NLUServiceService::upload_file] Getting result:', result, 'after ReadFileContent');
            if (result === true) {
                let tempCSVFile = `${tempUploadPath}/${tempZipFileName}`;
                logger.info('[NLUServiceService::upload_file] Getting tempCSVFile');
                return file_manager_1.DeleteFile(tempCSVFile);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(result => {
            if (result === true) {
                return redis_service_1.Redis.GetFileNLU(vcc, (nluFileRedisKey));
            }
            else {
                return Promise.resolve(null);
            }
        }).then(result => {
            let content = JSON.parse(result.data.replace('"', '\"'));
            let intents = [];
            let utterances = [];
            intents = content.intents ? content.intents : [];
            intents.map(i => {
                i.utterances.map(u => {
                    utterances.push(u);
                });
            });
            if (intents.length === 0)
                throw 'SERVER_ERROR_INTENTS_NOT_FOUND';
            intents.map(i => {
                if (i.name.toLowerCase() === 'none')
                    throw 'SERVER_ERROR_INTENT_ID_INVALID';
                if (validate_shared_1.IsStringAndEmpty(i.name))
                    throw 'SERVER_ERROR_INTENT_ID_MISSING';
                if (!validate_shared_1.IsStringAndValiadId(i.name))
                    throw 'SERVER_ERROR_INTENT_ID_INVALID';
            });
            if (res.headersSent)
                return;
            let data = {
                tempFolderName: tempFolderName,
                content: result.data,
                intents: intents.length,
                utterances: utterances.length
            };
            res.json({ status: resultUpload, description: (resultUpload ? 'FILE_STORED_SUCCESSFULLY' : 'FILE_UPLOAD_FAILED'), data });
        }).catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::upload_file] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_file_temporal/', (req, res, next) => {
        logger.info('[NLUService::delete_file_temporal] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nluFileRedisKey = 'NLU_FILE:' + userId;
        logger.info('[NLUService::delete_file_temporal] Getting vcc:userId : ', vcc, ':', userId);
        logger.info(`[NLUService::delete_file_temporal] Deleting  ${nluFileRedisKey} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            return redis_service_1.Redis.GetFileNLU(vcc, nluFileRedisKey);
        }).then(result => {
            if (result && result.data.length > 0) {
                return redis_service_1.Redis.DeleteFileNLU(vcc, nluFileRedisKey);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'CLEAR SUCCESS' : 'CLEAR ERROR '), data: result });
        }).catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::delete_file_temporal] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/export_nlu', (req, res, next) => {
        logger.info('[NLUService::export_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[NLUService::export_nlu] getting vcc:', vcc);
        let nlu = req.body.nlu;
        nlu.vcc = vcc;
        logger.info('[NLUService::export_nlu] getting nlu:', nlu);
        promises_shared_1.DummyPromise()
            .then(result => {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(result => {
            cognitiveService = result;
            return cognitiveService.ExportModel(nlu, res);
        })
            .then(result => {
            res.setHeader('Content-type', 'application/json; charset=utf-8');
            res.setHeader('Content-disposition', 'attachment; filename=');
            res.send(JSON.stringify(result, null, 3));
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::export_nlu] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get("/api/nlu/nlu-to-export/", (req, res) => {
        logger.info('[NLUService::nlu-to-export] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = req.query.vcc ? req.query.vcc.toString() : '';
        promises_shared_1.DummyPromise().then(_ => {
            return mssql_service_1.MSSql.GetNluIntentUtteranceToExport(vcc, 0);
        }).then(result => {
            if (!result && Object.keys(result).length == 0)
                throw 'DATA_NLU_NOT_FOUND';
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::nlu-to-export] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/retry_nlu/', (req, res) => {
        let vcc = res.locals.vcc;
        let appId = req.body.appId || null;
        promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.GetCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, appId);
        })
            .then((nluStrings) => {
            if (nluStrings && nluStrings.length > 0) {
                let nlu = JSON.parse(nluStrings[0]);
                let promises = [];
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', nlu.name, 2, 'RETRYING'));
                promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:Nlus`, nlu.appId, Object.assign(nlu, {
                    status: 0,
                    totalRetries: nlu.totalRetries + 1
                })));
                return Promise.all(promises);
            }
            else {
                throw 'NOT_FOUND_MODEL';
            }
        })
            .then((results) => {
            if (!results.includes(false)) {
                res.json({ status: true, data: true, description: 'SERVER_SUCCESS_MESSAGE' });
            }
            else {
                throw 'CANNOT_UPDATE_MODEL';
            }
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::retry_nlu] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/test_nlu_phrase/', (req, res) => {
        let nlu = req.body.nlu;
        let appId = nlu.appId || null;
        let phrase = nlu.phraseToTest;
        let apiIdLabel = nlu.cognitiveEngine;
        let subscriptionRegion = null;
        let subscriptionKey = null;
        let currentSubscriptionKey = {};
        let installationId = null;
        promises_shared_1.DummyPromise().then(_ => {
            return cognitive_analysis_implementation_1.GetCognitiveService(nlu.vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (apiIdLabel === "RASA") {
                return platform_redis_1.GetPlatformVersion();
            }
            else if (apiIdLabel === "LUIS") {
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(nlu.vcc, nlu.subscriptionKey);
            }
            else {
                return true;
            }
        })
            .then(result => {
            if (apiIdLabel === 'RASA') {
                installationId = result.installationId;
                return redis_service_1.Redis.GetRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase());
            }
            else
                return result;
        }).then(result => {
            if (apiIdLabel === 'RASA') {
                if (result === null)
                    return cognitiveService.RunApp(installationId + '::' + appId.toLowerCase(), 'TEST');
                else
                    result.created = moment.utc().format('YYYY-MM-DD HH:mm:ss');
                return redis_service_1.Redis.SaveRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase(), result);
            }
            else if (apiIdLabel === 'LUIS') {
                currentSubscriptionKey = result;
                subscriptionRegion = currentSubscriptionKey.subscriptionRegion;
                subscriptionKey = nlu.subscriptionKey;
                return null;
            }
            else {
                return null;
            }
        }).then(result => {
            if (apiIdLabel === 'RASA') {
                return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, phrase, null, null, null, 'TEST');
            }
            else
                return cognitiveService.ModelQuery(subscriptionRegion, subscriptionKey, appId, phrase, null, null, null);
        }).then(result => {
            if (apiIdLabel === "RASA")
                return FormatWithIntensDB(nlu, result);
            else
                return result;
        }).then(result => {
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::test_nlu_phrase] Getting err:', err);
            logger.error('[NLUService::test_nlu_phrase] Getting errmessage:', err.message);
            logger.error('[NLUService::test_nlu_phrase] Getting errorMsg:', errorMsg);
            logger.error('[NLUService::test_nlu_phrase] Getting clientMsg:', clientMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/stop_test_nlu_phrase/', (req, res) => {
        let nlu = req.body.nlu;
        let appId = nlu.appId || null;
        let apiIdLabel = nlu.cognitiveEngine;
        let installationId = null;
        promises_shared_1.DummyPromise().then(_ => {
            return cognitive_analysis_implementation_1.GetCognitiveService(nlu.vcc, null);
        }).then(result => {
            cognitiveService = result;
            if (apiIdLabel === "RASA") {
                return platform_redis_1.GetPlatformVersion();
            }
            else
                return null;
        })
            .then(result => {
            if (apiIdLabel === 'RASA') {
                installationId = result.installationId;
                return redis_service_1.Redis.GetRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase());
            }
        }).then(result => {
            if (apiIdLabel === "RASA") {
                if (result !== null)
                    return cognitiveService.StopApp(installationId + '::' + appId.toLowerCase(), result.port, 'TEST');
                return null;
            }
            else
                return null;
        }).then(result => {
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[NLUService::stop_test_nlu_phrase] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.NLUService = NLUService;
function FormatWithIntensDB(nlu, resFromModelQuery) {
    return promises_shared_1.DummyPromise()
        .then(result => {
        return mssql_service_1.MSSql.GetIntents(nlu.vcc, nlu.appId, null, null, null);
    })
        .then(result => {
        let res = result ? result : [];
        let arrRes = [];
        res.map(r => {
            arrRes.push({
                id: r.intentId,
                name: r.name
            });
        });
        let joinIntents = [];
        arrRes.map(x => {
            resFromModelQuery.intents.map(y => {
                if (x.id === y.intent)
                    joinIntents.push({
                        intent: x.name,
                        score: y.score
                    });
            });
        });
        resFromModelQuery.intents = joinIntents;
        return Promise.resolve(resFromModelQuery);
    }).catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[NLUService]::GetIntentsListFromDB] Error: ', error);
        return Promise.reject(errorMessage);
    });
}
exports.FormatWithIntensDB = FormatWithIntensDB;
function GetNLU(vcc, name) {
    logger.info('[NLUService::GetNLU] getting vcc:', vcc, 'name:', name);
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetNLU(vcc, name);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::GetNLU] getting errorMsg:', errorMsg);
    });
}
exports.GetNLU = GetNLU;
function SaveNLU(vcc, nlu) {
    logger.info('[NLUService::SaveNLU] getting vcc:', vcc, 'nlu:', nlu);
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveNLU(vcc, nlu.name, nlu.culture, nlu.description, nlu.appId, nlu.subscriptionKey, nlu.isSystemApp, nlu.needTrain, nlu.needPublish, nlu.isPublished, nlu.lastPublishedDate, nlu.isTrained, nlu.lastTrainedDate, nlu.createdDate, nlu.createdByUserId, nlu.lastModifiedDate, nlu.lastModifiedByUserId, nlu.trainVersion, nlu.trainId));
    return Promise.all(promiseArray)
        .then(results => {
        logger.info('[NLUService::SaveNLU] getting results:', results, 'after Promise.all MSSql.SaveNLU');
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNLU = SaveNLU;
function ImportNLUContent(vcc, nlu, cognitiveService) {
    logger.info('[NLUService::ImportNLUContent] getting vcc:', vcc, 'nlu:', nlu.appId);
    let nluContent = null;
    let nluFileRedisKey = 'NLU_FILE:' + nlu.createdByUserId;
    return promises_shared_1.DummyPromise().then(result => {
        return redis_service_1.Redis.GetFileNLU(nlu.vcc, (nluFileRedisKey));
    }).then(result => {
        if (!result)
            throw 'SERVER_ERROR_NLU_REDIS_DATA_NOT_FOUND';
        nluContent = JSON.parse(result.data.replace('"', '\"'));
        let intents = [];
        let promiseArray = [];
        nluContent.intents.map(i => {
            if (i.name.toLowerCase() === 'none')
                throw 'SERVER_ERROR_INTENT_ID_INVALID';
            let newIntent = {};
            newIntent.appId = nlu.appId;
            newIntent.name = i.name;
            newIntent.intentId = i.id;
            newIntent.createdDate = moment().utc().toDate();
            newIntent.createdByUserId = nlu.createdByUserId;
            newIntent.color = utilities_1.getRandomHexColor();
            promiseArray.push(intent_service_1.SaveIntent(vcc, newIntent));
            i.utterances.map(u => {
                promiseArray.push(mssql_service_1.MSSql.SaveUtterance(vcc, nlu.appId, newIntent.intentId, newIntent.name, u.id, u.text, nlu.createdByUserId));
            });
        });
        nluContent.entities.map(e => {
            promiseArray.push(mssql_service_1.MSSql.SaveEntity(vcc, nlu.appId, e.id, e.name, e.type, nlu.createdByUserId, null));
        });
        nluContent.utteranceEntities.map(ue => {
            promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, ue.utteranceId, ue.entityName, ue.start, ue.end, ue.startTokenIndex, ue.endTokenIndex));
        });
        return Promise.all(promiseArray);
    }).then(result => {
        return redis_service_1.Redis.DeleteFileNLU(nlu.vcc, (nluFileRedisKey));
    }).then(result => {
        return Promise.resolve(true);
    }).catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::ImportNLUContent] getting errorMsg:', errorMsg);
        return Promise.reject(errorMsg);
    });
}
function GetNLUToUtterancesBulk(searchOnDB = false) {
    let searchResult = [];
    let needUpdate = false;
    return promises_shared_1.DummyPromise()
        .then(result => {
        return redis_service_1.Redis.GetNLUToUtterancesBulk();
    })
        .then(result => {
        if (!result || searchOnDB) {
            needUpdate = true;
            return mssql_service_1.MSSql.GetNLUs('ALL', 'LastModifiedByUserId', 'dev-1', null, false);
        }
        else {
            return Promise.resolve(result);
        }
    })
        .then(result => {
        searchResult = result;
        if (needUpdate) {
            return redis_service_1.Redis.SaveNLUToUtterancesBulk(searchResult);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(result => {
        return Promise.resolve(searchResult);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::GetNLUToUtterancesBulk] getting errorMsg:', errorMsg);
        return Promise.resolve([]);
    });
}
exports.GetNLUToUtterancesBulk = GetNLUToUtterancesBulk;
function GetNLUToTrainDueToRasaUpdate(searchOnDB) {
    let searchResult = [];
    let needUpdate = false;
    return promises_shared_1.DummyPromise()
        .then(result => {
        return redis_service_1.Redis.GetNLUToTrainDueToRasaUpdate();
    })
        .then(result => {
        if (!result || searchOnDB) {
            needUpdate = true;
            return mssql_service_1.MSSql.GetNLUs('ALL', 'LastModifiedByUserId', 'dev-2', null, false);
        }
        else {
            return Promise.resolve(result);
        }
    })
        .then(result => {
        searchResult = result;
        if (needUpdate) {
            return redis_service_1.Redis.SaveNLUToTrainDueToRasaUpdate(searchResult);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(result => {
        return Promise.resolve(searchResult);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[NLUService::GetNLUToTrainDueToRasaUpdate] getting errorMsg:', errorMsg);
        return Promise.resolve([]);
    });
}
exports.GetNLUToTrainDueToRasaUpdate = GetNLUToTrainDueToRasaUpdate;
