"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportService = void 0;
const log4js = require("log4js");
const moment = require("moment");
const report_implementation_1 = require("../implementation/report.implementation");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
let logger = log4js.getLogger('ServerScripts');
function ReportService(app) {
    app.get('/api/report/get_reports/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[ReportService::get_reports] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let role = res.locals.userRole;
        let userId = res.locals.userId;
        if (userId === 'admin' && vcc === 'system') {
            role.id = 'provisioning';
        }
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            if (role.id == 'business_administrator' || role.id == 'provisioning') {
                return mssql_service_1.MSSql.GetReports(vcc, search_criteria, search_expression, role.id);
            }
            else {
                return mssql_service_1.MSSql.GetReports(vcc, search_criteria, search_expression, null);
            }
        })
            .then((resultReports) => {
            if (!resultReports)
                throw 'REPORT_SERVER_ERROR_CANNOT_EXECUTE';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultReports });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report/:report_id', (req, res, next) => {
        logger.info('[ReportService::get_report] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let role = res.locals.userRole;
        let reportId = req.params.report_id;
        let userId = res.locals.userId;
        if (userId === 'admin') {
            role.id = 'business_administrator';
        }
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!reportId)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            if (role.id == 'business_administrator') {
                return mssql_service_1.MSSql.GetReport(vcc, reportId, role.id);
            }
            else {
                return mssql_service_1.MSSql.GetReport(vcc, reportId, null);
            }
        }).then((report) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: report });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/create_report', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newReport = (req.body || {});
        newReport.createdDate = nowUtc;
        newReport.createdByUserId = userId;
        newReport.lastModifiedDate = nowUtc;
        newReport.lastModifiedByUserId = userId;
        newReport.startDate = (newReport.startDate ? new Date(newReport.startDate) : null);
        newReport.endDate = (newReport.endDate ? new Date(newReport.endDate) : null);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetReport(vcc, newReport.id);
        }).then((resultReport) => {
            if (resultReport)
                throw 'REPORT_SERVER_ERROR_EXISTS';
            return mssql_service_1.MSSql.SaveReport(vcc, newReport.id, newReport.description, newReport.type, newReport.procedure, newReport.startDate, newReport.endDate, newReport.frecuency, newReport.hourSpace, newReport.exactTime, newReport.weekday, newReport.dayMonth, newReport.summary, newReport.filter, newReport.email, newReport.createdDate, newReport.createdByUserId, newReport.lastModifiedDate, newReport.lastModifiedByUserId);
        }).then(resultSaveReport => {
            if (!resultSaveReport)
                throw 'REPORT_SERVER_ERROR_ON_SAVE';
            return mssql_service_1.MSSql.SaveSinglePermission(vcc, 'REPORT', newReport.id);
        }).then(result => {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + newReport.id + '\' creado correctamente');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_SAVE'), data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/save_report', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let role = res.locals.userRole;
        let nowUtc = moment().utc().toDate();
        let newReport = (req.body || {});
        newReport.createdDate = null;
        newReport.createdByUserId = null;
        newReport.lastModifiedDate = nowUtc;
        newReport.lastModifiedByUserId = userId;
        newReport.startDate = (newReport.startDate ? new Date(newReport.startDate) : null);
        newReport.endDate = (newReport.endDate ? new Date(newReport.endDate) : null);
        promises_shared_1.DummyPromise()
            .then(result => {
            let allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(sp => sp.type === 'REPORT').map(sp => sp.element) :
                []);
            if (!allowedReports.includes(newReport.id)) {
                res.sendStatus(403);
                throw 'REPORT_SERVER_ERROR_ACCESS_DENIED';
            }
            return mssql_service_1.MSSql.SaveReport(vcc, newReport.id, newReport.description, newReport.type, newReport.procedure, newReport.startDate, newReport.endDate, newReport.frecuency, newReport.hourSpace, newReport.exactTime, newReport.weekday, newReport.dayMonth, newReport.summary, newReport.filter, newReport.email, newReport.createdDate, newReport.createdByUserId, newReport.lastModifiedDate, newReport.lastModifiedByUserId);
        }).then(resultSaveReport => {
            if (!resultSaveReport)
                throw 'REPORT_SERVER_ERROR_ON_SAVE';
            return mssql_service_1.MSSql.SaveSinglePermission(vcc, 'REPORT', newReport.id);
        }).then(result => {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + newReport.id + '\' actualizado correctamente');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_SAVE'), data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/delete_report', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let role = res.locals.userRole;
        let report = req.body.report || {};
        let reportId = report.id;
        logger.info(`[ReportService::delete_report] Deleting report ${reportId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!report)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            let allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(sp => sp.type === 'REPORT').map(sp => sp.element) :
                []);
            if (!allowedReports.includes(reportId)) {
                res.sendStatus(403);
                throw 'REPORT_SERVER_ERROR_ACCESS_DENIED';
            }
            return mssql_service_1.MSSql.DeleteReport(vcc, reportId);
        }).then(resultDeleteReport => {
            if (!resultDeleteReport)
                throw 'REPORT_SERVER_ERROR_ON_SAVE';
            return mssql_service_1.MSSql.DeleteSinglePermission(vcc, 'REPORT', reportId);
        }).then(result => {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + reportId + '\' eliminado correctamente');
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_DELETE'), data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/delete_reports', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let role = res.locals.userRole;
        let reports = req.body.reports || [];
        logger.info(`[ReportService::delete_reports] Deleting reports ${reports.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!reports)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            let allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(sp => sp.type === 'REPORT').map(sp => sp.element) :
                []);
            let forbiddenReportId;
            reports.forEach(report => {
                if (!forbiddenReportId && !allowedReports.includes(report.id)) {
                    forbiddenReportId = report.id;
                }
            });
            if (forbiddenReportId) {
                res.sendStatus(403);
                throw 'REPORT_SERVER_ERROR_ACCESS_DENIED';
            }
            let promiseArray = [];
            reports.forEach(report => {
                promiseArray.push(mssql_service_1.MSSql.DeleteReport(vcc, report.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteReports => {
            let result;
            if (resultDeleteReports.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Reportes eliminados correctamente');
            }
            let promiseArray = [];
            reports.forEach(report => {
                promiseArray.push(mssql_service_1.MSSql.DeleteSinglePermission(vcc, 'REPORT', report.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteSinglePermissions => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteSinglePermissions, description: (resultDeleteSinglePermissions ? 'SERVER_SUCCESS_MESSAGE' : 'REPORT_SERVER_ERROR_ON_DELETE'), data: resultDeleteSinglePermissions });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report_procedures', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetReportProcedures();
        }).then(response => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report_procedure_parameters/:procedure', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let role = res.locals.userRole;
        let userId = res.locals.userId;
        let procedure = req.params.procedure;
        let fromBilling = false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (role.id === 'business_administrator' || userId === 'admin') {
                let report = new report_implementation_1.ReportImplementation();
                fromBilling = true;
                return report.GetReportProceduraParameters(procedure);
            }
            else {
                return mssql_service_1.MSSql.GetReportProcedureParameters(procedure);
            }
        }).then(response => {
            if (res.headersSent)
                return;
            let procedureParameter = [];
            if (response) {
                response.forEach((parameter) => {
                    if (userId === 'admin') {
                        if (parameter.parameter !== 'PlatformHash' && parameter.parameter !== '@VCC' && parameter.parameter !== '@UserId' && parameter.parameter !== '@ClientOffset' && !parameter.isOutput) {
                            if (!fromBilling) {
                                if (parameter.type === 'datetime') {
                                    parameter.type = 'date';
                                }
                                else {
                                    parameter.type = 'text';
                                }
                            }
                            parameter.parameter = parameter.parameter.replace('@', '');
                            procedureParameter.push(parameter);
                        }
                    }
                    else {
                        if (parameter.parameter !== 'SubAccountId' && parameter.parameter !== 'AccountId' && parameter.parameter !== 'InstallationId' && parameter.parameter !== 'PlatformHash' && parameter.parameter !== 'VCC' && parameter.parameter !== '@VCC' && parameter.parameter !== '@UserId' && parameter.parameter !== '@ClientOffset' && !parameter.isOutput) {
                            if (fromBilling) {
                                if (parameter.type === 'datetime') {
                                    parameter.type = 'date';
                                }
                                else {
                                    parameter.type = 'text';
                                }
                            }
                            parameter.parameter = parameter.parameter.replace('@', '');
                            procedureParameter.push(parameter);
                        }
                    }
                });
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: procedureParameter });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/get_filters_options', (req, res) => {
        let filters = req.body || [];
        let VCC = res.locals.vcc;
        let UserId = res.locals.userId;
        promises_shared_1.DummyPromise().then(_ => {
            let promises = [];
            for (let f of filters) {
                let parameters = [];
                if (f._arguments) {
                    let _arguments = f._arguments.split(",");
                    for (let a of _arguments) {
                        let value = '';
                        eval(`value=${a}`);
                        parameters.push({
                            column: a,
                            value
                        });
                    }
                }
                promises.push(mssql_service_1.MSSql.ExecuteReportProcedure(f.procedure, parameters));
            }
            return Promise.all(promises);
        }).then(results => {
            let response = {};
            let i = 0;
            for (let f of filters) {
                response[f.column] = results[i++];
            }
            res.send({
                status: true, description: 'Ok', data: response
            });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[ReportService::get_filters_options] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/execute_report', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let role = res.locals.userRole;
        let reportExecution = req.body.reportExecution || {};
        let procedure = reportExecution.procedure;
        let filters = reportExecution.filters;
        let clientOffset = reportExecution.clientOffset;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!filters && !procedure)
                throw 'SERVER_ERROR_MISSING_PARAMETERS';
            if (role.id === 'business_administrator' || userId === 'admin') {
                let report = new report_implementation_1.ReportImplementation();
                return report.GetReportProceduraParameters(procedure);
            }
            else {
                return mssql_service_1.MSSql.GetReportProcedureParameters(procedure);
            }
        })
            .then((resultParameters) => {
            let procedureParameters = [];
            resultParameters.forEach(parameter => {
                let column = parameter.parameter.replace('@', '');
                let colFilter = filters.find(filter => filter && filter.column === column);
                let value = (colFilter && colFilter.value ? colFilter.value : null);
                if (value && (column === 'EndDate' || column === 'StartDate')) {
                    let momentDate = moment.parseZone(value).isValid() && moment.parseZone(value) || null;
                    value = (momentDate) ? momentDate.utc().add(clientOffset, 'minutes').toDate() : null;
                }
                if (!value && column === 'VCC')
                    value = vcc;
                else if (!value && column === 'UserId')
                    value = userId;
                else if (!value && column === 'ClientOffset')
                    value = clientOffset;
                if (column !== 'RETURN_VALUE')
                    procedureParameters.push({
                        column: column,
                        value: value
                    });
            });
            if (role.id === 'business_administrator' || userId === 'admin') {
                let report = new report_implementation_1.ReportImplementation();
                return report.ReportExecution(procedureParameters, procedure, clientOffset);
            }
            else {
                return mssql_service_1.MSSql.ExecuteReportProcedure(procedure, procedureParameters);
            }
        }).then(responseReportData => {
            if (res.headersSent)
                return;
            let columns = [];
            let transformColumns = ['DateTime'];
            if (responseReportData) {
                for (let key in responseReportData.columns) {
                    let c = responseReportData.columns[key];
                    let t = c.type ? c.type.name || c.type.declaration : '';
                    if (transformColumns.includes(t)) {
                        columns.push({ name: c.name, type: t });
                    }
                }
            }
            if (columns.length > 0) {
                for (let row of responseReportData) {
                    for (let c of columns) {
                        let value = row[c.name];
                        if (value) {
                            let momentDate = moment.parseZone(value).isValid() && moment.parseZone(value) || null;
                            value = (momentDate) ? momentDate.utc().add(clientOffset, 'minutes').format('YYYY-MM-DD HH:mm:ss') : null;
                            row[c.name] = value;
                        }
                    }
                }
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: responseReportData });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ReportService = ReportService;
