"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveCategoryPermission = exports.SaveRolePermissionsGroup = exports.SaveRolesRoutes = exports.SaveRoles = exports.SaveRole = exports.GetRoleFunctions = exports.GetRole = exports.RoleService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const role_model_1 = require("../models/role.model");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const user_service_1 = require("./user.service");
let logger = log4js.getLogger('ServerScripts');
function RoleService(app) {
    app.get('/api/role/get_roles/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetRoles(vcc, search_criteria, search_expression);
        }).then(resultRoles => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultRoles });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/role/get_role/:role_id', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let roleId = req.params.role_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!roleId)
                throw 'ROLE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetRole(vcc, roleId);
        }).then(resultRole => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultRole });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/create_role', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newRole = (req.body || {});
        newRole.createdDate = nowUtc;
        newRole.createdByUserId = userId;
        newRole.lastModifiedDate = nowUtc;
        newRole.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetRole(vcc, newRole.id);
        }).then(resultRole => {
            if (resultRole)
                'ROLE_SERVER_ERROR_EXISTS';
            return SaveRole(vcc, newRole);
        }).then(resultSaveRole => {
            if (!resultSaveRole)
                throw 'ROLE_SERVER_ERROR_ON_SAVE';
            return redis_service_1.Redis.DeleteRole(vcc, newRole);
        }).then(resultSaveRole => {
            if (res.headersSent)
                return;
            logger.info('Rol \'' + newRole.id + '\' creado correctamente');
            res.json({ status: resultSaveRole, description: (resultSaveRole ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SERVER_ERROR_ON_SAVE'), data: resultSaveRole });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/save_role', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newRole = (req.body || {});
        newRole.createdDate = null;
        newRole.createdByUserId = null;
        newRole.lastModifiedDate = nowUtc;
        newRole.lastModifiedByUserId = userId;
        logger.info('[RoleService::/api/role/save_role] getting newRole:', newRole);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.DeleteRolePermission(vcc, newRole.id);
        })
            .then(result => {
            logger.info('[RoleService::/api/role/save_role] deleted permissions :', result);
            if (!result)
                throw 'CANNOT_DELETE_PERMISSION_ROLE';
            return mssql_service_1.MSSql.GetRole(vcc, newRole.id);
        }).then(resultGetRole => {
            logger.info('[RoleService::/api/role/save_role] getting resultGetRole:', resultGetRole);
            if (!resultGetRole)
                throw 'CANNOT_GET_ROLE_DATA';
            return SaveRole(vcc, newRole);
        }).then(resultSaveRole => {
            if (!resultSaveRole)
                throw 'ROLE_SAVE_FAILED';
            return redis_service_1.Redis.DeleteRole(vcc, newRole);
        }).then(resultSaveRole => {
            logger.info('[UserService::/api/role/save_user] getting resultSaveRole:', resultSaveRole, 'after Redis.DeleteRole');
            if (res.headersSent)
                return;
            logger.info('Rol \'' + newRole.id + '\' guardado correctamente');
            res.json({ status: resultSaveRole, description: (resultSaveRole ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SAVE_FAILED'), data: resultSaveRole });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/delete_role', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let role = req.body.role;
        let roleId = role.id;
        logger.info(`[RoleService::delete_role] Deleting role ${roleId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!role)
                throw 'ROLE_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteRole(vcc, roleId);
        }).then(resultDeleteRole => {
            if (!resultDeleteRole)
                throw 'ROLE_SERVER_ERROR_ON_DELETE';
            return redis_service_1.Redis.DeleteRole(vcc, role);
        }).then(resultDeleteRole => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteRole, description: (resultDeleteRole ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SERVER_ERROR_ON_DELETE'), data: resultDeleteRole });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/role/delete_roles', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let roles = req.body.roles || [];
        logger.info(`[RoleService::delete_roles] Deleting roles ${roles.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!roles)
                throw 'ROLE_SERVER_ERROR_MISSING_IDS';
            let promiseArray = [];
            roles.forEach(role => {
                promiseArray.push(mssql_service_1.MSSql.DeleteRole(vcc, role.id));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteRoles => {
            let result;
            if (resultDeleteRoles.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Roles eliminados correctamente');
                roles.forEach(role => {
                    redis_service_1.Redis.DeleteRole(vcc, role);
                });
            }
            return result;
        }).then(resultDeleteRoles => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteRoles, description: (resultDeleteRoles ? 'SERVER_SUCCESS_MESSAGE' : 'ROLE_SERVER_ERROR_ON_DELETE'), data: resultDeleteRoles });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/role/get_roles_permission_groups', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetRolePermissionsGroups(vcc);
        }).then(resultPermissionGroups => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultPermissionGroups });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/role/get_category_permission', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCategoryPermissions(vcc);
        }).then(resultCategoryPermission => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultCategoryPermission });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.RoleService = RoleService;
function GetRole(vcc, id) {
    let cacheUpdateNeeded = false;
    let role = null;
    return redis_service_1.Redis.GetRole(vcc, id)
        .then((redisRole) => {
        if (redisRole) {
            return Promise.resolve(redisRole);
        }
        else {
            logger.info('[RoleService::GetRole] The GetRole data for \'' + id + '\' was not found on cache. It will be searched on database.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetRole(vcc, id);
        }
    })
        .then((result) => {
        role = result;
        if (role) {
            return GetRoleFunctions(vcc, null, id, role);
        }
        else {
            throw 'ROLE_SERVER_ERROR_NOT_FOUND';
        }
    })
        .then(result => {
        if (result) {
            role.roleFunctions = result.data;
            if (cacheUpdateNeeded)
                redis_service_1.Redis.SaveRole(vcc, role);
        }
        return Promise.resolve(role);
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[RoleService::GetRole] Error al ejecutar GetRole: ' + errMsg);
        return Promise.reject(errMsg);
    });
}
exports.GetRole = GetRole;
function GetRoleFunctions(vcc, userId, roleId, role) {
    let roleFunctions = {};
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (roleId) {
            return Promise.resolve({ role: roleId });
        }
        else {
            return user_service_1.GetUser(vcc, userId);
        }
    })
        .then(result => {
        if (role) {
            return Promise.resolve(role);
        }
        else {
            return GetRole(vcc, result.role);
        }
    })
        .then(result => {
        roleFunctions.status = true;
        if (result.roleFunctions) {
            roleFunctions.data = result.roleFunctions;
        }
        else {
            roleFunctions.data = new role_model_1.RoleFunctions();
            let userCategories = [];
            result.categories.map(c => {
                userCategories.push(c.categoryId);
            });
            if (userCategories.includes('CATEG_GROUP_QUALITY_ADMIN'))
                roleFunctions.data.hasAdminFunctions = true;
            if (userCategories.includes('CATEG_GROUP_QUALITY_AGENT'))
                roleFunctions.data.hasAgentFunctions = true;
            if (userCategories.includes('CATEG_GROUP_QUALITY_AUDITOR'))
                roleFunctions.data.hasAuditorFunctions = true;
            if (userCategories.includes('CATEG_GROUP_QUALITY_SUPERVISOR'))
                roleFunctions.data.hasSupervisorFunctions = true;
        }
        return Promise.resolve(roleFunctions);
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[RoleService::GetRoleFunctions] Error on: ' + errMsg);
        roleFunctions.status = false;
        return Promise.resolve(roleFunctions);
    });
}
exports.GetRoleFunctions = GetRoleFunctions;
function SaveRole(vcc, role) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveRole(vcc, role.id, role.description, role.systemRole, role.type, role.createdDate, role.createdByUserId, role.lastModifiedDate, role.lastModifiedByUserId));
    if (role.permissions) {
        role.permissions.forEach(rolePermission => {
            promiseArray.push(mssql_service_1.MSSql.SaveRolePermission(vcc, role.id, rolePermission.groupId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRole = SaveRole;
function SaveRoles(vcc, role) {
    let promiseArray = [];
    if (role) {
        role.forEach(roles => {
            promiseArray.push(mssql_service_1.MSSql.SaveRole(vcc, roles.id, roles.description, roles.systemRole, roles.type, roles.createdDate, roles.createdByUserId, roles.lastModifiedDate, roles.lastModifiedByUserId));
            if (roles.permissions) {
                roles.permissions.forEach(rolePermission => {
                    promiseArray.push(mssql_service_1.MSSql.SaveRolePermission(vcc, rolePermission.roleId, rolePermission.groupId));
                });
            }
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRoles = SaveRoles;
function SaveRolesRoutes(vcc, role) {
    let promiseArray = [];
    if (role.permissions) {
        role.permissions.forEach(rolePermission => {
            promiseArray.push(mssql_service_1.MSSql.SaveRolePermission(vcc, rolePermission.roleId, rolePermission.groupId));
        });
    }
    if (role.routePermissions) {
        role.routePermissions.forEach(roleRoutePermissions => {
            promiseArray.push(mssql_service_1.MSSql.SaveRoutePermission(vcc, roleRoutePermissions.routeId, roleRoutePermissions.groupId, roleRoutePermissions.route, roleRoutePermissions.level));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRolesRoutes = SaveRolesRoutes;
function SaveRolePermissionsGroup(vcc, permissionsGroup) {
    let promiseArray = [];
    if (permissionsGroup) {
        permissionsGroup.forEach(rolePermissionsGroups => {
            promiseArray.push(mssql_service_1.MSSql.SaveRolePermissionsGroup(vcc, rolePermissionsGroups.groupId, rolePermissionsGroups.category, rolePermissionsGroups.type));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveRolePermissionsGroup = SaveRolePermissionsGroup;
function SaveCategoryPermission(vcc, categories) {
    let promiseArray = [];
    if (categories) {
        categories.forEach(category => {
            promiseArray.push(mssql_service_1.MSSql.SaveCategoryPermissions(vcc, category.category, category.position));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveCategoryPermission = SaveCategoryPermission;
