"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubgroupService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
let logger = log4js.getLogger('ServerScripts');
function SubgroupService(app) {
    app.get('/api/subgroup/get_subgroups/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[SubgroupService::get_subgroups] Received ' + req.method + ' on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetSubgroups(instance, search_criteria, search_expression, disabled);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let subgroupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: subgroupMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/create_subgroup', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newSubgroup = (req.body || {});
        newSubgroup.createdDate = nowUtc;
        newSubgroup.createdByUserId = userId;
        newSubgroup.lastModifiedDate = nowUtc;
        newSubgroup.lastModifiedByUserId = userId;
        newSubgroup.description = newSubgroup.description.trim();
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetSubgroupDescription(instance, newSubgroup.description);
        }).then(result => {
            if (result)
                throw 'SUBGROUP_SERVER_ERROR_EXISTS';
            newSubgroup.id = utilities_1.guid();
            return SaveSubgroup(instance, newSubgroup);
        }).then(result => {
            if (!result)
                throw 'SUBGROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/subgroup/get_subgroup/:subgroup_id', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let subgroupId = req.params.subgroup_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!subgroupId)
                throw 'SUBGROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetSubgroup(instance, subgroupId);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/save_subgroup', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newSubgroup = (req.body || {});
        newSubgroup.createdDate = null;
        newSubgroup.createdByUserId = null;
        newSubgroup.lastModifiedDate = nowUtc;
        newSubgroup.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetSubgroupDescription(instance, newSubgroup.description);
        }).then(result => {
            let subGrupDesc = {};
            subGrupDesc = result || subGrupDesc;
            if (result && subGrupDesc.id !== newSubgroup.id)
                throw 'SUBGROUP_SERVER_ERROR_EXISTS';
            return SaveSubgroup(instance, newSubgroup);
        }).then(resultSave => {
            if (!resultSave)
                throw 'SUBGROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(resultSave => {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/delete_subgroup', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let subgroup = (req.body || {});
        let subgroupId = subgroup.id;
        logger.info(`[SubGroupService::delete_subgroup] Deleting subgroup ${subgroupId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!subgroup)
                throw 'SUBGROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteSubgroup(instance, subgroupId);
        }).then(resultDelete => {
            if (!resultDelete)
                throw 'SUBGROUP_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDelete => {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_DELETE'), data: resultDelete });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/delete_subgroups', (req, res, next) => {
        logger.info('[subGroupService::delete_subgroups] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let groups = req.body.subgroups || [];
        let ids = groups.map(item => item.id);
        logger.info(`[SubGroupService::delete_subgroups] Deleting subgroups ${ids.join(',')} by the user ${res.locals.userId}`);
        let response;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetSubGroupsUsage(vcc, ids);
        })
            .then(resultNoDeleted => {
            let deleteData = ids;
            let noDelete = [];
            if (resultNoDeleted.length > 0) {
                deleteData = [];
                groups.forEach(item => {
                    if (!resultNoDeleted.find(noDeleted => noDeleted.idSubGroup === item.id)) {
                        deleteData.push(item.id);
                    }
                    else {
                        noDelete.push(item.description);
                    }
                });
            }
            return Promise.resolve({ noDelete, deleteData });
        }).then(resultDelete => {
            response = resultDelete;
            if (resultDelete.deleteData.length)
                return mssql_service_1.MSSql.DeleteSubGroups(vcc, resultDelete.deleteData);
            return Promise.resolve(true);
        }).then(resultDelete => {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[subgroupService::delete_subgroups] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.SubgroupService = SubgroupService;
function SaveSubgroup(instance, subgroup) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveSubgroup(instance, subgroup.id, subgroup.description, subgroup.createdDate, subgroup.createdByUserId, subgroup.lastModifiedDate, subgroup.lastModifiedByUserId, subgroup.enabled));
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
