"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanupTempFile = exports.JoinPath = exports.UploadFileToServer = exports.GetFolderFiles = exports.GetFolderTree = exports.UnZipFolder = exports.ZipFolder = exports.WriteFileContent = exports.BufferFile = exports.FileToBase64 = exports.ReadFileContent = exports.DeleteFile = exports.DeleteFolder = exports.CopyFolder = exports.CopyFile = exports.RenameFile = exports.TouchPath = exports.ExistInFileSystem = exports.CreateFolder = exports.ListFolderContent = exports.FileProperty = exports.DeleteFileSync = exports.PipeWriteFile = void 0;
const log4js = require("log4js");
const moment = require("moment");
const path = require("path");
const fs = require("fs");
const ncp = require("ncp");
const mkdirp = require("mkdirp");
const rimraf = require("rimraf");
const multer = require("multer");
const extract = require("extract-zip");
const child_process = require("child_process");
let fse = require('fs-extra');
let zipFolder = require('zip-folder');
let mv = require('mv');
const app_config_1 = require("../config/app.config");
const promises_shared_1 = require("./promises.shared");
let exec = child_process.exec;
let logger = log4js.getLogger('ServerScripts');
function PipeWriteFile(response, file) {
    let Promise = require('bluebird');
    let ended = false;
    return new Promise(function (resolve, reject) {
        response.pipe(file);
        function nice_ending() {
            if (!ended) {
                ended = true;
                resolve(true);
            }
        }
        function error_ending() {
            if (!ended) {
                ended = true;
                reject(false);
            }
        }
        file.on('finish', nice_ending);
        file.on('end', nice_ending);
        file.on('error', error_ending);
        file.on('close', error_ending);
    }).finally(() => file.close());
}
exports.PipeWriteFile = PipeWriteFile;
function DeleteFileSync(file) {
    var fs = require('fs');
    fs.unlinkSync(file);
}
exports.DeleteFileSync = DeleteFileSync;
function FileProperty(filePath) {
    return new Promise(function (resolve, reject) {
        try {
            let result = {};
            if (fs.existsSync(filePath) && fs.lstatSync(filePath).isFile()) {
                let fileStat = fs.lstatSync(filePath);
                let creationDate = moment(fileStat.birthtime);
                let updateDate = moment(fileStat.mtime);
                result = {
                    path: filePath,
                    isFile: fileStat.isFile(),
                    creationDate: creationDate,
                    updateDate: updateDate
                };
            }
            resolve(result);
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.FileProperty = FileProperty;
function ListFolderContent(folderPath) {
    return new Promise(function (resolve, reject) {
        try {
            let result = [];
            if (fs.existsSync(folderPath) && fs.lstatSync(folderPath).isDirectory()) {
                let files = fs.readdirSync(folderPath);
                files.forEach(fileName => {
                    let filePath = path.join(folderPath, fileName);
                    let fileStat = fs.lstatSync(filePath);
                    let creationDate = moment(fileStat.birthtime);
                    let updateDate = moment(fileStat.mtime);
                    result.push({
                        name: fileName,
                        path: filePath,
                        isFolder: fileStat.isDirectory(),
                        creationDate: creationDate,
                        updateDate: updateDate,
                    });
                });
            }
            resolve(result);
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.ListFolderContent = ListFolderContent;
function CreateFolder(folderName) {
    return ExistInFileSystem(folderName)
        .then(result => {
        if (result)
            return true;
        return mkdirp(folderName)
            .then(value => {
            return true;
        })
            .catch(err => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
            logger.error('Error al crear CreateFolder para \'' + folderName + '\': ' + errorMsg);
            return false;
        });
    });
}
exports.CreateFolder = CreateFolder;
function ExistInFileSystem(fileSystemPath) {
    return new Promise((resolve, reject) => {
        try {
            fs.stat(fileSystemPath, (err, stat) => {
                if (err === null || err === undefined)
                    return resolve(true);
                else {
                    logger.info(`[file-manager::ExistInFileSystem] Error on fs.stat searching ${fileSystemPath}: `, err);
                    return resolve(false);
                }
            });
        }
        catch (e) {
            logger.info(`[file-manager::ExistInFileSystem] Error searching ${fileSystemPath}:`, e);
            return resolve(false);
        }
    });
}
exports.ExistInFileSystem = ExistInFileSystem;
function TouchPath(fileSystemPath) {
    return new Promise(function (resolve, reject) {
        try {
            let now = moment().toDate();
            fs.utimes(fileSystemPath, now, now, function (err) {
                resolve(err === null || err === undefined);
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.TouchPath = TouchPath;
function RenameFile(currentSourcePath, newSourcePath) {
    return new Promise(function (resolve, reject) {
        try {
            fs.rename(currentSourcePath, newSourcePath, function (err) {
                resolve(err === null || err === undefined);
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.RenameFile = RenameFile;
function CopyFile(sourcePath, targetPath) {
    return new Promise(function (resolve, reject) {
        try {
            fse.copy(sourcePath, targetPath, function (err) {
                resolve(err === null || err === undefined);
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.CopyFile = CopyFile;
function CopyFolder(sourcePath, targetPath) {
    return new Promise(function (resolve, reject) {
        try {
            fs.stat(targetPath, function (err, stat) {
                resolve(err === null || err === undefined);
            });
        }
        catch (e) {
            reject(e);
        }
    })
        .then(resultExists => {
        if (resultExists)
            return true;
        return mkdirp(targetPath)
            .then(value => {
            return true;
        })
            .catch(err => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
            logger.error('Error al crear CopyFolder para \'' + targetPath + '\': ' + errorMsg);
            return false;
        });
    })
        .then(resultMkDir => {
        return new Promise(function (resolve, reject) {
            try {
                ncp.ncp(sourcePath, targetPath, function (err) {
                    resolve(err === null || err === undefined);
                });
            }
            catch (e) {
                reject(e);
            }
        });
    });
}
exports.CopyFolder = CopyFolder;
function DeleteFolder(folderPath) {
    return new Promise(function (resolve, reject) {
        try {
            rimraf(folderPath, function (err) {
                resolve(err === null || err === undefined);
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.DeleteFolder = DeleteFolder;
function DeleteFile(filePath, safe) {
    logger.info('[file-manager::DeleteFile] Deleting: ', filePath);
    return new Promise(function (resolve, reject) {
        try {
            fs.unlink(filePath, function (err) {
                if (err === null || err === undefined)
                    resolve(true);
                else {
                    if (safe) {
                        return resolve(true);
                    }
                    else {
                        logger.info('DeleteFile - err: ', err);
                        return reject(err);
                    }
                }
            });
        }
        catch (e) {
            if (safe) {
                return resolve(true);
            }
            else {
                reject(e);
            }
        }
    });
}
exports.DeleteFile = DeleteFile;
function ReadFileContent(fileName, encode, safe) {
    return new Promise(function (resolve, reject) {
        try {
            fs.readFile(fileName, encode, function (err, data) {
                logger.info('ReadFileContent - readFile - fileName: ', fileName);
                if (err) {
                    if (safe) {
                        return resolve(null);
                    }
                    else {
                        logger.info('ReadFileContent - readFile - err: ', err);
                        return reject(err);
                    }
                }
                else {
                    return resolve(data);
                }
            });
        }
        catch (e) {
            logger.info('ReadFileContent - error: ', e);
            if (safe)
                return resolve(null);
            else
                return reject(e);
        }
    });
}
exports.ReadFileContent = ReadFileContent;
function FileToBase64(file) {
    return new Promise(function (resolve, reject) {
        try {
            var buffer = fs.readFileSync(file);
            return resolve(Buffer.from(buffer).toString('base64'));
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.FileToBase64 = FileToBase64;
function BufferFile(file) {
    return new Promise(function (resolve, reject) {
        try {
            var buffer = fs.readFileSync(file);
            return resolve(buffer);
        }
        catch (e) {
            return reject(e);
        }
    });
}
exports.BufferFile = BufferFile;
function WriteFileContent(fileName, data, encode, safe) {
    return new Promise((resolve, reject) => {
        try {
            fs.writeFile(fileName, data, { encoding: encode }, function (err) {
                if (err) {
                    if (safe)
                        return resolve(null);
                    else
                        return reject(err);
                }
                else {
                    return resolve(true);
                }
            });
        }
        catch (e) {
            if (safe)
                return resolve(null);
            else
                return reject(e);
        }
    });
}
exports.WriteFileContent = WriteFileContent;
function ZipFolder(srcFolder, zipFilePath) {
    return new Promise((resolve, reject) => {
        try {
            zipFolder(srcFolder, zipFilePath, function (err) {
                resolve(err === null || err === undefined);
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.ZipFolder = ZipFolder;
function UnZipFolder(zipFilePath, destinationPath) {
    destinationPath = destinationPath || zipFilePath;
    return new Promise((resolve, reject) => {
        try {
            extract(zipFilePath, {
                dir: destinationPath
            }, function (err) {
                if (err) {
                    return reject(err);
                }
                else {
                    return resolve(true);
                }
            });
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.UnZipFolder = UnZipFolder;
function GetFolderTree(folderPath, extensionExclude, extensionFilter) {
    return new Promise(function (resolve, reject) {
        extensionExclude = extensionExclude || [];
        extensionFilter = extensionFilter || [];
        let walk = function (dir, obj, done) {
            fs.readdir(dir, function (err, list) {
                if (err)
                    return done(err);
                let pending = list.length;
                if (!pending)
                    return done(null);
                list.forEach(function (fileName) {
                    let filePath = path.resolve(dir, fileName);
                    fs.stat(filePath, function (err, stat) {
                        if (stat && stat.isDirectory()) {
                            let objDirectory = {
                                name: fileName,
                                relativePath: path.relative(folderPath, filePath).replace(/\\/g, '/'),
                                fullPath: filePath,
                                type: 'directory',
                                children: []
                            };
                            obj.push(objDirectory);
                            walk(filePath, objDirectory.children, function (err, res) {
                                objDirectory.size = objDirectory.children.reduce((prev, cur) => prev + cur.size, 0);
                                if (!--pending)
                                    done(null);
                            });
                        }
                        else {
                            let fileExtension = path.extname(fileName).toLowerCase().slice(1);
                            let skipFile = false;
                            if (fileExtension && extensionExclude.length && extensionExclude.indexOf(fileExtension) >= 0)
                                skipFile = true;
                            if (extensionFilter.length && (extensionFilter.indexOf(fileExtension) === -1 || !fileExtension))
                                skipFile = true;
                            if (!skipFile) {
                                let objFile = {
                                    name: fileName,
                                    type: 'file',
                                    extension: fileExtension,
                                    relativePath: path.relative(folderPath, filePath).replace(/\\/g, '/'),
                                    fullPath: filePath,
                                    size: stat.size
                                };
                                obj.push(objFile);
                            }
                            if (!--pending)
                                done(null);
                        }
                    });
                });
            });
        };
        let folderName = path.basename(folderPath);
        let objMainDirectory = {
            name: folderName,
            type: 'directory',
            relativePath: path.relative(folderPath, folderPath).replace(/\\/g, '/'),
            fullPath: folderPath,
            children: []
        };
        let folderTree = [];
        folderTree.push(objMainDirectory);
        walk(folderPath, objMainDirectory.children, function (err, results) {
            if (err)
                return reject(err);
            objMainDirectory.size = objMainDirectory.children.reduce((prev, cur) => prev + cur.size, 0);
            resolve(folderTree);
        });
    });
}
exports.GetFolderTree = GetFolderTree;
function GetFolderFiles(folderPath, extensionExclude, extensionFilter, recursive) {
    return new Promise(function (resolve, reject) {
        extensionExclude = extensionExclude || [];
        extensionFilter = extensionFilter || [];
        recursive = (recursive === true);
        let walk = function (dir, obj, done) {
            fs.readdir(dir, function (err, list) {
                if (err)
                    return done(err);
                let pending = list.length;
                if (!pending)
                    return done(null);
                list.forEach(function (fileName) {
                    let filePath = path.resolve(dir, fileName);
                    fs.stat(filePath, function (err, stat) {
                        if (stat && stat.isDirectory()) {
                            let objDirectory = {
                                name: fileName,
                                relativePath: path.relative(folderPath, filePath).replace(/\\/g, '/'),
                                fullPath: filePath,
                                type: 'directory',
                                children: []
                            };
                            obj.push(objDirectory);
                            if (recursive) {
                                walk(filePath, obj, function (err, res) {
                                    if (!--pending)
                                        done(null);
                                });
                            }
                            else {
                                if (!--pending)
                                    done(null);
                            }
                        }
                        else {
                            let fileExtension = path.extname(fileName).toLowerCase().slice(1);
                            let skipFile = false;
                            if (fileExtension && extensionExclude.length && extensionExclude.indexOf(fileExtension) >= 0)
                                skipFile = true;
                            if (extensionFilter.length && (extensionFilter.indexOf(fileExtension) === -1 || !fileExtension))
                                skipFile = true;
                            if (!skipFile) {
                                let objFile = {
                                    name: fileName,
                                    type: 'file',
                                    extension: fileExtension,
                                    relativePath: path.relative(folderPath, filePath).replace(/\\/g, '/'),
                                    fullPath: filePath,
                                    size: stat.size
                                };
                                obj.push(objFile);
                            }
                            if (!--pending)
                                done(null);
                        }
                    });
                });
            });
        };
        let folderFiles = [];
        walk(folderPath, folderFiles, function (err, results) {
            if (err) {
                let errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
                logger.error('Error al consultar archivos de la carpeta snapshot \'' + folderPath + '\': ' + errorMsg);
                resolve([]);
            }
            else {
                resolve(folderFiles);
            }
        });
    });
}
exports.GetFolderFiles = GetFolderFiles;
function UploadFileToServer(fileName, destFolder, req, res, types, fieldName, single, replace, fileSizeLimit) {
    single = (single === false ? single : true);
    fieldName = (fieldName ? fieldName : 'file');
    replace = (replace === false ? replace : true);
    fileSizeLimit = (fileSizeLimit ? fileSizeLimit : app_config_1.FILE_UPLOAD_LIMIT);
    req.multerCounter = 0;
    let storageEngine = multer.diskStorage({
        destination: function (req, file, cb) {
            cb(null, destFolder);
        },
        filename: function (req, file, cb) {
            cb(null, fileName || file.originalname);
        }
    });
    let fileFilter = function (req, file, cb) {
        req.multerCounter++;
        let typesRegEx = '\.(' + (types.toLowerCase() || '.+') + ')$';
        let regEx = new RegExp(typesRegEx);
        if (types && !file.originalname.toLowerCase().match(regEx)) {
            logger.error('El archivo recibido no está dentro de los tipos permitidos  (' + types.toLowerCase() + ').');
            return cb(new Error('Only ' + types.toLowerCase() + ' files are allowed!'));
        }
        if (replace) {
            cb(null, true);
        }
        else {
            ExistInFileSystem(path.join(destFolder, file.originalname))
                .then(result => {
                return cb(null, !result);
            });
        }
    };
    let upload;
    if (single) {
        upload = multer({
            storage: storageEngine,
            fileFilter: fileFilter,
            limits: { fileSize: fileSizeLimit },
        }).single(fieldName);
    }
    else {
        upload = multer({
            storage: storageEngine,
            fileFilter: fileFilter,
            limits: { fileSize: fileSizeLimit },
        }).array(fieldName);
    }
    let tempFilename = path.join(destFolder, fileName);
    return promises_shared_1.DummyPromise()
        .then(result => {
        if (!fileName)
            return true;
        return DeleteFolder(tempFilename);
    }).then(resultDelete => {
        if (!resultDelete)
            throw 'FILE_SERVER_ERROR_CANNOT_DELETE ' + tempFilename + '\'';
        return new Promise((resolve, reject) => {
            return upload(req, res, function (err) {
                if (err && err.code === 'LIMIT_FILE_SIZE') {
                    logger.error('El archivo recibido supera el límite permitido (' + fileSizeLimit.toString() + 'bytes).');
                    reject('ERROR_FILE_UPLOAD_LIMIT' + '|' + (Math.round(100.00 * fileSizeLimit / 1024 / 1024) / 100.00).toString());
                }
                else if (err) {
                    logger.error('Error al subir archivo: ' + err);
                    reject(err);
                }
                else {
                    resolve(true);
                }
            });
        });
    });
}
exports.UploadFileToServer = UploadFileToServer;
function JoinPath(route, fileName) {
    return path.join(route, fileName);
}
exports.JoinPath = JoinPath;
function CleanupTempFile(path, persistence) {
    logger.info('[file-manager:CleanupTempFile]', path, persistence);
    return ListFolderContent(path)
        .then(elements => {
        let promiseArray = [];
        elements.forEach((element) => {
            let secondsOld = moment().diff(element.updateDate, 'seconds');
            if (secondsOld >= persistence) {
                if (!element.isFolder) {
                    promiseArray.push(DeleteFile(element.path));
                }
            }
            else if (element.isFolder) {
                CleanupTempFile(element.path, persistence);
            }
        });
        return Promise.all(promiseArray);
    }).then((results) => {
        let result = (results && results.length > 0) ? !results.includes(false) : true;
        return Promise.resolve(result);
    }).catch(error => {
        logger.error('[file-manager:CleanupTempFile] error: ', error);
        return Promise.reject(error);
    });
}
exports.CleanupTempFile = CleanupTempFile;
