"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiI6GetInteractions = exports.ApiI6GetCampaignMembers = exports.ApiI6GetAsyncInteractionContent = exports.ApiI6GetInteractionFile = exports.ApiI6GetInteractionParts = exports.ApiI6GetInteractionData = exports.ApiI6ConfiguredChannels = exports.ApiI6GetCampaignDispositions = exports.ApiI6SearchCampaigns = exports.ApiI6SearchUsers = exports.SaveUserByAPIi6 = exports.ApiI6GetUser = exports.SearchUsers = exports.GetRequestQuote = exports.Login = void 0;
const log4js = require("log4js");
const moment = require("moment");
const integration_config_1 = require("../config/integration.config");
const redis_service_1 = require("../db/redis.service");
const request_shared_1 = require("./request.shared");
const arrays_shared_1 = require("./arrays.shared");
const promises_shared_1 = require("./promises.shared");
let loggerAllegro = log4js.getLogger('ServerScripts');
function Login(vcc, user, password) {
    let i6IntegrationData = null;
    let needUpdateOnRedis = false;
    if (!user)
        user = `${integration_config_1.USER_API_I6}@${vcc}`;
    if (!password)
        password = integration_config_1.PASS_API_I6;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return redis_service_1.Redis.GetI6Integration(vcc);
    })
        .then(result => {
        if (result?.accessToken)
            return Promise.resolve(result);
        needUpdateOnRedis = true;
        loggerAllegro.info('[http-request-occ::Login] Login por API i6', JSON.stringify({ user: user, password: password }));
        let data = { user: user, password: password };
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/login/`,
            body: data,
            headers: {
                'Content-Type': 'application/json'
            },
            retryProtocol: true
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        if (needUpdateOnRedis) {
            let status = response?.status ? true : false;
            let message = response ? response : '';
            i6IntegrationData = { status: status, accessToken: message.token };
            return redis_service_1.Redis.SaveI6Integration(vcc, i6IntegrationData);
        }
        else {
            i6IntegrationData = response;
            return Promise.resolve(true);
        }
    })
        .then(_ => {
        return Promise.resolve(i6IntegrationData);
    })
        .catch((err) => {
        loggerAllegro.error('[http-request-occ::Login] Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.Login = Login;
function GetRequestQuote(vcc) {
    let redisData = null;
    let i6IRequestQuote = null;
    let needUpdateOnRedis = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return Login(vcc);
    })
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        if (result?.requestQuote)
            return Promise.resolve(result);
        redisData = result;
        needUpdateOnRedis = true;
        loggerAllegro.info('[http-request-occ::GetRequestQuote] Getting request quote');
        let options = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/vcc/get/`,
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(options);
    })
        .then((response) => {
        if (needUpdateOnRedis) {
            if (!response?.vcc?.ApiConfiguration?.urlsConfig?.resources)
                throw 'API_I6_GET_REQUEST_QUOTE_FAILED';
            i6IRequestQuote = response?.vcc?.ApiConfiguration?.urlsConfig?.resources;
            redisData.requestQuote = i6IRequestQuote;
            return redis_service_1.Redis.SaveI6Integration(vcc, redisData);
        }
        else {
            i6IRequestQuote = response.requestQuote;
            return Promise.resolve(true);
        }
    })
        .then(result => {
        loggerAllegro.debug("allegroi6quote", i6IRequestQuote);
        return Promise.resolve({ status: true, body: { i6IRequestQuote } });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::GetRequestQuote] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.GetRequestQuote = GetRequestQuote;
function SearchUsers(accessToken, request) {
    loggerAllegro.info('[http-request-occ::SearchUsers] Request (inConcert): Listado de usuarios por API i6');
    let postOptions = {
        uri: `http://${integration_config_1.DNS}/inconcert/api/users/search/`,
        body: request,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        },
        retryProtocol: true
    };
    return request_shared_1.SimplePostRequest(postOptions)
        .then((response) => {
        return Promise.resolve(response);
    })
        .catch((err) => {
        loggerAllegro.error('[http-request-occ::SearchUsers] : Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SearchUsers = SearchUsers;
function ApiI6GetUser(vcc, userId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetUser] Getting campaigns');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let getOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/users/get/${userId}`,
            method: 'GET',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then((response) => {
        return Promise.resolve({ status: true, body: response });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetUser] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetUser = ApiI6GetUser;
function SaveUserByAPIi6(vcc, user) {
    loggerAllegro.info('[http-request-occ::SaveUserByAPIi6] Request (inConcert): Creación de usuario por API i6');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/users/save/`,
            body: user,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        return Promise.resolve(response);
    })
        .catch((err) => {
        loggerAllegro.error('[http-request-occ::SaveUserByAPIi6] Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SaveUserByAPIi6 = SaveUserByAPIi6;
function ApiI6SearchUsers(vcc, campaignIds, applicationId = 'interaction') {
    loggerAllegro.info('[http-request-occ::ApiI6SearchUsers] Getting users');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/users/search/`,
            body: {
                query: { Application: applicationId }
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true,
            timeout: 1500000
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
        let agents = [];
        response['rows'].map(r => {
            agents.push({
                vcc: vcc,
                campaignId: null,
                campaigns: [],
                userId: r.Id,
                userName: (r.Name ? r.Name.trim() + ' ' : '') + (r.FirstSurname ? r.FirstSurname.trim() : '')
            });
        });
        return Promise.resolve({ status: true, body: agents });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchUsers] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchUsers = ApiI6SearchUsers;
function ApiI6SearchCampaigns(vcc) {
    loggerAllegro.info('[http-request-occ::ApiI6SearchCampaigns] Getting campaigns');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/campaigns/search/`,
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
        let result;
        result = response['rows'].map(r => {
            let campaign = r.Id.split('@');
            let dispositionCodes = [];
            if (r.dispositionCodes) {
                r.dispositionCodes.map(dc => {
                    if (dc.childs) {
                        dc.childs.map(c1 => {
                            if (c1.childs) {
                                c1.childs.map(c2 => {
                                    if (c2.childs) {
                                        c2.childs.map(c3 => {
                                            dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: c3.name });
                                        });
                                    }
                                    else {
                                        dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: null });
                                    }
                                });
                            }
                            else {
                                dispositionCodes.push({ parent: dc.name, child: c1.name, child2: null, child3: null });
                            }
                        });
                    }
                    else {
                        dispositionCodes.push({ parent: dc.name, child: null, child2: null, child3: null });
                    }
                });
            }
            return { id: campaign[0], description: r.Description, dispositionCodes: dispositionCodes };
        });
        return Promise.resolve({ status: true, body: result });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchCampaigns] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchCampaigns = ApiI6SearchCampaigns;
function ApiI6GetCampaignDispositions(vcc) {
    loggerAllegro.info('[http-request-occ::ApiI6GetCampaignDispositions] Getting disposition codes');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/campaigns/get_dispositions/' + null,
            method: 'GET',
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(postOptions);
    })
        .then((response) => {
        return Promise.resolve({ status: true, body: response });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetCampaignDispositions] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetCampaignDispositions = ApiI6GetCampaignDispositions;
function ApiI6ConfiguredChannels(vcc, campaign) {
    loggerAllegro.info('[http-request-occ::ApiI6ConfiguredChannels] Getting campaigns:', campaign.toString());
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/campaigns/get_configured_channels/`,
            body: {
                campaign: campaign && campaign.length && campaign.length === 1 ? campaign[0] : null
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_TYPES_FAILED';
        let keys = Object.keys(response.channels);
        keys = keys.filter(k => k !== 'WEBCONTACT');
        let result = {};
        if (campaign && (campaign.length === 0 || campaign.length && campaign.length === 1)) {
            result = keys.map(k => {
                return {
                    instance: vcc,
                    id: k,
                    description: k
                };
            });
        }
        else {
            let channelCampaigns = [];
            let channels = [];
            keys.map(k => {
                let campaigns = Object.keys(response.channels[k]);
                campaigns = campaigns.map(c => c = c.split('@')[0]);
                channelCampaigns.push({
                    channel: k,
                    campaigns: campaigns
                });
            });
            channelCampaigns.map(cc => {
                cc.campaigns.map(c => {
                    if (campaign.includes(c))
                        channels.push(cc.channel);
                });
            });
            channels = arrays_shared_1.GetUniqueFromArray(channels);
            result = channels.map(c => {
                return {
                    instance: vcc,
                    id: c,
                    description: c
                };
            });
        }
        result = arrays_shared_1.SortObjectArrayByValue(result, 'id', 'A');
        return Promise.resolve({ status: true, body: result });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6ConfiguredChannels] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6ConfiguredChannels = ApiI6ConfiguredChannels;
function ApiI6GetInteractionData(vcc, interactionId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionData] Getting interaction parts');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let getOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/interactions/get/${interactionId}`,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then((response) => {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_DATA_FAILED';
        return Promise.resolve({ status: true, body: response.interaction });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionData] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionData = ApiI6GetInteractionData;
function ApiI6GetInteractionParts(vcc, interactionId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractionParts] Getting interaction parts');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let getOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/interactions/media/list/${interactionId}`,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then((response) => {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_PARTS_FAILED';
        return Promise.resolve({ status: true, body: response.media });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractionParts] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionParts = ApiI6GetInteractionParts;
function ApiI6GetInteractionFile(vcc, interactionId, section, part, output = 'buffer') {
    let logLabel = `[http-request-occ::ApiI6GetInteractionFile] `;
    loggerAllegro.info(`${logLabel}Getting audio file`);
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let getOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/interactions/media/get/?interaction=${interactionId}&id=${interactionId}&section=${section}&part=${part}`,
            headers: {
                'Authorization': result.accessToken
            },
            responseType: 'stream',
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then(result => {
        if (result && result.body && result.body.includes('404'))
            throw 'SERVER_ERROR_FILE_NOT_FOUND';
        loggerAllegro.info(`${logLabel}OK`);
        if (output === 'stream')
            return Promise.resolve(result);
        else {
            let buffer = null;
            const bufs = [];
            return new Promise((resolve, reject) => {
                result.on('error', err => {
                    loggerAllegro.error(`${logLabel}SERVER_ERROR_CREATE_READ_STREAM_FAILED`);
                    reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                });
                result.once('close', () => {
                    try {
                        buffer = Buffer.concat(bufs);
                        return resolve(buffer);
                    }
                    catch (e) {
                        return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                    }
                });
                result.on('data', d => {
                    try {
                        bufs.push(d);
                    }
                    catch (e) {
                        return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                    }
                });
            });
        }
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        loggerAllegro.error(`${logLabel}Error ${errorMessage}`);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractionFile = ApiI6GetInteractionFile;
function ApiI6GetAsyncInteractionContent(vcc, channel, contentId) {
    loggerAllegro.info('[http-request-occ::ApiI6GetAsyncInteractionContent] Getting text content');
    let section = 0;
    switch (channel) {
        case 'MAIL':
            section = -1;
            break;
    }
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let getOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/repository/download/?id=${contentId}&section=${section}`,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimpleGetRequest(getOptions);
    })
        .then(result => {
        return Promise.resolve({ status: true, body: result });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetAsyncInteractionContent] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetAsyncInteractionContent = ApiI6GetAsyncInteractionContent;
function ApiI6GetCampaignMembers(vcc, campaigns) {
    loggerAllegro.info('[http-request-occ::ApiI6GetCampaignMembers] Getting campaign members');
    return Login(vcc)
        .then(result => {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/campaigns/get_campaign_members/`,
            body: {
                campaigns: campaigns
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGN_MEMBERS_FAILED';
        let campaigns = [];
        response.rows.map(r => {
            let campaign = {};
            campaign.id = r.Id ? r.Id.split('@')[0] : null;
            if (campaign.id) {
                let agents = [];
                let supervisors = [];
                let agentWg = [];
                let supervisorWg = [];
                r.members.map(m => {
                    if (m.apps_permisson) {
                        if (m.apps_permisson.interaction) {
                            agents.push(m.Id.split('@')[0]);
                            let agent = {};
                            agent.id = m.Id.split('@')[0];
                            agent.workGroupId = m.WorkgroupId ? m.WorkgroupId : null;
                            agentWg.push(agent);
                        }
                        if (m.apps_permisson.supervisor) {
                            supervisors.push(m.Id.split('@')[0]);
                            let supervisor = {};
                            supervisor.id = m.Id.split('@')[0];
                            supervisor.workGroups = [];
                            if (m.supervisor_workgroups)
                                Object.keys(m.supervisor_workgroups).map(w => {
                                    if (m.supervisor_workgroups[w])
                                        supervisor.workGroups.push(w);
                                });
                            supervisorWg.push(supervisor);
                        }
                    }
                });
                campaign.supervisorAgents = [];
                supervisorWg.map(swg => {
                    let supervisorAgents = {};
                    supervisorAgents.id = swg.id;
                    supervisorAgents.agents = [];
                    if (swg.workGroups.length) {
                        swg.workGroups.map(wg => {
                            agentWg.filter(awg => awg.workGroupId === wg).map(awg => supervisorAgents.agents.push(awg.id));
                        });
                    }
                    else {
                        agentWg.map(awg => supervisorAgents.agents.push(awg.id));
                    }
                    campaign.supervisorAgents.push(supervisorAgents);
                });
                campaign.agents = arrays_shared_1.GetUniqueFromArray(agents);
                campaign.supervisors = arrays_shared_1.GetUniqueFromArray(supervisors);
                campaigns.push(campaign);
            }
        });
        return Promise.resolve({ status: true, body: campaigns });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6SearchUsers] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetCampaignMembers = ApiI6GetCampaignMembers;
function ApiI6GetInteractions(vcc, channel, filters, defaultRowsNumber, page) {
    let promiseArray = [];
    if (filters.interactionIds.length) {
        filters.interactionIds.forEach(item => {
            promiseArray.push(ApiI6GetInteractionData(vcc, item));
        });
    }
    if (!page)
        page = 1;
    if (!defaultRowsNumber)
        defaultRowsNumber = 500;
    let from = (page - 1) * defaultRowsNumber;
    loggerAllegro.info('[http-request-occ::ApiI6GetInteractions] Getting interactions');
    let occFilters = {
        type: channel,
        state: 'FINISHED',
        date_last_event_from: moment(filters.dateIni).utc().format('YYYY-MM-DD HH:mm:ss'),
        date_last_event_to: moment(filters.dateFin).utc().format('YYYY-MM-DD HH:mm:ss'),
    };
    if (filters?.campaigns?.length)
        occFilters.campaign = filters.campaigns;
    if (filters?.agents?.length)
        occFilters.agent = filters.agents;
    if (filters?.origins?.length)
        occFilters.initiative = filters.origins[0] === 'E' ? 'INBOUND' : 'OUTBOUND';
    if (filters?.contactAddress?.length)
        occFilters.contact_id = filters.contactAddress;
    if (filters?.results?.length)
        occFilters.disposition_code = filters.results;
    if (filters?.minDuration || filters?.maxDuration) {
        occFilters.min_duration = filters.minDuration;
        occFilters.max_duration = filters.maxDuration;
    }
    const data = filters.interactionIds.length ? Promise.all(promiseArray) : Login(vcc);
    return data
        .then(result => {
        if (filters.interactionIds.length) {
            return Promise.resolve(result);
        }
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        let postOptions = {
            uri: `http://${integration_config_1.DNS}/inconcert/api/interactions/load/`,
            body: {
                query: occFilters,
                last: from,
                rows: defaultRowsNumber
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            retryProtocol: true
        };
        return request_shared_1.SimplePostRequest(postOptions);
    })
        .then((response) => {
        let interactions = [];
        if (filters.interactionIds.length) {
            response.forEach(item => {
                if (item.status) {
                    const { body } = item;
                    if (body.type === channel) {
                        if (!occFilters.campaign) {
                            interactions.push({
                                agent: body.__last_step.agent,
                                campaign: body.campaign,
                                id: body.id,
                                disposition_code: body.dispositionCode,
                                start_timestamp: moment.utc(new Date(body.startTimestamp * 1000)).format('YYYY-MM-DD HH:mm:ss'),
                                state_timestamp: body.timestamp,
                                total_characters: body.interactionData.totals.totalCharacters || 0,
                                total_records_time_sec: body.interactionData.totals.totalRecordsTimeSec || 0,
                                type: body.type,
                                last_event_timestamp: body.timestamp,
                                state: body.state
                            });
                        }
                        else if (occFilters.campaign && occFilters.campaign.includes(body.campaign.split('@')[0])) {
                            interactions.push({
                                agent: body.__last_step.agent,
                                campaign: body.campaign,
                                id: body.id,
                                disposition_code: body.dispositionCode,
                                start_timestamp: moment.utc(new Date(body.startTimestamp * 1000)).format('YYYY-MM-DD HH:mm:ss'),
                                state_timestamp: body.timestamp,
                                total_characters: body.interactionData.totals.totalCharacters || 0,
                                total_records_time_sec: body.interactionData.totals.totalRecordsTimeSec || 0,
                                type: body.type,
                                last_event_timestamp: body.timestamp,
                                state: body.state
                            });
                        }
                    }
                }
            });
        }
        else {
            if (response?.status === false)
                throw 'SERVER_ERROR_GET_API_I6_LOAD_INTERACTIONS_FAILED';
            if (response.content.total_interactions > 0) {
                let interactionIds = Object.keys(response.content.interactions);
                interactions = interactionIds.map(ii => response.content.interactions[ii]);
                interactions = interactions.filter(i => {
                    if (i.type === 'CALL') {
                        return i.total_records_time_sec ? true : false;
                    }
                    else
                        return true;
                });
                let interactionEndDate = null;
                let hourFormat = 'hh:mm:ss';
                let referenceMinDate = moment('00:00:00', hourFormat);
                let mintHour = filters.hourIni.split(':');
                referenceMinDate.set('hour', Number(mintHour[0]));
                referenceMinDate.set('minute', Number(mintHour[1]));
                referenceMinDate.set('second', Number(mintHour[2]));
                let referenceMaxDate = moment('00:00:00', hourFormat);
                let maxHour = filters.hourFin.split(':');
                referenceMaxDate.set('hour', Number(maxHour[0]));
                referenceMaxDate.set('minute', Number(maxHour[1]));
                referenceMaxDate.set('second', Number(maxHour[2]));
                interactions = interactions.filter(i => {
                    interactionEndDate = moment.utc(i.last_event_timestamp);
                    let currentHour = moment.utc(`${interactionEndDate.hour()}:${interactionEndDate.minutes()}:${interactionEndDate.seconds()}`, hourFormat);
                    if (referenceMaxDate.isSameOrAfter(referenceMinDate))
                        return currentHour.isBetween(referenceMinDate, referenceMaxDate);
                    else
                        return !currentHour.isBetween(referenceMaxDate, referenceMinDate);
                });
            }
        }
        return Promise.resolve({
            status: true,
            body: {
                interactions,
                total: filters.interactionIds.length ? interactions.length : response.content.total_interactions
            }
        });
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[http-request-occ::ApiI6GetInteractions] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetInteractions = ApiI6GetInteractions;
