"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetVCC = exports.GetRequestUser = exports.GetRequestInstance = exports.GetDomainFromRequestHost = exports.ValidateDomainRedirect = exports.GetRequestData = exports.GetRequestDataByVCC = void 0;
const url = require("url");
const jwt = require("jsonwebtoken");
const log4js = require("log4js");
const jwt_config_1 = require("../config/jwt.config");
const vcc_model_1 = require("../models/vcc.model");
const domain_model_1 = require("../models/domain.model");
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const api_shared_1 = require("./api.shared");
const billing_implementation_1 = require("../implementation/billing.implementation");
let logger = log4js.getLogger('ServerScripts');
function GetRequestDataByVCC(req, res, next) {
    logger.info('[http.shared::GetRequestDataByVCC] Calling function GetRequestDataByVCC');
    let host = req.headers['host'] || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    res.locals.protocol = req.protocol;
    res.locals.host = req.headers['host'];
    res.locals.pageUrl = req.protocol + '://' + req.get('host') + req.originalUrl;
    res.locals.pageReferrer = req.headers['referer'];
    res.locals.origin = req.headers['origin'];
    let requestDomain = url.parse(host).hostname;
    res.locals.requestDomain = requestDomain;
    res.locals.requestDomain = requestDomain;
    let nameVCC;
    let credential = req.body.userId;
    let originalUrl = req.originalUrl;
    logger.info('[http.shared::GetRequestDataByVCC] Checking url: ' + originalUrl);
    if (originalUrl === '/auth/login') {
        logger.info('[http.shared::GetRequestDataByVCC] Url /auth/login');
        let credentials = credential.split('@');
        nameVCC = credentials.length > 1 ? credentials[1] : '';
        nameVCC = nameVCC !== 'system' ? nameVCC : '';
        if (credential === 'admin')
            nameVCC = 'system';
    }
    else {
        logger.info('[http.shared::GetRequestDataByVCC] Calling GetVccFromSession from GetRequestDataByVCC');
        let requestSession = (req.headers['authorization'] || '').replace('bearer ', '');
        let vcc = GetVccFromSession(requestSession);
        nameVCC = vcc;
    }
    logger.info('[http.shared::GetRequestDataByVCC] Got vcc ' + nameVCC);
    if (!nameVCC) {
        return next();
    }
    let requestData = new vcc_model_1.RequestData;
    redis_service_1.Redis.GetRequestData(nameVCC)
        .then(result => {
        if (result && result.vcc !== null) {
            logger.info('[http.shared::GetRequestDataByVCC] Received requestData for vcc from redis: ' + JSON.stringify(result.vcc));
            res.locals.vcc = result.vcc.vcc;
            res.locals.instance = result.vcc.vcc;
            res.locals.type = result.vcc.type;
            res.locals.domain = result.Domain;
            res.locals.api = {
                'speechmatics': result.api.find((x) => {
                    return x.apiName === 'SpeechMatics';
                }), 'idatha': result.api.find((x) => {
                    return x.apiName === 'IDATHA';
                })
            };
            res.locals.azure = result.azureSubscriptionKey.find((x) => {
                return x.subscriptionName === 'DEMO';
            });
            return next();
        }
        else {
            logger.info('[http.shared::GetRequestDataByVCC] RequestData for vcc doesn\'t exists on redis');
            logger.info('[http.shared::GetRequestDataByVCC] Getting vcc data from GetVCC local function');
            return GetVCC(nameVCC)
                .then(resultGetVCC => {
                requestData.vcc = resultGetVCC;
                logger.info('[http.shared::GetRequestDataByVCC] Received result from GetVCC local function');
                logger.info('[http.shared::GetRequestDataByVCC] Setting vcc and instance on res.local');
                res.locals.vcc = resultGetVCC.vcc;
                res.locals.instance = resultGetVCC.vcc;
                res.locals.type = resultGetVCC.type;
                if (resultGetVCC.vcc) {
                    logger.info('[http.shared::GetRequestDataByVCC] Getting domain');
                    return GetDomain(resultGetVCC.vcc, requestDomain);
                }
                else {
                    logger.info('[http.shared::GetRequestDataByVCC] Vcc doesn\'t exists on result');
                    return null;
                }
            })
                .then(resultDomain => {
                logger.info('[http.shared::GetRequestDataByVCC] Receiving result from GetDomain local function');
                logger.info('[http.shared::GetRequestDataByVCC] Setting domain on res.local');
                requestData.domain = resultDomain;
                res.locals.domain = resultDomain;
                logger.info('[http.shared::GetRequestDataByVCC] Calling GetApi function for system');
                return api_shared_1.GetAPIDataFromConfig('system');
            }).then((result) => {
                logger.info('[http.shared::GetRequestDataByVCC] Setting apis data on res.locals');
                requestData.api = result;
                res.locals.api = {
                    'speechmatics': result.find((x) => {
                        return x.apiName === 'SpeechMatics';
                    }), 'idatha': result.find((x) => {
                        return x.apiName === 'IDATHA';
                    })
                };
                return Promise.resolve(true);
            }).then(result => {
                logger.info('[http.shared::GetRequestDataByVCC] Calling MSSql.GetSubscriptionKeys for vcc ' + nameVCC);
                return mssql_service_1.MSSql.GetSubscriptionKeys(nameVCC);
            }).then(result => {
                logger.info('[http.shared::GetRequestDataByVCC] Setting MSSql.GetSubscriptionKeys result into res.locals.azure');
                requestData.azureSubscriptionKey = result;
                res.locals.azure = result.find((x) => {
                    return x.subscriptionName === 'Subscription_key_1';
                });
                logger.info('[http.shared::GetRequestDataByVCC] Saving request data on redis');
                redis_service_1.Redis.SaveRequestData(nameVCC, requestData);
                return next();
            })
                .catch((err) => {
                let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error executing request');
                logger.error('[http.shared::GetRequestDataByVCC] Error on \'' + req.path + '\': ' + errMsg);
                if (res.headersSent)
                    return;
                res.status(401).json({ description: 'Undefined domain (' + requestDomain + ')', error: 'VCC_SERVER_ERROR_NOT_FOUND' });
            });
        }
    })
        .catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error executing request');
        logger.error('[http.shared::GetRequestDataByVCC] Error on \'' + req.path + '\': ' + errMsg);
        if (res.headersSent)
            return;
        res.status(401).json({ description: 'Undefined domain (' + requestDomain + ')', error: 'VCC_SERVER_ERROR_NOT_FOUND' });
    });
}
exports.GetRequestDataByVCC = GetRequestDataByVCC;
function GetVccFromSession(session) {
    try {
        logger.info('[http.shared::GetVccFromSession] Checking if session exists');
        if (session) {
            logger.info('[http.shared::GetVccFromSession] Session exists');
            let decoded = jwt.verify(session, jwt_config_1.JWT_SECRET);
            if (decoded.vcc)
                return decoded.vcc.toLowerCase();
        }
    }
    catch (err) {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('[http.shared::GetVccFromSession] Error decoding session: ' + errMsg);
    }
    return null;
}
function GetRequestData(req, res, next) {
    let host = req.headers['host'] || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    res.locals.protocol = req.protocol;
    res.locals.host = req.headers['host'];
    res.locals.pageUrl = req.protocol + '://' + req.get('host') + req.originalUrl;
    res.locals.pageReferrer = req.headers['referer'];
    res.locals.origin = req.headers['origin'];
    let requestDomain = url.parse(host).hostname;
    res.locals.requestDomain = requestDomain;
    GetRequestInstance(host)
        .then(resultInstance => {
        res.locals.vcc = resultInstance;
        res.locals.instance = resultInstance;
        if (resultInstance) {
            return GetDomain(resultInstance, requestDomain);
        }
        else {
            return null;
        }
    }).then(resultDomain => {
        res.locals.domain = resultDomain;
        return GetVCC(res.locals.vcc);
    }).then(result => {
        res.locals.type = result.type;
        return next();
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
        if (res.headersSent)
            return;
        res.status(401).json({ description: 'Undefined domain (' + requestDomain + ')', error: 'VCC_SERVER_ERROR_NOT_FOUND' });
    });
}
exports.GetRequestData = GetRequestData;
function ValidateDomainRedirect(req, res, next) {
    let domain = res.locals.domain;
    if (domain && domain.isContentDomain && domain.contentDefaultRedirect && domain.contentDefaultRedirect !== req.url && !req.url.match('^\/(api|auth|public|internal|popup|form|builder|payments|preview|distributed|static|template|assets|favicon)(.*)$')) {
        res.redirect(301, domain.contentDefaultRedirect);
    }
    else {
        next();
    }
}
exports.ValidateDomainRedirect = ValidateDomainRedirect;
function GetDomainFromRequestHost(requestHost) {
    let host = requestHost || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    let requestDomain = url.parse(host).hostname;
    return requestDomain;
}
exports.GetDomainFromRequestHost = GetDomainFromRequestHost;
function GetRequestInstance(requestHost) {
    let host = requestHost || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    let domain = url.parse(host).hostname;
    return GetInstanceByDomain(domain)
        .then((vcc) => {
        if (vcc) {
            return Promise.resolve(vcc);
        }
        else {
            logger.warn('El dominio \'' + domain + '\' no esta asociado a ninguna instancia. Se usara instancia por defecto.');
            return Promise.resolve('default');
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al obtener instancia asociada al request: ' + errMsg);
        return Promise.resolve('default');
    });
}
exports.GetRequestInstance = GetRequestInstance;
function GetRequestUser(authorizationHeader) {
    let session = authorizationHeader || '';
    let userId;
    try {
        session = session.replace(/^Bearer /i, '');
        if (session) {
            let decoded = jwt.verify(session, jwt_config_1.JWT_SECRET);
            if (decoded.userId)
                userId = decoded.userId;
        }
    }
    catch (err) {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('Error al decodificar datos de la sesion: ' + errMsg);
    }
    return userId;
}
exports.GetRequestUser = GetRequestUser;
function GetInstanceByDomain(domain) {
    let cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetInstanceByDomain(domain)
        .then((redisInstance) => {
        if (redisInstance || redisInstance === '') {
            return Promise.resolve(redisInstance);
        }
        else {
            logger.info('El dato de GetInstanceByDomain para \'' + domain + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetInstanceByDomain(domain);
        }
    }).then((vcc) => {
        if (cacheUpdateNeeded)
            redis_service_1.Redis.SaveInstanceByDomain(domain, vcc || '');
        if (vcc || vcc === '') {
            return Promise.resolve(vcc);
        }
        else {
            throw 'Se consulto dato de GetInstanceByDomain para \'' + domain + '\' que no se encuentra definido en base de datos.';
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetInstanceByDomain: ' + errMsg);
    });
}
function GetDomain(vcc, domain) {
    let cacheUpdateNeeded = false;
    let defaultDomain = new domain_model_1.Domain(domain, domain, null, false, '80', null, false, false, null, false, null, null, null, null, null, null);
    return redis_service_1.Redis.GetDomain(vcc, domain)
        .then((redisDomain) => {
        if (redisDomain || JSON.stringify(redisDomain) === JSON.stringify(defaultDomain)) {
            return Promise.resolve(redisDomain);
        }
        else {
            logger.info('El dato de GetDomain para \'' + domain + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetDomain(vcc, domain);
        }
    }).then((resultDomain) => {
        if (cacheUpdateNeeded)
            redis_service_1.Redis.SaveDomain(vcc, resultDomain || defaultDomain);
        if (resultDomain || JSON.stringify(resultDomain) === JSON.stringify(defaultDomain)) {
            return Promise.resolve(resultDomain);
        }
        else {
            throw 'Se consulto dato de GetDomain para \'' + domain + '\' que no se encuentra definido en base de datos.';
        }
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar GetDomain: ' + errMsg);
    });
}
function GetVCC(id) {
    logger.info('[http.shared::GetVCC] Starting GetVCC function');
    let cacheUpdateNeeded = false;
    let vcc = {};
    let vccAux = {};
    logger.info('[http.shared::GetVCC] Checking if vcc exists on redis');
    return redis_service_1.Redis.GetVCC(id, id)
        .then((redisVCC) => {
        if (redisVCC) {
            logger.info('[http.shared::GetVCC] Received vcc exists on redis');
            return Promise.resolve(redisVCC);
        }
        else {
            logger.info('[http.shared::GetVCC] GetVCC for  \'' + id + '\' doen\'t exists on cache, requesting to database.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetVCC(id);
        }
    }).then((resultVCC) => {
        logger.info('[http.shared::GetVCC] Received result from database');
        if (resultVCC) {
            logger.info('[http.shared::GetVCC] Result from database exists');
            vcc = resultVCC;
            let billing = new billing_implementation_1.BillingImplementation();
            return billing.GetVCCStatus(vcc.vcc);
        }
        else {
            logger.info('[http.shared::GetVCC] Result from database doesn\' exists');
            throw 'Se consulto dato de GetVCC para \'' + id + '\' que no se encuentra definido en base de datos.';
        }
    }).then((result) => {
        let statusVCC = result;
        if (vcc && cacheUpdateNeeded && statusVCC) {
            logger.info('[http.shared::GetVCC] Saving database result on redis');
            let vccRedis = {};
            vccRedis.vcc = vcc.vcc;
            vccRedis.id = vcc.id;
            vccRedis.type = vcc.type;
            vccRedis.description = vcc.description;
            vccRedis.enableRealTime = vcc.enableRealTime;
            vccRedis.transcriptionDefaultLanguage = vcc.transcriptionDefaultLanguage;
            vccRedis.cognitiveEngine = vcc.cognitiveEngine;
            vccRedis.enableGenerativeEngine = vcc.enableGenerativeEngine;
            vccRedis.country = vcc.country;
            vccRedis.language = vcc.language;
            vccRedis.timeZone = vcc.timeZone;
            vccRedis.createdDate = vcc.createdDate;
            vccRedis.createdByUserId = vcc.createdByUserId;
            vccRedis.lastModifiedDate = vcc.lastModifiedDate;
            vccRedis.lastModifiedByUserId = vcc.lastModifiedByUserId;
            vccRedis.cognitiveEngineCharsLimit = vcc.cognitiveEngine === 'RASA' ? config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE : config_1.COGNITIVE_SERVICES_LUIS_CHARS_PER_PHRASE;
            vccRedis.products = [];
            statusVCC.products.forEach(x => {
                let product = {};
                product.product = x.product;
                product.availableLicense = x.valid;
                vccRedis.products.push(product);
            });
            redis_service_1.Redis.SaveVCC(id, vccRedis);
        }
        return Promise.resolve(vcc);
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error on request');
        logger.error('[http.shared::GetVCC] Error executing GetVCC: ' + errMsg);
        return Promise.reject(err);
    });
}
exports.GetVCC = GetVCC;
