"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetUserIdFromSession = exports.ValidateUserSession = void 0;
const jwt = require("jsonwebtoken");
const log4js = require("log4js");
const jwt_config_1 = require("../config/jwt.config");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const user_service_1 = require("../services/user.service");
const role_service_1 = require("../services/role.service");
let logger = log4js.getLogger('ServerScripts');
function ValidateUserSession(req, res, next) {
    if (!req.url.match('^\/(api|auth|builder)\/'))
        return next();
    let vcc = res.locals.vcc;
    let requestSession;
    let decodedRequestSession;
    let decodedStoredSession;
    let decodedUserId;
    let isLoginRequest = !!req.url.match('^\/auth\/login');
    let isLogoutRequest = !!req.url.match('^\/auth\/logout');
    let isPublicResource = !!req.url.match('^\/api\/public');
    if (isLoginRequest || isPublicResource)
        return next();
    let userId;
    let user;
    let roleId;
    let role;
    promises_shared_1.DummyPromise()
        .then(result => {
        if (req.url.match('^\/builder\/')) {
            requestSession = req.cookies['Session'];
        }
        else {
            requestSession = (req.headers['authorization'] || '').replace('bearer ', '');
        }
        if (!requestSession)
            throw 'SESSION_SERVER_ERROR_MISSING_TOKEN';
        res.locals.session = requestSession;
        if (res.locals.session && !req.cookies['Session'] && !isLogoutRequest) {
            res.cookie('Session', res.locals.session, { httpOnly: true });
        }
        userId = GetUserIdFromSession(res.locals.session);
        if (!userId)
            throw 'SESSION_SERVER_ERROR_CANNOT_GET_USER';
        res.locals.userId = userId;
        return user_service_1.GetUser(vcc, userId);
    }).then(resultUser => {
        if (!resultUser)
            throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
        user = resultUser;
        roleId = user.role;
        res.locals.user = user;
        return role_service_1.GetRole(vcc, roleId);
    }).then(resultRole => {
        if (!resultRole)
            throw 'AUTH_SERVER_ERROR_CANNOT_GET_SESSION';
        role = resultRole;
        res.locals.userRole = resultRole;
        try {
            decodedRequestSession = jwt.verify(requestSession, jwt_config_1.JWT_SECRET);
        }
        catch (e) {
            decodedRequestSession = null;
        }
        if (!decodedRequestSession)
            throw 'SESSION_SERVER_ERROR_CANNOT_DECODE_TOKEN';
        decodedUserId = (decodedRequestSession.userId || '').toLowerCase();
        return redis_service_1.Redis.GetSession(vcc, decodedUserId);
    }).then(storedSession => {
        if (storedSession === requestSession)
            return true;
        if (!storedSession && decodedRequestSession)
            return redis_service_1.Redis.SaveSession(vcc, decodedUserId, requestSession);
        try {
            decodedStoredSession = jwt.verify(storedSession, jwt_config_1.JWT_SECRET);
        }
        catch (e) {
            decodedStoredSession = null;
        }
        if (decodedRequestSession && decodedStoredSession && decodedRequestSession.iat >= decodedStoredSession.iat)
            return redis_service_1.Redis.SaveSession(vcc, decodedUserId, requestSession);
        logger.warn('Sesión inválida: El token recibido no es válido.');
        if (res.headersSent)
            return;
        if (req.url.match('^\/builder\/')) {
            res.status(401).type('txt').send('Invalid session');
        }
        else {
            res.status(401).json({ description: 'Invalid session token', error: 'Invalid session' });
        }
        return false;
    }).then(result => {
        if (result)
            return next();
    }).catch((err) => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
        if (res.headersSent)
            return;
        if (req.url.match('^\/builder\/')) {
            res.status(401).type('txt').send('Invalid session');
        }
        else {
            res.status(401).json({ description: 'Invalid session token', error: 'Invalid session' });
        }
    });
}
exports.ValidateUserSession = ValidateUserSession;
function GetUserIdFromSession(session) {
    try {
        if (session) {
            let decoded = jwt.verify(session, jwt_config_1.JWT_SECRET);
            if (decoded.userId)
                return decoded.userId.toLowerCase();
        }
    }
    catch (err) {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('Error el decodificar datos de la sesion: ' + errMsg);
    }
    return null;
}
exports.GetUserIdFromSession = GetUserIdFromSession;
